(************** Content-type: application/mathematica **************

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    612781,      16602]*)
(*NotebookOutlinePosition[    636525,      17349]*)
(*  CellTagsIndexPosition[    630749,      17164]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["TARCER 1.95", "Title",
  TextAlignment->Center,
  CellTags->"T1.1"],

Cell[TextData[{
  "A ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " program for the reduction of two-loop propagator integrals "
}], "Text",
  TextAlignment->Center],

Cell[TextData[{
  "Rolf Mertig\n",
  ButtonBox["Mertig Research & Consulting\n",
    ButtonData:>{
      URL[ "http://www.mertig.com"], None},
    ButtonStyle->"Hyperlink"],
  "and\nRainer Scharf\n",
  ButtonBox["Universit\[ADoubleDot]t Leipzig",
    ButtonData:>{
      URL[ "http://www.physik.uni-leipzig.de/~scharf/"], None},
    ButtonStyle->"Hyperlink"]
}], "Text",
  Editable->False,
  TextAlignment->Center,
  CellTags->"T1.2"],

Cell["April 23, 1999", "Text",
  TextAlignment->Center],

Cell["\<\
COMMENTS (FO 8.9.2002.): Changed context of TFI to \
HighEnergyPhysics`fcloops`TFI` and updated other HighEnergyPhysics`* \
contexts.\
\>", "Text"],

Cell["\<\
COMMENTS (FO 15.3.2001.): Changed PreFactor into Prefactor1 to \
avoid conflict with FeynArts and FeynCalc.\
\>", "Text"],

Cell["\<\
COMMENTS (FO 15.3.2001.): Changed context of Zeta2 and made \
definition dependent on not having loaded FeynCalc.\
\>", "Text"],

Cell[TextData[{
  "COMMENTS (RM 25.4.): ",
  ButtonBox["TFISimplify",
    ButtonData:>"T1.7.1.1",
    ButtonStyle->"Hyperlink"],
  " (like TFIRecurse, but without trule[9])."
}], "Text"],

Cell[TextData[{
  "COMMENTS (RM 23.4.): bug fixed in ",
  ButtonBox["tlrule[17]",
    ButtonData:>"tlrule17",
    ButtonStyle->"Hyperlink"],
  "."
}], "Text"],

Cell[TextData[{
  "COMMENTS (RM 6.4.):  ",
  ButtonBox["special TVR relations",
    ButtonData:>"specialTVR",
    ButtonStyle->"Hyperlink"],
  " added to TVR."
}], "Text"],

Cell[TextData[{
  "COMMENTS (RM 22.3.):  ",
  ButtonBox["TVI Symmetry",
    ButtonData:>"Tsymmetry",
    ButtonStyle->"Hyperlink"],
  " added to TVR."
}], "Text"],

Cell[TextData[{
  "COMMENTS (RM 18.3.): More rules added to ",
  ButtonBox["$TTable",
    ButtonData:>"$TTable",
    ButtonStyle->"Hyperlink"],
  ". "
}], "Text"],

Cell[TextData[{
  "COMMENTS (RM 18.2.): More rules added to ",
  ButtonBox["$TTable",
    ButtonData:>"$TTable",
    ButtonStyle->"Hyperlink"],
  ". "
}], "Text"],

Cell[TextData[{
  "COMMENTS (RM 21.2.): Included ",
  ButtonBox["$TTable",
    ButtonData:>"$TTable",
    ButtonStyle->"Hyperlink"],
  ". added tlrule[25]"
}], "Text"],

Cell["\<\
COMMENTS (RM 20.2.): $RankLimit = {0,6} ;  Changed TarcerRecurse \
such that special TJR and TVR (reducing D -> D-2) are used first.\
\>", "Text"],

Cell["\<\
COMMENTS (RM 15.2.): $RankLimit = {0,8} ;  
                                              The following test now works:
                                              
                                                 Print[\"test 1 :  TFI[D, pp, \
{0, 0, 0, 1, 3}, {{1, MW}, {1, MZ}, {0, 0}, {1, MZ}, {1,MW}}] = \",
                                                            Timing[t1 \
=TarcerRecurse[TFI[D, pp, {0, 0, 0, 1, 3}, {{1, MW}, {1, MZ}, {0, 0}, {1, \
MZ}, {1,MW}}]]
                                                                       ] // \
InputForm
                                                         ]

                                                  Print[\"test 2 :  TFI[d, \
pp, 0, {0, 0, 0, 0, 4},{{1, m1}, {1, m2}, 0, 0, 0}] = \",
                                                           Timing[t2 \
=TarcerRecurse[TFI[D, pp,  0, {0, 0, 0, 0, 4},{{1, m1}, {1, m2}, 0, 0, 0}]];
                                                                       ] // \
InputForm
                                                          ]
\
\>", "Text"],

Cell["\<\
COMMENTS (RM 13.2.): tlrule[11] and trule[12] made more \
restrictive: inhibiting thus a recursion if {n5,m5} equals {0,0}.\
\>", "Text"],

Cell["\<\
COMMENTS (RM 12.2.): tlrule[15] made safer (in order to not get   0 \
:> 0 );
                                             TFIRecurse made safer (to not \
get into a recursion);
                                                  There is still something \
wrong with the tlrule's ; since 
                                         TFIRecurse[TFI[D, pp, {0, 0, 0, 1, \
3}, {{1, MW}, {1, MZ}, {0, 0}, {1, MZ}, {1,   MW}}]]
                                         doesn't work ..\
\>", "Text"],

Cell["\<\
COMMENTS (RM 5.2.): tlrule[10] slightly changed to cover more \
cases.\
\>", "Text"],

Cell["\<\
COMMENTS (RM 2.2.):  FunctionExpand added in tlrule[9]. Option \
TimeConstraint added to TarcerRecurse (because TimeConstrained is buggy under \
Linux).\
\>", "Text"],

Cell[TextData[{
  "COMMENTS (RM 1.2.):  Changed TarcerRecurse. New TLR ",
  ButtonBox["definition",
    ButtonData:>"TLRdef",
    ButtonStyle->"Hyperlink"],
  ". \n                          default $RankLimit={0,6};"
}], "Text"],

Cell[BoxData[
    \(\($Remember = False;\)\)], "Input"],

Cell["\<\
COMMENTS (RM 19.1.):   changes are:  tlrule[22] and tlrule[23], \
minor efficiency changes (Expand3 = Identity) and other small programming \
optimizations.\
\>", "Text"],

Cell["\<\
COMMENTS (RM 22.1.):   changes are: bug in tlrule[23] fixed, new \
tlrule[24], new option Factor to TarcerReduce (doesn't change anything by \
default, but allows for improved Factor-ing).\
\>", "Text"],

Cell[TextData[{
  "See also: ",
  ButtonBox["hep-ph/9801383",
    ButtonData:>{
      URL[ "http://xxx.lanl.gov/abs/hep-ph/9801383"], None},
    ButtonStyle->"Hyperlink"],
  ", published in Comput. Phys. Commun. 111 (1998) 265-273"
}], "Text",
  Editable->False],

Cell[CellGroupData[{

Cell["Table of contents", "Section",
  CellDingbat->"\[FilledSmallCircle]"],

Cell[TextData[ButtonBox["Preliminaries",
  ButtonData:>"T1.1.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Usages",
  ButtonData:>"T1.2.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Creating the tarcerij.mx file",
  ButtonData:>"T1.2.1.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Prologue (setting of $RankLimit)",
  ButtonData:>"T1.3.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Typesetting rules",
  ButtonData:>"T1.3.1.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["The Palettes",
  ButtonData:>"T1.3.2.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["OperatorApply",
  ButtonData:>"T1.3.3.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MassDerivative",
  ButtonData:>"T1.3.4.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["TarasovT",
  ButtonData:>"T1.3.5.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["ApplyTarasovT",
  ButtonData:>"T1.3.6.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Misc",
  ButtonData:>"T1.3.7.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Colors",
  ButtonData:>"T1.3.7.1.1",
  ButtonStyle->"Hyperlink"]], "SmallText"],

Cell[TextData[ButtonBox["PQ",
  ButtonData:>"T1.3.7.2.1",
  ButtonStyle->"Hyperlink"]], "SmallText"],

Cell[TextData[ButtonBox["PNQ",
  ButtonData:>"T1.3.7.3.1",
  ButtonStyle->"Hyperlink"]], "SmallText"],

Cell[TextData[ButtonBox["FactorC",
  ButtonData:>"T1.3.7.4.1",
  ButtonStyle->"Hyperlink"]], "SmallText"],

Cell[TextData[ButtonBox["TFI",
  ButtonData:>"T1.3.7.5.1",
  ButtonStyle->"Hyperlink"]], "SmallText"],

Cell[TextData[ButtonBox["TJI",
  ButtonData:>"T1.3.7.6.1",
  ButtonStyle->"Hyperlink"]], "SmallText"],

Cell[TextData[ButtonBox["TBI",
  ButtonData:>"T1.3.7.7.1",
  ButtonStyle->"Hyperlink"]], "SmallText"],

Cell[TextData[ButtonBox["TAI",
  ButtonData:>"T1.3.7.8.1",
  ButtonStyle->"Hyperlink"]], "SmallText"],

Cell[TextData[ButtonBox["Definitions and notation",
  ButtonData:>"T1.4.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[StyleBox[ButtonBox["ToArgs",
  ButtonData:>"T1.4.1.1",
  ButtonStyle->"Hyperlink"],
  FontColor->RGBColor[0.2, 0.6, 1]]], "Text"],

Cell[TextData[StyleBox[ButtonBox["Cayley",
  ButtonData:>"T1.4.2.1",
  ButtonStyle->"Hyperlink"],
  FontColor->RGBColor[1, 0, 0.4]]], "Text"],

Cell[TextData[Cell[BoxData[
    FormBox[
      ButtonBox[
        StyleBox[
          RowBox[{
            RowBox[{
              StyleBox["Cayley",
                FontColor->RGBColor[1, 0, 0.4]], "[", "i", "]"}], 
            "=", \(\[CapitalDelta]\_i\)}],
          SingleLetterItalics->False],
        ButtonFunction:>(FrontEndExecute[ {
            FrontEnd`NotebookLocate[ #2]}]&),
        ButtonData:>"T1.4.3.1",
        Active->True,
        Background->GrayLevel[1],
        ButtonNote->ButtonData], TextForm]]]], "Text"],

Cell[TextData[Cell[BoxData[
    FormBox[
      ButtonBox[
        StyleBox[
          RowBox[{
            RowBox[{
              StyleBox["CayleyD",
                FontColor->RGBColor[1, 0.2, 1]], "[", "i", "]"}], 
            "=", \(\[CapitalDelta]\_i/\[CapitalDelta]\)}],
          SingleLetterItalics->False],
        ButtonFunction:>(FrontEndExecute[ {
            FrontEnd`NotebookLocate[ #2]}]&),
        ButtonData:>"T1.4.4.1",
        Active->True,
        Background->GrayLevel[1],
        ButtonNote->ButtonData], TextForm]]]], "Text"],

Cell[TextData[ButtonBox["Utility Functions",
  ButtonData:>"T1.5.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun and friends",
  ButtonData:>"T1.5.1.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Interchange",
  ButtonData:>"T1.5.1.1.1",
  ButtonStyle->"Hyperlink"]], "SmallText"],

Cell[TextData[{
  ButtonBox["\[Nu]",
    ButtonData:>"T1.5.2.1",
    ButtonStyle->"Hyperlink"],
  ButtonBox["explicit",
    ButtonData:>"T1.5.2.1",
    ButtonStyle->"Hyperlink"]
}], "Text"],

Cell[TextData[ButtonBox["TA",
  ButtonData:>"T1.5.2.1.1",
  ButtonStyle->"Hyperlink"]], "SmallText"],

Cell[TextData[ButtonBox["CheckRecursion",
  ButtonData:>"T1.5.3.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["CheckTVIRecursion",
  ButtonData:>"T1.5.3.1.1",
  ButtonStyle->"Hyperlink"]], "SmallText"],

Cell[TextData[ButtonBox["CheckTJIRecursion",
  ButtonData:>"T1.5.3.2.1",
  ButtonStyle->"Hyperlink"]], "SmallText"],

Cell[TextData[ButtonBox["Recurrence Relations Input and Recursion Function \
Generation",
  ButtonData:>"T1.6.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (35)",
  ButtonData:>"T1.6.1.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (35)",
  ButtonData:>"T1.6.2.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[Cell[BoxData[
    FormBox[
      ButtonBox[
        StyleBox[
          RowBox[{
            FormBox[\(1\^\(\(+\)\(\ \)\)\),
              "TraditionalForm"], \(\(\ \)(\(35  _\) 1)\)}],
          SingleLetterItalics->False],
        ButtonFunction:>(FrontEndExecute[ {
            FrontEnd`NotebookLocate[ #2]}]&),
        ButtonData:>"T1.6.2.1.1",
        Active->True,
        Background->GrayLevel[1],
        ButtonNote->ButtonData], TraditionalForm]]]], "SmallText"],

Cell[TextData[Cell[BoxData[
    FormBox[
      ButtonBox[
        StyleBox[
          RowBox[{
            FormBox[\(2\^\(\(+\)\(\ \)\)\),
              "TraditionalForm"], \(\(\ \)(\(35  _\) 2)\)}],
          SingleLetterItalics->False],
        ButtonFunction:>(FrontEndExecute[ {
            FrontEnd`NotebookLocate[ #2]}]&),
        ButtonData:>"T1.6.2.2.1",
        Active->True,
        Background->GrayLevel[1],
        ButtonNote->ButtonData], TraditionalForm]]]], "SmallText"],

Cell[TextData[Cell[BoxData[
    FormBox[
      ButtonBox[
        StyleBox[
          RowBox[{
            FormBox[\(3\^\(\(+\)\(\ \)\)\),
              "TraditionalForm"], \(\(\ \)(\(35  _\) 3)\)}],
          SingleLetterItalics->False],
        ButtonFunction:>(FrontEndExecute[ {
            FrontEnd`NotebookLocate[ #2]}]&),
        ButtonData:>"T1.6.2.3.1",
        Active->True,
        Background->GrayLevel[1],
        ButtonNote->ButtonData], TraditionalForm]]]], "SmallText"],

Cell[TextData[Cell[BoxData[
    FormBox[
      ButtonBox[
        StyleBox[
          RowBox[{
            FormBox[\(4\^\(\(+\)\(\ \)\)\),
              "TraditionalForm"], \(\(\ \)(\(35  _\) 4)\)}],
          SingleLetterItalics->False],
        ButtonFunction:>(FrontEndExecute[ {
            FrontEnd`NotebookLocate[ #2]}]&),
        ButtonData:>"T1.6.2.4.1",
        Active->True,
        Background->GrayLevel[1],
        ButtonNote->ButtonData], TraditionalForm]]]], "SmallText"],

Cell[TextData[ButtonBox["Input (3501)",
  ButtonData:>"T1.6.3.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[Cell[BoxData[
    FormBox[
      ButtonBox[
        StyleBox[
          
          RowBox[{\(\(MakeFun\ \((3501)\)\ \[CapitalDelta]\)\(=\)\(0\)\(\ \
\)\), 
            FormBox[\((p\^2 = m\^2)\),
              "TraditionalForm"]}],
          SingleLetterItalics->False],
        ButtonFunction:>(FrontEndExecute[ {
            FrontEnd`NotebookLocate[ #2]}]&),
        ButtonData:>"T1.6.4.1",
        Active->True,
        Background->GrayLevel[1],
        ButtonNote->ButtonData], TraditionalForm]]]], "Text"],

Cell[TextData[ButtonBox["Input (3502)",
  ButtonData:>"T1.6.5.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[Cell[BoxData[
    FormBox[
      ButtonBox[
        StyleBox[
          
          RowBox[{\(\(MakeFun\ \((3502)\)\ \[CapitalDelta]\)\(=\)\(0\)\(\ \
\)\), 
            FormBox[\((p\^2 = m\^2)\),
              "TraditionalForm"]}],
          SingleLetterItalics->False],
        ButtonFunction:>(FrontEndExecute[ {
            FrontEnd`NotebookLocate[ #2]}]&),
        ButtonData:>"T1.6.6.1",
        Active->True,
        Background->GrayLevel[1],
        ButtonNote->ButtonData], TraditionalForm]]]], "Text"],

Cell[TextData[Cell[BoxData[
    FormBox[
      ButtonBox[
        StyleBox[
          
          RowBox[{\(\(MakeFun\ \((3503)\)\ \[CapitalDelta]\)\(=\)\(0\)\(\ \
\)\), 
            FormBox[\((p\^2 = m\^2)\),
              "TraditionalForm"]}],
          SingleLetterItalics->False],
        ButtonFunction:>(FrontEndExecute[ {
            FrontEnd`NotebookLocate[ #2]}]&),
        ButtonData:>"T1.6.7.1",
        Active->True,
        Background->GrayLevel[1],
        ButtonNote->ButtonData], TraditionalForm]]]], "Text"],

Cell[TextData[Cell[BoxData[
    FormBox[
      ButtonBox[
        StyleBox[
          
          RowBox[{\(\(MakeFun\ \((3504)\)\ \[CapitalDelta]\)\(=\)\(0\)\(\ \
\)\), 
            FormBox[\((p\^2 = m\^2)\),
              "TraditionalForm"]}],
          SingleLetterItalics->False],
        ButtonFunction:>(FrontEndExecute[ {
            FrontEnd`NotebookLocate[ #2]}]&),
        ButtonData:>"T1.6.8.1",
        Active->True,
        Background->GrayLevel[1],
        ButtonNote->ButtonData], TraditionalForm]]]], "Text"],

Cell[TextData[ButtonBox["Input (41)",
  ButtonData:>"T1.6.9.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (41)",
  ButtonData:>"T1.6.10.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (43)",
  ButtonData:>"T1.6.11.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (44)",
  ButtonData:>"T1.6.12.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (45)",
  ButtonData:>"T1.6.13.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (43) and (44)",
  ButtonData:>"T1.6.14.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (50)",
  ButtonData:>"T1.6.15.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (51)",
  ButtonData:>"T1.6.16.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (51)",
  ButtonData:>"T1.6.17.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (52)",
  ButtonData:>"T1.6.18.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (52)",
  ButtonData:>"T1.6.19.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (53)",
  ButtonData:>"T1.6.20.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (54)",
  ButtonData:>"T1.6.21.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (54)",
  ButtonData:>"T1.6.22.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (55)",
  ButtonData:>"T1.6.23.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (55)",
  ButtonData:>"T1.6.24.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (56)",
  ButtonData:>"T1.6.25.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (56)",
  ButtonData:>"T1.6.26.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (59)",
  ButtonData:>"T1.6.27.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (59)",
  ButtonData:>"T1.6.28.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (60)",
  ButtonData:>"T1.6.29.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (60)",
  ButtonData:>"T1.6.30.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (61)",
  ButtonData:>"T1.6.31.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (61)",
  ButtonData:>"T1.6.32.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (6153)",
  ButtonData:>"T1.6.33.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (62)",
  ButtonData:>"T1.6.34.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (62)",
  ButtonData:>"T1.6.35.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (63)",
  ButtonData:>"T1.6.36.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (63)",
  ButtonData:>"T1.6.37.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (6552)",
  ButtonData:>"T1.6.38.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (6552)",
  ButtonData:>"T1.6.39.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (6553)",
  ButtonData:>"T1.6.40.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (66)",
  ButtonData:>"T1.6.41.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (66)",
  ButtonData:>"T1.6.42.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (alternative67)",
  ButtonData:>"T1.6.43.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (67)",
  ButtonData:>"T1.6.44.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (67)",
  ButtonData:>"T1.6.45.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (6713)",
  ButtonData:>"T1.6.46.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (68)",
  ButtonData:>"T1.6.47.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (68)",
  ButtonData:>"T1.6.48.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (6813)",
  ButtonData:>"T1.6.49.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (70)",
  ButtonData:>"T1.6.50.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (70)",
  ButtonData:>"T1.6.51.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (71)",
  ButtonData:>"T1.6.52.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (71)",
  ButtonData:>"T1.6.53.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (7123)",
  ButtonData:>"T1.6.54.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (7113)",
  ButtonData:>"T1.6.55.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (78)",
  ButtonData:>"T1.6.56.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (78)",
  ButtonData:>"T1.6.57.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (80)",
  ButtonData:>"T1.6.58.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (80)",
  ButtonData:>"T1.6.59.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (81)",
  ButtonData:>"T1.6.60.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (81)",
  ButtonData:>"T1.6.61.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (8123)",
  ButtonData:>"T1.6.62.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (8113)",
  ButtonData:>"T1.6.63.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (82)",
  ButtonData:>"T1.6.64.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (82)",
  ButtonData:>"T1.6.65.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (8213)",
  ButtonData:>"T1.6.66.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (8212)",
  ButtonData:>"T1.6.67.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (83)",
  ButtonData:>"T1.6.68.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (83)",
  ButtonData:>"T1.6.69.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (87)",
  ButtonData:>"T1.6.70.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["(88)",
  ButtonData:>"T1.6.70.1.1",
  ButtonStyle->"Hyperlink"]], "SmallText"],

Cell[TextData[ButtonBox["(89)",
  ButtonData:>"T1.6.70.2.1",
  ButtonStyle->"Hyperlink"]], "SmallText"],

Cell[TextData[ButtonBox["MakeFun (87)",
  ButtonData:>"T1.6.71.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (90)",
  ButtonData:>"T1.6.72.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (90)",
  ButtonData:>"T1.6.73.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (91)",
  ButtonData:>"T1.6.74.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (911)",
  ButtonData:>"T1.6.75.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (912)",
  ButtonData:>"T1.6.76.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (913)",
  ButtonData:>"T1.6.77.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (914)",
  ButtonData:>"T1.6.78.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (92)",
  ButtonData:>"T1.6.79.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (93)",
  ButtonData:>"T1.6.80.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (93)",
  ButtonData:>"T1.6.81.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (94)",
  ButtonData:>"T1.6.82.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (94)",
  ButtonData:>"T1.6.83.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (9401)",
  ButtonData:>"T1.6.84.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (9402)",
  ButtonData:>"T1.6.85.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (95)",
  ButtonData:>"T1.6.86.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (95)",
  ButtonData:>"T1.6.87.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (96)",
  ButtonData:>"T1.6.88.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (96)",
  ButtonData:>"T1.6.89.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (97)",
  ButtonData:>"T1.6.90.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (97)",
  ButtonData:>"T1.6.91.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (98)",
  ButtonData:>"T1.6.92.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (98)",
  ButtonData:>"T1.6.93.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (99)",
  ButtonData:>"T1.6.94.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (99)",
  ButtonData:>"T1.6.95.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (9912)",
  ButtonData:>"T1.6.96.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (9913)",
  ButtonData:>"T1.6.97.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (100)",
  ButtonData:>"T1.6.98.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (100)",
  ButtonData:>"T1.6.99.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (1002)",
  ButtonData:>"T1.6.100.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (1003)",
  ButtonData:>"T1.6.101.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (1004)",
  ButtonData:>"T1.6.102.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (1005)",
  ButtonData:>"T1.6.103.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (1006)",
  ButtonData:>"T1.6.104.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (150)",
  ButtonData:>"T1.6.105.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (150)",
  ButtonData:>"T1.6.106.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (151)",
  ButtonData:>"T1.6.107.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (151)",
  ButtonData:>"T1.6.108.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (152)",
  ButtonData:>"T1.6.109.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (152)",
  ButtonData:>"T1.6.110.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (153)",
  ButtonData:>"T1.6.111.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (153)",
  ButtonData:>"T1.6.112.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (200)",
  ButtonData:>"T1.6.113.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (200)",
  ButtonData:>"T1.6.114.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (201)",
  ButtonData:>"T1.6.115.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (202)",
  ButtonData:>"T1.6.116.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (203)",
  ButtonData:>"T1.6.117.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (204)",
  ButtonData:>"T1.6.118.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["MakeFun (205)",
  ButtonData:>"T1.6.119.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Input (1xxxx)",
  ButtonData:>"T1.6.120.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["five -> four",
  ButtonData:>"T1.6.121.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["four -> three",
  ButtonData:>"T1.6.122.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["three -> two",
  ButtonData:>"T1.6.123.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["two -> one",
  ButtonData:>"T1.6.124.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["TarcerRecurse and TFIRecurse",
  ButtonData:>"T1.7.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["TFIRecurse",
  ButtonData:>"T1.7.1.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["T*IS",
  ButtonData:>"T1.7.2.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["TComment",
  ButtonData:>"T1.7.3.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["TarcerRecurse",
  ButtonData:>"T1.7.4.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["TarcerExpand",
  ButtonData:>"T1.8.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Construct T's",
  ButtonData:>"T1.9.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["$BasisIntegrals",
  ButtonData:>"T1.10.1",
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[TextData[ButtonBox["Save",
  ButtonData:>"T1.9.1",
  ButtonStyle->"Hyperlink"]], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Preliminaries", "Section",
  CellDingbat->"\[FilledSmallCircle]",
  CellTags->"T1.1.1"],

Cell[TextData[{
  "This is free software.\nYou may use it, distribute it, change it, improve \
it, etc..\nIf you have ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " questions, ask them to support@wolfram.com or to your local ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " reseller."
}], "SmallText",
  Editable->False],

Cell["\<\
 We consider the following type of integrals, where a,b,u,v,r,s,t \
are non-negative integers.\
\>", "Text"],

Cell[TextData[{
  Cell[BoxData[
      \(TraditionalForm\`\(\(TFI\)\([\)\(d, 
        p\^2, \[CapitalDelta]p, {a, b}, {u, v, r, s, t}\)\)\)]],
  ", ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            RowBox[{
              RowBox[{
                RowBox[{
                  RowBox[{
                    RowBox[{
                      FormBox[\({{\[Nu]\_1, m\_1}, {\(\[Nu]\_2\)\(,\)\),
                        "TraditionalForm"], \(m\_2\)}], "}"}], 
                  ",", \({\[Nu]\_3, m\_3}\), ",", \({\[Nu]\_4, m\_4}\), 
                  ",", \({\[Nu]\_5, m\_5}\)}], "}"}], "]"}], " ", "="}], 
          " "}], TraditionalForm]]],
  "\n\n",
  Cell[BoxData[
      \(TraditionalForm\`1\/\[Pi]\^d\)]],
  "\[Integral] \[Integral] ",
  Cell[BoxData[
      FormBox[
        FractionBox[
          RowBox[{
            
            FormBox[\(\(d\^d\) k\_1\ \(d\^d\) 
                k\_2\ \(\((\[CapitalDelta]\[CenterDot]
                        k\_1)\)\^a\) \((\[CapitalDelta]\[CenterDot]k\_1)\)\^b\
\),
              
              "TraditionalForm"], \(\((k\_1\%2)\)\^u\), \(\((k\_2\%2)\)\^v\), \
\(\((p\[CenterDot]k\_1)\)\^r\), \(\((p\[CenterDot]
                    k\_2)\)\^s\), \(\((k\_1\[CenterDot]
                    k\_2)\)\^t\)}], \(\(\(\(\(\([k\_1\%2 - 
                                        m\_1\%2]\)\^\[Nu]\_1\)[
                                  k\_2\%2 - 
                                    m\_2\%2]\^\[Nu]\_2\)[\((k\_1 - p)\)\^2 - 
                              m\_3\%2]\^\[Nu]\_3\)[\((k\_2 - p)\)\^2 - 
                        m\_4\%2]\^\[Nu]\_4\)[\((k\_1 - k\_2)\)\^2 - 
                  m\_5\%2]\^\[Nu]\_5\)], TraditionalForm]]]
}], "Text"],

Cell["\<\
Putting the        $Post = (#;)&;       instruction after   
Unset[$Post]   suppresses all output cells.
Putting Unset[$Post] behind  $Post=(#:)&;
will produce a lot of typsetted reduction formulae.\
\>", "Text"],

Cell[BoxData[
    \(\(Unset[$Post];\)\)], "Input"],

Cell[BoxData[
    \(\($Post\  = \ \((\(#;\))\) &;\)\)], "Input"],

Cell[BoxData[
    \(\(If[$Notebooks =!= True, $Post\  = \ \((\(#;\))\) &];\)\)], "Input"],

Cell[BoxData[
    \(\($HistoryLength = 0;\)\)], "Input"],

Cell["\<\
This determines the directory into which the resulting file \
tarcer.mx is written.\
\>", "Text"],

Cell[BoxData[
    \(If[$Notebooks, 
      SetDirectory[
        ToFileName[
          First["\<FileName\>" /. 
              NotebookInformation[InputNotebook[]]]]]]\)], "Input"],

Cell[TextData[{
  "Somehow ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " crashes on my (R.M.) good old NeXTStation if I don't open all cells first \
...; this opens all cells automatically:"
}], "SmallText"],

Cell[BoxData[
    \(If[$System === "\<NeXT\>", 
      SelectionMove[EvaluationNotebook[], All, Notebook]; 
      FrontEndExecute[{FrontEnd`FrontEndToken[
            EvaluationNotebook[], "\<SelectionOpenAllGroups\>"]}]]\)], "Input"],

Cell[BoxData[
    \(\(Off[General::spell];\)\)], "Input"],

Cell[BoxData[
    \(\(Off[General::spell1];\)\)], "Input"],

Cell[CellGroupData[{

Cell["Creating the tarcerij.mx file", "Subsubsection",
  CellTags->"T1.2.1.1"],

Cell[TextData[{
  "Running this notebook will produce a tarcerij.mx file where ij will be \
replaced by \nthe two number set by the variable $RankLimit (default 33). \n\
In the same directory as this notebook which can be loaded  into ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " by    <<tarcer33.mx,\nor, when running in Terminal mode, on Unix systems, \
already when starting ",
  StyleBox["Mathematica, ",
    FontSlant->"Italic"],
  "as \nmyshellprompt>  math -initfile tarcer33.mx"
}], "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Usages", "Section",
  CellDingbat->"\[FilledSmallCircle]",
  CellTags->"T1.2.1"],

Cell[BoxData[
    \(\(BeginPackage["\<HighEnergyPhysics`Tarcer`\>"];\)\)], "Input"],

Cell[BoxData[
    \(\(TarcerRecurse::usage = "\<TarcerRecurse[expr] applies recurrence \
relations for all TFI's (and TVI, TJI, TBI, TAI) in expr. For evaluation and \
Laurent expansion of the resulting basis integrals use \
TarcerExpand.\>";\)\)], "Input"],

Cell[BoxData[
    \(\(TarcerExpand::usage = "\<TarcerExpand[expr, d -> (4+e), order] or \
TarcerExpand[epxr, d -> (4-e), order] or TarcerExpand[expr, d -> (4 - 2 \
om)substitutes integrals as specified by the option TarcerRules, and does a \
Laurent expansion in the variable around 0 at the right hand side of the \
second argument (i.e., you can use whatever convention you like ...). The \
same effect as TarcerExpand[TarcerRecurse[expr], ...] can be achieved by \
setting the option TarcerRecurse of TarcerExpand to True. TarcerExpand[expr, \
d -> (4+e)] is equivalent to TarcerExpand[expr, d -> (4+e), 0].\>";\)\)], \
"Input"],

Cell[BoxData[
    \(\(TarcerReduce::usage = "\<TarcerReduce is an option to TarcerExpand. \
It should be set to a list of basic integrals.\>";\)\)], "Input"],

Cell[BoxData[
    \(\(Cayley::usage = "\<Cayley[m1,m2,m3,m4,m5,Sqrt[pp]] is the Cayley \
determinant.\>";\)\)], "Input"],

Cell[BoxData[
    \(\(Cayleyu::usage = "\<Cayleyu are the sub-Cayley-determinants as \
described in Tarasov's paper.\>";\)\)], "Input"],

Cell[BoxData[
    \(\(CayleyD::usage = "\<CayleyD are the sub-Cayley-determinants as \
described in Tarasov's paper.\>";\)\)], "Input"],

Cell[BoxData[
    \(\(Prefactor1::usage = "\<Prefactor1[expr] may be used to prohibit \
TarcerExpand from Taylor-expanding expr.\>";\)\)], "Input"],

Cell[BoxData[
    \(\(SEpsilon::usage = "\<Sepsilon[d] is an abbreviation for Exp[(d-4)/2 \
EulerGamma].\>";\)\)], "Input"],

Cell[BoxData[
    \(\(HighEnergyPhysics`fcloops`TFI`TFI::usage = "\<TFI[d, pp, dp, \
{a,b},{u,v,r,s,t},{ {n1,m1},{n2,m2},{n3,m3},{n4,m4},{n5,m5} }] is the massive \
2-loop selfenergy type integral with five propagators in d dimensions. \n\t\t\
See the TARCER paper for a detailed description of the notation \
used.\>";\)\)], "Input"],

Cell[BoxData[
    \(\(TFIRecurse::usage = "\<TFIRecurse[expr] applies the recursion \
algorithm implemented in TARCER.nb (following Tarasov) to TFI's (and TVI's, \
TJI's, TBI's, TAI's in expr.\>";\)\)], "Input"],

Cell[BoxData[
    \(\(TFISimplify::usage = "\<TFISimplify[expr] applies the recursion \
algorithm implemented in TARCER.nb (following Tarasov) to TFI's (and TVI's, \
TJI's, TBI's, TAI's in expr, except k1.k2.\>";\)\)], "Input"],

Cell[BoxData[
    \(\(TVI::usage = "\<TVI[d,pp,{ {n1,m1},{n2,m2},{n3,m3},{n4,m4}}] is the \
massive 2-loop integral with four propagators in d dimensions. \>";\)\)], \
"Input"],

Cell[BoxData[
    \(\(TJI::usage = "\<TJI[d, pp, { {n1,m1},{n2,m2},{n3,m3} }] is the \
massive 2-loop integral with three propagators in d dimensions. \>";\)\)], \
"Input"],

Cell[BoxData[
    \(\(TKI::usage = "\<TKI[d, 0, { {n1,m1},{n2,m2},{n3,m3} }] is the bubble \
massive 2-loop selfenergy type integral with three propagators in d \
dimensions. \>";\)\)], "Input"],

Cell[BoxData[
    \(\(TBI::usage = "\<TBI[d, pp, { {n1,m1},{n2,m2} }] is the massive 1-loop \
selfenergy type integral with two propagators in d dimensions.\>";\)\)], \
"Input"],

Cell[BoxData[
    \(\(TAI::usage = "\<TAI[d, pp, { {n1,m1} }] is the massive 1-loop \
selfenergy type integral with one propagator in d dimensions.\>";\)\)], \
"Input"],

Cell[BoxData[
    \(\(TFR::usage = "\<TFR is similar in syntax to TFI but applies once the \
recurrence relations derived for TFI.\>";\)\)], "Input"],

Cell[BoxData[
    \(\(TVR::usage = "\<TVR is similar in syntax to TVI but applies once the \
recurrence relations derived for TVI.\>";\)\)], "Input"],

Cell[BoxData[
    \(\(TJR::usage = "\<TJR is similar in syntax to TJI but applies once the \
recurrence relations derived for TJI.\>";\)\)], "Input"],

Cell[BoxData[
    \(\(TBR::usage = "\<TBR is similar in syntax to TBI but applies once the \
recurrence relations derived for TBI.\>";\)\)], "Input"],

Cell[BoxData[
    \(\(TAR::usage = "\<TAR is similar in syntax to TAI but applies once the \
recurrence relations derived for TAI.\>";\)\)], "Input"],

Cell[BoxData[
    \(\($RankLimit::usage = "\<$RankLimit is the list of integers specifying \
the highest possible sums of exponents of scalar products; i.e., {a+b, r+s} \
means that at most (Delta.k1)^a * (Delta.k2)^2 * (p.k1)^r * (p.k2)^s in the \
numerator will be calculated (a,b,r,s have to be non-negative integares). \
$RankLimit has to be set in the TARCER.nb notebook. In the tarcer.mx file \
this is not possible anymore.\>";\)\)], "Input"],

Cell[BoxData[
    \(\($TarcerRecursed::usage = "\<If $Comment is set to True all integrals\n\
\t\twhich are reduced are collected in the global variable \
$TarcerRecured.\>";\)\)], "Input"],

Cell[BoxData[
    \(\($BasisIntegrals::usage = "\<$BasisIntegrals is a list of some basis \
integrals. It is the default setting of TarcerReduce of \
TarcerExpand.\>";\)\)], "Input"],

Cell[BoxData[
    \(\($Comment::usage = "\<$Comment can be set to True or False. If set to \
False no comments are generated.\>";\)\)], "Input"],

Cell[BoxData[
    \(\($CommentNotebook::usage = "\<$Comment can be set to True or False. \
Setting it to True will generate a separate notebook (if and only if $Comment \
is True) where the used recursion relations are displayed.\>";\)\)], "Input"],

Cell[BoxData[
    \(\($TTable::usage = "\<$TTable is a list of precomputed reductions. \
$TTable is the default setting of the option Table of TarcerRecurse. \
\>";\)\)], "Input"],

Cell[BoxData[
    \(\(Begin["\<`Private`\>"];\)\)], "Input"],

Cell[BoxData[
    \(\(Expand3 = Identity;\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Prologue (setting of $RankLimit)", "Section",
  CellDingbat->"\[FilledSmallCircle]",
  CellTags->"T1.3.1"],

Cell[TextData[{
  "This sets the limit up to which the ",
  Cell[BoxData[
      \(TraditionalForm\`T\)]],
  "-operators are constructed automatically, i.e., $RankLimit = {a+b, r+s} , \
concerning: ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(\((\[CapitalDelta]\[CenterDot]
              k\_1)\)\^a\)\(\ \)\)\)]],
  Cell[BoxData[
      \(TraditionalForm\`\((\[CapitalDelta]\[CenterDot]k\_2)\)\^b\)]],
  " ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          RowBox[{"(", 
            FormBox[\(p\[CenterDot]k\_1\),
              "TraditionalForm"], ")"}], "r"], TraditionalForm]]],
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          RowBox[{"(", 
            FormBox[\(p\[CenterDot]k\_2\),
              "TraditionalForm"], ")"}], "s"], TraditionalForm]]]
}], "Text"],

Cell[BoxData[
    \(\($RankLimit = {0, 6};\)\)], "Input"],

Cell["Check recursion relations by changing False to True here:", "Text"],

Cell[BoxData[
    \(\($CheckRecursion = False;\)\)], "Input"],

Cell["\<\
Change False to True for getting the input palettes for entering \
new recursion relations.\
\>", "Text"],

Cell[BoxData[
    \(\($PutPalettes = False;\)\)], "Input"],

Cell[TextData[{
  "Notice:  Instead of using shift operators",
  Cell[BoxData[
      FormBox[
        RowBox[{
          FormBox[
            RowBox[{"  ", 
              StyleBox[\(1\^+\),
                FontWeight->"Bold"]}],
            "TraditionalForm"], ",", 
          StyleBox[\(2\^+\),
            FontWeight->"Bold"]}], TraditionalForm]]],
  " etc. the \[Nu]-indices are colored blue; instead of ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          FormBox[
            RowBox[{"  ", 
              StyleBox[\(1\^-\),
                FontWeight->"Bold"]}],
            "TraditionalForm"], ",", 
          StyleBox[\(2\^-\),
            FontWeight->"Bold"]}], TraditionalForm]]],
  " etc. the \[Nu]-indices are colored red."
}], "Text"],

Cell[CellGroupData[{

Cell["Typesetting rules", "Subsection",
  CellTags->"T1.3.1.1"],

Cell[BoxData[
    \(\(atime = AbsoluteTime[];\)\)], "Input"],

Cell[BoxData[
    \(SEpsilon /: MakeBoxes[SEpsilon[d_], fmt_] := 
      InterpretationBox @@ {StyleBox[
            SubscriptBox["\<S\>", First[Variables[d]]], 
            FontWeight \[Rule] "\<Bold\>"], SEpsilon[d], 
          Editable \[Rule] False}\)], "Input"],

Cell[BoxData[
    \(MakeBoxes[SEpsilon[d_]\^n_, fmt_] := 
      InterpretationBox @@ {StyleBox[
            SubsuperscriptBox["\<S\>", First[Variables[d]], n], 
            FontWeight \[Rule] "\<Bold\>"], SEpsilon[d], 
          Editable \[Rule] False}\)], "Input"],

Cell[BoxData[
    \(Prefactor1 /: MakeBoxes[Prefactor1[d_], fmt_] := 
      InterpretationBox @@ {MakeBoxes[d, fmt], Prefactor1[d], 
          Editable -> False}\)], "Input"],

Cell["\<\
Don't mess with the InterpretationBox's !! (important for the \
palette and OperatorApply).\
\>", "Text"],

Cell[BoxData[
    \(MakeBoxes[Schiebe[i_, pm_String]\ , fmt_] := 
      InterpretationBox[
        StyleBox[SuperscriptBox[i, pm], FontWeight -> "\<Bold\>"], 
        Schiebe[i, pm], Editable -> True]\)], "Input"],

Cell[BoxData[
    \(\(CayleyD /: MakeBoxes[\(CayleyD[i_Integer]\)[args_], fmt_] := 
        FractionBox[
          SubscriptBox["\<\[CapitalDelta]\>", \ 
            i], "\<\[CapitalDelta]\>"];\)\)], "Input"],

Cell[BoxData[
    \(\(CayleyD /: MakeBoxes[\(CayleyD[i_Integer, j_Integer]\)[args_], fmt_] := 
        FractionBox[
          SubscriptBox["\<\[CapitalDelta]\>", 
            10\ i + j], "\<\[CapitalDelta]\>"];\)\)], "Input"],

Cell[BoxData[
    \(\(CayleyD /: 
        MakeBoxes[\(CayleyD[i_Integer, j_Integer, k_Integer]\)[args_], fmt_] := 
        FractionBox[
          SubscriptBox["\<\[CapitalDelta]\>", 
            100\ \ i + 10  j + k], "\<\[CapitalDelta]\>"];\)\)], "Input"],

Cell[BoxData[
    \(\(Cayley /: MakeBoxes[\(Cayley[i_Integer]\)[args_], fmt_] := 
        SubscriptBox["\<\[CapitalDelta]\>", \ i];\)\)], "Input"],

Cell[BoxData[
    \(\(Cayley /: MakeBoxes[\(Cayley[i_Integer, j_Integer]\)[args_], fmt_] := 
        SubscriptBox["\<\[CapitalDelta]\>", 10\ i + j];\)\)], "Input"],

Cell[BoxData[
    \(\(Cayley /: 
        MakeBoxes[\(Cayley[i_Integer, j_Integer, k_Integer]\)[args_], fmt_] := 
        SubscriptBox["\<\[CapitalDelta]\>", 
          100\ \ i + 10  j + k];\)\)], "Input"],

Cell[BoxData[
    \(\(Cayleyu /: 
        MakeBoxes[\(Cayleyu[i_Integer, j_Integer, k_Integer]\)[args_], fmt_] := 
        SubscriptBox["\<u\>", 100\ \ i + 10  j + k];\)\)], "Input"],

Cell["\<\
MakeBoxes[PP, fm_] := InterpretationBox[SuperscriptBox[p, 2], PP]; 
MakeBoxes[m1^(p_), _] := InterpretationBox[SubsuperscriptBox[m, 1, p], m1^p]; \

MakeBoxes[m2^(p_), _] := InterpretationBox[SubsuperscriptBox[m, 2, p], m2^p]; \

MakeBoxes[m3^(p_), _] := InterpretationBox[SubsuperscriptBox[m, 3, p], m3^p]; \

MakeBoxes[m4^(p_), _] := InterpretationBox[SubsuperscriptBox[m, 4, p], m4^p]; \

MakeBoxes[m5^(p_), _] := InterpretationBox[SubsuperscriptBox[m, 5, p], m5^p]; \

m6 = Sqrt[PP]; \
\>", "Input"],

Cell[BoxData[
    \(\(MakeBoxes[m1, _] := 
        InterpretationBox[SubscriptBox[m, 1], m1];\)\)], "Input"],

Cell[BoxData[
    \(\(MakeBoxes[m2, _] := 
        InterpretationBox[SubscriptBox[m, 2], m2];\)\)], "Input"],

Cell[BoxData[
    \(\(MakeBoxes[m3, _] := 
        InterpretationBox[SubscriptBox[m, 3], m3];\)\)], "Input"],

Cell[BoxData[
    \(\(MakeBoxes[m4, _] := 
        InterpretationBox[SubscriptBox[m, 4], m4];\)\)], "Input"],

Cell[BoxData[
    \(\(MakeBoxes[m5, _] := 
        InterpretationBox[SubscriptBox[m, 5], m5];\)\)], "Input"],

Cell[BoxData[
    \(\(mbt[z_] := ToBoxes[z, TraditionalForm];\)\)], "Input"],

Cell[BoxData[
    \(\(redblue[z_ /; Head[z] =!= Plus] := mbt[z];\)\)], "Input"],

Cell[BoxData[
    \(\(redblue[z_ - 1] := 
        StyleBox[mbt[z], FontColor \[Rule] RGBColor[1, 0, 0]];\)\)], "Input"],

Cell[BoxData[
    \(\(redblue[z_Subscript - 2] := 
        StyleBox[
          SubscriptBox[
            OverscriptBox[
              z\[LeftDoubleBracket]1\[RightDoubleBracket], "\<_\>"], 
            z\[LeftDoubleBracket]2\[RightDoubleBracket]], 
          FontColor \[Rule] RGBColor[1, 0, 0]];\)\)], "Input"],

Cell[BoxData[
    \(\(redblue[z_ - 2] := 
        StyleBox[OverscriptBox[mbt[z], "\<_\>"], 
          FontColor \[Rule] RGBColor[1, 0, 0]];\)\)], "Input"],

Cell[BoxData[
    \(\(redblue[z_ - 3] := 
        StyleBox[UnderscriptBox[OverscriptBox[mbt[z], "\<_\>"], "\<_\>"], 
          FontColor \[Rule] RGBColor[1, 0, 0]];\)\)], "Input"],

Cell[BoxData[
    \(\(redblue[z_ + 1] := 
        StyleBox[mbt[z], FontColor \[Rule] RGBColor[0, 0, 1]];\)\)], "Input"],

Cell[BoxData[
    \(\(redblue[z_ + 2] := 
        StyleBox[OverscriptBox[mbt[z], "\<_\>"], 
          FontColor \[Rule] RGBColor[0, 0, 1]];\)\)], "Input"],

Cell[BoxData[
    \(\(redblue[z_Subscript + 2] := 
        StyleBox[
          SubscriptBox[
            OverscriptBox[
              z\[LeftDoubleBracket]1\[RightDoubleBracket], "\<_\>"], 
            z\[LeftDoubleBracket]2\[RightDoubleBracket]], 
          FontColor \[Rule] RGBColor[0, 0, 1]];\)\)], "Input"],

Cell[BoxData[
    \(\(TFI /: 
        MakeBoxes[TFI[d_, pp_, {ur__}, {den__}], 
          fmt_] := \n\t\((InterpretationBox @@ {SubsuperscriptBox[
                  StyleBox["\<F\>", SingleLetterItalics -> False, 
                    FontWeight -> "\<Bold\>"], 
                  RowBox @@ {\((redblue /@ {den})\)}, 
                  RowBox[{"\<(\>", 
                      ToBoxes[d, TraditionalForm], "\<)\>", "\< \>", 
                      ur}]], \n\t\tTFI[d, pp, {ur}, {den}], 
                Editable -> True})\) /; 
          MemberQ[{StandardForm, TraditionalForm}, fmt];\)\)], "Input"],

Cell[BoxData[
    \(\(TFI /: 
        MakeBoxes[TFI[d_, pp_, dp_, any__, {den__}], 
          fmt_] := \n\t\(\((InterpretationBox @@ {SubsuperscriptBox[
                    StyleBox["\<F\>", SingleLetterItalics -> False, 
                      FontWeight -> "\<Bold\>"], 
                    RowBox @@ {\((redblue /@ {den})\)}, 
                    RowBox[{"\<(\>", 
                        ToBoxes[d, TraditionalForm], "\<)\>", "\< \>", 
                        Apply[Sequence, Flatten[{any}]]}]], \n\t\tTFI[d, pp, 
                    dp, any, {den}], Editable -> True})\) /; 
            MemberQ[{StandardForm, TraditionalForm}, fmt]\) /; 
          Head[dp] =!= List;\)\)], "Input"],

Cell[BoxData[
    \(\(TFI /: 
        MakeBoxes[TFI[dpp__, {den__}], 
          fmt_] := \n\t\((InterpretationBox @@ {SubsuperscriptBox[
                  StyleBox["\<F\>", SingleLetterItalics -> False, 
                    FontWeight -> "\<Bold\>"], 
                  RowBox @@ {\((redblue /@ {den})\)}, 
                  RowBox[{"\<(\>", 
                      ToBoxes[First[{dpp}], 
                        TraditionalForm], "\<)\>"}]], \n\t\tTFI[dpp, {den}], 
                Editable -> True})\) /; 
          MemberQ[{StandardForm, TraditionalForm}, fmt] && 
            Head[Last[{dpp}]] =!= List;\)\)], "Input"],

Cell[BoxData[
    \(\(TVI /: 
        MakeBoxes[TVI[dpp__, {den__}], 
          fmt_] := \n\((\t\((InterpretationBox @@ {SubsuperscriptBox[
                      StyleBox["\<V\>", SingleLetterItalics -> False, 
                        FontWeight -> "\<Bold\>"], 
                      RowBox @@ {\((redblue /@ {den})\)}, 
                      RowBox[{"\<(\>", 
                          ToBoxes[{dpp}[\([1]\)], 
                            TraditionalForm], "\<)\>"}]], \n\t\(Hold[TVI]\)[
                      dpp, {den}], Editable -> True})\) /. 
              Hold[TVI] -> TVI)\) /; 
          MemberQ[{StandardForm, TraditionalForm}, fmt] && 
            Head[Last[{dpp}]] =!= List;\)\)], "Input"],

Cell[BoxData[
    \({TVI[d, {\[Alpha], \[Beta], \[Gamma], \[Delta]}], 
      TVI[d, {\[Alpha] + 1, \[Beta], \[Gamma], \[Delta]}], 
      TVI[d, {\[Alpha] - 1, \[Beta], \[Gamma], \[Delta]}]}\)], "Input"],

Cell[BoxData[
    \(TJI /: 
      MakeBoxes[TJI[d_, 0, {den__}], 
        fmt_] := \t\((\n\((\((InterpretationBox @@ {SubsuperscriptBox[
                        StyleBox["\<K\>", SingleLetterItalics -> False, 
                          FontWeight -> "\<Bold\>"], 
                        RowBox @@ {\((redblue /@ {den})\)}, 
                        RowBox[{"\<(\>", 
                            ToBoxes[d, 
                              TraditionalForm], "\<)\>"}]], \n\t\t\(Hold[
                          TJI]\)[d, 0, {den}], Editable -> True})\) /. 
                Hold[TJI] -> TJI)\) /; 
            MemberQ[{StandardForm, TraditionalForm}, fmt]\n\t)\) /; 
        MemberQ[{StandardForm, TraditionalForm}, fmt]\)], "Input"],

Cell[BoxData[
    \(TJI /: 
      MakeBoxes[TJI[dpp__, {den__}], 
        fmt_] := \t\((\((InterpretationBox @@ {SubsuperscriptBox[
                    StyleBox["\<J\>", SingleLetterItalics -> False, 
                      FontWeight -> "\<Bold\>"], 
                    RowBox @@ {\((redblue /@ {den})\)}, 
                    RowBox[{"\<(\>", 
                        ToBoxes[{dpp}[\([1]\)], 
                          TraditionalForm], "\<)\>"}]], \n\t\t\(Hold[TJI]\)[
                    dpp, {den}], Editable -> True})\) /. Hold[TJI] -> TJI)\) /; 
        MemberQ[{StandardForm, TraditionalForm}, fmt]\)], "Input"],

Cell[BoxData[
    \(\(TKI /: 
        MakeBoxes[TKI[d_, pp___, {den__}], 
          fmt_] := \n\t\((SubsuperscriptBox[
              StyleBox["\<K\>", SingleLetterItalics -> False, 
                FontWeight -> "\<Bold\>"], 
              RowBox @@ {\((redblue /@ {den})\)}, 
              RowBox[{"\<(\>", ToBoxes[d, TraditionalForm], "\<)\>"}]])\) /; 
          MemberQ[{StandardForm, TraditionalForm}, fmt];\)\)], "Input"],

Cell[BoxData[
    \(TKI /: 
      MakeBoxes[TKI[dpp__, {den__}], 
        fmt_] := \t\((\((InterpretationBox @@ {SubsuperscriptBox[
                    StyleBox["\<K\>", SingleLetterItalics -> False, 
                      FontWeight -> "\<Bold\>"], 
                    RowBox @@ {\((redblue /@ {den})\)}, 
                    RowBox[{"\<(\>", 
                        ToBoxes[{dpp}[\([1]\)], 
                          TraditionalForm], "\<)\>"}]], \n\t\t\(Hold[TKI]\)[
                    dpp, {den}], Editable -> True})\) /. Hold[TKI] -> TKI)\) /; 
        MemberQ[{StandardForm, TraditionalForm}, fmt]\)], "Input"],

Cell[BoxData[
    \({TKI[d, {\[Alpha], \[Beta], \[Gamma]}], 
      TKI[d, {\[Alpha] + 1, \[Beta], \[Gamma]}], 
      TKI[d, {\[Alpha] - 1, \[Beta], \[Gamma]}]}\)], "Input"],

Cell[BoxData[
    \(\(TBI /: 
        MakeBoxes[TBI[d_, pp___, {den__}], 
          fmt_] := \n\t\((SubsuperscriptBox[
              StyleBox["\<B\>", SingleLetterItalics -> False, 
                FontWeight -> "\<Bold\>"], 
              RowBox @@ {\((redblue /@ {den})\)}, 
              RowBox[{"\<(\>", ToBoxes[d, TraditionalForm], "\<)\>"}]])\) /; 
          MemberQ[{StandardForm, TraditionalForm}, fmt];\)\)], "Input"],

Cell[BoxData[
    \(\(TBI /: 
        MakeBoxes[TBI[d_, pp___, {den__}], 
          fmt_] := \n\t\((InterpretationBox @@ {SubsuperscriptBox[
                  StyleBox["\<B\>", SingleLetterItalics -> False, 
                    FontWeight -> "\<Bold\>"], 
                  RowBox @@ {\((redblue /@ {den})\)}, 
                  RowBox[{"\<(\>", 
                      ToBoxes[d, TraditionalForm], "\<)\>"}]], \n\t\tTBI[d, 
                  pp, {den}], Editable -> True})\) /; 
          MemberQ[{StandardForm, TraditionalForm}, fmt];\)\)], "Input"],

Cell[BoxData[
    \({TBI[d, {\[Alpha], \[Beta]}], TBI[d, {\[Alpha] + 1, \[Beta]}], 
      TBI[d, {\[Alpha] - 1, \[Beta]}]}\)], "Input"],

Cell[BoxData[
    \(\(TAI /: 
        MakeBoxes[TAI[d_, pp___, {den__}], 
          fmt_] := \n\t\((InterpretationBox @@ {SubsuperscriptBox[
                  StyleBox["\<A\>", SingleLetterItalics -> False, 
                    FontWeight -> "\<Bold\>"], 
                  RowBox @@ {\((redblue /@ {den})\)}, 
                  RowBox[{"\<(\>", 
                      ToBoxes[d, TraditionalForm], "\<)\>"}]], \n\t\tTAI[
                  d, {den}], Editable -> True})\) /; 
          MemberQ[{StandardForm, TraditionalForm}, fmt];\)\)], "Input"],

Cell[BoxData[
    \({TAI[d, {\[Alpha]}], TAI[d, {\[Alpha] + 1}], 
      TAI[d, {\[Alpha] - 1}]}\)], "Input"],

Cell["\<\
T*IC are just like T*I , but used only for commentary in the 
recursion (TComment).\
\>", "Text"],

Cell[BoxData[
    \(\(TFIC /: 
        MakeBoxes[TFIC[d_, pp_, {0, 0, i_, 0, 0}, {den__}], 
          fmt_] := \n\t\((InterpretationBox @@ {SubsuperscriptBox[
                  StyleBox["\<F\>", SingleLetterItalics -> False, 
                    FontWeight -> "\<Bold\>"], 
                  RowBox @@ {\((redblue /@ {den})\)}, 
                  RowBox[{"\<(\>", 
                      ToBoxes[d, TraditionalForm], "\<)\>", "\< \>", 
                      i, "\<0\>"}]], \n\t\tTFIC[d, 
                  pp, {0, 0, i, 0, 0}, {den}], Editable -> True})\) /; 
          MemberQ[{StandardForm, TraditionalForm}, fmt];\)\)], "Input"],

Cell[BoxData[
    \(\(TFIC /: 
        MakeBoxes[TFIC[d_, pp_, {0, 0, 0, i_, 0}, {den__}], 
          fmt_] := \n\t\((InterpretationBox @@ {SubsuperscriptBox[
                  StyleBox["\<F\>", SingleLetterItalics -> False, 
                    FontWeight -> "\<Bold\>"], 
                  RowBox @@ {\((redblue /@ {den})\)}, 
                  RowBox[{"\<(\>", 
                      ToBoxes[d, TraditionalForm], "\<)\>", "\< \>", "\<0\>", 
                      i}]], \n\t\tTFIC[d, pp, {0, 0, 0, i, 0}, {den}], 
                Editable -> True})\) /; 
          MemberQ[{StandardForm, TraditionalForm}, fmt];\)\)], "Input"],

Cell[BoxData[
    \(\(TFIC /: 
        MakeBoxes[TFIC[dpp__, {den__}], 
          fmt_] := \n\t\((InterpretationBox @@ {SubsuperscriptBox[
                  StyleBox["\<F\>", SingleLetterItalics -> False, 
                    FontWeight -> "\<Bold\>"], 
                  RowBox @@ {\((redblue /@ {den})\)}, 
                  RowBox[{"\<(\>", 
                      ToBoxes[First[{dpp}], 
                        TraditionalForm], "\<)\>"}]], \n\t\tTFIC[dpp, {den}], 
                Editable -> True})\) /; 
          MemberQ[{StandardForm, TraditionalForm}, fmt] && 
            Head[Last[{dpp}]] =!= List;\)\)], "Input"],

Cell[BoxData[
    \(\(TVIC /: 
        MakeBoxes[TVIC[dpp__, {den__}], 
          fmt_] := \n\((\t\((InterpretationBox @@ {SubsuperscriptBox[
                      StyleBox["\<V\>", SingleLetterItalics -> False, 
                        FontWeight -> "\<Bold\>"], 
                      RowBox @@ {\((redblue /@ {den})\)}, 
                      RowBox[{"\<(\>", 
                          ToBoxes[{dpp}[\([1]\)], 
                            TraditionalForm], "\<)\>"}]], \n\t\(Hold[TVIC]\)[
                      dpp, {den}], Editable -> True})\) /. 
              Hold[TVIC] -> TVIC)\) /; 
          MemberQ[{StandardForm, TraditionalForm}, fmt] && 
            Head[Last[{dpp}]] =!= List;\)\)], "Input"],

Cell[BoxData[
    \(TJIC /: 
      MakeBoxes[TJIC[d_, 0, {den__}], 
        fmt_] := \t\((\n\((\((InterpretationBox @@ {SubsuperscriptBox[
                        StyleBox["\<K\>", SingleLetterItalics -> False, 
                          FontWeight -> "\<Bold\>"], 
                        RowBox @@ {\((redblue /@ {den})\)}, 
                        RowBox[{"\<(\>", 
                            ToBoxes[d, 
                              TraditionalForm], "\<)\>"}]], \n\t\t\(Hold[
                          TJIC]\)[d, 0, {den}], Editable -> True})\) /. 
                Hold[TJIC] -> TJIC)\) /; 
            MemberQ[{StandardForm, TraditionalForm}, fmt]\n\t)\) /; 
        MemberQ[{StandardForm, TraditionalForm}, fmt]\)], "Input"],

Cell[BoxData[
    \(TJIC /: 
      MakeBoxes[TJIC[dpp__, {den__}], 
        fmt_] := \t\((\((InterpretationBox @@ {SubsuperscriptBox[
                    StyleBox["\<J\>", SingleLetterItalics -> False, 
                      FontWeight -> "\<Bold\>"], 
                    RowBox @@ {\((redblue /@ {den})\)}, 
                    RowBox[{"\<(\>", 
                        ToBoxes[{dpp}[\([1]\)], 
                          TraditionalForm], "\<)\>"}]], \n\t\t\(Hold[TJIC]\)[
                    dpp, {den}], Editable -> True})\) /. 
            Hold[TJIC] -> TJIC)\) /; 
        MemberQ[{StandardForm, TraditionalForm}, fmt]\)], "Input"],

Cell[BoxData[
    \(\(TBIC /: 
        MakeBoxes[TBIC[d_, pp___, {den__}], 
          fmt_] := \n\t\((SubsuperscriptBox[
              StyleBox["\<B\>", SingleLetterItalics -> False, 
                FontWeight -> "\<Bold\>"], 
              RowBox @@ {\((redblue /@ {den})\)}, 
              RowBox[{"\<(\>", ToBoxes[d, TraditionalForm], "\<)\>"}]])\) /; 
          MemberQ[{StandardForm, TraditionalForm}, fmt];\)\)], "Input"],

Cell[BoxData[
    \(\(TBIC /: 
        MakeBoxes[TBIC[d_, pp___, {den__}], 
          fmt_] := \n\t\((InterpretationBox @@ {SubsuperscriptBox[
                  StyleBox["\<B\>", SingleLetterItalics -> False, 
                    FontWeight -> "\<Bold\>"], 
                  RowBox @@ {\((redblue /@ {den})\)}, 
                  RowBox[{"\<(\>", 
                      ToBoxes[d, TraditionalForm], "\<)\>"}]], \n\t\tTBIC[d, 
                  pp, {den}], Editable -> True})\) /; 
          MemberQ[{StandardForm, TraditionalForm}, fmt];\)\)], "Input"],

Cell[BoxData[
    \(\(TAIC /: 
        MakeBoxes[TAIC[d_, pp___, {den__}], 
          fmt_] := \n\t\((InterpretationBox @@ {SubsuperscriptBox[
                  StyleBox["\<A\>", SingleLetterItalics -> False, 
                    FontWeight -> "\<Bold\>"], 
                  RowBox @@ {\((redblue /@ {den})\)}, 
                  RowBox[{"\<(\>", 
                      ToBoxes[d, TraditionalForm], "\<)\>"}]], \n\t\tTAIC[
                  d, {den}], Editable -> True})\) /; 
          MemberQ[{StandardForm, TraditionalForm}, fmt];\)\)], "Input"],

Cell[BoxData[
    \(\(TFR /: 
        MakeBoxes[TFR[d_, pp_, {0, 0, i_, 0, 0}, {den__}], 
          fmt_] := \n\t\((InterpretationBox @@ {SubsuperscriptBox[
                  StyleBox["\<F\>", SingleLetterItalics -> False, 
                    FontWeight -> "\<Bold\>"], 
                  RowBox @@ {\((redblue /@ {den})\)}, 
                  RowBox[{"\<(\>", 
                      ToBoxes[d, TraditionalForm], "\<)\>", "\< \>", 
                      i, "\<0\>"}]], \n\t\tTFR[d, 
                  pp, {0, 0, i, 0, 0}, {den}], Editable -> True})\) /; 
          MemberQ[{StandardForm, TraditionalForm}, fmt];\)\)], "Input"],

Cell[BoxData[
    \(\(TFR /: 
        MakeBoxes[TFR[d_, pp_, {0, 0, 0, i_, 0}, {den__}], 
          fmt_] := \n\t\((InterpretationBox @@ {SubsuperscriptBox[
                  StyleBox["\<F\>", SingleLetterItalics -> False, 
                    FontWeight -> "\<Bold\>"], 
                  RowBox @@ {\((redblue /@ {den})\)}, 
                  RowBox[{"\<(\>", 
                      ToBoxes[d, TraditionalForm], "\<)\>", "\< \>", "\<0\>", 
                      i}]], \n\t\tTFR[d, pp, {0, 0, 0, i, 0}, {den}], 
                Editable -> True})\) /; 
          MemberQ[{StandardForm, TraditionalForm}, fmt];\)\)], "Input"],

Cell[BoxData[
    \(\(TFR /: 
        MakeBoxes[TFR[dpp__, {den__}], 
          fmt_] := \n\t\((InterpretationBox @@ {SubsuperscriptBox[
                  StyleBox["\<F\>", SingleLetterItalics -> False, 
                    FontWeight -> "\<Bold\>"], 
                  RowBox @@ {\((redblue /@ {den})\)}, 
                  RowBox[{"\<(\>", 
                      ToBoxes[First[{dpp}], 
                        TraditionalForm], "\<)\>"}]], \n\t\tTFR[dpp, {den}], 
                Editable -> True})\) /; 
          MemberQ[{StandardForm, TraditionalForm}, fmt] && 
            Head[Last[{dpp}]] =!= List;\)\)], "Input"],

Cell[BoxData[
    \(\(TVR /: 
        MakeBoxes[TVR[dpp__, {den__}], 
          fmt_] := \n\((\t\((InterpretationBox @@ {SubsuperscriptBox[
                      StyleBox["\<V\>", SingleLetterItalics -> False, 
                        FontWeight -> "\<Bold\>"], 
                      RowBox @@ {\((redblue /@ {den})\)}, 
                      RowBox[{"\<(\>", 
                          ToBoxes[{dpp}[\([1]\)], 
                            TraditionalForm], "\<)\>"}]], \n\t\(Hold[TVR]\)[
                      dpp, {den}], Editable -> True})\) /. 
              Hold[TVR] -> TVR)\) /; 
          MemberQ[{StandardForm, TraditionalForm}, fmt] && 
            Head[Last[{dpp}]] =!= List;\)\)], "Input"],

Cell[BoxData[
    \(TJR /: 
      MakeBoxes[TJR[d_, 0, {den__}], 
        fmt_] := \t\((\n\((\((InterpretationBox @@ {SubsuperscriptBox[
                        StyleBox["\<K\>", SingleLetterItalics -> False, 
                          FontWeight -> "\<Bold\>"], 
                        RowBox @@ {\((redblue /@ {den})\)}, 
                        RowBox[{"\<(\>", 
                            ToBoxes[d, 
                              TraditionalForm], "\<)\>"}]], \n\t\t\(Hold[
                          TJR]\)[d, 0, {den}], Editable -> True})\) /. 
                Hold[TJR] -> TJR)\) /; 
            MemberQ[{StandardForm, TraditionalForm}, fmt]\n\t)\) /; 
        MemberQ[{StandardForm, TraditionalForm}, fmt]\)], "Input"],

Cell[BoxData[
    \(TJR /: 
      MakeBoxes[TJR[dpp__, {den__}], 
        fmt_] := \t\((\((InterpretationBox @@ {SubsuperscriptBox[
                    StyleBox["\<J\>", SingleLetterItalics -> False, 
                      FontWeight -> "\<Bold\>"], 
                    RowBox @@ {\((redblue /@ {den})\)}, 
                    RowBox[{"\<(\>", 
                        ToBoxes[{dpp}[\([1]\)], 
                          TraditionalForm], "\<)\>"}]], \n\t\t\(Hold[TJR]\)[
                    dpp, {den}], Editable -> True})\) /. Hold[TJR] -> TJR)\) /; 
        MemberQ[{StandardForm, TraditionalForm}, fmt]\)], "Input"],

Cell[BoxData[
    \(\(TBR /: 
        MakeBoxes[TBR[d_, pp___, {den__}], 
          fmt_] := \n\t\((SubsuperscriptBox[
              StyleBox["\<B\>", SingleLetterItalics -> False, 
                FontWeight -> "\<Bold\>"], 
              RowBox @@ {\((redblue /@ {den})\)}, 
              RowBox[{"\<(\>", ToBoxes[d, TraditionalForm], "\<)\>"}]])\) /; 
          MemberQ[{StandardForm, TraditionalForm}, fmt];\)\)], "Input"],

Cell[BoxData[
    \(\(TBR /: 
        MakeBoxes[TBR[d_, pp___, {den__}], 
          fmt_] := \n\t\((InterpretationBox @@ {SubsuperscriptBox[
                  StyleBox["\<B\>", SingleLetterItalics -> False, 
                    FontWeight -> "\<Bold\>"], 
                  RowBox @@ {\((redblue /@ {den})\)}, 
                  RowBox[{"\<(\>", 
                      ToBoxes[d, TraditionalForm], "\<)\>"}]], \n\t\tTBR[d, 
                  pp, {den}], Editable -> True})\) /; 
          MemberQ[{StandardForm, TraditionalForm}, fmt];\)\)], "Input"],

Cell[BoxData[
    \(\(TAR /: 
        MakeBoxes[TAR[d_, pp___, {den__}], 
          fmt_] := \n\t\((InterpretationBox @@ {SubsuperscriptBox[
                  StyleBox["\<A\>", SingleLetterItalics -> False, 
                    FontWeight -> "\<Bold\>"], 
                  RowBox @@ {\((redblue /@ {den})\)}, 
                  RowBox[{"\<(\>", 
                      ToBoxes[d, TraditionalForm], "\<)\>"}]], \n\t\tTAR[
                  d, {den}], Editable -> True})\) /; 
          MemberQ[{StandardForm, TraditionalForm}, fmt];\)\)], "Input"],

Cell[BoxData[
    \(\(n1 /: MakeBoxes[n1, TraditionalForm] := 
        SubscriptBox["\<\[Nu]\>", 1];\)\)], "Input"],

Cell[BoxData[
    \(\(n2 /: MakeBoxes[n2, TraditionalForm] := 
        SubscriptBox["\<\[Nu]\>", 2];\)\)], "Input"],

Cell[BoxData[
    \(\(n3 /: MakeBoxes[n3, TraditionalForm] := 
        SubscriptBox["\<\[Nu]\>", 3];\)\)], "Input"],

Cell[BoxData[
    \(\(n4 /: MakeBoxes[n4, TraditionalForm] := 
        SubscriptBox["\<\[Nu]\>", 4];\)\)], "Input"],

Cell[BoxData[
    \(\(n5 /: MakeBoxes[n5, TraditionalForm] := 
        SubscriptBox["\<\[Nu]\>", 5];\)\)], "Input"],

Cell[BoxData[
    \(\(MakeBoxes[ParD[i_]\^j_, TraditionalForm] := 
        SubsuperscriptBox["\<\[PartialD]\>", i, j];\)\)], "Input"],

Cell[BoxData[
    \(\(MakeBoxes[ParD[i_], TraditionalForm] ^= 
        SubscriptBox["\<\[PartialD]\>", i];\)\)], "Input"],

Cell[BoxData[
    \(\(MakeBoxes[DPlus, TraditionalForm] := 
        StyleBox[SuperscriptBox["\<d\>", "\<+\>"], 
          FontWeight \[Rule] "\<Bold\>", 
          SingleLetterItalics \[Rule] False];\)\)], "Input"],

Cell[BoxData[
    \(MakeBoxes[TpD[_, b_], TraditionalForm] := 
      MakeBoxes[b, TraditionalForm]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["The Palettes", "Subsection",
  CellTags->"T1.3.2.1"],

Cell[BoxData[
    \(\(If[$PutPalettes === 
          True, \n\t\(\(tobut[a_]\  := \ 
            ButtonBox[ToBoxes[a, StandardForm], \ 
              ButtonStyle -> "\<Paste\>"];\)\n
        \ \(If[Head[pal1] === NotebookObject, \n\tNotebookClose[pal1]];\)\n
        \t\(pal1 = 
            NotebookPut[
              Notebook[{Cell[
                    BoxData[
                      GridBox[\n\t\t\t\t\t\t\tJoin[
                          Transpose@{\n\t\t\t\t\t\t\t\t\tTable[
                                tobut@\(m\_j\^2\), {j, 
                                  5}], \n\t\t\t\t\t\t\t\t\tTable[
                                tobut@\[Nu]\_j, {j, 
                                  5}], \n\t\t\t\t\t\t\t\t\tTable[
                                tobut@Schiebe[i, "\<+\>"], {i, 
                                  5}], \n\t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
Table[tobut@Schiebe[i, "\<-\>"], {i, 5}]}, \n\t\t\t\t\t\t\t\(\({\t
                            tobut /@ {PP, \[CapitalDelta]\_\[Placeholder], 
                                u\_\[Placeholder], \[ScriptCapitalC]\ }}\)\(\n\
\)
                          \)\t\t\t\t\t\t\t], 
                        RowSpacings -> 0, \n\ \ \ \ \ \ ColumnSpacings -> 
                          0]], NotebookDefault]}, \n\tEditable -> True, \n
                WindowToolbars -> {}, \nWindowSize -> {Fit, \ Fit}, \n
                WindowMargins -> {{0, \ Automatic}, \ {0, Automatic}}, \n
                WindowFrame -> "\<Palette\>", \nWindowElements -> {}, \n
                WindowFrameElements -> "\<CloseBox\>", \n
                WindowClickSelect -> False, \n
                ScrollingOptions -> {"\<PagewiseScrolling\>" -> True}, \n
                ShowCellBracket -> False, \n
                CellMargins -> {{0, \ 0}, \ {0, \ 0}}, \nActive -> True, \n
                CellOpen -> True, \nShowCellLabel -> False, \n
                ShowCellTags -> False, \n
                ImageMargins -> {{0, \ 0}, \ {0, \ 0}}, \nMagnification -> 2, 
                Visible -> True]];\)\n
        \(SetOptions[pal1, 
            WindowMargins -> {{0, Automatic}, {Automatic, 0}}];\)\n
        \(SetOptions[pal1, Visible -> True, WindowSize -> {Fit, Fit}];\)\n
        \ \(If[Head[pal2] === NotebookObject, \n\tNotebookClose[pal2]];\)\n
        \t\(pal2 = 
            NotebookPut[
              Notebook[{Cell[
                    BoxData[
                      GridBox[\n\t\t\t\t\t\t\tJJJ = 
                          Join[{tobut /@ {TFI[
                                    d + 2, {\[Nu]\_1, \[Nu]\_2, \[Nu]\_3, \
\[Nu]\_4, \[Nu]\_5}], TVI[d + 2, {\[Nu]\_1, \[Nu]\_2, \[Nu]\_3, \[Nu]\_4}], 
                                  TJI[d + 2, {\[Nu]\_1, \[Nu]\_2, \[Nu]\_3}], 
                                  TKI[d, {\[Nu]\_1, \[Nu]\_2, \[Nu]\_3}], 
                                  TBI[d, {\[Nu]\_1, \[Nu]\_2}], 
                                  TAI[d, {\[Nu]\_1}]}}, \n\t\t\t\t\t\t\t\t\
\(\({tobut /@ {TFI[d, {\[Nu]\_1, \[Nu]\_2, \[Nu]\_3, \[Nu]\_4, \[Nu]\_5}], 
                                  TVI[d, {\[Nu]\_1, \[Nu]\_2, \[Nu]\_3, \[Nu]\
\_4}], TJI[d, {\[Nu]\_1, \[Nu]\_2, \[Nu]\_3}], 
                                  TKI[d - 2, {\[Nu]\_1, \[Nu]\_2, \[Nu]\_3}], 
                                  TBI[d - 2, {\[Nu]\_1, \[Nu]\_2}], 
                                  TAI[d - 2, {\[Nu]\_1}]}}\)\(\n\)
                            \)\t\t\t\t\t\t\t], 
                        RowSpacings -> 0, \n\ \ \ \ \ \ ColumnSpacings -> 
                          0]], NotebookDefault]}, \n\tEditable -> True, \n
                WindowToolbars -> {}, \nWindowSize -> {FitAll, \ FitAll}, \n
                WindowMargins -> {{0, Automatic}, \ {0, Automatic}}, \n
                WindowFrame -> "\<Palette\>", \nWindowElements -> {}, \n
                WindowFrameElements -> "\<CloseBox\>", \n
                WindowClickSelect -> False, \n
                ScrollingOptions -> {"\<PagewiseScrolling\>" -> True}, \n
                ShowCellBracket -> False, \n
                CellMargins -> {{0, 0}, \ {0, 0}}, \nActive -> True, \n
                CellOpen -> True, \nShowCellLabel -> False, \n
                ShowCellTags -> False, \n
                ImageMargins -> {{0, 0}, \ {0, \ 0}}, \nMagnification -> 2, 
                Visible -> True]];\)\n
        \(SetOptions[pal2, 
            WindowMargins -> {{Automatic, 15}, {Automatic, 4}}];\)\n
        \(SetOptions[pal2, Visible -> True, WindowSize -> {Fit, Fit}];\)\n
        \tSetOptions[pal2, WindowSize -> {Fit, Fit}]\n
        \)\t];\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["OperatorApply", "Subsection",
  CellTags->"T1.3.3.1"],

Cell[BoxData[
    \(\(OperatorApplyF[x_ == y_] := 
        Block[{sch}, \n\t\tsch\  = \ {Schiebe[i_, "\<+\>"]\^ni_. \ TFI[
                    d_, {n15__}] :> \n\t\t\t\t\tTFI[d, 
                  ReplacePart[{n15}, {n15}[\([i]\)] + ni, 
                    i]], \n\t\t\t\t\(\(Schiebe[i_, "\<-\>"]\^ni_. \ TFI[
                    d_, {n15__}] :> \n\t\t\t\t\tTFI[d, 
                  ReplacePart[{n15}, {n15}[\([i]\)] - ni, i]]\)\(\n\)
              \)\t\t\t}; 
          Map[Collect[Expand[#] //. sch, TFI[__], Factor] &, \n\t\t\tx == 
              y]];\)\)], "Input"],

Cell[BoxData[
    \(\(OperatorApplyV[x_ == y_] := 
        Block[{sch}, \n\t\tsch\  = \ {Schiebe[i_, "\<+\>"]\^ni_. \ TVI[
                    d_, {n15__}] :> \n\t\t\t\t\tTVI[d, 
                  ReplacePart[{n15}, {n15}[\([i]\)] + ni, 
                    i]], \n\t\t\t\t\(\(Schiebe[i_, "\<-\>"]\^ni_. \ TVI[
                    d_, {n15__}]\)\(:>\)\(\n\)\(\t\t\t\t\t\)\(TVI[d, 
                  ReplacePart[{n15}, {n15}[\([i]\)] - ni, 
                    i]]\)\(\n\)\)\t\t\t}; 
          Map[Collect[Expand[#] //. sch, TVI[__], Factor] &, \n\t\t\tx == 
              y]];\)\)], "Input"],

Cell[BoxData[
    \(\(OperatorApplyJ[x_ == y_] := 
        Block[{sch}, \n\t\tsch\  = \ {Schiebe[i_, "\<+\>"]\^ni_. \ TJI[
                    d_, {n15__}] :> \n\t\t\t\t\tTJI[d, 
                  ReplacePart[{n15}, {n15}[\([i]\)] + ni, 
                    i]], \n\t\t\t\t\(\(Schiebe[i_, "\<-\>"]\^ni_. \ TJI[
                    d_, {n15__}]\)\(:>\)\(\n\)\(\t\t\t\t\t\)\(TJI[d, 
                  ReplacePart[{n15}, {n15}[\([i]\)] - ni, 
                    i]]\)\(\n\)\)\t\t\t}; 
          Map[Collect[Expand[#] //. sch, TJI[__], Factor] &, \n\t\t\tx == 
              y]];\)\)], "Input"],

Cell[BoxData[
    \(\(OperatorApplyK[x_ == y_] := 
        Block[{sch}, \n\t\tsch\  = \ {Schiebe[i_, "\<+\>"]\^ni_. \ TKI[
                    d_, {n15__}] :> \n\t\t\t\t\tTKI[d, 
                  ReplacePart[{n15}, {n15}[\([i]\)] + ni, 
                    i]], \n\t\t\t\t\(\(Schiebe[i_, "\<-\>"]\^ni_. \ TKI[
                    d_, {n15__}]\)\(:>\)\(\n\)\(\t\t\t\t\t\)\(TKI[d, 
                  ReplacePart[{n15}, {n15}[\([i]\)] - ni, 
                    i]]\)\(\n\)\)\t\t\t}; 
          Map[Collect[Expand[#] //. sch, TKI[__], Factor] &, \n\t\t\tx == 
              y]];\)\)], "Input"],

Cell[BoxData[
    \(\(OperatorApplyK[x_ == y_] := 
        Block[{sch}, \n\t\tsch\  = \ {Schiebe[i_, "\<+\>"]\^ni_. \ TKI[
                    d_, {n15__}] :> \n\t\t\t\t\tTKI[d, 
                  ReplacePart[{n15}, {n15}[\([i]\)] + ni, 
                    i]], \n\t\t\t\t\(\(Schiebe[i_, "\<-\>"]\^ni_. \ TKI[
                    d_, {n15__}]\)\(:>\)\(\n\)\(\t\t\t\t\t\)\(TKI[d, 
                  ReplacePart[{n15}, {n15}[\([i]\)] - ni, 
                    i]]\)\(\n\)\)\t\t\t}; 
          Map[Collect[Expand[#] //. sch, TKI[__], Factor] &, \n\t\t\tx == 
              y]];\)\)], "Input"],

Cell[BoxData[
    \(\(OperatorApplyB[x_ == y_] := 
        Block[{sch}, \n\t\tsch\  = \ {Schiebe[i_, "\<+\>"]\^ni_. \ TBI[
                    d_, {n15__}] :> \n\t\t\t\t\tTBI[d, 
                  ReplacePart[{n15}, {n15}[\([i]\)] + ni, 
                    i]], \n\t\t\t\t\(\(Schiebe[i_, "\<-\>"]\^ni_. \ TBI[
                    d_, {n15__}]\)\(:>\)\(\n\)\(\t\t\t\t\t\)\(TBI[d, 
                  ReplacePart[{n15}, {n15}[\([i]\)] - ni, 
                    i]]\)\(\n\)\)\t\t\t}; 
          Map[Collect[Expand[#] //. sch, TBI[__], Factor] &, \n\t\t\tx == 
              y]];\)\)], "Input"],

Cell[BoxData[
    \(\(OperatorApplyA[x_ == y_] := 
        Block[{sch}, \n\t\tsch\  = \ {Schiebe[i_, "\<+\>"]\^ni_. \ TAI[
                    d_, {n15__}] :> \n\t\t\t\t\tTAI[d, 
                  ReplacePart[{n15}, {n15}[\([i]\)] + ni, 
                    i]], \n\t\t\t\t\(\(Schiebe[i_, "\<-\>"]\^ni_. \ TAI[
                    d_, {n15__}]\)\(:>\)\(\n\)\(\t\t\t\t\t\)\(TAI[d, 
                  ReplacePart[{n15}, {n15}[\([i]\)] - ni, 
                    i]]\)\(\n\)\)\t\t\t}; 
          Map[Collect[Expand[#] //. sch, TAI[__], Factor] &, \n\t\t\tx == 
              y]];\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MassDerivative", "Subsection",
  CellTags->"T1.3.4.1"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(MassDerivative[
          STLI[any__, {\n\ \ \ \ \ \ \ {al_, m1_}, {be_, m2_}, {ga_, 
                m3_}, {de_, m4_}, {ep_, m5_}}], {{0, d1_}, {0, d2_}, \ {0, 
              d3_}, \ {0, d4_}, \ {0, d5_}}]\), 
        StyleBox[" ",
          FontWeight->"Plain"], 
        StyleBox[":=",
          FontWeight->"Plain"], 
        "\n", \(\((\ \(Pochhammer[al, d1]\ Pochhammer[be, d2]\ Pochhammer[ga, 
                  d3]\)*\n\ \ Pochhammer[de, d4]\ Pochhammer[ep, 
                d5]\ )\)\ *\n\ 
          STLI[any, {{al + d1, m1}, \ {be + d2, m2}, \ {ga + d3, 
                m3}, \ {de + d4, m4}, \ {ep + d5, m5}}]\)}], ";"}]], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["TarasovT", "Subsection",
  CellTags->"T1.3.5.1"],

Cell["\<\
TarasovT[abrs__, qq_ /; Head[qq] =!= List] := 
  TarasovT[abrs, qq, {\"g\", \"g\", \"g\", \"g\", \"g\"}]\
\>", "Input"],

Cell[BoxData[
    \(\(TarasovT[r_, \ s_, \ 
          qq_, \ {alp1_, alp2_, \ alp3_, alp4_, alp5_}]\  := \n
        Block[{\ al1, al2, al3, al4, al5, alrul, new, Q1, Q11, Q12, Q2, Q22, 
            dum1, dum2}, \n\ \(If[alp1\  === \ 0, \ al1\  = \ 0]; \ 
          If[alp2\  === \ 0, \ al2\  = \ 0];\n
          \ \(If[alp3\  === \ 0, \ al3\  = \ 0];\)\n
          If[alp4\  === \ 0, \ al4\  = \ 0]; \ 
          If[alp5\  === \ 0, \ al5\  = \ 0];\n
          \ \(alrul\  = \ 
              Select[{al1 :> I\ ParD[1], \ al2 :> I\ ParD[2], \ 
                  al3 :> I\ ParD[3], \n\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ al4 :> 
                    I\ ParD[4], \ 
                  al5 :> I\ ParD[5]}, \ \(! MatchQ[#, \ 0 :> _]\) &];\)\n
          \ \(Q1\  = \ 
              al3\ al5\  + \ al4\ al5\  + \ al2\ al3\  + \ al3\ al4;\)\n
          \ \(Q2\  = \ 
              al4\ al5\  + \ al3\ al5\  + \ al1\ al4\  + \ al3\ al4;\)\n
          \ \(Q11\  = 1/\((\(-4\))\)\ \((\ al2\  + \ al4\  + \ al5\ )\);\)\n
          \ \(Q22\  = 1/\((\(-4\))\)\ \((\ al1\  + \ al3\  + \ al5\ )\);\)\n
          \ \(Q12\  = 1/\((\(-2\))\)\ al5;\)\n
          \t\tnew = \n\ 
            I^\((\(-r\) - s)\) \((\ 
                  D[\ Exp[\ 
                      I\ qq\ \((\n\ \ \ \ be1\ Q1\  + \ be2\ Q2\  + 
                            be1^2\ Q11\  + be2^2\ Q22\  + \ 
                            be1\ be2\ Q12\n\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ )\)\ rho\ ], {be1, r}, {be2, 
                      s}\n\t\t\t\t]\ )\) /. \n\ \((\ 
                Join[alrul, \ {be1\  :> \ 0, be2\  :> \ 0, \ 
                    rho\  :> \ \(-Pi^2\)/
                          Pi^2\ DPlus\ }])\); \(\(Map[\((Select[#, \
\((FreeQ[#, ParD] && FreeQ[#, DPlus])\) &]\ TpD[qq, 
                      Select[#, \((\(! \((FreeQ[#, ParD] && 
                                  FreeQ[#, 
                                    DPlus])\)\))\) &]])\) &, \(\((Expand[new, 
                    ParD] + \ dum1\ dum2)\)\(\n\)\)\t\t]\)\(/.\)\({dum1 :> 0, 
              dum2 :> 0}\)\(\n\)\)\)\t\t];\)\)], "Input"],

Cell[BoxData[
    \(\(\(\n\)\(TarasovT[r_, \ s_, \ pp_, \ dp_ /; Head[dp] =!= List]\  := \ 
      TarasovT[r, s, pp, 
        dp, {"\<g\>", "\<g\>", "\<g\>", "\<g\>", "\<g\>"}]\)\)\)], "Input"],

Cell[TextData[{
  Cell[BoxData[
      \(TraditionalForm\`\((\[CapitalDelta]\ q\_1)\)\^a\)]],
  Cell[BoxData[
      \(TraditionalForm\`\((\[CapitalDelta]\ q\_2)\)\^b\)]],
  Cell[BoxData[
      \(TraditionalForm\`\((p\ q\_1)\)\^r\)]],
  Cell[BoxData[
      \(TraditionalForm\`\((p\ q\_2)\)\^s\)]]
}], "Text"],

Cell[BoxData[
    \(\(\(\n\)\(TarasovT[a_, b_, r_, s_, pp_, 
          dp_, {alp1_, alp2_, alp3_, alp4_, alp5_}] := 
        Block[{al1, al2, al3, al4, al5, alrul, new, Q1, Q11, Q12, Q2, Q22, 
            dum1, dum2}, If[alp1 === 0, al1 = 0]; If[alp2 === 0, al2 = 0]; 
          If[alp3 === 0, al3 = 0]; If[alp4 === 0, al4 = 0]; 
          If[alp5 === 0, al5 = 0]; 
          alrul = Select[{al1 \[RuleDelayed] I\ ParD[1], 
                al2 \[RuleDelayed] I\ ParD[2], al3 \[RuleDelayed] I\ ParD[3], 
                al4 \[RuleDelayed] I\ ParD[4], 
                al5 \[RuleDelayed] 
                  I\ ParD[5]}, \(\(\[InvisibleSpace]\)\(! \((MatchQ[#1, 
                      0 \[RuleDelayed] _])\)\)\) &]; 
          Q1 = al3\ al5 + al4\ al5 + al2\ al3 + al3\ al4; 
          Q2 = al4\ al5 + al3\ al5 + al1\ al4 + al3\ al4; 
          Q11 = \(-\(1\/4\)\)\ \((al2 + al4 + al5)\); 
          Q22 = \(-\(1\/4\)\)\ \((al1 + al3 + al5)\); Q12 = \(-\(al5\/2\)\); 
          new = I\^\(\(-r\) - s - a - b\)\ \[PartialD]\_\({be1, r}, {be2, s}, \
{ga1, a}, {ga2, b}\)Exp[
                    I\ \((\((be1\ pp + ga1\ dp)\)\ Q1 + \((be2\ pp + 
                                ga2\ dp)\)\ Q2 + 
                          be1\ \((be1\ pp + 2\ ga1\ dp)\)\ Q11 + 
                          be2\ \((be2\ pp + 
                                2\ ga2\ dp)\)\ Q22 + \((be1\ be2\ pp + \((be1\
\ ga2 + be2\ ga1)\)\ dp)\)\ Q12)\)\ rho] /. 
              Join[alrul, {be1 \[RuleDelayed] 0, be2 \[RuleDelayed] 0, 
                  ga1 :> 0, ga2 :> 0, 
                  rho \[RuleDelayed] \(-\(\(\[Pi]\^2\ \
DPlus\)\/\[Pi]\^2\)\)}]; \((Select[#1, 
                      FreeQ[#1, ParD] && FreeQ[#1, DPlus] &]\ TpD[pp, 
                      Select[#1, \(\(\[InvisibleSpace]\)\(! \((FreeQ[#1, 
                                  ParD] && 
                                FreeQ[#1, DPlus])\)\)\) &]] &)\) /@ \((Expand[
                    new] + dum1\ dum2)\) /. {dum1 \[RuleDelayed] 0, 
              dum2 \[RuleDelayed] 0}];\)\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["ApplyTarasovT", "Subsection",
  CellTags->"T1.3.6.1"],

Cell[BoxData[
    \(ApplyTarasovT[tij_, \ exp_] := 
      Block[{dump}, \n\t\tdump = 
          Product[ParD[j]^dummy, {j, 5}]; \n\t\tExpand[
            tij\ exp] /. \n\((STLI[de_, pp_, pe__List]\ TpD[tpp_, 
                  DPlus^i_. \ t_])\) :> \n\t
            MassDerivative[
              STLI[\ de /. de :> \((de\  + \ 2\ i)\), pp, 
                pe], \n\t\t\t\t\t\t\t\(\(t\ dump\  /. Times -> List\) /. 
                  Power[_, n_] :> {0, n}\) /. dummy -> 0]]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Misc", "Subsection",
  CellTags->"T1.3.7.1"],

Cell[BoxData[
    \(Prefactor1 /: \t\tPrefactor1[a_]^n_ := Prefactor1[a^n]\)], "Input"],

Cell[CellGroupData[{

Cell["Colors", "Subsubsection",
  CellTags->"T1.3.7.1.1"],

Cell[TextData[{
  StyleBox["TFI",
    FontColor->RGBColor[1, 0.6, 0]],
  " ",
  StyleBox["TVI ",
    FontColor->RGBColor[0.2, 0.4, 0.8]],
  StyleBox["TFR ",
    FontColor->RGBColor[0, 0.6, 0]],
  StyleBox["TVR ",
    FontColor->RGBColor[0.2, 0.8, 1]],
  StyleBox["MakeFun ",
    FontColor->RGBColor[1, 0.6, 0.6]],
  StyleBox["Cayley ",
    FontColor->RGBColor[1, 0, 0.4]],
  StyleBox["CayleyD ",
    FontColor->RGBColor[1, 0.2, 1]],
  StyleBox["Cayleyu",
    FontColor->RGBColor[0.8, 0.8, 0.4]],
  StyleBox[" ",
    FontColor->RGBColor[1, 0.2, 1]],
  StyleBox["IFF ",
    FontColor->RGBColor[0.6, 0.4, 1]],
  StyleBox["ToArgs ",
    FontColor->RGBColor[0.2, 0.6, 1]],
  StyleBox["Block",
    FontSize->16,
    FontWeight->"Bold",
    FontColor->RGBColor[1, 0.2, 0.2]]
}], "Text"],

Cell[BoxData[
    \(\(MakeBoxes[Block, StandardForm] := 
        StyleBox["\<Block\>", FontSize \[Rule] 16, 
          FontWeight \[Rule] "\<Bold\>", 
          FontColor \[Rule] RGBColor[1, 0.2, 0.2]];\)\)], "Input"],

Cell[BoxData[
    \(\(MakeBoxes[Cayley, StandardForm] := 
        StyleBox["\<Cayley\>", 
          FontColor \[Rule] RGBColor[1, 0, 0.4]];\)\)], "Input"],

Cell[BoxData[
    \(\(MakeBoxes[CayleyD, StandardForm] := 
        StyleBox["\<CayleyD\>", 
          FontColor \[Rule] RGBColor[1, 0.2, 1]];\)\)], "Input"],

Cell[BoxData[
    \(\(MakeBoxes[Cayleyu, StandardForm] := 
        StyleBox["\<Cayleyu\>", 
          FontColor \[Rule] RGBColor[0.8, 0.8, 0.4]];\)\)], "Input"],

Cell[BoxData[
    \(\(MakeBoxes[IFF, StandardForm] := 
        StyleBox["\<IFF\>", 
          FontColor \[Rule] RGBColor[0.6, 0.4, 1]];\)\)], "Input"],

Cell[BoxData[
    \(\(MakeBoxes[MakeFun, StandardForm] := 
        StyleBox["\<MakeFun\>", 
          FontColor \[Rule] RGBColor[1, 0.6, 0.6]];\)\)], "Input"],

Cell[BoxData[
    \(\(MakeBoxes[TFI, StandardForm] := 
        StyleBox["\<TFI\>", 
          FontColor \[Rule] RGBColor[1, 0.6, 0]];\)\)], "Input"],

Cell[BoxData[
    \(\(MakeBoxes[TFR, StandardForm] := 
        StyleBox["\<TFR\>", 
          FontColor \[Rule] RGBColor[0, 0.6, 0]];\)\)], "Input"],

Cell[BoxData[
    \(\(MakeBoxes[ToArgs, StandardForm] := 
        StyleBox["\<ToArgs\>", 
          FontColor \[Rule] RGBColor[0.2, 0.6, 1]];\)\)], "Input"],

Cell[BoxData[
    \(\(MakeBoxes[TVI, StandardForm] := 
        StyleBox["\<TVI\>", 
          FontColor \[Rule] RGBColor[0.2, 0.4, 0.8]];\)\)], "Input"],

Cell[BoxData[
    \(MakeBoxes[TVR, StandardForm] := 
      StyleBox["\<TVR\>", FontColor \[Rule] RGBColor[0.2, 0.8, 1]]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["PQ", "Subsubsection",
  CellTags->"T1.3.7.2.1"],

Cell[BoxData[
    \(\(PQ[_Integer?Positive]\  := \ True;\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["PNQ", "Subsubsection",
  CellTags->"T1.3.7.3.1"],

Cell[BoxData[
    \(\(PNQ[0] = True;\)\)], "Input"],

Cell[BoxData[
    \(\(PNQ[_Integer?Positive]\  := \ True;\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["FactorC", "Subsubsection",
  CellTags->"T1.3.7.4.1"],

Cell[BoxData[
    \(FactorC[z_] := \(FactorC[z] = Factor[z]\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["TFI", "Subsubsection",
  CellTags->"T1.3.7.5.1"],

Cell[BoxData[
    \(\(TFI[depp__, {a___, b_Integer, c___}] := 
        TFI[depp, {a, {b, 0}, c}];\)\)], "Input"],

Cell[BoxData[
    \(\(TFI[depp__, {a___, {0, m_ /; m =!= 0}, b___}] := 
        TFI[depp, {a, {0, 0}, b}];\)\)], "Input"],

Cell[BoxData[
    \(\(TFI[a__, {0, 0}, b__List] := TFI[a, b];\)\)], "Input"],

Cell[BoxData[
    \(\(TFI[d_, pp_, dp_, {0, 0}, {x1_, x2_, x3_, x4_, x5_}, list_List] := 
        TFI[d, pp, {x1, x2, x3, x4, x5}, list];\)\)], "Input"],

Cell[BoxData[
    \(\(TFI[d_, pp_, dp_, {a_, b_}, {0, 0, 0, 0, 0}, list_List] := 
        TFI[d, pp, dp, {a, b}, list];\)\)], "Input"],

Cell[BoxData[
    \(\(TFI[d_, pp_, dp_ /; Head[dp] =!= List, {0, 0, 0, 0, 0}, list_List] := 
        TFI[d, pp, list];\)\)], "Input"],

Cell[BoxData[
    \(\(TFI[d_, pp_, dp_ /; Head[dp] =!= List, list_List] := 
        TFI[d, pp, list];\)\)], "Input"],

Cell[BoxData[
    \(\(TFI[d_, pp_, dp_ /; Head[dp] =!= List, {x1_, x2_, x3_, x4_, x5_}, 
          list_List] := TFI[d, pp, {x1, x2, x3, x4, x5}, list];\)\)], "Input"],

Cell[BoxData[
    \(\(TFI[__, {{_, _}, {0, 0}, {_, _}, {0, 0}, {_, 0}}] := 0;\)\)], "Input"],

Cell[BoxData[
    \(\(TFI[__, {{0, 0}, {_, _}, {0, 0}, {_, _}, {_, 0}}] := 0;\)\)], "Input"],

Cell[BoxData[
    \(\(TFI[__, {{0, 0}, {_, _}, {_, 0}, {_, _}, {0, 0}}] := 0;\)\)], "Input"],

Cell[BoxData[
    \(\(TFI[__, {{_, 0}, {_, _}, {0, 0}, {_, _}, {0, 0}}] := 0;\)\)], "Input"],

Cell[BoxData[
    \(\(TFI[__, {{_, _}, {_, 0}, {_, _}, {0, 0}, {0, 0}}] := 0;\)\)], "Input"],

Cell[BoxData[
    \(\(TFI[__, {{_, _}, {0, 0}, {_, _}, {_, 0}, {0, 0}}] := 0;\)\)], "Input"],

Cell[BoxData[
    \(\(TFI[__, {{0, 0}, {0, 0}, {0, 0}, {_, 0}, {_, _}}] := 0;\)\)], "Input"],

Cell[BoxData[
    \(\(TFI[__, {{0, 0}, {0, 0}, {_, 0}, {0, 0}, {_, _}}] := 0;\)\)], "Input"],

Cell[BoxData[
    \(\(TFI[__, {{0, 0}, {_, 0}, {0, 0}, {0, 0}, {_, _}}] := 0;\)\)], "Input"],

Cell[BoxData[
    \(\(TFI[__, {{_, 0}, {0, 0}, {0, 0}, {0, 0}, {_, _}}] := 0;\)\)], "Input"],

Cell[BoxData[
    \(\(TFI[__, {{_, _}, {_, 0}, {0, 0}, {0, 0}, {0, 0}}] := 0;\)\)], "Input"],

Cell[BoxData[
    \(\(TFI[__, {{_, 0}, {_, _}, {0, 0}, {0, 0}, {0, 0}}] := 0;\)\)], "Input"],

Cell[BoxData[
    \(\(TFI[__, {{_, _}, {0, 0}, {0, 0}, {_, 0}, {0, 0}}] := 0;\)\)], "Input"],

Cell[BoxData[
    \(\(TFI[__, {{_, 0}, {0, 0}, {0, 0}, {_, _}, {0, 0}}] := 0;\)\)], "Input"],

Cell[BoxData[
    \(\(TFI[__, {{0, 0}, {_, 0}, {_, _}, {0, 0}, {0, 0}}] := 0;\)\)], "Input"],

Cell[BoxData[
    \(\(TFI[__, {{0, 0}, {_, _}, {_, 0}, {0, 0}, {0, 0}}] := 0;\)\)], "Input"],

Cell[BoxData[
    \(\(TFI[__, {{0, 0}, {0, 0}, {_, _}, {_, 0}, {0, 0}}] := 0;\)\)], "Input"],

Cell[BoxData[
    \(\(TFI[__, {{0, 0}, {0, 0}, {_, 0}, {_, _}, {0, 0}}] := 0;\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["TJI", "Subsubsection",
  CellTags->"T1.3.7.6.1"],

Cell[BoxData[
    \(TJI[de_, 
        pp_, {{n1_Integer, m1_}, {n2_Integer, m2_}, {n3_Integer, 
            m3_}}] := \((TJI[de, pp, {{n1, m1}, {n2, m2}, {n3, m3}}] = 
            TJI[de, pp, 
              Reverse[Sort[{{n1, m1}, {n2, m2}, {n3, 
                      m3}}]]])\) /; \(\(\[InvisibleSpace]\)\(! \
\((OrderedQ[{{n3, m3}, {n2, m2}, {n1, m1}}])\)\)\)\)], "Input"],

Cell[BoxData[
    \(TJI[d, pp, {{1, m1}, {1, m2}, {1, m3}}]\)], "Input"],

Cell[BoxData[
    \(TJI[d_, 0, {{_, 0}, {_, 0}, {_, 0}}] := 0\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["TBI", "Subsubsection",
  CellTags->"T1.3.7.7.1"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        StyleBox["TBI",
          FontColor->RGBColor[1, 0.6, 0]], "[", \(de_, 
        pp_, {{n1_Integer, m1_}, {n2_Integer, m2_}}\), "]"}], ":=", 
      RowBox[{
        RowBox[{"(", 
          RowBox[{\(TJI[de, pp, {{n1, m1}, {n2, m2}}]\), "=", 
            RowBox[{
              StyleBox["TBI",
                FontColor->RGBColor[1, 0.6, 0]], "[", \(de, pp, 
              Reverse@\(Sort@{{n1, m1}, {n2, m2}}\)\), "]"}]}], ")"}], "/;", 
        "\[InvisibleSpace]", \(! \((OrderedQ[{{n2, m2}, {n1, 
                  m1}}])\)\)}]}]], "Input"],

Cell[BoxData[
    \(TBI[d_, 0, {{_, 0}, {_, 0}}] := 0\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["TAI", "Subsubsection",
  CellTags->"T1.3.7.8.1"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          StyleBox["TAI",
            FontColor->RGBColor[1, 0.6, 0]], "[", \(de_, 
          pp_, {{n1_Integer, 0}}\), "]"}], ":=", "0"}], ";"}]], "Input"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Definitions and notation", "Section",
  CellDingbat->"\[FilledSmallCircle]",
  CellTags->"T1.4.1"],

Cell[BoxData[
    RowBox[{
      RowBox[{"\[CapitalDelta]\[ScriptCapitalC]", "=", 
        RowBox[{\(-\(1\/2\)\), 
          RowBox[{"Det", "[", " ", 
            RowBox[{"(", GridBox[{
                  {"0", "1", "1", "1", "1"},
                  {"1", "0", \(\[Mu]\_6\), \(\[Mu]\_4\), \(\[Mu]\_3\)},
                  {"1", \(\[Mu]\_6\), "0", \(\[Mu]\_2\), \(\[Mu]\_1\)},
                  {"1", \(\[Mu]\_4\), \(\[Mu]\_2\), "0", \(\[Mu]\_5\)},
                  {"1", \(\[Mu]\_3\), \(\[Mu]\_1\), \(\[Mu]\_5\), "0"}
                  }], ")"}], "]"}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(\(checkli\  = \ {\n\t\t\[ScriptCapitalC] -> \[CapitalDelta]\
\[ScriptCapitalC], \n\[CapitalDelta]\_i_ :> \[PartialD]\_\(\[Mu]\_i\)\
\[CapitalDelta]\[ScriptCapitalC], \n\t\t\[Mu]\_6 :> PP, \[Mu]\_i_ :> 
            ToExpression["\<m\>" <> ToString[i]]^2, \n\t\tm\_1 :> \@m1\^2, 
          m\_2 :> \@m2\^2, m\_3 :> \@m3\^2, m\_4 :> \@m4\^2, 
          m\_5 :> \@m5\^2, \n\[CapitalDelta]\_\(i_, j_, k_\) :> \[Mu]\_i\%2 + \
\[Mu]\_j\%2 + \[Mu]\_k\%2 - 
              2 \((\(\[Mu]\_i\) \[Mu]\_j + \(\[Mu]\_j\) \[Mu]\_k + \(\[Mu]\_k\
\) \[Mu]\_i)\), \n
          u\_\(i_, j_, k_\) :> \[Mu]\_i - \[Mu]\_j - \[Mu]\_k, \n\
\[CapitalSigma]\_i_ :> 
            3\ d\  - 2\ \((\[Sum]\_\(j = 1\)\%i \[Nu]\_j)\) - 
              2, \n\t\[Nu]\_1 \[RuleDelayed] n1, \[Nu]\_2 \[RuleDelayed] 
            n2, \[Nu]\_3 \[RuleDelayed] n3, \[Nu]\_4 \[RuleDelayed] 
            n4, \[Nu]\_5 \[RuleDelayed] n5};\)\)], "Input"],

Cell[TextData[{
  "Define symmetry property for ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalDelta]\_\(i, j, k\)\)]]
}], "Text"],

Cell[BoxData[
    \(\[CapitalDelta]\_\(i_, j_, k_\) := \[CapitalDelta]\_\(Sequence @@ \
Sort[{i, j, k}]\) /; \(! OrderedQ[{i, j, k}]\)\)], "Input"],

Cell[BoxData[
    \(\[CapitalDelta]\_\(3, 1, 2\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    StyleBox["ToArgs",
      FontColor->RGBColor[0.2, 0.6, 1]]], "Subsection",
  CellTags->"T1.4.1.1"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          StyleBox["ToArgs",
            FontColor->RGBColor[0.2, 0.6, 1]], "[", "z_", "]"}], 
        ":=", \(Expand[z //. checkli]\)}], ";"}]], "Input"],

Cell[BoxData[
    \(ToArgs[\[CapitalSigma]\_4] // InputForm\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    StyleBox["Cayley",
      FontColor->RGBColor[1, 0, 0.4]]], "Subsection",
  CellTags->"T1.4.2.1"],

Cell[TextData[{
  "This is (40).  ( p = ",
  Cell[BoxData[
      \(TraditionalForm\`\@p\^2\)]],
  ")"
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"setd", "[", 
        RowBox[{
          RowBox[{"hCayley", "[", 
            RowBox[{
              RowBox[{
                InterpretationBox[\(m\_1\),
                  m1], ":", "_"}], ",", 
              RowBox[{
                InterpretationBox[\(m\_2\),
                  m2], ":", "_"}], ",", 
              RowBox[{
                InterpretationBox[\(m\_3\),
                  m3], ":", "_"}], ",", 
              RowBox[{
                InterpretationBox[\(m\_4\),
                  m4], ":", "_"}], ",", 
              RowBox[{
                InterpretationBox[\(m\_5\),
                  m5], ":", "_"}], ",", "p_"}], "]"}], ",", 
          RowBox[{"set", "[", 
            RowBox[{
              RowBox[{"hCayley", "[", 
                RowBox[{
                  InterpretationBox[\(m\_1\),
                    m1], ",", 
                  InterpretationBox[\(m\_2\),
                    m2], ",", 
                  InterpretationBox[\(m\_3\),
                    m3], ",", 
                  InterpretationBox[\(m\_4\),
                    m4], ",", 
                  InterpretationBox[\(m\_5\),
                    m5], ",", "p"}], "]"}], ",", 
              RowBox[{"factor", "[", 
                RowBox[{
                  RowBox[{
                    StyleBox["ToArgs",
                      FontColor->RGBColor[0.2, 0.6, 1]], "[", 
                    "\[ScriptCapitalC]", "]"}], "/.", 
                  RowBox[{
                    InterpretationBox[\(p\^2\),
                      PP], "\[Rule]", \(p\^2\)}]}], "]"}]}], "]"}]}], "]"}], "/.",
       
      RowBox[{"{", 
        RowBox[{
          RowBox[{"hCayley", "\[RuleDelayed]", 
            StyleBox["Cayley",
              FontColor->RGBColor[1, 0, 0.4]]}], 
          ",", \(setd \[RuleDelayed] SetDelayed\), 
          ",", \(set \[RuleDelayed] Set\), 
          ",", \(factor \[RuleDelayed] Factor\)}], "}"}]}]], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        StyleBox["Cayley",
          FontColor->RGBColor[1, 0, 0.4]], "[", "i", "]"}], 
      "=", \(\[CapitalDelta]\_i\)}]], "Subsection",
  CellTags->"T1.4.3.1"],

Cell[BoxData[
    RowBox[{"Do", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"setd", "[", 
            RowBox[{
              RowBox[{\(hCayley[i]\), "[", 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{
                      InterpretationBox[\(m\_1\),
                        m1], ":", "_"}], ",", 
                    RowBox[{
                      InterpretationBox[\(m\_2\),
                        m2], ":", "_"}], ",", 
                    RowBox[{
                      InterpretationBox[\(m\_3\),
                        m3], ":", "_"}], ",", 
                    RowBox[{
                      InterpretationBox[\(m\_4\),
                        m4], ":", "_"}], ",", 
                    RowBox[{
                      InterpretationBox[\(m\_5\),
                        m5], ":", "_"}], ",", "p_"}], "}"}], "]"}], ",", 
              RowBox[{"set", "[", 
                RowBox[{
                  RowBox[{\(hCayley[i]\), "[", 
                    RowBox[{"{", 
                      RowBox[{
                        InterpretationBox[\(m\_1\),
                          m1], ",", 
                        InterpretationBox[\(m\_2\),
                          m2], ",", 
                        InterpretationBox[\(m\_3\),
                          m3], ",", 
                        InterpretationBox[\(m\_4\),
                          m4], ",", 
                        InterpretationBox[\(m\_5\),
                          m5], ",", "p"}], "}"}], "]"}], ",", 
                  RowBox[{"factor", "[", 
                    RowBox[{
                      RowBox[{
                        StyleBox["ToArgs",
                          FontColor->RGBColor[0.2, 0.6, 1]], 
                        "[", \(\[CapitalDelta]\_i\), "]"}], "/.", 
                      RowBox[{
                        InterpretationBox[\(p\^2\),
                          PP], "\[Rule]", \(p\^2\)}]}], "]"}]}], "]"}]}], 
            "]"}], "/.", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{"hCayley", "\[RuleDelayed]", 
                StyleBox["Cayley",
                  FontColor->RGBColor[1, 0, 0.4]]}], 
              ",", \(setd \[RuleDelayed] SetDelayed\), 
              ",", \(set \[RuleDelayed] Set\), 
              ",", \(factor \[RuleDelayed] Factor\)}], "}"}]}], 
        ",", \({i, 6}\)}], "]"}]], "Input"],

Cell[BoxData[
    RowBox[{
      StyleBox["ToArgs",
        FontColor->RGBColor[0.2, 0.6, 1]], 
      "[", \(\[CapitalDelta]\_\(1, 2, 6\)\), "]"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{
            StyleBox["Cayley",
              FontColor->RGBColor[1, 0, 0.4]], "[", \(i_, j_, k_\), "]"}], 
          "[", "m_List", "]"}], ":=", 
        RowBox[{
          RowBox[{
            RowBox[{
              StyleBox["Cayley",
                FontColor->RGBColor[1, 0, 0.4]], "[", \(i, j, k\), "]"}], "[",
             "m", "]"}], 
          "=", \(Block[{mm = m}, \n\t\t\tWhile[Length[mm] < 7, 
              AppendTo[mm, Last[mm]]]; \n\t\t\tFactorC[
              mm\[LeftDoubleBracket]i\[RightDoubleBracket]\^4 - 
                2\ mm\[LeftDoubleBracket]j\[RightDoubleBracket]\^2\ mm\
\[LeftDoubleBracket]i\[RightDoubleBracket]\^2 - 
                2\ mm\[LeftDoubleBracket]k\[RightDoubleBracket]\^2\ mm\
\[LeftDoubleBracket]i\[RightDoubleBracket]\^2 + 
                mm\[LeftDoubleBracket]j\[RightDoubleBracket]\^4 + 
                mm\[LeftDoubleBracket]k\[RightDoubleBracket]\^4 - 
                2\ mm\[LeftDoubleBracket]j\[RightDoubleBracket]\^2\ mm\
\[LeftDoubleBracket]k\[RightDoubleBracket]\^2]]\)}]}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{
            StyleBox["Cayleyu",
              FontColor->RGBColor[0.8, 0.8, 0.4]], "[", \(i_, j_, k_\), "]"}],
           "[", "m_List", "]"}], ":=", 
        RowBox[{
          RowBox[{
            RowBox[{
              StyleBox["Cayleyu",
                FontColor->RGBColor[0.8, 0.8, 0.4]], "[", \(i, j, k\), "]"}], 
            "[", "mm", "]"}], 
          "=", \(Block[{mm = m}, \n\t\t\t\t\tWhile[Length[mm] < 7, 
              AppendTo[mm, Last[mm]]]; 
            FactorC[mm\[LeftDoubleBracket]i\[RightDoubleBracket]\^2 - 
                mm\[LeftDoubleBracket]j\[RightDoubleBracket]\^2 - 
                mm\[LeftDoubleBracket]k\[RightDoubleBracket]\^2]]\)}]}], 
      ";"}]], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        StyleBox["CayleyD",
          FontColor->RGBColor[1, 0.2, 1]], "[", "i", "]"}], 
      "=", \(\[CapitalDelta]\_i\/\[CapitalDelta]\)}]], "Subsection",
  CellTags->"T1.4.4.1"],

Cell[BoxData[
    RowBox[{"Do", "[", 
      RowBox[{
        RowBox[{
          RowBox[{"setd", "[", 
            RowBox[{
              RowBox[{\(hCayleyD[i]\), "[", 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{
                      InterpretationBox[\(m\_1\),
                        m1], ":", "_"}], ",", 
                    RowBox[{
                      InterpretationBox[\(m\_2\),
                        m2], ":", "_"}], ",", 
                    RowBox[{
                      InterpretationBox[\(m\_3\),
                        m3], ":", "_"}], ",", 
                    RowBox[{
                      InterpretationBox[\(m\_4\),
                        m4], ":", "_"}], ",", 
                    RowBox[{
                      InterpretationBox[\(m\_5\),
                        m5], ":", "_"}], ",", "p_"}], "}"}], "]"}], ",", 
              RowBox[{"set", "[", 
                RowBox[{
                  RowBox[{\(hCayleyD[i]\), "[", 
                    RowBox[{"{", 
                      RowBox[{
                        InterpretationBox[\(m\_1\),
                          m1], ",", 
                        InterpretationBox[\(m\_2\),
                          m2], ",", 
                        InterpretationBox[\(m\_3\),
                          m3], ",", 
                        InterpretationBox[\(m\_4\),
                          m4], ",", 
                        InterpretationBox[\(m\_5\),
                          m5], ",", "p"}], "}"}], "]"}], ",", 
                  RowBox[{"factor", "[", 
                    FractionBox[
                      RowBox[{
                        RowBox[{
                          StyleBox["ToArgs",
                            FontColor->RGBColor[0.2, 0.6, 1]], 
                          "[", \(\[CapitalDelta]\_i\), "]"}], "/.", 
                        RowBox[{
                          InterpretationBox[\(p\^2\),
                            PP], "\[Rule]", \(p\^2\)}]}], 
                      RowBox[{"\[ScriptCapitalC]", "[", 
                        RowBox[{
                          InterpretationBox[\(m\_1\),
                            m1], ",", 
                          InterpretationBox[\(m\_2\),
                            m2], ",", 
                          InterpretationBox[\(m\_3\),
                            m3], ",", 
                          InterpretationBox[\(m\_4\),
                            m4], ",", 
                          InterpretationBox[\(m\_5\),
                            m5], ",", "p"}], "]"}]], "]"}]}], "]"}]}], "]"}], 
          "/.", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{"hCayleyD", "\[RuleDelayed]", 
                StyleBox["CayleyD",
                  FontColor->RGBColor[1, 0.2, 1]]}], 
              ",", \(setd \[RuleDelayed] SetDelayed\), 
              ",", \(set \[RuleDelayed] Set\), 
              ",", \(factor \[RuleDelayed] Factor\), ",", 
              RowBox[{"\[ScriptCapitalC]", "\[RuleDelayed]", 
                StyleBox["Cayley",
                  FontColor->RGBColor[1, 0, 0.4]]}]}], "}"}]}], 
        ",", \({i, 6}\)}], "]"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"setd", "[", 
        RowBox[{
          RowBox[{\(hCayleyD[1, 2, 5]\), "[", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{
                  InterpretationBox[\(m\_1\),
                    m1], ":", "_"}], ",", 
                RowBox[{
                  InterpretationBox[\(m\_2\),
                    m2], ":", "_"}], ",", 
                RowBox[{
                  InterpretationBox[\(m\_3\),
                    m3], ":", "_"}], ",", 
                RowBox[{
                  InterpretationBox[\(m\_4\),
                    m4], ":", "_"}], ",", 
                RowBox[{
                  InterpretationBox[\(m\_5\),
                    m5], ":", "_"}], ",", "p_"}], "}"}], "]"}], ",", 
          RowBox[{"set", "[", 
            RowBox[{
              RowBox[{\(hCayleyD[1, 2, 5]\), "[", 
                RowBox[{"{", 
                  RowBox[{
                    InterpretationBox[\(m\_1\),
                      m1], ",", 
                    InterpretationBox[\(m\_2\),
                      m2], ",", 
                    InterpretationBox[\(m\_3\),
                      m3], ",", 
                    InterpretationBox[\(m\_4\),
                      m4], ",", 
                    InterpretationBox[\(m\_5\),
                      m5], ",", "p"}], "}"}], "]"}], ",", 
              RowBox[{"factor", "[", 
                FractionBox[
                  RowBox[{
                    RowBox[{
                      StyleBox["ToArgs",
                        FontColor->RGBColor[0.2, 0.6, 1]], 
                      "[", \(\[CapitalDelta]\_\(1, 2, 5\)\), "]"}], "/.", 
                    RowBox[{
                      InterpretationBox[\(p\^2\),
                        PP], "\[Rule]", \(p\^2\)}]}], 
                  RowBox[{"\[ScriptCapitalC]", "[", 
                    RowBox[{
                      InterpretationBox[\(m\_1\),
                        m1], ",", 
                      InterpretationBox[\(m\_2\),
                        m2], ",", 
                      InterpretationBox[\(m\_3\),
                        m3], ",", 
                      InterpretationBox[\(m\_4\),
                        m4], ",", 
                      InterpretationBox[\(m\_5\),
                        m5], ",", "p"}], "]"}]], "]"}]}], "]"}]}], "]"}], "/.",
       
      RowBox[{"{", 
        RowBox[{
          RowBox[{"hCayleyD", "\[RuleDelayed]", 
            StyleBox["CayleyD",
              FontColor->RGBColor[1, 0.2, 1]]}], 
          ",", \(setd \[RuleDelayed] SetDelayed\), 
          ",", \(set \[RuleDelayed] Set\), 
          ",", \(factor \[RuleDelayed] Factor\), ",", 
          RowBox[{"\[ScriptCapitalC]", "\[RuleDelayed]", 
            StyleBox["Cayley",
              FontColor->RGBColor[1, 0, 0.4]]}]}], "}"}]}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"setd", "[", 
        RowBox[{
          RowBox[{\(hCayleyD[1, 3, 6]\), "[", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{
                  InterpretationBox[\(m\_1\),
                    m1], ":", "_"}], ",", 
                RowBox[{
                  InterpretationBox[\(m\_2\),
                    m2], ":", "_"}], ",", 
                RowBox[{
                  InterpretationBox[\(m\_3\),
                    m3], ":", "_"}], ",", 
                RowBox[{
                  InterpretationBox[\(m\_4\),
                    m4], ":", "_"}], ",", 
                RowBox[{
                  InterpretationBox[\(m\_5\),
                    m5], ":", "_"}], ",", "p_"}], "}"}], "]"}], ",", 
          RowBox[{"set", "[", 
            RowBox[{
              RowBox[{\(hCayleyD[1, 3, 6]\), "[", 
                RowBox[{"{", 
                  RowBox[{
                    InterpretationBox[\(m\_1\),
                      m1], ",", 
                    InterpretationBox[\(m\_2\),
                      m2], ",", 
                    InterpretationBox[\(m\_3\),
                      m3], ",", 
                    InterpretationBox[\(m\_4\),
                      m4], ",", 
                    InterpretationBox[\(m\_5\),
                      m5], ",", "p"}], "}"}], "]"}], ",", 
              RowBox[{"factor", "[", 
                FractionBox[
                  RowBox[{
                    RowBox[{
                      StyleBox["ToArgs",
                        FontColor->RGBColor[0.2, 0.6, 1]], 
                      "[", \(\[CapitalDelta]\_\(1, 3, 6\)\), "]"}], "/.", 
                    RowBox[{
                      InterpretationBox[\(p\^2\),
                        PP], "\[Rule]", \(p\^2\)}]}], 
                  RowBox[{"\[ScriptCapitalC]", "[", 
                    RowBox[{
                      InterpretationBox[\(m\_1\),
                        m1], ",", 
                      InterpretationBox[\(m\_2\),
                        m2], ",", 
                      InterpretationBox[\(m\_3\),
                        m3], ",", 
                      InterpretationBox[\(m\_4\),
                        m4], ",", 
                      InterpretationBox[\(m\_5\),
                        m5], ",", "p"}], "]"}]], "]"}]}], "]"}]}], "]"}], "/.",
       
      RowBox[{"{", 
        RowBox[{
          RowBox[{"hCayleyD", "\[RuleDelayed]", 
            StyleBox["CayleyD",
              FontColor->RGBColor[1, 0.2, 1]]}], 
          ",", \(setd \[RuleDelayed] SetDelayed\), 
          ",", \(set \[RuleDelayed] Set\), 
          ",", \(factor \[RuleDelayed] Factor\), ",", 
          RowBox[{"\[ScriptCapitalC]", "\[RuleDelayed]", 
            StyleBox["Cayley",
              FontColor->RGBColor[1, 0, 0.4]]}]}], "}"}]}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"setd", "[", 
        RowBox[{
          RowBox[{\(hCayleyD[3, 4, 5]\), "[", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{
                  InterpretationBox[\(m\_1\),
                    m1], ":", "_"}], ",", 
                RowBox[{
                  InterpretationBox[\(m\_2\),
                    m2], ":", "_"}], ",", 
                RowBox[{
                  InterpretationBox[\(m\_3\),
                    m3], ":", "_"}], ",", 
                RowBox[{
                  InterpretationBox[\(m\_4\),
                    m4], ":", "_"}], ",", 
                RowBox[{
                  InterpretationBox[\(m\_5\),
                    m5], ":", "_"}], ",", "p_"}], "}"}], "]"}], ",", 
          RowBox[{"set", "[", 
            RowBox[{
              RowBox[{\(hCayleyD[3, 4, 5]\), "[", 
                RowBox[{"{", 
                  RowBox[{
                    InterpretationBox[\(m\_1\),
                      m1], ",", 
                    InterpretationBox[\(m\_2\),
                      m2], ",", 
                    InterpretationBox[\(m\_3\),
                      m3], ",", 
                    InterpretationBox[\(m\_4\),
                      m4], ",", 
                    InterpretationBox[\(m\_5\),
                      m5], ",", "p"}], "}"}], "]"}], ",", 
              RowBox[{"factor", "[", 
                FractionBox[
                  RowBox[{
                    RowBox[{
                      StyleBox["ToArgs",
                        FontColor->RGBColor[0.2, 0.6, 1]], 
                      "[", \(\[CapitalDelta]\_\(3, 4, 5\)\), "]"}], "/.", 
                    RowBox[{
                      InterpretationBox[\(p\^2\),
                        PP], "\[Rule]", \(p\^2\)}]}], 
                  RowBox[{"\[ScriptCapitalC]", "[", 
                    RowBox[{
                      InterpretationBox[\(m\_1\),
                        m1], ",", 
                      InterpretationBox[\(m\_2\),
                        m2], ",", 
                      InterpretationBox[\(m\_3\),
                        m3], ",", 
                      InterpretationBox[\(m\_4\),
                        m4], ",", 
                      InterpretationBox[\(m\_5\),
                        m5], ",", "p"}], "]"}]], "]"}]}], "]"}]}], "]"}], "/.",
       
      RowBox[{"{", 
        RowBox[{
          RowBox[{"hCayleyD", "\[RuleDelayed]", 
            StyleBox["CayleyD",
              FontColor->RGBColor[1, 0.2, 1]]}], 
          ",", \(setd \[RuleDelayed] SetDelayed\), 
          ",", \(set \[RuleDelayed] Set\), 
          ",", \(factor \[RuleDelayed] Factor\), ",", 
          RowBox[{"\[ScriptCapitalC]", "\[RuleDelayed]", 
            StyleBox["Cayley",
              FontColor->RGBColor[1, 0, 0.4]]}]}], "}"}]}]], "Input"],

Cell[BoxData[
    RowBox[{
      StyleBox["ToArgs",
        FontColor->RGBColor[0.2, 0.6, 1]], 
      "[", \(\[CapitalDelta]\_\(i, j, k\)\), "]"}]], "Input"],

Cell[BoxData[
    RowBox[{
      StyleBox["ToArgs",
        FontColor->RGBColor[0.2, 0.6, 1]], 
      "[", \(\[CapitalDelta]\_\(1, 2, 6\)\), "]"}]], "Input"],

Cell[BoxData[
    RowBox[{
      StyleBox["ToArgs",
        FontColor->RGBColor[0.2, 0.6, 1]], "[", \(u\_\(i, j, k\)\), 
      "]"}]], "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Utility Functions", "Section",
  CellDingbat->"\[FilledSmallCircle]",
  CellTags->"T1.5.1"],

Cell[CellGroupData[{

Cell["MakeFun and friends", "Subsection",
  CellTags->"T1.5.1.1"],

Cell["\<\
This function generates the recursion function definitions for 
TARCER starting from the entered relations.\
\>", "Text"],

Cell["The idea is to recurse TFR into TFI's step by step.", "Text"],

Cell[TextData[{
  "Define: S cD[ijk__] := ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[ScriptCapitalC]\^\(-1\)\) \
\[CapitalDelta]\_ijk\)]]
}], "Text"],

Cell[BoxData[
    \(\(nutonand\[CapitalDelta]u[w_, 
          5] := \(w /. {\[Nu]\_1 \[RuleDelayed] n1, \[Nu]\_2 \[RuleDelayed] 
                n2, \[Nu]\_3 \[RuleDelayed] n3, \[Nu]\_4 \[RuleDelayed] 
                n4, \[Nu]\_5 \[RuleDelayed] 
                n5, \[CapitalDelta]\_ijk__ \[RuleDelayed] \[ScriptCapitalC]\ \
cD[ijk], u\_ijk__ \[RuleDelayed] cu[ijk], \n\t\tD\_ijk__ :> 
                TA[D, ijk], \[Rho]\_ijk__ :> 
                TA[\[Rho], ijk], \[Phi]\_\(\(ijk__\)\(\ \)\) :> 
                TA[\[Phi], ijk], \n\t\t\t\[Sigma]\_\(i_, j_, k_\) :> 
                TA[\[Sigma], d, i, j, k, \[Nu]\_i, \[Nu]\_j, \[Nu]\_k], 
              h\_\(i_, j_, k_\) :> 
                TA[h, d, i, j, k, \[Nu]\_i, \[Nu]\_j, \[Nu]\_k], 
              S\_\(i_, j_, k_\) :> 
                TA[S, d, i, j, 
                  k, \[Nu]\_i, \[Nu]\_j, \[Nu]\_k], \n\t\t\t\(\(\
\[CapitalSigma]\_i_\)\(:>\)\(ToArgs[\[CapitalSigma]\_i]\)\(\n\)\)\t\t}\) /. {\
\t\[Nu]\_1 \[RuleDelayed] n1, \[Nu]\_2 \[RuleDelayed] 
              n2, \[Nu]\_3 \[RuleDelayed] n3, \[Nu]\_4 \[RuleDelayed] 
              n4, \[Nu]\_5 \[RuleDelayed] n5, m\_1 :> m1, m\_2 :> m2, 
            m\_3 :> m3, m\_4 :> m4, m\_5 :> m5};\)\)], "Input"],

Cell[BoxData[
    \(\(nutonand\[CapitalDelta]u[
          w_] := \(w /. {\[Nu]\_1 \[RuleDelayed] n1, \[Nu]\_2 \[RuleDelayed] 
                n2, \[Nu]\_3 \[RuleDelayed] n3, \[Nu]\_4 \[RuleDelayed] 
                n4, \[Nu]\_5 \[RuleDelayed] 
                n5, \[CapitalDelta]\_ijk__ \[RuleDelayed] cD[ijk], 
              u\_ijk__ \[RuleDelayed] cu[ijk], 
              D\_ijk__ \[RuleDelayed] 
                TA[D, ijk], \[Rho]\_ijk__ \[RuleDelayed] 
                TA[\[Rho], ijk], \[Phi]\_ijk__ \[RuleDelayed] 
                TA[\[Phi], ijk], \[Sigma]\_\(i_, j_, k_\) \[RuleDelayed] 
                TA[\[Sigma], d, i, j, k, \[Nu]\_i, \[Nu]\_j, \[Nu]\_k], 
              h\_\(i_, j_, k_\) \[RuleDelayed] 
                TA[h, d, i, j, k, \[Nu]\_i, \[Nu]\_j, \[Nu]\_k], 
              S\_\(i_, j_, k_\) \[RuleDelayed] 
                TA[S, d, i, j, 
                  k, \[Nu]\_i, \[Nu]\_j, \[Nu]\_k], \[CapitalSigma]\_i_ \
\[RuleDelayed] ToArgs[\[CapitalSigma]\_i]}\) /. {\[Nu]\_1 \[RuleDelayed] 
              n1, \[Nu]\_2 \[RuleDelayed] n2, \[Nu]\_3 \[RuleDelayed] 
              n3, \[Nu]\_4 \[RuleDelayed] n4, \[Nu]\_5 \[RuleDelayed] n5, 
            m\_1 \[RuleDelayed] m1, m\_2 \[RuleDelayed] m2, 
            m\_3 \[RuleDelayed] m3, m\_4 \[RuleDelayed] m4, 
            m\_5 \[RuleDelayed] m5};\)\)], "Input"],

Cell[BoxData[
    \(\(addPeP[TFR[d_, pp_, li1_, rest_]] := 
        TFR[d, pp, li1, rest];\)\)], "Input"],

Cell[BoxData[
    \(\(addPeP[TFI[d_, pp_, li1_List]] := TFI[d, pp, li1];\)\)], "Input"],

Cell[BoxData[
    \(\(addPeP[TFR[d_, pp_, dp_Symbol, li1_, rest_]] := 
        TFR[d, pp, dp, li1, rest];\)\)], "Input"],

Cell[BoxData[
    \(\(addPeP[TFI[d_, pp_, dp_Symbol, li1_List]] := 
        TFI[d, pp, dp, li1];\)\)], "Input"],

Cell[BoxData[
    \(\(addPeP[TVI[d_, pp_, li1_List]] := TVI[d, pp, li1];\)\)], "Input"],

Cell[BoxData[
    \(nnnnnaddPeP[any_[d_, pp_, li1_List, rest__List]] := 
      any[d, pp, li1, rest] /; any =!= List\)], "Input"],

Cell[BoxData[
    \(\(addPeP[z_List] := Append[z, \@PP];\)\)], "Input"],

Cell["addPeP[TFI[d_, z_List]] := TFI[d, PP, z]; ", "Input"],

Cell[BoxData[
    \(\(addPeP[TVI[d_, z_List]] := TVI[d, PP, z];\)\)], "Input"],

Cell[BoxData[
    \(\(addPeP[TJI[d_, z_List]] := TJI[d, PP, z];\)\)], "Input"],

Cell[BoxData[
    \(\(addPeP[TFR[d_, z_List]] := TFR[d, PP_, z];\)\)], "Input"],

Cell[BoxData[
    \(\(addPeP[TFR[d_, pl_List, z_List]] := TFR[d, PP_, pl, z];\)\)], "Input"],

Cell[BoxData[
    \(\(addPeP[TVR[d_, z__List]] := TVR[d, PP_, z];\)\)], "Input"],

Cell[BoxData[
    \(\(addPeP[TJR[d_, z__List]] := TJR[d, PP_, z];\)\)], "Input"],

Cell[BoxData[
    \(\(nutomass[
          w_] := \(\(w /. {\[Nu]\_1 + i1_.  \[RuleDelayed] {n1 + i1, 
                    m1}, \[Nu]\_2 + i2_.  \[RuleDelayed] {n2 + i2, 
                    m2}, \[Nu]\_3 + i3_.  \[RuleDelayed] {n3 + i3, 
                    m3}, \[Nu]\_4 + i4_.  \[RuleDelayed] {n4 + i4, 
                    m4}, \[Nu]\_5 + i5_.  \[RuleDelayed] {n5 + i5, 
                    m5}}\) /. {in1_Integer, in2_Integer, in3_Integer, 
                in4_Integer} \[RuleDelayed] {{in1, m1}, {in2, m2}, {in3, 
                  m3}, {in4, m4}}\) /. {in1_Integer, in2_Integer, 
              in3_Integer} \[RuleDelayed] {{in1, m1}, {in2, m2}, {in3, 
                m3}};\)\)], "Input"],

Cell[BoxData[
    \(\(Clear[maybeF];\)\)], "Input"],

Cell[BoxData[
    \(\(maybeF[z_Times] := 
        FactorC[z] /; 
          FreeQ[z, TFI] && FreeQ[z, TVI] && FreeQ[z, TJI] && 
            Length[Cases[z, cD[__], \(-1\)]] > 1;\)\)], "Input"],

Cell[BoxData[
    \(maybeF[z_] := z; SetAttributes[top, Listable];\)], "Input"],

Cell["This is the function for the information of side conditions.", "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"SetAttributes", "[", 
        RowBox[{
          StyleBox["IFF",
            FontColor->RGBColor[0.6, 0.4, 1]], ",", "HoldAll"}], "]"}], 
      ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"Clear", "[", 
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "]"}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          StyleBox["MakeFun",
            FontColor->RGBColor[1, 0.6, 0.6]], "[", 
          RowBox[{
            RowBox[{
              RowBox[{"c_.", " ", 
                RowBox[{
                  StyleBox["TFI",
                    FontColor->RGBColor[1, 0.6, 0]], "[", "t__", "]"}]}], "==",
               "rhs_"}], ",", "eqn_String", ",", 
            RowBox[{
              StyleBox["IFF",
                FontColor->RGBColor[0.6, 0.4, 1]], "[", "conds_", "]"}], 
            ",", \(mrel_ : {}\)}], "]"}], ":=", 
        RowBox[{
          StyleBox["Block",
            CellFrame->True,
            FontSize->16,
            FontWeight->"Bold",
            FontColor->RGBColor[1, 0.2, 0.2]], "[", 
          RowBox[{\({nm}\), ",", 
            
            RowBox[{\(nmrel = Select[Flatten[{mrel}], FreeQ[#1, Factor] &]\), 
              ";", 
              RowBox[{
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                        RowBox[{
                          RowBox[{
                            RowBox[{
                              RowBox[{
                                RowBox[{
                                  RowBox[{
                                    RowBox[{"HoldForm", "@@", 
                                      RowBox[{"{", 
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{"setdel", "[", 
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{"addPeP", "[", 
                                        RowBox[{
                                        StyleBox["TFR",
                                        FontColor->RGBColor[0, 0.6, 0]], 
                                        "@@", \(top /@ nutomass[{t}]\)}], 
                                        "]"}], "/.", "nmrel"}], ",", 
                                        RowBox[{\(Hold[Condition]\), "[", 
                                        RowBox[{
                                        
                                        RowBox[{\(Hold[CompoundExpression]\), 
                                        "[", "\t", 
                                        
                                        RowBox[{\(\(Hold[TComment]\)[
                                        eqn, \(\(\(addPeP[nutomass[TFR[t]]] /. 
                                        TFR -> TFIC\) /. {Pattern :> pat, 
                                        Blank :> blank}\) /. pat[PP, 
                                        blank[]] :> PP\) /. nmrel]\), ",", 
                                        RowBox[{
                                        RowBox[{"Collect", "[", 
                                        RowBox[{
                                        RowBox[{"TT", "=", 
                                        RowBox[{"Expand3", "[", 
                                        RowBox[{
                                        "nutonand\[CapitalDelta]u", "[", 
                                        RowBox[{
                                        RowBox[{\(1\/c\), 
                                        RowBox[{"(", 
                                        RowBox[{"rhs", "/.", 
                                        RowBox[{
                                        RowBox[{
                                        StyleBox["TFI",
                                        FontColor->RGBColor[1, 0.6, 0]], "[", 
                                        "te__", "]"}], "\[RuleDelayed]", 
                                        RowBox[{"addPeP", "[", 
                                        RowBox[{"nutomass", "[", 
                                        RowBox[{
                                        StyleBox["TFI",
                                        FontColor->RGBColor[1, 0.6, 0]], "[", 
                                        "te", "]"}], "]"}], "]"}]}]}], 
                                        ")"}]}], ",", "5"}], "]"}], "]"}]}], 
                                        ",", 
                                        RowBox[{
                                        StyleBox["TFI",
                                        FontColor->RGBColor[1, 0.6, 0]], "[", 
                                        "__", "]"}], 
                                        ",", \(maybeF[\((Collect[
                                        Numerator[#1], cD[__], 
                                        Factor]/Collect[Denominator[#1], 
                                        cD[__], Factor] &)\) /@ 
                                        Factor[#1]] &\)}], "]"}], "/.", 
                                        "nmrel"}]}], "]"}], ",", 
                                        RowBox[{
                                        "nutonand\[CapitalDelta]u", "[", 
                                        RowBox[{
                                        RowBox[{
                                        StyleBox["IFF",
                                        FontColor->RGBColor[0.6, 0.4, 1]], 
                                        "[", "conds", "]"}], ",", "5"}], 
                                        "]"}]}], "]"}]}], "]"}], 
                                        "/.", \(top[i_Integer] \[Rule] i\)}], 
                                        "/.", \(top \[RuleDelayed] \
\((patternTest[pattern[#1, blank[]], PQ] &)\)\)}], "/.", 
                                        RowBox[{
                                        RowBox[{"patternTest", "[", 
                                        RowBox[{
                                        RowBox[{"pattern", "[", 
                                        RowBox[{
                                        RowBox[{"dm_", "/;", 
                                        RowBox[{"MemberQ", "[", 
                                        RowBox[{
                                        RowBox[{"{", 
                                        RowBox[{"d", ",", 
                                        InterpretationBox[\(p\^2\),
                                        PP], ",", "DP", ",", 
                                        InterpretationBox[\(m\_1\),
                                        m1], ",", 
                                        InterpretationBox[\(m\_2\),
                                        m2], ",", 
                                        InterpretationBox[\(m\_3\),
                                        m3], ",", 
                                        InterpretationBox[\(m\_4\),
                                        m4], ",", 
                                        InterpretationBox[\(m\_5\),
                                        m5]}], "}"}], ",", "dm"}], "]"}]}], 
                                        ",", \(blank[]\)}], "]"}], ",", "_"}],
                                         "]"}], 
                                        "\[RuleDelayed]", \(pattern[dm, 
                                        blank[]]\)}]}], "/.", "\n", 
                                        "\t\t\t\t\t", \({blank \[RuleDelayed] 
                                        Blank, 
                                        pattern \[RuleDelayed] Pattern, 
                                        patternTest \[RuleDelayed] 
                                        PatternTest}\)}], "}"}]}], "/.", 
                                    RowBox[{\(cD[ijk__]\), "\[Rule]", 
                                      RowBox[{
                                        RowBox[{"Hold", "[", 
                                        RowBox[{
                                        StyleBox["CayleyD",
                                        FontColor->RGBColor[1, 0.2, 1]], "[", 
                                        "ijk", "]"}], "]"}], 
                                        "[", \(addPeP[
                                        BLA = \(nutomass[Last[{t}]] /. 
                                        nmrel\) /. {nuu_, ma_} \[RuleDelayed] 
                                        ma]\), "]"}]}]}], 
                                  "/.", \(addPeP -> Identity\)}], 
                                "/.", \(Hold[CompoundExpression] \[Rule] 
                                  CompoundExpression\)}], "/.", 
                              RowBox[{
                                RowBox[{\(Hold[Condition]\), "[", 
                                  RowBox[{"a_", ",", 
                                    RowBox[{
                                      StyleBox["IFF",
                                        FontColor->RGBColor[0.6, 0.4, 1]], 
                                      "[", "b_", "]"}]}], "]"}], 
                                "\[Rule]", \(a /; b\)}]}], 
                            "/.", \(Hold[Set] \[Rule] Set\)}], 
                          "/.", \(Hold[TComment] -> TComment\)}], "/.", 
                        RowBox[{
                          RowBox[{"Hold", "[", 
                            RowBox[{
                              StyleBox["CayleyD",
                                FontColor->RGBColor[1, 0.2, 1]], "[", "ijk__",
                               "]"}], "]"}], "\[Rule]", 
                          RowBox[{
                            StyleBox["CayleyD",
                              FontColor->RGBColor[1, 0.2, 1]], "[", "ijk", 
                            "]"}]}]}], 
                      "/.", \({setdel \[RuleDelayed] SetDelayed}\)}], ")"}], 
                  "/.", \(Identity -> identit\)}], 
                "/.", \(identit[a_] -> a\)}]}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          StyleBox["MakeFun",
            FontColor->RGBColor[1, 0.6, 0.6]], "[", 
          RowBox[{
            RowBox[{
              RowBox[{"c_.", " ", 
                RowBox[{
                  StyleBox["TFI",
                    FontColor->RGBColor[1, 0.6, 0]], "[", "t__", "]"}]}], "==",
               "rhs_"}], ",", "eqn_String", ",", \(mrel_ : {}\)}], "]"}], ":=",
         
        RowBox[{
          RowBox[{
            StyleBox["Block",
              CellFrame->True,
              FontSize->16,
              FontWeight->"Bold",
              FontColor->RGBColor[1, 0.2, 0.2]], "[", 
            RowBox[{\({nm}\), ",", 
              
              RowBox[{\(nmrel = 
                  Select[Flatten[{mrel}], FreeQ[#1, Factor] &]\), ";", 
                RowBox[{
                  RowBox[{
                    RowBox[{
                      RowBox[{"(", 
                        RowBox[{
                          RowBox[{
                            RowBox[{
                              RowBox[{
                                RowBox[{
                                  RowBox[{
                                    RowBox[{
                                      RowBox[{"HoldForm", "@@", 
                                        RowBox[{"{", 
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{"setdel", "[", 
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{"addPeP", "[", 
                                        RowBox[{
                                        StyleBox["TFR",
                                        FontColor->RGBColor[0, 0.6, 0]], 
                                        "@@", \(top /@ nutomass[{t}]\)}], 
                                        "]"}], "/.", "nmrel"}], ",", 
                                        
                                        RowBox[{\(Hold[CompoundExpression]\), 
                                        "[", 
                                        
                                        RowBox[{\(\(Hold[TComment]\)[
                                        eqn, \(\(\(addPeP[nutomass[TFR[t]]] /. 
                                        TFR -> TFIC\) /. {Pattern :> pat, 
                                        Blank :> blank}\) /. pat[PP, 
                                        blank[]] :> PP\) /. nmrel]\), ",", 
                                        RowBox[{
                                        RowBox[{"Collect", "[", 
                                        RowBox[{
                                        RowBox[{"TT", "=", 
                                        RowBox[{"Expand3", "[", 
                                        RowBox[{
                                        "nutonand\[CapitalDelta]u", "[", 
                                        RowBox[{\(1\/c\), 
                                        RowBox[{"(", 
                                        RowBox[{"rhs", "/.", 
                                        RowBox[{
                                        RowBox[{
                                        StyleBox["TFI",
                                        FontColor->RGBColor[1, 0.6, 0]], "[", 
                                        "te__", "]"}], "\[RuleDelayed]", 
                                        RowBox[{"addPeP", "[", 
                                        RowBox[{"nutomass", "[", 
                                        RowBox[{
                                        StyleBox["TFI",
                                        FontColor->RGBColor[1, 0.6, 0]], "[", 
                                        "te", "]"}], "]"}], "]"}]}]}], 
                                        ")"}]}], "]"}], "]"}]}], ",", 
                                        RowBox[{
                                        StyleBox["TFI",
                                        FontColor->RGBColor[1, 0.6, 0]], "[", 
                                        "__", "]"}], 
                                        ",", \(maybeF[\((Collect[
                                        Numerator[#1], cD[__], 
                                        Factor]/Collect[Denominator[#1], 
                                        cD[__], Factor] &)\) /@ 
                                        Factor[#1]] &\)}], "]"}], "/.", 
                                        "nmrel"}]}], "]"}]}], "]"}], 
                                        "/.", \(top[i_Integer] \[Rule] i\)}], 
                                        "/.", \(top \[RuleDelayed] \
\((patternTest[pattern[#1, blank[]], PQ] &)\)\)}], "/.", 
                                        RowBox[{
                                        RowBox[{"patternTest", "[", 
                                        RowBox[{
                                        RowBox[{"pattern", "[", 
                                        RowBox[{
                                        RowBox[{"dm_", "/;", 
                                        RowBox[{"MemberQ", "[", 
                                        RowBox[{
                                        RowBox[{"{", 
                                        RowBox[{"d", ",", 
                                        InterpretationBox[\(p\^2\),
                                        PP], ",", "DP", ",", 
                                        InterpretationBox[\(m\_1\),
                                        m1], ",", 
                                        InterpretationBox[\(m\_2\),
                                        m2], ",", 
                                        InterpretationBox[\(m\_3\),
                                        m3], ",", 
                                        InterpretationBox[\(m\_4\),
                                        m4], ",", 
                                        InterpretationBox[\(m\_5\),
                                        m5]}], "}"}], ",", "dm"}], "]"}]}], 
                                        ",", \(blank[]\)}], "]"}], ",", "_"}],
                                         "]"}], "\[RuleDelayed]", \(pattern[
                                        dm, blank[]]\)}]}], "/.", 
                                        "\t", \({blank \[RuleDelayed] Blank, 
                                        pattern \[RuleDelayed] Pattern, 
                                        patternTest \[RuleDelayed] 
                                        PatternTest}\)}], "}"}]}], "/.", 
                                      RowBox[{\(cD[ijk__]\), "\[Rule]", 
                                        RowBox[{
                                        RowBox[{"Hold", "[", 
                                        RowBox[{
                                        StyleBox["CayleyD",
                                        FontColor->RGBColor[1, 0.2, 1]], "[", 
                                        "ijk", "]"}], "]"}], 
                                        "[", \(addPeP[\(\(nutomass[
                                        Last[{t}]] /. addPeP -> Identity\) /. 
                                        nmrel\) /. {nuu_, ma_} \[RuleDelayed] 
                                        ma]\), "]"}]}]}], 
                                    "/.", \(Hold[CompoundExpression] \[Rule] 
                                      CompoundExpression\)}], "/.", 
                                  RowBox[{
                                    RowBox[{\(Hold[Condition]\), "[", 
                                      RowBox[{"a_", ",", 
                                        RowBox[{
                                        StyleBox["IFF",
                                        FontColor->RGBColor[0.6, 0.4, 1]], 
                                        "[", "b_", "]"}]}], "]"}], 
                                    "\[Rule]", \(a /; b\)}]}], 
                                "/.", \(Hold[Set] \[Rule] Set\)}], 
                              "/.", \(Hold[TComment] -> TComment\)}], "/.", 
                            RowBox[{
                              RowBox[{"Hold", "[", 
                                RowBox[{
                                  StyleBox["CayleyD",
                                    FontColor->RGBColor[1, 0.2, 1]], "[", 
                                  "ijk__", "]"}], "]"}], "\[Rule]", 
                              RowBox[{
                                StyleBox["CayleyD",
                                  FontColor->RGBColor[1, 0.2, 1]], "[", "ijk",
                                 "]"}]}]}], 
                          "/.", \({setdel \[RuleDelayed] SetDelayed}\)}], 
                        ")"}], "/.", \(addPeP -> Identity\)}], 
                    "/.", \(Identity -> identit\)}], 
                  "/.", \(identit[a_] -> a\)}]}]}], "]"}], "/;", 
          RowBox[{\(Head[mrel]\), "=!=", 
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          StyleBox["MakeFun",
            FontColor->RGBColor[1, 0.6, 0.6]], "[", 
          RowBox[{
            RowBox[{
              RowBox[{"c_.", " ", 
                RowBox[{
                  StyleBox["TVI",
                    FontColor->RGBColor[0.2, 0.4, 0.8]], "[", "t__", "]"}]}], 
              "==", "rhs_"}], ",", "eqn_String", ",", 
            RowBox[{
              StyleBox["IFF",
                FontColor->RGBColor[0.6, 0.4, 1]], "[", "conds_", "]"}], 
            ",", \(mrel_ : {}\)}], "]"}], ":=", 
        RowBox[{
          StyleBox["Block",
            FontSize->16,
            FontWeight->"Bold",
            FontColor->RGBColor[1, 0.2, 0.2]], "[", 
          RowBox[{\({nm}\), ",", 
            
            RowBox[{\(nmrel = Select[Flatten[{mrel}], FreeQ[#1, Factor] &]\), 
              ";", 
              RowBox[{"nmrel", "=", 
                RowBox[{"Join", "[", 
                  RowBox[{"nmrel", ",", 
                    RowBox[{"{", 
                      RowBox[{
                        RowBox[{\(m\_1\), "\[RuleDelayed]", 
                          InterpretationBox[\(m\_1\),
                            m1]}], ",", 
                        RowBox[{\(m\_2\), "\[RuleDelayed]", 
                          InterpretationBox[\(m\_2\),
                            m2]}], ",", 
                        RowBox[{\(m\_3\), "\[RuleDelayed]", 
                          InterpretationBox[\(m\_3\),
                            m3]}], ",", 
                        RowBox[{\(m\_4\), "\[RuleDelayed]", 
                          InterpretationBox[\(m\_4\),
                            m4]}], 
                        ",", \(\[CapitalSigma]\_i_ :> 
                          ToArgs[\[CapitalSigma]\_i]\)}], "}"}]}], "]"}]}], 
              ";", 
              RowBox[{
                RowBox[{
                  RowBox[{
                    RowBox[{
                      RowBox[{
                        RowBox[{
                          RowBox[{
                            RowBox[{
                              RowBox[{
                                RowBox[{
                                  RowBox[{
                                    RowBox[{
                                      RowBox[{
                                        RowBox[{
                                        RowBox[{"HoldForm", "@@", 
                                        RowBox[{"{", 
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{"setdel", "[", 
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{"addPeP", "[", 
                                        RowBox[{
                                        StyleBox["TVR",
                                        FontColor->RGBColor[0.2, 0.8, 1]], 
                                        "@@", \(top /@ nutomass[{t}]\)}], 
                                        "]"}], "/.", "nmrel"}], ",", 
                                        RowBox[{\(Hold[Condition]\), "[", 
                                        RowBox[{
                                        
                                        RowBox[{\(Hold[CompoundExpression]\), 
                                        "[", 
                                        
                                        RowBox[{\(\(Hold[TComment]\)[
                                        eqn, \(\(\(addPeP[nutomass[TVR[t]]] /. 
                                        TVR -> TVIC\) /. {Pattern :> pat, 
                                        Blank :> blank}\) /. pat[PP, 
                                        blank[]] :> PP\) /. nmrel]\), ",", 
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{"Collect", "[", 
                                        RowBox[{
                                        RowBox[{"TT", "=", 
                                        RowBox[{"Expand3", "[", 
                                        RowBox[{"nutonand\[CapitalDelta]u", 
                                        "[", 
                                        RowBox[{\(1\/c\), 
                                        RowBox[{"(", 
                                        RowBox[{"rhs", "/.", 
                                        RowBox[{
                                        RowBox[{
                                        StyleBox["TVI",
                                        FontColor->RGBColor[0.2, 0.4, 0.8]], 
                                        "[", "te__", "]"}], "\[RuleDelayed]", 
                                        
                                        RowBox[{"addPeP", "[", 
                                        RowBox[{"nutomass", "[", 
                                        RowBox[{
                                        StyleBox["TVI",
                                        FontColor->RGBColor[0.2, 0.4, 0.8]], 
                                        "[", "te", "]"}], "]"}], "]"}]}]}], 
                                        ")"}]}], "]"}], "]"}]}], ",", 
                                        RowBox[{
                                        StyleBox["TVI",
                                        FontColor->RGBColor[0.2, 0.4, 0.8]], 
                                        "[", "__", "]"}], 
                                        ",", \(maybeF[\((Collect[
                                        Numerator[#1], cu[__], 
                                        Factor]/Collect[Denominator[#1], 
                                        cu[__], Factor] &)\) /@ 
                                        Factor[#1]] &\)}], "]"}], 
                                        "/.", \(\[ScriptCapitalC] -> 1\)}], "/.",
                                         "nmrel"}]}], "]"}], ",", 
                                        RowBox[{
                                        "nutonand\[CapitalDelta]u", "[", 
                                        RowBox[{
                                        StyleBox["IFF",
                                        FontColor->RGBColor[0.6, 0.4, 1]], 
                                        "[", "conds", "]"}], "]"}]}], "]"}]}],
                                         "]"}], "/.", \(top[
                                        i_Integer] \[Rule] i\)}], 
                                        "/.", \(top \[RuleDelayed] \
\((patternTest[pattern[#1, blank[]], PQ] &)\)\)}], "/.", 
                                        RowBox[{
                                        RowBox[{"patternTest", "[", 
                                        RowBox[{
                                        RowBox[{"pattern", "[", 
                                        RowBox[{
                                        RowBox[{"dm_", "/;", 
                                        RowBox[{"MemberQ", "[", 
                                        RowBox[{
                                        RowBox[{"{", 
                                        RowBox[{"d", ",", 
                                        InterpretationBox[\(p\^2\),
                                        PP], ",", 
                                        InterpretationBox[\(m\_1\),
                                        m1], ",", 
                                        InterpretationBox[\(m\_2\),
                                        m2], ",", 
                                        InterpretationBox[\(m\_3\),
                                        m3], ",", 
                                        InterpretationBox[\(m\_4\),
                                        m4]}], "}"}], ",", "dm"}], "]"}]}], 
                                        ",", \(blank[]\)}], "]"}], ",", "_"}],
                                         "]"}], "\[RuleDelayed]", \(pattern[
                                        dm, blank[]]\)}]}], "/.", 
                                        "\t", \({blank \[RuleDelayed] Blank, 
                                        pattern \[RuleDelayed] Pattern, 
                                        patternTest \[RuleDelayed] 
                                        PatternTest}\)}], "}"}]}], "/.", "\n",
                                         "\t\t\t", \(HoldPattern[
                                        TA[aijk__]] \[Rule] \(Hold[
                                        TA[aijk]]\)[
                                        addPeP[\(nutomass[Last[{t}]] /. 
                                        nmrel\) /. {nuu_, ma_} \[RuleDelayed] 
                                        ma]]\)}], "/.", "\n", 
                                        "\t\t\t", \(cD[
                                        ijk__] \[Rule] \(Hold[Cayley[ijk]]\)[
                                        addPeP[\(nutomass[Last[{t}]] /. 
                                        nmrel\) /. {nuu_, ma_} \[RuleDelayed] 
                                        ma]]\)}], "/.", 
                                      RowBox[{\(cu[ijk__]\), "\[Rule]", 
                                        RowBox[{
                                        RowBox[{"Hold", "[", 
                                        RowBox[{
                                        StyleBox["Cayleyu",
                                        FontColor->RGBColor[0.8, 0.8, 0.4]], 
                                        "[", "ijk", "]"}], "]"}], 
                                        "[", \(addPeP[\(\(nutomass[
                                        Last[{t}]] /. addPeP -> Identity\) /. 
                                        nmrel\) /. {nuu_, ma_} \[RuleDelayed] 
                                        ma]\), "]"}]}]}], 
                                    "/.", \(Hold[CompoundExpression] \[Rule] 
                                      CompoundExpression\)}], "/.", 
                                  RowBox[{
                                    RowBox[{\(Hold[Condition]\), "[", 
                                      RowBox[{"a_", ",", 
                                        RowBox[{
                                        StyleBox["IFF",
                                        FontColor->RGBColor[0.6, 0.4, 1]], 
                                        "[", "b_", "]"}]}], "]"}], 
                                    "\[Rule]", \(a /; b\)}]}], 
                                "/.", \(Hold[Set] \[Rule] Set\)}], 
                              "/.", \(Hold[TComment] -> TComment\)}], 
                            "/.", \(addPeP -> Identity\)}], "/.", 
                          RowBox[{
                            RowBox[{"Hold", "[", 
                              RowBox[{
                                StyleBox["Cayleyu",
                                  FontColor->RGBColor[0.8, 0.8, 0.4]], "[", 
                                "ijk__", "]"}], "]"}], "\[Rule]", 
                            RowBox[{
                              StyleBox["Cayleyu",
                                FontColor->RGBColor[0.8, 0.8, 0.4]], "[", 
                              "ijk", "]"}]}]}], "/.", 
                        RowBox[{
                          RowBox[{"Hold", "[", 
                            RowBox[{
                              StyleBox["Cayley",
                                FontColor->RGBColor[1, 0.2, 1]], "[", "ijk__",
                               "]"}], "]"}], "\[Rule]", 
                          RowBox[{
                            StyleBox["Cayley",
                              FontColor->RGBColor[1, 0.2, 1]], "[", "ijk", 
                            "]"}]}]}], "/.", "\n", 
                      "\t\t\t", \(Hold[TA] -> TA\)}], 
                    "/.", \({setdel \[RuleDelayed] SetDelayed}\)}], 
                  "/.", \(Identity -> identit\)}], 
                "/.", \(identit[a_] -> a\)}]}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          StyleBox["MakeFun",
            FontColor->RGBColor[1, 0.6, 0.6]], "[", 
          RowBox[{
            RowBox[{
              RowBox[{"c_.", " ", 
                RowBox[{
                  StyleBox["TVI",
                    FontColor->RGBColor[0.2, 0.4, 0.8]], "[", "t__", "]"}]}], 
              "==", "rhs_"}], ",", "eqn_String", ",", \(mrel_ : {}\)}], "]"}],
         ":=", 
        RowBox[{
          RowBox[{
            StyleBox["Block",
              FontSize->16,
              FontWeight->"Bold",
              FontColor->RGBColor[1, 0.2, 0.2]], "[", 
            RowBox[{\({nm}\), ",", 
              
              RowBox[{\(nmrel = 
                  Select[Flatten[{mrel}], FreeQ[#1, Factor] &]\), ";", 
                RowBox[{"nmrel", "=", 
                  RowBox[{"Join", "[", 
                    RowBox[{"nmrel", ",", 
                      RowBox[{"{", 
                        RowBox[{
                          RowBox[{\(m\_1\), "\[RuleDelayed]", 
                            InterpretationBox[\(m\_1\),
                              m1]}], ",", 
                          RowBox[{\(m\_2\), "\[RuleDelayed]", 
                            InterpretationBox[\(m\_2\),
                              m2]}], ",", 
                          RowBox[{\(m\_3\), "\[RuleDelayed]", 
                            InterpretationBox[\(m\_3\),
                              m3]}], ",", 
                          RowBox[{\(m\_4\), "\[RuleDelayed]", 
                            InterpretationBox[\(m\_4\),
                              m4]}], ",", 
                          RowBox[{\(\[CapitalSigma]\_i_\), "\[RuleDelayed]", 
                            RowBox[{
                              StyleBox["ToArgs",
                                FontColor->RGBColor[0.2, 0.6, 1]], 
                              "[", \(\[CapitalSigma]\_i\), "]"}]}]}], "}"}]}],
                     "]"}]}], ";", 
                RowBox[{
                  RowBox[{
                    RowBox[{
                      RowBox[{
                        RowBox[{
                          RowBox[{
                            RowBox[{
                              RowBox[{
                                RowBox[{
                                  RowBox[{
                                    RowBox[{
                                      RowBox[{"HoldForm", "@@", 
                                        RowBox[{"{", 
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{"setdel", "[", 
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{"addPeP", "[", 
                                        RowBox[{
                                        StyleBox["TVR",
                                        FontColor->RGBColor[0.2, 0.8, 1]], 
                                        "@@", \(top /@ nutomass[{t}]\)}], 
                                        "]"}], "/.", "nmrel"}], ",", 
                                        
                                        RowBox[{\(Hold[CompoundExpression]\), 
                                        "[", 
                                        
                                        RowBox[{\(\(Hold[TComment]\)[
                                        eqn, \(\(\(addPeP[nutomass[TVR[t]]] /. 
                                        TVR -> TVIC\) /. {Pattern :> pat, 
                                        Blank :> blank}\) /. pat[PP, 
                                        blank[]] :> PP\) /. nmrel]\), "\n", 
                                        "\t\t\t\t\t\t", ",", 
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{"Collect", "[", 
                                        RowBox[{
                                        RowBox[{"TT", "=", 
                                        RowBox[{"Expand3", "[", 
                                        RowBox[{
                                        "nutonand\[CapitalDelta]u", "[", 
                                        RowBox[{\(1\/c\), 
                                        RowBox[{"(", 
                                        RowBox[{"rhs", "/.", 
                                        RowBox[{
                                        RowBox[{
                                        StyleBox["TVI",
                                        FontColor->RGBColor[0.2, 0.4, 0.8]], 
                                        "[", "te__", "]"}], "\[RuleDelayed]", 
                                        
                                        RowBox[{"addPeP", "[", 
                                        RowBox[{"nutomass", "[", 
                                        RowBox[{
                                        StyleBox["TVI",
                                        FontColor->RGBColor[0.2, 0.4, 0.8]], 
                                        "[", "te", "]"}], "]"}], "]"}]}]}], 
                                        ")"}]}], "]"}], "]"}]}], ",", 
                                        RowBox[{
                                        StyleBox["TVI",
                                        FontColor->RGBColor[0.2, 0.4, 0.8]], 
                                        "[", "__", "]"}], 
                                        ",", \(maybeF[\((Collect[
                                        Numerator[#1], cu[__], 
                                        Factor]/Collect[Denominator[#1], 
                                        cu[__], Factor] &)\) /@ 
                                        Factor[#1]] &\)}], "]"}], "/.", 
                                        "nmrel"}], 
                                        "/.", \(\[ScriptCapitalC] \[Rule] 
                                        1\)}]}], "]"}]}], "]"}], 
                                        "/.", \(top[i_Integer] \[Rule] i\)}], 
                                        "/.", \(top \[RuleDelayed] \
\((patternTest[pattern[#1, blank[]], PQ] &)\)\)}], "/.", 
                                        RowBox[{
                                        RowBox[{"patternTest", "[", 
                                        RowBox[{
                                        RowBox[{"pattern", "[", 
                                        RowBox[{
                                        RowBox[{"dm_", "/;", 
                                        RowBox[{"MemberQ", "[", 
                                        RowBox[{
                                        RowBox[{"{", 
                                        RowBox[{"d", ",", 
                                        InterpretationBox[\(p\^2\),
                                        PP], ",", 
                                        InterpretationBox[\(m\_1\),
                                        m1], ",", 
                                        InterpretationBox[\(m\_2\),
                                        m2], ",", 
                                        InterpretationBox[\(m\_3\),
                                        m3], ",", 
                                        InterpretationBox[\(m\_4\),
                                        m4]}], "}"}], ",", "dm"}], "]"}]}], 
                                        ",", \(blank[]\)}], "]"}], ",", "_"}],
                                         "]"}], "\[RuleDelayed]", \(pattern[
                                        dm, blank[]]\)}]}], 
                                        "/.", \({blank \[RuleDelayed] Blank, 
                                        pattern \[RuleDelayed] Pattern, 
                                        patternTest \[RuleDelayed] 
                                        PatternTest}\)}], "}"}]}], "/.", 
                                      RowBox[{\(cD[ijk__]\), "\[Rule]", 
                                        RowBox[{
                                        RowBox[{"Hold", "[", 
                                        RowBox[{
                                        StyleBox["Cayley",
                                        FontColor->RGBColor[1, 0, 0.4]], "[", 
                                        "ijk", "]"}], "]"}], 
                                        "[", \(addPeP[\(nutomass[Last[{t}]] /. 
                                        nmrel\) /. {nuu_, 
                                        ma_Symbol} \[RuleDelayed] ma]\), 
                                        "]"}]}]}], "/.", 
                                    RowBox[{\(cu[ijk__]\), "\[Rule]", 
                                      RowBox[{
                                        RowBox[{"Hold", "[", 
                                        RowBox[{
                                        StyleBox["Cayleyu",
                                        FontColor->RGBColor[0.8, 0.8, 0.4]], 
                                        "[", "ijk", "]"}], "]"}], 
                                        "[", \(addPeP[\(nutomass[Last[{t}]] /. 
                                        nmrel\) /. {nuu_, 
                                        ma_Symbol} \[RuleDelayed] ma]\), 
                                        "]"}]}]}], 
                                  "/.", \(Hold[CompoundExpression] \[Rule] 
                                    CompoundExpression\)}], 
                                "/.", \(Hold[Set] \[Rule] Set\)}], 
                              "/.", \(Hold[TComment] -> TComment\)}], 
                            "/.", \(addPeP -> Identity\)}], "/.", 
                          RowBox[{
                            RowBox[{"Hold", "[", 
                              RowBox[{
                                StyleBox["Cayleyu",
                                  FontColor->RGBColor[0.8, 0.8, 0.4]], "[", 
                                "ijk__", "]"}], "]"}], "\[Rule]", 
                            RowBox[{
                              StyleBox["Cayleyu",
                                FontColor->RGBColor[0.8, 0.8, 0.4]], "[", 
                              "ijk", "]"}]}]}], "/.", 
                        RowBox[{
                          RowBox[{"Hold", "[", 
                            RowBox[{
                              StyleBox["Cayley",
                                FontColor->RGBColor[1, 0, 0.4]], "[", "ijk__",
                               "]"}], "]"}], "\[Rule]", 
                          RowBox[{
                            StyleBox["Cayley",
                              FontColor->RGBColor[1, 0, 0.4]], "[", "ijk", 
                            "]"}]}]}], 
                      "/.", \({setdel \[RuleDelayed] SetDelayed}\)}], 
                    "/.", \(Identity -> identit\)}], 
                  "/.", \(identit[a_] -> a\)}]}]}], "]"}], "/;", 
          RowBox[{\(Head[mrel]\), "=!=", 
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          StyleBox["MakeFun",
            FontColor->RGBColor[1, 0.6, 0.6]], "[", 
          RowBox[{\(c_. \ TJI[t__] == rhs_\), ",", "eqn_String", ",", 
            RowBox[{
              StyleBox["IFF",
                FontColor->RGBColor[0.6, 0.4, 1]], "[", "conds_", "]"}], 
            ",", \(mrel_ : {}\)}], "]"}], ":=", 
        RowBox[{
          StyleBox["Block",
            FontSize->16,
            FontWeight->"Bold",
            FontColor->RGBColor[1, 0.2, 0.2]], "[", 
          RowBox[{\({nm, nmrel, \ result}\), ",", 
            
            RowBox[{\(\[ScriptCapitalC] = 1\), 
              ";", \(nmrel = Select[Flatten[{mrel}], FreeQ[#1, Factor] &]\), 
              ";", 
              RowBox[{"nmrel", "=", 
                RowBox[{"Join", "[", 
                  RowBox[{"nmrel", ",", 
                    RowBox[{"{", 
                      RowBox[{
                        RowBox[{\(m\_1\), "\[RuleDelayed]", 
                          InterpretationBox[\(m\_1\),
                            m1]}], ",", 
                        RowBox[{\(m\_2\), "\[RuleDelayed]", 
                          InterpretationBox[\(m\_2\),
                            m2]}], ",", 
                        RowBox[{\(m\_3\), "\[RuleDelayed]", 
                          InterpretationBox[\(m\_3\),
                            m3]}], ",", 
                        RowBox[{\(m\_4\), "\[RuleDelayed]", 
                          InterpretationBox[\(m\_4\),
                            m4]}], ",", 
                        RowBox[{\(\[CapitalSigma]\_i_\), "\[RuleDelayed]", 
                          RowBox[{
                            StyleBox["ToArgs",
                              FontColor->RGBColor[0.2, 0.6, 1]], 
                            "[", \(\[CapitalSigma]\_i\), "]"}]}]}], "}"}]}], 
                  "]"}]}], ";", "\n", "\t\t", 
              RowBox[{"result", " ", "=", " ", 
                RowBox[{
                  RowBox[{
                    RowBox[{
                      RowBox[{
                        RowBox[{
                          RowBox[{
                            RowBox[{
                              RowBox[{
                                RowBox[{
                                  RowBox[{
                                    RowBox[{
                                      RowBox[{"HoldForm", "@@", 
                                        RowBox[{"{", 
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{
                                        RowBox[{"setdel", "[", 
                                        
                                        RowBox[{\(addPeP[TJR @@ \(top /@ 
                                        nutomass[{tT = t}]\)] /. nmrel\), ",",
                                         
                                        RowBox[{\(Hold[Condition]\), "[", 
                                        
                                        RowBox[{\(\(Hold[
                                        CompoundExpression]\)[\(Hold[
                                        TComment]\)[eqn, \(\(\(addPeP[
                                        nutomass[TJR[t]]] /. TJR -> 
                                        TJIC\) /. {Pattern :> pat, Blank :> 
                                        blank}\) /. pat[PP, blank[]] :> PP\) /. 
                                        nmrel]\t, TJ = \(Collect[TT = Expand3[
                                        nutonand\[CapitalDelta]u[\(1\/c\) \
\((rhs /. TJI[te__] \[RuleDelayed] addPeP[nutomass[TJI[te]]])\)]], TJI[__], 
                                        maybeF[\((Collect[Numerator[#1], 
                                        cu[__], 
                                        Factor]/Collect[Denominator[#1], 
                                        cu[__], Factor] &)\) /@ 
                                        Factor[#1]] &] /. \[ScriptCapitalC] \
\[Rule] 1\) /. nmrel]\), ",", 
                                        RowBox[{
                                        "nutonand\[CapitalDelta]u", "[", 
                                        RowBox[{
                                        StyleBox["IFF",
                                        FontColor->RGBColor[0.6, 0.4, 1]], 
                                        "[", "conds", "]"}], "]"}]}], "]"}]}],
                                         "]"}], "/.", \(top[
                                        i_Integer] \[Rule] i\)}], 
                                        "/.", \(top \[RuleDelayed] \
\((patternTest[pattern[#1, blank[]], PQ] &)\)\)}], "/.", 
                                        RowBox[{
                                        RowBox[{"patternTest", "[", 
                                        RowBox[{
                                        RowBox[{"pattern", "[", 
                                        RowBox[{
                                        RowBox[{"dm_", "/;", 
                                        RowBox[{"MemberQ", "[", 
                                        RowBox[{
                                        RowBox[{"{", 
                                        RowBox[{"d", ",", 
                                        InterpretationBox[\(p\^2\),
                                        PP], ",", 
                                        InterpretationBox[\(m\_1\),
                                        m1], ",", 
                                        InterpretationBox[\(m\_2\),
                                        m2], ",", 
                                        InterpretationBox[\(m\_3\),
                                        m3], ",", 
                                        InterpretationBox[\(m\_4\),
                                        m4]}], "}"}], ",", "dm"}], "]"}]}], 
                                        ",", \(blank[]\)}], "]"}], ",", "_"}],
                                         "]"}], "\[RuleDelayed]", \(pattern[
                                        dm, blank[]]\)}]}], 
                                        "/.", \({blank \[RuleDelayed] Blank, 
                                        pattern \[RuleDelayed] Pattern, 
                                        patternTest \[RuleDelayed] 
                                        PatternTest}\)}], "}"}]}], 
                                      "/.", \(HoldPattern[
                                        TA[aijk__]] \[Rule] \(\(Hold[TA]\)[
                                        aijk]\)[
                                        addPeP[\(nutomass[Last[{t}]] /. 
                                        nmrel\) /. {nuu_, ma_} \[RuleDelayed] 
                                        ma]]\)}], "/.", 
                                    RowBox[{\(cD[ijk__]\), "\[Rule]", 
                                      RowBox[{
                                        RowBox[{"Hold", "[", 
                                        RowBox[{
                                        StyleBox["Cayley",
                                        FontColor->RGBColor[1, 0, 0.4]], "[", 
                                        "ijk", "]"}], "]"}], 
                                        "[", \(addPeP[\(nutomass[Last[{t}]] /. 
                                        nmrel\) /. {nuu_, 
                                        ma_Symbol} \[RuleDelayed] ma]\), 
                                        "]"}]}]}], "/.", 
                                  RowBox[{\(cu[ijk__]\), "\[Rule]", 
                                    RowBox[{
                                      RowBox[{"Hold", "[", 
                                        RowBox[{
                                        StyleBox["Cayleyu",
                                        FontColor->RGBColor[0.8, 0.8, 0.4]], 
                                        "[", "ijk", "]"}], "]"}], 
                                      "[", \(addPeP[\(nutomass[Last[{t}]] /. 
                                        nmrel\) /. {nuu_, 
                                        ma_Symbol} \[RuleDelayed] ma]\), 
                                      "]"}]}]}], 
                                "/.", \(Hold[CompoundExpression] \[Rule] 
                                  CompoundExpression\)}], "/.", 
                              RowBox[{
                                RowBox[{\(Hold[Condition]\), "[", 
                                  RowBox[{"a_", ",", 
                                    RowBox[{
                                      StyleBox["IFF",
                                        FontColor->RGBColor[0.6, 0.4, 1]], 
                                      "[", "b_", "]"}]}], "]"}], 
                                "\[Rule]", \(a /; b\)}]}], 
                            "/.", \(Hold[Set] \[Rule] Set\)}], 
                          "/.", \(Hold[TComment] -> TComment\)}], "/.", 
                        RowBox[{
                          RowBox[{"Hold", "[", 
                            RowBox[{
                              StyleBox["Cayleyu",
                                FontColor->RGBColor[0.8, 0.8, 0.4]], "[", 
                              "ijk__", "]"}], "]"}], "\[Rule]", 
                          RowBox[{
                            StyleBox["Cayleyu",
                              FontColor->RGBColor[0.8, 0.8, 0.4]], "[", "ijk",
                             "]"}]}]}], "/.", 
                      RowBox[{
                        RowBox[{"Hold", "[", 
                          RowBox[{
                            StyleBox["Cayley",
                              FontColor->RGBColor[1, 0, 0.4]], "[", "ijk__", 
                            "]"}], "]"}], "\[Rule]", 
                        RowBox[{
                          StyleBox["Cayley",
                            FontColor->RGBColor[1, 0, 0.4]], "[", "ijk", 
                          "]"}]}]}], "/.", \(Hold[TA] \[Rule] TA\)}], 
                  "/.", \({setdel \[RuleDelayed] SetDelayed}\)}]}], 
              ";", \(\[ScriptCapitalC] =  . \), ";", " ", "result"}]}], 
          "]"}]}], ";"}]], "Input"],

Cell[BoxData[
    \(\(SetAttributes[paTT, HoldFirst];\)\)], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{
          RowBox[{
            RowBox[{"c_.", " ", 
              RowBox[{
                StyleBox["TKI",
                  FontColor->RGBColor[0.2, 0.4, 0.8]], "[", "t__", "]"}]}], "==",
             "rhs_"}], ",", "eqn_String", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", "conds_", "]"}], 
          ",", \(mrel_ : {}\)}], "]"}], ":=", 
      " ", \(\(\(\(MakeFun[c\ TJI[t] == rhs /. TKI -> TJI, eqn, IFF[conds], 
                mrel] /. {PP :> 0, Sqrt[PP] :> 0, Pattern :> paTT}\) /. 
            paTT[0, _] -> 0\) /. paTT -> Pattern\) /. 
        TJIC[d_, li_List] :> TJIC[d, 0, li]\)}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          StyleBox["MakeFun",
            FontColor->RGBColor[1, 0.6, 0.6]], "[", 
          RowBox[{
            RowBox[{
              RowBox[{"c_.", " ", 
                RowBox[{
                  StyleBox["TBI",
                    FontColor->RGBColor[0.2, 0.4, 0.8]], "[", "t__", "]"}]}], 
              "==", "rhs_"}], ",", "eqn_String", ",", 
            RowBox[{
              StyleBox["IFF",
                FontColor->RGBColor[0.6, 0.4, 1]], "[", "conds_", "]"}], 
            ",", \(mrel_ : {}\)}], "]"}], ":=", 
        " ", \(MakeFun[c\ TJI[t] == rhs /. TBI -> TJI, eqn, IFF[conds], 
            mrel] /. {\n\t\t\tTJI :> TBI, TJR :> TBR, TJIC :> TBIC}\)}], 
      ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          StyleBox["MakeFun",
            FontColor->RGBColor[1, 0.6, 0.6]], "[", 
          RowBox[{
            RowBox[{
              RowBox[{"c_.", " ", 
                RowBox[{
                  StyleBox["TAI",
                    FontColor->RGBColor[0.2, 0.4, 0.8]], "[", "t__", "]"}]}], 
              "==", "rhs_"}], ",", "eqn_String", ",", 
            RowBox[{
              StyleBox["IFF",
                FontColor->RGBColor[0.6, 0.4, 1]], "[", "conds_", "]"}], 
            ",", \(mrel_ : {}\)}], "]"}], ":=", 
        " ", \(MakeFun[c\ TJI[t] == rhs /. TAI -> TJI, eqn, IFF[conds], 
            mrel] /. {\n\t\t\tTJI :> TAI, TJR :> TAR, TJIC :> TAIC}\)}], 
      ";"}]], "Input"],

Cell[CellGroupData[{

Cell["Interchange", "Subsubsection",
  CellTags->"T1.5.1.1.1"],

Cell[BoxData[
    RowBox[{\(Interchange[z_, i_ \[LeftRightArrow] j_]\), ":=", 
      RowBox[{\(\[Nu]explicit[
            z] /. {\[Nu]\_i :> \[Nu]\_j, \[Nu]\_j :> \[Nu]\_i, m\_i :> m\_j, 
            m\_j :> m\_i, \[CapitalDelta]\_ij__ :> \((\[CapitalDelta]\_ij /. \
{i :> j, j :> i})\), 
            u\_ij__ :> \((u\_ij /. {i :> j, 
                    j :> i})\), \n\t\th\_ij__ :> \((h\_ij /. {i :> j, 
                    j :> i})\), \[Phi]\_ij__ :> \((\[Phi]\_ij /. {i :> j, 
                    j :> i})\), \[Rho]\_ij__ :> \((\[Rho]\_ij /. {i :> j, 
                    j :> i})\), \[Sigma]\_ij__ :> \((\[Sigma]\_ij /. {i :> j, 
                    j :> i})\), D\_ij__ :> \((D\_ij /. {i :> j, j :> i})\)}\),
         "/.", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{
              RowBox[{
                StyleBox["TFI",
                  FontColor->RGBColor[1, 0.6, 0]], "[", \(depp__, args_List\),
                 "]"}], ":>", 
              RowBox[{
                RowBox[{
                  StyleBox["TFI",
                    FontColor->RGBColor[1, 0.6, 0]], "[", \(depp, 
                  Sort[args]\), "]"}], "/;", 
                "\[InvisibleSpace]", \(! \((OrderedQ[args])\)\)}]}], ",", 
            "\n", "\t", 
            RowBox[{
              RowBox[{
                StyleBox["TVI",
                  FontColor->RGBColor[1, 0.6, 0]], "[", \(depp__, args_List\),
                 "]"}], ":>", 
              RowBox[{
                RowBox[{
                  StyleBox["TVI",
                    FontColor->RGBColor[1, 0.6, 0]], "[", \(depp, 
                  Sort[args]\), "]"}], "/;", 
                "\[InvisibleSpace]", \(! \((OrderedQ[args])\)\)}]}], ",", 
            "\n", "\t", 
            RowBox[{
              RowBox[{
                StyleBox["TJI",
                  FontColor->RGBColor[1, 0.6, 0]], "[", \(depp__, args_List\),
                 "]"}], ":>", 
              RowBox[{
                RowBox[{
                  StyleBox["TJI",
                    FontColor->RGBColor[1, 0.6, 0]], "[", \(depp, 
                  Sort[args]\), "]"}], "/;", 
                "\[InvisibleSpace]", \(! \((OrderedQ[args])\)\)}]}], ",", 
            "\n", "\t\t\t", 
            RowBox[{
              RowBox[{
                StyleBox["TBI",
                  FontColor->RGBColor[1, 0.6, 0]], "[", \(depp__, args_List\),
                 "]"}], ":>", 
              RowBox[{
                RowBox[{
                  StyleBox["TBI",
                    FontColor->RGBColor[1, 0.6, 0]], "[", \(depp, 
                  Sort[args]\), "]"}], "/;", 
                "\[InvisibleSpace]", \(! \((OrderedQ[args])\)\)}]}]}], 
          "}"}]}]}]], "Input"],

Cell[BoxData[
    \(\(Interchange[z_, i_ \[LeftRightArrow] j_, vw__] := 
        Interchange[Interchange[z, i \[LeftRightArrow] j], vw];\)\)], "Input"],

Cell[BoxData[
    \(\(Interchange[z_, {i_ \[LeftRightArrow] j_}] := 
        Interchange[Interchange[z, i \[LeftRightArrow] j]];\)\)], "Input"],

Cell[BoxData[
    \(\(Interchange[z_, {i_ \[LeftRightArrow] j_, vw__}] := 
        Interchange[Interchange[z, i \[LeftRightArrow] j], vw];\)\)], "Input"],

Cell[BoxData[
    \(Interchange[
      test[\[CapitalDelta]\_\(x, b, y\), \[Nu]\_x, \[Nu]\_y, \[Nu]\_a, \
\[Nu]\_b, m\_x], \ x \[LeftRightArrow] y, a \[LeftRightArrow] b]\)], "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["\[Nu]explicit", "Subsection",
  CellTags->"T1.5.2.1"],

Cell[BoxData[
    \(\[Nu]explicit[
        z_] := \(z /. \[CapitalSigma]\_i_ :> \((Expand[
                3\ d\  - 2\  - 
                  2\ \(\[Sum]\_\(j = 1\)\%i \[Nu]\_j\)])\)\) /. \n\t{\t\
\[Sigma]\_\(i_, j_, k_\) :> \(-\(1\/4\)\) \((d - \[Nu]\_i - 
                    2  \[Nu]\_j)\) \[Phi]\_\(i, j, k\) - \(1\/4\) \((d - 
                    2  \[Nu]\_i - \[Nu]\_j)\) \[Phi]\_\(j, i, k\) - \n\t\t\t\t\
\t\(1\/4\) \((2  d - 2  \[Nu]\_i - 2  \[Nu]\_j - \[Nu]\_k - 
                    1)\) \[Phi]\_\(k, i, j\), \n\t\t\t\
h\_\(\(i_\)\(,\)\(j_\)\(,\)\(k_\)\(\ \)\) :> \n\t\t\t\t\(-\(1\/2\)\) \((d - 
                    2  \[Nu]\_j - \[Nu]\_k)\) \(m\_k\^2\) \[Phi]\_\(i, j, k\) \
- \(1\/2\) \((2  d - \[Nu]\_i - 2  \[Nu]\_j - 2  \[Nu]\_k - 
                    1)\) \(m\_i\^2\) \[Phi]\_\(k, i, j\) + \n\t\t\t\t\t\((d - \
\[Nu]\_j - 2\ \[Nu]\_k)\) \[Rho]\_\(i, j, k\), \n
          S\_\(\(i_\)\(,\)\(j_\)\(,\)\(k_\)\(\ \)\) :> \n\t\t\t\t\(-\((d - 
                      2  \[Nu]\_j - \[Nu]\_k)\)\) \(m\_k\^2\) \[Phi]\_\(j, i, \
k\)\  - \((d - \[Nu]\_j - 
                    2  \[Nu]\_k)\) \(m\_j\^2\) \[Phi]\_\(k, i, j\) + \n\t\t\t\
\t\t2 \((2  d - \[Nu]\_i - 2  \[Nu]\_j - 2  \[Nu]\_k - 
                    1)\) \[Rho]\_\(i, j, k\)}\)], "Input"],

Cell[CellGroupData[{

Cell["TA", "Subsubsection",
  CellTags->"T1.5.2.1.1"],

Cell[BoxData[
    \(MakeBoxes[\(TA[a_, d___Symbol, ijk___Integer]\)[__], fmt_] := 
      SubscriptBox[a, RowBox[{ijk}]]\)], "Input"],

Cell[BoxData[
    \(MakeBoxes[\(TA[a_, d___Symbol, ijk___Integer, ___Symbol]\)[__], fmt_] := 
      SubscriptBox[a, RowBox[{ijk}]]\)], "Input"],

Cell[BoxData[
    \(\(\(TA[args__]\)[
          mar_List] := \((\(TA[args]\)[mar] = \(ta[args]\)[
              mar])\);\)\)], "Input"],

Cell[TextData[{
  "Assume that the last argument of {em__} is always p =",
  Cell[BoxData[
      FormBox[
        SqrtBox[
          RowBox[{" ", 
            FormBox[\(p\^2\),
              "TraditionalForm"]}]], TraditionalForm]]],
  "."
}], "Text"],

Cell["(72)", "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(\(ta[D, i_Integer, j_Integer, k_Integer]\)[{em__}]\), ":=", 
        RowBox[{"With", "[", 
          
          RowBox[{\({q = Last[{em}], 
              mi = {em}\[LeftDoubleBracket]i\[RightDoubleBracket], 
              mj = {em}\[LeftDoubleBracket]j\[RightDoubleBracket], 
              mk = {em}\[LeftDoubleBracket]k\[RightDoubleBracket]}\), ",", 
            RowBox[{"Factor", "[", 
              
              RowBox[{\(q\^8\), "-", \(4\ q\^6\ \((mi\^2 + mj\^2 + mk\^2)\)\),
                 "+", \(q\^4\ \((6\ \((mi\^4 + mj\^4 + mk\^4)\) + 
                      4\ \((mi\^2\ mj\^2 + mk\^2\ mi\^2 + 
                            mk\^2\ mj\^2)\))\)\), 
                "-", \(4\ q\^2\ \((mi\^6 + mj\^6 + mk\^6 - 
                      mi\^2\ \((mj\^4 + mk\^4)\) - 
                      mj\^2\ \((mi\^4 + mk\^4)\) - 
                      mk\^2\ \((mi\^4 + mj\^4)\) + 
                      10\ mi\^2\ mj\^2\ mk\^2)\)\), "+", 
                SuperscriptBox[
                  RowBox[{
                    RowBox[{
                      StyleBox["Cayley",
                        FontColor->RGBColor[1, 0, 0.4]], "[", \(i, j, k\), 
                      "]"}], "[", \({em}\), "]"}], "2"]}], "]"}]}], "]"}]}], 
      ";"}]], "Input"],

Cell["(73)", "Text"],

Cell[BoxData[
    \(\(\(ta[\[Rho], i_Integer, j_Integer, k_Integer]\)[{em__}] := 
        With[{q = Last[{em}], 
            mi = {em}\[LeftDoubleBracket]i\[RightDoubleBracket], 
            mj = {em}\[LeftDoubleBracket]j\[RightDoubleBracket], 
            mk = {em}\[LeftDoubleBracket]k\[RightDoubleBracket]}, 
          Factor[\(-q\^6\) + 3\ q\^4\ \((mi\^2 + mj\^2 + mk\^2)\) - 
              q\^2\ \((3\ \((mi\^4 + mj\^4 + mk\^4)\) + 
                    2\ \((mi\^2\ mj\^2 + mk\^2\ mi\^2 + 
                          mk\^2\ mj\^2)\))\) + \ \((mi\^6 + mj\^6 + mk\^6 - 
                  mi\^2\ \((mj\^4 + mk\^4)\) - mj\^2\ \((mi\^4 + mk\^4)\) - 
                  mk\^2\ \((mi\^4 + mj\^4)\) + 
                  10\ mi\^2\ mj\^2\ mk\^2)\)]];\)\)], "Input"],

Cell["(74)", "Text"],

Cell[BoxData[
    \(\(\(ta[\[Phi], i_Integer, j_Integer, k_Integer]\)[{em__}] := 
        With[{q = Last[{em}], 
            mi = {em}\[LeftDoubleBracket]i\[RightDoubleBracket], 
            mj = {em}\[LeftDoubleBracket]j\[RightDoubleBracket], 
            mk = {em}\[LeftDoubleBracket]k\[RightDoubleBracket]}, 
          4  Factor[
              q\^4\  + 
                2 \( q\^2\) \((mi\^2 - mj\^2 - 
                      mk\^2)\) + \((mj\^2 - mk\^2)\)\^2 + \(mi\^2\) \((2 
                         mj\^2 + 2  mk\^2 - 3  mi\^2)\)]];\)\)], "Input"],

Cell["(75)", "Text"],

Cell[BoxData[
    \(\(\(ta[\[Sigma], d_, i_Integer, j_Integer, 
            k_Integer, \[Nu]i_Integer, \[Nu]j_Integer, \
\[Nu]k_Integer]\)[{em__}] := 
        Factor[\(\(-\(1\/4\)\) \((d - \[Nu]i - 2  \[Nu]j)\) \(TA[\[Phi], i, 
                    j, k]\)[{em}]\)\ \(-\(\(1\/4\) \((d - 
                      2  \[Nu]i - \[Nu]j)\) \(TA[\[Phi], j, i, 
                      k]\)[{em}]\)\) - \(\(1\/4\) \((2  d - 2  \[Nu]i - 
                  2  \[Nu]j - \[Nu]k - 1)\) \(\(TA[\[Phi], k, i, 
                  j]\)[{em}]\)\(\ \)\)\ \ ];\)\)], "Input"],

Cell["(76)", "Text"],

Cell[BoxData[
    \(\(\(ta[h, d_, i_Integer, j_Integer, 
            k_Integer, \[Nu]i_Integer, \[Nu]j_Integer, \
\[Nu]k_Integer]\)[{em__}] := \n\t
        With[{q = Last[{em}], 
            mi = {em}\[LeftDoubleBracket]i\[RightDoubleBracket], 
            mj = {em}\[LeftDoubleBracket]j\[RightDoubleBracket], 
            mk = {em}\[LeftDoubleBracket]k\[RightDoubleBracket]}, 
          Factor[\(-\(1\/2\)\) \((d - 
                    2  \[Nu]j - \[Nu]k)\) \(mk\^2\) \(TA[\[Phi], i, j, 
                    k]\)[{em}] - \(1\/2\) \((2  d - \[Nu]i - 2  \[Nu]j - 
                    2  \[Nu]k - 1)\) \(mi\^2\) \(TA[\[Phi], k, i, 
                    j]\)[{em}] + \(\((d - \[Nu]j - 
                    2\ \[Nu]k)\) \(\(TA[\[Rho], i, j, 
                    k]\)[{em}]\)\(\ \)\)\ \ ]];\)\)], "Input"],

Cell["(79)", "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(\(ta[S, d_, i_Integer, j_Integer, 
            k_Integer, \[Nu]i_Integer, \[Nu]j_Integer, \
\[Nu]k_Integer]\)[{em__}]\), ":=", "\n", "\t\t", 
        RowBox[{"With", "[", 
          
          RowBox[{\({q = Last[{em}], 
              mi = {em}\[LeftDoubleBracket]i\[RightDoubleBracket], 
              mj = {em}\[LeftDoubleBracket]j\[RightDoubleBracket], 
              mk = {em}\[LeftDoubleBracket]k\[RightDoubleBracket]}\), ",", 
            RowBox[{"Factor", "[", 
              
              RowBox[{\(\(-\((d - 
                        2  \[Nu]j - \[Nu]k)\)\) \(mk\^2\) \(TA[\[Phi], j, i, 
                      k]\)[{em}]\), " ", "-", 
                
                RowBox[{\((d - \[Nu]j - 
                      2  \[Nu]k)\), \(mj\^2\), \(\(TA[\[Phi], k, i, 
                      j]\)[{em}]\), "test", "\n", 
                  FormBox["test",
                    "TraditionalForm"], 
                  "2", \((2  d - \[Nu]i - 2  \[Nu]j - 2  \[Nu]k - 
                      1)\), \(\(TA[\[Rho], i, j, k]\)[{em}]\)}]}], "   ", 
              "]"}]}], "]"}]}], ";"}]], "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["CheckRecursion", "Subsection",
  CellTags->"T1.5.3.1"],

Cell["\<\
The following functions check the recurrence relations for J and V \
(using the final form as generated by MakeFun) via their Mellin-Integral \
Kernels.\
\>", "Text"],

Cell["\<\
This test is applicable in the general case  and if some of the 
masses vanish, but not for other special cases !!!\
\>", "Text"],

Cell[CellGroupData[{

Cell["CheckTVIRecursion", "Subsubsection",
  CellTags->"T1.5.3.1.1"],

Cell[BoxData[
    \(\(TVIKernel[d_, PP, \ 
          lis_] := \n\(\((\(-PP\))\)\^\(d - n1 - n2 - n3 - 
                n4\)\) \(\((m1\^2\/\(-PP\))\)\^s1\) \
\(\((m2\^2\/\(-PP\))\)\^s2\) \(\((m3\^2\/\(-PP\))\)\^s3\) \
\((m4\^2\/\(-PP\))\)\^s4*
          TVIpsi\ [
            d, \ \((lis /. {m1 -> s1, m2 -> s2, m3 -> s3, 
                  m4 -> s4})\)];\)\)], "Input"],

Cell[BoxData[
    \(\(TVIPsi[
          d_, \n\t{{n1_, \ s1_}, \ {n2_, s2_}, {n3_, s3_}, {n4_, 
              s4_}}] := \(\((\(-1\))\)\^\(n1 + n2 + n3 + n4 + 
                1\)\) \(\(gamma[\(-s1\)] gamma[d/2 - n1 - s1]\)\/gamma[
                n1]\) \(\(gamma[\(-s2\)] gamma[d/2 - n2 - s2]\)\/gamma[
                n2]\) \(\(gamma[\(-s3\)] gamma[d/2 - n3 - s3]\)\/gamma[
                n3]\) \(\(gamma[\(-s4\)] gamma[n4 + s4]\)\/gamma[n4]\) 
          gamma[n1 + n2 + n3 + n4 + s1 + s2 + s3 + s4 - d]/
            gamma[3  d/2 - n1 - n2 - n3 - n4 - s1 - s2 - s3 - s4] \(gamma[
                n1 + n3 + s1 + s3 - d/2]\/gamma[
                n1 + n3 + n4 + s1 + s3 + s4 - d/2]\) 
          gamma[d - n1 - n3 - n4 - s1 - s3 - s4]\/gamma[d - n1 - n3 - s1 - \
s3];\)\)], "Input"],

Cell[BoxData[
    \(CheckTVIRecursion[expr_] := \ 
      If[\ $CheckRecursion === True, \n\t
        Block[{rel, kern, \ nkern}, \n
          rel = \((\((Extract[
                    expr /. TVIC -> TVIkernel, {1, 2, 1, 1, 2}])\) - 
                Extract[expr /. TVI -> TVIkernel, {1, 2, 1, 2}])\); \n
          If[\ {TVIkernel} === 
              Union[Head /@ 
                  Cases[rel, ff_[__] /; Context[ff] =!= "\<System`\>", 
                    4]], \t\n
            rel = Collect[Numerator[Together[rel]], \ TVI[__], Factor]; \n
            kern = rel /. TVIkernel -> TVIKernel; \n
            kern = kern /. {PP -> \(-1\), m1 -> Sqrt[z1], m2 -> Sqrt[z2], 
                  m3 -> Sqrt[z3], m4 -> Sqrt[z4]}; \n
            kern = PowerExpand[kern]; \nkern = Expand[kern]; \n
            kern = kern /. {\(z1\^\(s1 + w1_. \)\) \(z2\^\(s2 + 
                            w2_. \)\) \(z3\^\(s3 + w3_. \)\) \(z4\^\(s4 + 
                            w4_. \)\) 
                      fac_ :> \((fac /. {s1 -> s1 - w1, s2 -> s2 - w2, 
                          s3 -> s3 - w3, s4 -> s4 - w4})\)}\ ; \n
            kern = \(\(kern /. 
                    TVIpsi -> TVIPsi\) /. {gamma[zz_] :> 
                      gamma[Expand[zz]]}\) /. {gamma[zz_ + nn_Integer] :> 
                    gamma[zz] Pochhammer[zz, nn]}; \n\t\t\t\tKERN = kern; \n
            kern = kern /. {gamma[zz_] :> 1}; \n
            kern = \(kern /. {\((\(-1\))\)\^\(nn_ + ww_Integer\) :> \
\((\(-1\))\)\^ww}\) /. {\((\(-1\))\)\^nn_ :> 1}; \n
            nkern = Table[\n\t\t\tkern /. 
                  Thread[{d, \ n1, \ n2, \ n3, n4, s1, s2, \ s3, s4} -> 
                      Table[Random[], {9}]], {3}]; \n
            nkern\ \ , \n\t\tPrint["\<Matching Error\>"]\ ]\ ], \n\t\t\
If[$Notebooks, Print["\<Off\>"]]]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["CheckTJIRecursion", "Subsubsection",
  CellTags->"T1.5.3.2.1"],

Cell[BoxData[
    \(\(TJIKernel[d_, PP, \ 
          lis_] := \n\(\((\(-PP\))\)\^\(d - n1 - n2 - 
                n3\)\) \(\((m1\^2\/\(-PP\))\)\^s1\) \
\(\((m2\^2\/\(-PP\))\)\^s2\) \((m3\^2\/\(-PP\))\)\^s3*
          TJIpsi\ [
            d, \ \((lis /. {m1 -> s1, m2 -> s2, m3 -> s3})\)];\)\)], "Input"],

Cell[BoxData[
    \(TJIPsi\ [d_, {{n1_, \ s1_}, \ {n2_, s2_}, {n3_, s3_}}] := 
      Evaluate[TVIPsi[d, \n\t{{n1, \ s1}, \ {n2, s2}, {n3, s3}, {0, 0}}]/
          gamma[0]]\)], "Input"],

Cell[BoxData[
    \(\(Clear[CheckTJIRecursion];\)\)], "Input"],

Cell[BoxData[
    \(CheckTJIRecursion[expr_] := \ 
      If[\ $CheckRecursion === True, \n\t
        Block[{rel, kern, nkern}, \n
          rel = \((\((Extract[
                    expr /. TJIC -> TJIkernel, {1, 2, 1, 1, 2}])\) - 
                Extract[
                  expr /. TJI -> TJIkernel, {1, 2, 1, 2}])\); \n\t\t\tREL = 
            rel; \t\t\t\n
          If[{TJIkernel} === 
              Union[Head /@ 
                  Cases[rel, ff_[__] /; Context[ff] =!= "\<System`\>", 
                    4]]\ , \t\t\t\n
            rel = Collect[Numerator[Together[rel]], \ TJI[__], Factor]; \n
            kern = rel /. TJIkernel -> TJIKernel; \n
            kern = kern /. {PP -> \(-1\), m1 -> Sqrt[z1], m2 -> Sqrt[z2], 
                  m3 -> Sqrt[z3]}; \nkern = PowerExpand[kern]; \n
            kern = Expand[kern]; \n
            kern = kern /. {\(z1\^\(s1 + w1_. \)\) \(z2\^\(s2 + 
                            w2_. \)\) \(z3\^\(s3 + w3_. \)\) 
                      fac_ :> \((fac /. {s1 -> s1 - w1, s2 -> s2 - w2, 
                          s3 -> s3 - w3})\)}; \ \n
            kern = \(\(kern /. 
                    TJIpsi -> TJIPsi\) /. {gamma[zz_] :> 
                      gamma[Expand[zz]]}\) /. {gamma[zz_ + nn_Integer] :> 
                    gamma[zz] Pochhammer[zz, nn]}; \n\ \ \ \ \ \ KERN = 
              kern; \nkern = kern /. {gamma[zz_] :> 1}; \n
            kern = \(kern /. {\((\(-1\))\)\^\(nn_ + ww_Integer\) :> \
\((\(-1\))\)\^ww}\) /. {\((\(-1\))\)\^nn_ :> 1}; \n
            nkern = Table[\n\t\t\tkern /. 
                  Thread[{d, \ n1, \ n2, \ n3, s1, s2, \ s3} -> 
                      Table[Random[], {7}]], {3}]; \n
            nkern\ \ , \n\t\tPrint["\<Matching Error\>"]\ ]\ ], \n\t\t\
If[$Notebooks, Print["\<Off\>"]]]\)], "Input"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Recurrence Relations Input and Recursion Function Generation", "Section",
  CellDingbat->"\[FilledSmallCircle]",
  CellTags->"T1.6.1"],

Cell[TextData[{
  "Most ",
  Cell[BoxData[
      \(TraditionalForm\`e\_\(\(...\)\(\ \)\)\)]],
  "equations below were entered by the palettes which show up when \
$PutPalettes is set to True.\nExamples of alternative (hand) input are  (50) \
, (201) -- (205)."
}], "Text"],

Cell[CellGroupData[{

Cell["Input (35)", "Subsection",
  CellTags->"T1.6.1.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_35\), "=", "\n", 
        RowBox[{
          RowBox[{"2", \(\[Nu]\_1\), "\[ScriptCapitalC]", " ", 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["F",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_5\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TFI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4], 
                Subscript[ \[Nu], 5]}],
              Editable->True]}], "\[Equal]", " ", 
          RowBox[{
            RowBox[{"(", 
              
              RowBox[{\(\((d - 
                      2  \[Nu]\_1 - \[Nu]\_3 - \[Nu]\_5)\) \[CapitalDelta]\_1\
\), "+", 
                RowBox[{\(\[CapitalDelta]\_\(3, 4, 5\)\), 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{\(\[Nu]\_5\), 
                        InterpretationBox[
                          StyleBox[\(5\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 5, "+"],
                          Editable->True], 
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[
                              StyleBox[\(2\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 2, "-"],
                              Editable->True], "-", 
                            InterpretationBox[
                              StyleBox[\(1\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 1, "-"],
                              Editable->True]}], ")"}]}], "-", 
                      RowBox[{\(\[Nu]\_3\), 
                        InterpretationBox[
                          StyleBox[\(3\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 3, "+"],
                          Editable->True], 
                        InterpretationBox[
                          StyleBox[\(1\^-\),
                            FontWeight->"Bold"],
                          Schiebe[ 1, "-"],
                          Editable->True]}]}], ")"}]}], "+", 
                RowBox[{\(\[CapitalDelta]\_2\), 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{\(\[Nu]\_1\), 
                        InterpretationBox[
                          StyleBox[\(1\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 1, "+"],
                          Editable->True], 
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[
                              StyleBox[\(5\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 5, "-"],
                              Editable->True], "-", 
                            InterpretationBox[
                              StyleBox[\(2\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 2, "-"],
                              Editable->True]}], ")"}]}], "+", 
                      RowBox[{\(\[Nu]\_3\), 
                        InterpretationBox[
                          StyleBox[\(3\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 3, "+"],
                          Editable->True], 
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[
                              StyleBox[\(5\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 5, "-"]], "-", 
                            InterpretationBox[
                              StyleBox[\(4\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 4, "-"],
                              Editable->True]}], ")"}]}], "+", \(\[Nu]\_5\), 
                      "-", \(\[Nu]\_1\)}], ")"}]}], "+", "\n", "\t\t\t\t", 
                RowBox[{\(\[CapitalDelta]\_6\), 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{\(\[Nu]\_1\), 
                        InterpretationBox[
                          StyleBox[\(1\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 1, "+"],
                          Editable->True], 
                        InterpretationBox[
                          StyleBox[\(3\^-\),
                            FontWeight->"Bold"],
                          Schiebe[ 3, "-"],
                          Editable->True]}], "+", 
                      RowBox[{\(\[Nu]\_5\), 
                        InterpretationBox[
                          StyleBox[\(5\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 5, "+"],
                          Editable->True], 
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[
                              StyleBox[\(3\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 3, "-"],
                              Editable->True], "-", 
                            InterpretationBox[
                              StyleBox[\(4\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 4, "-"],
                              Editable->True]}], ")"}]}], "+", \(\[Nu]\_3\), 
                      "-", \(\[Nu]\_1\)}], ")"}]}]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["F",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_5\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TFI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4], 
                Subscript[ \[Nu], 5]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_35 = OperatorApplyF[e\_35]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (35)", "Subsection",
  CellTags->"T1.6.2.1"],

Cell[BoxData[
    \(h\_351 = s\_35 /. \[Nu]\_1 -> \((\[Nu]\_1 - 1)\)\)], "Input"],

Cell[CellGroupData[{

Cell[TextData[{
  Cell[BoxData[
      \(TraditionalForm\`1\^\(\(+\)\(\ \)\)\)]],
  " (35_1)"
}], "Subsubsection",
  CellTags->"T1.6.2.1.1"],

Cell[BoxData[
    RowBox[{\(f\_351\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_351\), ",", "\"\<(35_1)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", 
            RowBox[{\(\[Nu]\_1 > 1\), "&&", 
              RowBox[{
                RowBox[{
                  StyleBox["Cayley",
                    FontColor->RGBColor[1, 0, 0.4]], "[", 
                  RowBox[{
                    InterpretationBox[\(m\_1\),
                      m1], ",", 
                    InterpretationBox[\(m\_2\),
                      m2], ",", 
                    InterpretationBox[\(m\_3\),
                      m3], ",", 
                    InterpretationBox[\(m\_4\),
                      m4], ",", 
                    InterpretationBox[\(m\_5\),
                      m5], ",", 
                    SqrtBox[
                      InterpretationBox[\(p\^2\),
                        PP]]}], "]"}], "=!=", "0"}]}], "]"}]}], 
        "]"}]}]], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
  Cell[BoxData[
      \(TraditionalForm\`2\^\(\(+\)\(\ \)\)\)]],
  " (35_2)"
}], "Subsubsection",
  CellTags->"T1.6.2.2.1"],

Cell[BoxData[
    RowBox[{\(f\_352\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(Interchange[h\_351, 1 \[LeftRightArrow] 2, 
            3 \[LeftRightArrow] 4]\), ",", "\"\<(35_2)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", 
            RowBox[{\(\[Nu]\_2 > 1\), "&&", 
              RowBox[{
                RowBox[{
                  StyleBox["Cayley",
                    FontColor->RGBColor[1, 0, 0.4]], "[", 
                  RowBox[{
                    InterpretationBox[\(m\_1\),
                      m1], ",", 
                    InterpretationBox[\(m\_2\),
                      m2], ",", 
                    InterpretationBox[\(m\_3\),
                      m3], ",", 
                    InterpretationBox[\(m\_4\),
                      m4], ",", 
                    InterpretationBox[\(m\_5\),
                      m5], ",", 
                    SqrtBox[
                      InterpretationBox[\(p\^2\),
                        PP]]}], "]"}], "=!=", "0"}]}], "]"}]}], 
        "]"}]}]], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
  Cell[BoxData[
      \(TraditionalForm\`3\^\(\(+\)\(\ \)\)\)]],
  " (35_3)"
}], "Subsubsection",
  CellTags->"T1.6.2.3.1"],

Cell[BoxData[
    RowBox[{\(f\_353\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(Interchange[h\_351, 1 \[LeftRightArrow] 3, 
            2 \[LeftRightArrow] 4]\), ",", "\"\<(35_3)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", 
            RowBox[{\(\[Nu]\_3 > 1\), "&&", 
              RowBox[{
                RowBox[{
                  StyleBox["Cayley",
                    FontColor->RGBColor[1, 0, 0.4]], "[", 
                  RowBox[{
                    InterpretationBox[\(m\_1\),
                      m1], ",", 
                    InterpretationBox[\(m\_2\),
                      m2], ",", 
                    InterpretationBox[\(m\_3\),
                      m3], ",", 
                    InterpretationBox[\(m\_4\),
                      m4], ",", 
                    InterpretationBox[\(m\_5\),
                      m5], ",", 
                    SqrtBox[
                      InterpretationBox[\(p\^2\),
                        PP]]}], "]"}], "=!=", "0"}]}], "]"}]}], 
        "]"}]}]], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
  Cell[BoxData[
      \(TraditionalForm\`4\^\(\(+\)\(\ \)\)\)]],
  " (35_4)"
}], "Subsubsection",
  CellTags->"T1.6.2.4.1"],

Cell[BoxData[
    RowBox[{\(f\_354\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(Interchange[h\_351, 1 \[LeftRightArrow] 4, 
            2 \[LeftRightArrow] 3]\), ",", "\"\<(35_4)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", 
            RowBox[{\(\[Nu]\_4 > 1\), "&&", 
              RowBox[{
                RowBox[{
                  StyleBox["Cayley",
                    FontColor->RGBColor[1, 0, 0.4]], "[", 
                  RowBox[{
                    InterpretationBox[\(m\_1\),
                      m1], ",", 
                    InterpretationBox[\(m\_2\),
                      m2], ",", 
                    InterpretationBox[\(m\_3\),
                      m3], ",", 
                    InterpretationBox[\(m\_4\),
                      m4], ",", 
                    InterpretationBox[\(m\_5\),
                      m5], ",", 
                    SqrtBox[
                      InterpretationBox[\(p\^2\),
                        PP]]}], "]"}], "=!=", "0"}]}], "]"}]}], 
        "]"}]}]], "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (3501)", "Subsection",
  CellTags->"T1.6.3.1"],

Cell[BoxData[
    \(Cayley[m1, m5, m5, m4, m5, m5] // Factor\)], "Input"],

Cell[TextData[Cell[BoxData[
    \(TraditionalForm\`m\_1 = 0\ \ ; \ \ m\_2 = \(m\_3 = m\_5\); \ \ \ 
    p\^2 = m\_5\%2\)]]], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_3501\), "=", 
        RowBox[{
          RowBox[{\((d - 2  \[Nu]\_1 - \[Nu]\_3 - \[Nu]\_5)\), 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["F",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_5\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TFI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4], 
                Subscript[ \[Nu], 5]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", " ", 
              RowBox[{
                RowBox[{\(\[Nu]\_5\), 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(1\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 1, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(2\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 2, "-"],
                        Editable->True]}], ")"}], 
                  InterpretationBox[
                    StyleBox[\(5\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 5, "+"],
                    Editable->True]}], "+", 
                RowBox[{\(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(1\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "-"],
                    Editable->True], 
                  InterpretationBox[
                    RowBox[{
                      StyleBox[\(3\^+\),
                        FontWeight->"Bold"], " "}],
                    Schiebe[ 3, "+"],
                    Editable->True]}]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["F",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_5\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TFI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4], 
                Subscript[ \[Nu], 5]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_3501 = OperatorApplyF[e\_3501]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
  "MakeFun (3501) \[CapitalDelta]=0 ",
  Cell[BoxData[
      \(TraditionalForm\`\((p\^2 = m\^2)\)\)]]
}], "Subsection",
  CellTags->"T1.6.4.1"],

Cell[BoxData[
    \(h\_3501 = s\_3501\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_3501\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(s\_3501\), ",", "\"\<(3501)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", \(PP === m5^2\), "]"}], 
          ",", \({m1 :> 0, m2 :> m5, m3 :> m5}\)}], "]"}]}]], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (3502)", "Subsection",
  CellTags->"T1.6.5.1"],

Cell[BoxData[
    \(Cayley[m1, m5, m5, m4, m5, m5] // Factor\)], "Input"],

Cell[TextData[Cell[BoxData[
    \(TraditionalForm\`m\_4 = 0\ \ ; \ \ m\_2 = \(m\_3 = m\_5\); \ \ \ 
    p\^2 = m\_5\%2\)]]], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_3502\), "=", 
        RowBox[{
          RowBox[{\((d - 2  \[Nu]\_1 - \[Nu]\_3 - \[Nu]\_5)\), 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["F",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_5\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TFI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4], 
                Subscript[ \[Nu], 5]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", " ", 
              RowBox[{
                RowBox[{\(\[Nu]\_5\), 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(1\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 1, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(2\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 2, "-"],
                        Editable->True]}], ")"}], 
                  InterpretationBox[
                    StyleBox[\(5\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 5, "+"],
                    Editable->True]}], "+", 
                RowBox[{\(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(1\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "-"],
                    Editable->True], 
                  InterpretationBox[
                    RowBox[{
                      StyleBox[\(3\^+\),
                        FontWeight->"Bold"], " "}],
                    Schiebe[ 3, "+"],
                    Editable->True]}]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["F",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_5\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TFI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4], 
                Subscript[ \[Nu], 5]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_3502 = OperatorApplyF[e\_3502]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
  "MakeFun (3502) \[CapitalDelta]=0 ",
  Cell[BoxData[
      \(TraditionalForm\`\((p\^2 = m\^2)\)\)]]
}], "Subsection",
  CellTags->"T1.6.6.1"],

Cell[BoxData[
    \(h\_3502 = s\_3502\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_3502\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_3502\), ",", "\"\<(3502)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", \(PP === m5^2\), "]"}], 
          ",", \({m4 :> 0, m2 :> m5, m3 :> m5}\)}], "]"}]}]], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
  "MakeFun (3503) \[CapitalDelta]=0 ",
  Cell[BoxData[
      \(TraditionalForm\`\((p\^2 = m\^2)\)\)]]
}], "Subsection",
  CellTags->"T1.6.7.1"],

Cell[BoxData[
    \(Cayley[m5, 0, m3, m5, m5, m5]\)], "Input"],

Cell[BoxData[
    \(h\_3503 = 
      Interchange[s\_3502, 1 \[LeftRightArrow] 2, \ 
        3 \[LeftRightArrow] 4]\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_3503\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_3503\), ",", "\"\<(3503)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", \(PP === m5^2\), "]"}], 
          ",", \({m2 :> 0, m1 :> m5, m4 :> m5}\)}], "]"}]}]], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
  "MakeFun (3504) \[CapitalDelta]=0 ",
  Cell[BoxData[
      \(TraditionalForm\`\((p\^2 = m\^2)\)\)]]
}], "Subsection",
  CellTags->"T1.6.8.1"],

Cell[BoxData[
    \(Cayley[m5, m2, 0, m5, m5, m5]\)], "Input"],

Cell[BoxData[
    \(h\_3504 = 
      Interchange[s\_3502, 1 \[LeftRightArrow] 3, \ 
        2 \[LeftRightArrow] 4]\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_3504\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_3504\), ",", "\"\<(3504)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", \(PP === m5^2\), "]"}], 
          ",", \({m3 :> 0, m1 :> m5, m4 :> m5}\)}], "]"}]}]], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (41)", "Subsection",
  CellTags->"T1.6.9.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_41\), "=", "\n", 
        RowBox[{
          RowBox[{"2", \(\[Nu]\_5\), " ", "\[ScriptCapitalC]", " ", 
            InterpretationBox[
              StyleBox[\(5\^+\),
                FontWeight->"Bold"],
              Schiebe[ 5, "+"]], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["F",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_5\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TFI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4], 
                Subscript[ \[Nu], 5]}]]}], " ", "\[Equal]", " ", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{\(\[CapitalDelta]\_\(1, 3, 6\)\), 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{\(\[Nu]\_3\), 
                        InterpretationBox[
                          StyleBox[\(3\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 3, "+"]], 
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[
                              StyleBox[\(4\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 4, "-"]], "-", 
                            InterpretationBox[
                              StyleBox[\(5\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 5, "-"]]}], ")"}]}], "+", 
                      RowBox[{\(\[Nu]\_1\), 
                        InterpretationBox[
                          StyleBox[\(1\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 1, "+"]], 
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[
                              StyleBox[\(2\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 2, "-"]], "-", 
                            InterpretationBox[
                              StyleBox[\(5\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 5, "-"]]}], ")"}]}]}], ")"}]}], 
                "+", \(\((d - \[Nu]\_1 - \[Nu]\_3 - 
                      2  \[Nu]\_5)\) \[CapitalDelta]\_5\), "+", 
                RowBox[{\(\[CapitalDelta]\_2\), 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{\(\[Nu]\_5\), 
                        InterpretationBox[
                          StyleBox[\(5\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 5, "+"]], 
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[
                              StyleBox[\(1\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 1, "-"]], "-", 
                            InterpretationBox[
                              StyleBox[\(2\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 2, "-"]]}], ")"}]}], "+", 
                      RowBox[{\(\[Nu]\_3\), 
                        InterpretationBox[
                          StyleBox[\(3\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 3, "+"]], 
                        InterpretationBox[
                          StyleBox[\(1\^-\),
                            FontWeight->"Bold"],
                          Schiebe[ 1, "-"]]}], "+", \(\[Nu]\_1\), 
                      "-", \(\[Nu]\_5\)}], ")"}]}], "+", 
                RowBox[{\(\[CapitalDelta]\_4\), 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{\(\[Nu]\_1\), 
                        InterpretationBox[
                          StyleBox[\(1\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 1, "+"]], 
                        InterpretationBox[
                          StyleBox[\(3\^-\),
                            FontWeight->"Bold"],
                          Schiebe[ 3, "-"]]}], "+", 
                      RowBox[{\(\[Nu]\_5\), 
                        InterpretationBox[
                          StyleBox[\(5\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 5, "+"]], 
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[
                              StyleBox[\(3\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 3, "-"]], "-", 
                            InterpretationBox[
                              StyleBox[\(4\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 4, "-"]]}], ")"}]}], "+", \(\[Nu]\_3\),
                       "-", \(\[Nu]\_5\)}], ")"}]}]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["F",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_5\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TFI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4], 
                Subscript[ \[Nu], 5]}]]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_41 = OperatorApplyF[e\_41]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (41)", "Subsection",
  CellTags->"T1.6.10.1"],

Cell[BoxData[
    \(h\_41 = s\_41 /. \[Nu]\_5 \[Rule] \[Nu]\_5 - 1\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_41\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_41\), ",", "\"\<(41)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", 
            RowBox[{\(\[Nu]\_5 > 1\), "&&", 
              RowBox[{
                RowBox[{
                  StyleBox["Cayley",
                    FontColor->RGBColor[1, 0, 0.4]], "[", 
                  RowBox[{
                    InterpretationBox[\(m\_1\),
                      m1], ",", 
                    InterpretationBox[\(m\_2\),
                      m2], ",", 
                    InterpretationBox[\(m\_3\),
                      m3], ",", 
                    InterpretationBox[\(m\_4\),
                      m4], ",", 
                    InterpretationBox[\(m\_5\),
                      m5], ",", 
                    SqrtBox[
                      InterpretationBox[\(p\^2\),
                        PP]]}], "]"}], "=!=", "0"}]}], "]"}]}], 
        "]"}]}]], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (43)", "Subsection",
  CellTags->"T1.6.11.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(p\^2\)]],
  " and ( ",
  Cell[BoxData[
      \(TraditionalForm\`m\_5\%2 == 0\)]],
  "  &&  ",
  Cell[BoxData[
      \(TraditionalForm\`m\_1\%2\)]],
  "==",
  Cell[BoxData[
      \(TraditionalForm\`m\_2\%2\)]],
  " && ",
  Cell[BoxData[
      \(TraditionalForm\`m\_3\%2\)]],
  "== ",
  Cell[BoxData[
      \(TraditionalForm\`m\_4\%2\)]],
  " ) \[DoubleLongRightArrow] \[CapitalDelta] == 0"
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_43\), "=", 
        RowBox[{
          RowBox[{\((d - \[Nu]\_1 - \[Nu]\_3 - 2  \[Nu]\_5)\), 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["F",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_5\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TFI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4], 
                Subscript[ \[Nu], 5]}],
              Editable->True]}], "\[Equal]", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{\(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True], 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(5\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 5, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(2\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 2, "-"],
                        Editable->True]}], ")"}]}], "+", 
                RowBox[{\(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True], 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(5\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 5, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(4\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 4, "-"],
                        Editable->True]}], ")"}]}]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["F",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_5\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TFI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4], 
                Subscript[ \[Nu], 5]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_43 = OperatorApplyF[e\_43]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (44)", "Subsection",
  CellTags->"T1.6.12.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(p\^2\)]],
  " and ( ",
  Cell[BoxData[
      \(TraditionalForm\`m\_5\%2 == 0\)]],
  "  &&  ",
  Cell[BoxData[
      \(TraditionalForm\`m\_1\%2\)]],
  "==",
  Cell[BoxData[
      \(TraditionalForm\`m\_2\%2\)]],
  " && ",
  Cell[BoxData[
      \(TraditionalForm\`m\_3\%2\)]],
  "== ",
  Cell[BoxData[
      \(TraditionalForm\`m\_4\%2\)]],
  " ) \[DoubleLongRightArrow] \[CapitalDelta] == 0"
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_44\), "=", 
        RowBox[{
          RowBox[{\((d - \[Nu]\_2 - \[Nu]\_4 - 2  \[Nu]\_5)\), 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["F",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_5\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TFI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4], 
                Subscript[ \[Nu], 5]}],
              Editable->True]}], "\[Equal]", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{\(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True], 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(5\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 5, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(1\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 1, "-"],
                        Editable->True]}], ")"}]}], "+", 
                RowBox[{\(\[Nu]\_4\), 
                  InterpretationBox[
                    StyleBox[\(4\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 4, "+"],
                    Editable->True], 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(5\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 5, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(3\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 3, "-"],
                        Editable->True]}], ")"}]}]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["F",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_5\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TFI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4], 
                Subscript[ \[Nu], 5]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_44 = OperatorApplyF[e\_44]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (45)", "Subsection",
  CellTags->"T1.6.13.1"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_45\), "=", " ", 
        RowBox[{
          RowBox[{\(\[CapitalDelta]\_\(1, 3, 6\)\), 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["F",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_5\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TFI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4], 
                Subscript[ \[Nu], 5]}],
              Editable->True]}], "\[Equal]", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{\(u\_\(6, 1, 3\)\), 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{\(\[Nu]\_1\), 
                        InterpretationBox[
                          StyleBox[\(1\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 1, "+"],
                          Editable->True], 
                        InterpretationBox[
                          StyleBox[\(3\^-\),
                            FontWeight->"Bold"],
                          Schiebe[ 3, "-"],
                          Editable->True]}], "+", 
                      RowBox[{\(\[Nu]\_5\), 
                        InterpretationBox[
                          StyleBox[\(5\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 5, "+"],
                          Editable->True], 
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[
                              StyleBox[\(3\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 3, "-"],
                              Editable->True], "-", 
                            InterpretationBox[
                              StyleBox[\(4\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 4, "-"],
                              Editable->True]}], ")"}]}]}], ")"}]}], "-", 
                RowBox[{"2", \(u\_\(1, 3, 6\)\), 
                  RowBox[{"(", " ", 
                    RowBox[{
                      RowBox[{\(\[Nu]\_2\), 
                        InterpretationBox[
                          StyleBox[\(2\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 2, "+"],
                          Editable->True], 
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[
                              StyleBox[\(5\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 5, "-"],
                              Editable->True], "-", 
                            InterpretationBox[
                              StyleBox[\(1\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 1, "-"],
                              Editable->True]}], ")"}]}], "+", " ", 
                      RowBox[{\(\[Nu]\_4\), 
                        InterpretationBox[
                          StyleBox[\(4\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 4, "+"],
                          Editable->True], 
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[
                              StyleBox[\(5\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 5, "-"],
                              Editable->True], "-", 
                            InterpretationBox[
                              StyleBox[\(3\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 3, "-"],
                              Editable->True]}], ")"}]}], 
                      "+", \(-\[CapitalSigma]\_5\), "-", \(\[Nu]\_1\), 
                      "+", \(3  \[Nu]\_3\)}], ")"}]}], "+", 
                RowBox[{"2", \(m\_3\%2\), 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{\(\[Nu]\_5\), 
                        InterpretationBox[
                          StyleBox[\(5\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 5, "+"],
                          Editable->True], 
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[
                              StyleBox[\(1\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 1, "-"],
                              Editable->True], "-", 
                            InterpretationBox[
                              StyleBox[\(2\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 2, "-"],
                              Editable->True]}], ")"}]}], "+", 
                      RowBox[{\(\[Nu]\_3\), 
                        InterpretationBox[
                          StyleBox[\(3\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 3, "+"],
                          Editable->True], 
                        InterpretationBox[
                          StyleBox[\(1\^-\),
                            FontWeight->"Bold"],
                          Schiebe[ 1, "-"],
                          Editable->True]}], "+", \(\[Nu]\_1\), 
                      "-", \(\[Nu]\_3\)}], ")"}]}]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["F",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_5\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TFI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4], 
                Subscript[ \[Nu], 5]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_45 = OperatorApplyF[e\_45]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (43) and (44)", "Subsection",
  CellTags->"T1.6.14.1"],

Cell[BoxData[
    \(f\_43 = 
      MakeFun[s\_43, "\<(43)\>", 
        IFF[\[Nu]\_1 + \[Nu]\_3 + 2  \[Nu]\_5 - \((d /. dd_Symbol :> 4)\) =!= 
            0], {m5 :> 0, m4 :> m3, m2 :> m1}]\)], "Input"],

Cell[BoxData[
    \(f\_44 = 
      MakeFun[s\_44, "\<(44)\>", {m5 :> 0, m4 :> m3, m2 :> m1}]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (50)", "Subsection",
  CellTags->"T1.6.15.1"],

Cell["\<\
Subscript[f, 50] = 
   HoldForm[TFR[d_, PP_, 
      {{(n1_)?PQ, m1_}, {(n2_)?PQ, m2_}, {(n3_)?PQ, m3_}, 
       {(n4_)?PQ, m4_}, {(n5_)?PQ, m5_}}] := 
     (TComment[\"(50)\", TFIC[d, PP, 
          {{n1, m1}, {n2, m2}, {n3, m3}, {n4, m4}, 
           {n5, m5}}]]; 
        Block[{r1 = 3 - d, r2 = 4 - d, 
          r3 = 
           -(m1^2*m2^2*m3^2) + m2^4*m3^2 + m2^2*m3^4 + 
            m1^4*m4^2 - m1^2*m2^2*m4^2 - 
            m1^2*m3^2*m4^2 - m2^2*m3^2*m4^2 + 
            m1^2*m4^4 + m1^2*m2^2*m5^2 - 
            m2^2*m3^2*m5^2 - m1^2*m4^2*m5^2 + 
            m3^2*m4^2*m5^2 + m1^2*m3^2*PP - 
            m2^2*m3^2*PP - m1^2*m4^2*PP + m2^2*m4^2*PP - 
            m1^2*m5^2*PP - m2^2*m5^2*PP - m3^2*m5^2*PP - 
            m4^2*m5^2*PP + m5^4*PP + m5^2*PP^2, 
          r4 = -4 + d, r5 = -3 + d, r6 = -7 + 2*d, 
          r7 = -(m2^2*r4) + m3^2*r5 - m5^2*r5 + m4^2*r6, 
          r8 = -10 + 3*d, r9 = -m3^2 + m4^2, 
          r10 = -11 + 3*d, 
          r11 = -(m4^2*r10) - 3*m5^2*r5 + m3^2*r6, 
          r12 = 
           m2^2*r11 + m5^4*r5 + m2^4*r8 + m4^2*r4*r9 + 
            m5^2*r5*r9, r13 = -6 + d, 
          r14 = -(m4^2*r13) + m5^2*r5, 
          r15 = -3*m3^2 - m4^2*r10 + m5^2*r5, 
          r16 = 
           m3^2*r14 + m2^2*r15 + m2^4*r4 - 2*m3^4*r5 - 
            3*m4^2*m5^2*r5 + m5^4*r5 + m4^4*r8, 
          r17 = m3^2 + 2*m4^2, r18 = -1 + d, 
          r19 = -(m3^2*r13) - 2*m4^2*r18 + 2*m5^2*r5, 
          r20 = 
           2*m2^4 - 3*m3^4 + d*m3^4 - 3*m3^2*m4^2 + 
            2*m4^4 + m2^2*r19 - 2*m5^4*r5 + m5^2*r17*r5, 
          r21 = 2 + d, r22 = 
           2*m5^2 - m4^2*r21 + m2^2*r4 - m3^2*r4, 
          r23 = -7 + d, r24 = 6*m4^2 - 4*m5^2, 
          r25 = 3*m3^2 + m4^2*r18 + m5^2*r23, 
          r26 = 
           -6*m4^4 - 2*m4^2*m5^2*r23 + m3^2*r24 + 
            2*m2^2*r25 - 2*m2^4*r4 - m3^4*r4, 
          r27 = m3^4 + m3^2*m4^2 - 2*m4^4, 
          r28 = m3^2 + m4^2*r23, 
          r29 = -2*m4^2*r18 + m3^2*r21 + 2*m5^2*r23, 
          r30 = -6*m2^4 + 2*m5^2*r28 - m2^2*r29 + r27*r4, 
          r31 = m1^4*r22 + m1^2*r26 + m3^2*r30 + m1^6*r4, 
          r32 = m3 - m4, r33 = m3 + m4, 
          r34 = 2*m3^2 + m4^2, r35 = -17 + 3*d, 
          r36 = m3^2*r23 + m4^2*r35, r37 = -9 + d, 
          r38 = 17 - 3*d, 
          r39 = 5*m3^2 + m4^2*r37 + m5^2*r38, 
          r40 = 1 + d, r41 = 
           5*m4^2 - m5^2*r23 + m2^2*r4 - m3^2*r40, 
          r42 = 
           m5^4*r35 - m5^2*r36 + m2^2*r39 - 2*m1^4*r4 + 
            m2^4*r4 - r32*r33*r34*r4 + m1^2*r41, 
          r43 = m1^2 - m2^2 + m3^2 - m4^2, 
          r44 = 2*m5^2*r13 + r4*r43, 
          r45 = 
           m3^4*r12 + m1^4*r16 + m1^2*m3^2*r20 + 
            PP*r31 + PP^2*r42 + PP^3*r44 + m1^6*r7, 
          r46 = m2^2 + m4^2, r47 = 13 - 3*d, 
          r48 = m4^2*r47 - m5^2*r5 + m3^2*r6, 
          r49 = m5^4*r5 + m5^2*r32*r33*r5 + m4^2*r4*r9, 
          r50 = -18 + 4*d, r51 = -(m4^2*r13) - 3*m5^2*r5, 
          r52 = 
           2*m3^4*r5 + m4^2*m5^2*r5 + m5^4*r5 + 
            m4^4*r50 + m3^2*r51, 
          r53 = -3*m4^2 + 2*m5^2*r5, 
          r54 = 
           m4^4*r47 + 2*m3^4*r5 + m4^2*m5^2*r5 - 
            2*m5^4*r5 + m3^2*r53, 
          r55 = m4^2*r13 + 6*m3^2*r5 - 2*m5^2*r5, 
          r56 = 3*m4^2 + m3^2*r5 - m5^2*r5, 
          r57 = 7 - 2*d, 
          r58 = m3^2*r5 + 3*m5^2*r5 + m4^2*r57, 
          r59 = 
           m2^6*r4 + m2^2*m4^2*r55 + m2^4*r56 + m4^4*r58, 
          r60 = m4^2 - m3^2*r5, r61 = 10 - 3*d, 
          r62 = 2*m5^2 + 3*m4^2*r4 + m3^2*r61, 
          r63 = m4^2 + m5^2*r5, 
          r64 = 
           -4*m4^2*m5^2 + 3*m4^4*r4 - 2*m3^4*r5 + 
            2*m3^2*r63, r65 = m4^2 + m3^2*r5 - m5^2*r5, 
          r66 = 2*m3^2*r5 + 2*m5^2*r5 + m4^2*r61, 
          r67 = 3*m2^4*r4 + 2*m2^2*r65 + m4^2*r66, 
          r68 = 
           -3*m2^6*r4 - 2*m1^4*m4^2*r5 + 
            2*m4^2*m5^2*r60 + m2^4*r62 + m2^2*r64 + 
            m1^2*r67 - 3*m4^4*r4*r9, r69 = -13 + 3*d, 
          r70 = m3^2*r5 + m4^2*r69, 
          r71 = m4^2 - 3*m2^2*r4 - m3^2*r5 + m5^2*r5, 
          r72 = -5 + d, r73 = m3^2 + m5^2*r69 + m4^2*r72, 
          r74 = 
           3*m2^4*r4 - m5^4*r5 + m5^2*r70 + m1^2*r71 + 
            m2^2*r73 + 3*m4^2*r4*r9, 
          r75 = m1^2 - m2^2 + m3^2 - m4^2 - 2*m5^2, 
          r76 = 
           m2^8*r4 + m2^6*r48 + m4^4*r49 + 
            2*m1^4*m4^2*r46*r5 + m2^4*r52 + 
            m2^2*m4^2*r54 - m1^2*r59 + PP*r68 + 
            PP^2*r74 + PP^3*r4*r75, 
          r77 = m3^2 - m4^2 + m5^2, 
          r78 = -2*m2^2 + m4^2 + m5^2, 
          r79 = -2*m1^2 - m3^2 - m4^2 + m5^2, 
          r80 = 
           m1^4 + PP^2 - m1^2*r77 + m3^2*r78 + PP*r79, 
          r81 = m3^2 + m4^2, r82 = m3^2 + m4^2 + m5^2, 
          r83 = m1^2 + m3^2 - m5^2, 
          r84 = 
           m1^4 + 2*m3^2*m4^2 - m4^4 + PP^2 - m2^2*r77 + 
            m5^2*r81 - m1^2*r82 - 2*PP*r83, 
          r85 = m1 - m3, r86 = -PP + r85^2, 
          r87 = m1 + m3, r88 = -PP + r87^2, 
          r89 = m2 - m4, r90 = m2 + m4, 
          r91 = -m1^2 - m2^2 - m3^2 - m4^2 + 2*m5^2, 
          r92 = PP^2 + r85*r87*r89*r90 + PP*r91, 
          r93 = m2^2 - 3*m4^2, r94 = m4 - m5, 
          r95 = m4 + m5, r96 = 2*m3^2 - m4^2 + m5^2, 
          r97 = -m1^2 - m2^2 - 2*m3^2 + 3*m5^2, 
          r98 = 
           PP^2 + m1^2*r93 + m2^2*r94*r95 + m4^2*r96 + 
            PP*r97, r99 = -2 + d, r100 = -m1^2 + m2^2, 
          r101 = m2^2 + m1^2*r72, 
          r102 = m3^2*r101 - 2*m3^4*r4 + m2^2*r100*r4, 
          r103 = 9 - 2*d, r104 = m2^2*r23 + m3^2*r72, 
          r105 = 
           2*m1^4 + m2^4*r103 + m1^2*r104 + m2^2*m3^2*r5, 
          r106 = -9 + 2*d, r107 = m1 - m2, 
          r108 = m1 + m2, 
          r109 = -(r106*r107*r108) + m3^2*r4, 
          r110 = 
           m2^2*r102 + m4^2*r105 + m4^4*r109 - m4^6*r4, 
          r111 = m1^2 + m2^2, 
          r112 = m2^2*r4 - 2*m1^2*r72, r113 = -19 + 5*d, 
          r114 = m3^2*r112 + m3^4*r113 + m2^2*r111*r4, 
          r115 = m3^2*r5 + m2^2*r6, 
          r116 = 
           2*m1^4 + 2*m1^2*r115 + m2^4*r13 + m3^4*r72 - 
            2*m2^2*m3^2*r99, 
          r117 = m2^2*r106 + m3^2*r4 - m1^2*r99, 
          r118 = 
           m2^2*r114 - m4^2*r116 + m4^4*r117 - m4^6*r6, 
          r119 = -(m2^2*r5) + 2*m3^2*r57 + m1^2*r72, 
          r120 = m1^2*r18 + 2*m3^2*r4 - m2^2*r5, 
          r121 = m2^2*r119 + m4^2*r120 + 2*m4^4*r5, 
          r122 = m3^2*r5 + m4^2*r72, 
          r123 = -2*m2^2*r4 - m4^2*r69 + m3^2*r72, 
          r124 = 
           m2^2*r122 + m1^2*r123 + 2*m2^4*r4 + 
            2*m4^2*r32*r33*r4, 
          r125 = -3*m4^2 + m2^2*r4 - m3^2*r72, 
          r126 = m3^2 - m4^2*r72, 
          r127 = 
           -2*m3^2*m4^2 - 2*m1^2*r125 - 2*m2^2*r126 + 
            m4^4*r18 - 4*m2^4*r4 + m3^4*r72, 
          r128 = 
           -(m2^2*r18) + 2*m4^2*r5 + m1^2*r72 + 
            2*m3^2*r72, 
          r129 = 
           m5^2*r127 - m5^4*r128 - r124*r32*r33 + 
            m5^6*r72, r130 = -m1^2 + m2^2 + m3^2 - m4^2, 
          r131 = -14 + 3*d, 
          r132 = 
           -(m3^2*r13) + m4^2*r131 + m1^2*r4 + 5*m2^2*r4, 
          r133 = -2*m5^4 + m5^2*r132 + r130*r32*r33*r4, 
          r134 = 
           m5^2*r118 + m5^4*r121 + PP*r129 + PP^2*r133 + 
            r110*r32*r33 - 2*m5^2*PP^3*r4 + 
            m5^6*r5*r89*r90, r135 = m3 - m4 - m5, 
          r136 = m3 + m4 - m5, r137 = m3 - m4 + m5, 
          r138 = m3 + m4 + m5, r139 = -10 + d, 
          r140 = 13 - 2*d, r141 = d*m4^2 + 2*m3^2*r5, 
          r142 = -8 + d, 
          r143 = m4^2*r142 - m3^2*r99 + m5^2*r99, 
          r144 = 
           m3^2*m4^2*r139 + m4^4*r140 - m5^2*r141 + 
            m2^2*r143 + m3^4*r5 + m5^4*r5, 
          r145 = 3*m3^4 - 12*m3^2*m4^2 + 7*m4^4, 
          r146 = 3*m3^2 + 4*m4^2, r147 = m3^2 - 11*m4^2, 
          r148 = 
           7*m3^4 + 10*m3^2*m4^2 - 17*m4^4 - 5*m5^4 - 
            2*m5^2*r147, r149 = m2^2 + 2*m3^2 + m4^2, 
          r150 = 
           -m2^4 + m2^2*m3^2 + m3^4 + 3*m2^2*m4^2 + 
            m3^2*m4^2 - m4^4 + m5^4 - m5^2*r149, 
          r151 = 
           3*m3^6 + 6*m3^4*m4^2 - 17*m3^2*m4^4 + 
            8*m4^6 + 3*m5^6 - m5^2*r145 - m5^4*r146 + 
            m2^2*r148 - d*r150*r77 + 6*m2^4*r94*r95, 
          r152 = -(m3^2*m4) + m4^3, r153 = 11 - 2*d, 
          r154 = -(d*m3^2*m4^2) + m4^4*r153 + m3^4*r5, 
          r155 = m4^2 + m3^2*r5, 
          r156 = r142*r32*r33 + m5^2*r99, 
          r157 = 11 - 3*d, r158 = -8 + 3*d, 
          r159 = m5^2*r158 + m4^2*r21, 
          r160 = 
           -3*m3^4 + 2*m4^2*m5^2*r106 + m5^4*r157 + 
            m3^2*r159 - m4^4*r18, 
          r161 = 
           -2*m2^6 + m5^2*r154 - 2*m5^4*r155 + 
            m2^4*r156 + m2^2*r160 + r152^2*r4 + m5^6*r5, 
          r162 = m4^2*r23 - 3*m3^2*r72, 
          r163 = -4*m2^2 + m4^2*r18 + m3^2*r23, 
          r164 = -m3^4 + m4^4, 
          r165 = 
           -(m2^4*r13) + m2^2*r162 + m1^2*r163 + 
            r164*r4 + m1^4*r99, r166 = -19 + 4*d, 
          r167 = m4^2 + m3^2*r72, r168 = m3^2 + m4^2*r69, 
          r169 = -2*r168 + 4*m2^2*r4, 
          r170 = 
           -m3^4 + m2^4*r13 - m4^4*r166 + 2*m2^2*r167 + 
            m1^2*r169 + 2*m3^2*m4^2*r72 + m1^4*r99, 
          r171 = m1^2 - m2^2 + 4*m4^2, 
          r172 = 
           -(m5^2*r170) + r165*r32*r33 + m5^6*r72 - 
            m5^4*r171*r72, r173 = m2^2 + m3^2, 
          r174 = -(m4^2*r13) + r131*r173 + m1^2*r8, 
          r175 = m5^2*r174 - 2*m5^4*r4 + r130*r32*r33*r4, 
          r176 = 
           2*m1^6*m4^2 + m1^4*r144 + m1^2*r151 + 
            m3^2*r161 + PP*r172 + PP^2*r175 - 
            2*m5^2*PP^3*r4, r177 = m3^2 + m4^2 - m5^2, 
          r178 = -2*m1^2 + m3^2 - m4^2 + m5^2, 
          r179 = -m3^2 + m4^2 + m5^2, 
          r180 = m2^2*r177 + m4^2*r178 + PP*r179, 
          r181 = -2*m2^2 - m3^2 + m4^2 + m5^2, 
          r182 = m1^2*r177 + m3^2*r181 + PP*r77, 
          r183 = -(m2^2*r13) - 2*m3^2*r5, 
          r184 = 
           m1^2*r183 - 2*m3^4*r4 + m2^4*r57 + m1^4*r72 - 
            m2^2*m3^2*r99, r185 = 3*m2^2*r4 + m3^2*r72, 
          r186 = 
           m2^4*r106 + 2*m1^2*r185 + m1^4*r40 - 
            2*m2^2*m3^2*r6, r187 = r111*r13 - m3^2*r4, 
          r188 = 
           m2^2*r184 + m4^2*r186 - m4^4*r187 + m4^6*r4, 
          r189 = 2*m1^2 + m3^2*r10 + 2*m2^2*r5, 
          r190 = m2^2*r5 + m3^2*r72 + 2*m1^2*r99, 
          r191 = m2^2*r189 - m4^2*r190 - m4^4*r5, 
          r192 = -(m3^2*r5) - m4^2*r72, 
          r193 = 2*m2^2*r4 + m4^2*r69 - m3^2*r72, 
          r194 = 
           m2^2*r192 + m1^2*r193 - 2*m2^4*r4 + 
            2*m4^2*r4*r9, r195 = m2^2 + m4^2 + m3^2*r72, 
          r196 = -(m3^2*r6) - m4^2*r72, 
          r197 = 
           m2^4*r157 + 2*m1^2*r195 + 2*m2^2*r196 + 
            2*m4^2*r17*r4 + m1^4*r72, 
          r198 = 
           m4^2*r10 - 2*m2^2*r5 + 2*m1^2*r72 + m3^2*r72, 
          r199 = 
           r107*r108*r194 + m5^2*r197 - m5^4*r198 + 
            m5^6*r72, r200 = m1^2 - m2^2 - m3^2 + m4^2, 
          r201 = m1^2 + m2^2 + m3^2 + 5*m4^2, 
          r202 = 2*m5^4 - r107*r108*r200 - m5^2*r201, 
          r203 = 
           r107*r108*r110 - m5^2*r188 - m5^4*r191 + 
            PP*r199 + 2*m5^2*PP^3*r4 + PP^2*r202*r4 + 
            m5^6*r5*r89*r90, r204 = m4^2 + m5^2, 
          r205 = -2*m5^2*r5 + m4^2*r99, r206 = 18 - 5*d, 
          r207 = m4^2*r206 + m5^2*r8 - 2*m3^2*r99, 
          r208 = 
           m2^2*r207 + 2*m2^4*r4 - m3^4*r5 - 
            m3^2*r204*r5 + r205*r94*r95, 
          r209 = 2*m3^2*m4^2 + m4^4*r142 + 2*m3^4*r5, 
          r210 = -(m4^2*r18) + m3^2*r5, r211 = 3 - 2*d, 
          r212 = m5^2 + m4^2*r157 + m3^2*r211, 
          r213 = m4^2*r106 + m3^2*r4, 
          r214 = 
           -(d*m3^4) + 10*m3^2*m4^2 - 2*d*m3^2*m4^2 - 
            2*m4^4 + d*m4^4 + 2*m5^2*r213 - 2*m5^4*r4, 
          r215 = 
           d*m3^2*m4^4 - 2*m4^6 - m5^2*r209 + 
            m5^4*r210 + m2^4*r212 + m2^2*r214 + 
            m2^6*r4 + m5^6*r5 - m3^4*m4^2*r99, 
          r216 = m4^2*r13 + m3^2*r99, 
          r217 = m3^2*r5 - m4^2*r72, r218 = 13 - 4*d, 
          r219 = 3*m3^2 + m5^2*r218 + m4^2*r23, 
          r220 = m4^2*r142 + m5^2*r61, 
          r221 = 
           2*m3^4 + 2*m4^2*m5^2 - m4^4*r13 + m3^2*r220 + 
            2*m5^4*r6, r222 = 
           m4^2*m5^2*r216 + m5^4*r217 + m2^4*r219 + 
            m2^2*r221 - m5^6*r5 + m2^6*r99, 
          r223 = -(m3^2*r10) + m4^2*r5, 
          r224 = 
           4*m3^2*m4^2 + m1^2*r122 + m4^4*r13 + 
            m2^2*r223 + m1^4*r4 - m2^4*r4 - m3^4*r99, 
          r225 = -m3^2 + m2^2*r72 - m4^2*r72, 
          r226 = -2*m4^2*r106 + 2*m3^2*r69, 
          r227 = 
           m1^4*r106 + m2^4*r106 - m4^4*r13 - 
            2*m1^2*r225 + m2^2*r226 - 4*m3^2*m4^2*r4 - 
            m3^4*r99, r228 = m3^2 + 3*m4^2, 
          r229 = 
           r107*r108*r224 - m5^2*r227 + m5^6*r72 - 
            m5^4*r228*r72, 
          r230 = m4^2*r131 + m1^2*r4 + m2^2*r4 + m3^2*r8, 
          r231 = 
           2*m5^4*r106 - m5^2*r230 - r107*r108*r200*r4, 
          r232 = 
           m1^4*r208 - m1^2*r215 - m3^2*r222 + PP*r229 + 
            PP^2*r231 + 2*m5^2*PP^3*r4 + m1^6*r7, 
          r233 = -2*m3^2 + m4^2, 
          r234 = -m2^2 + m4^2 - 2*m5^2, 
          r235 = -m1^2 + m2^2 + m5^2, 
          r236 = 
           m1^4 + m5^4 + m2^2*r233 + m1^2*r234 + 
            PP*r235 - m5^2*r46, r237 = -m2^2 + m5^2, 
          r238 = m2^2 + m3^2 - 2*m4^2 + m5^2, 
          r239 = -m1^2 + m2^2 - m5^2, 
          r240 = m1^4 + m3^2*r237 - m1^2*r238 + PP*r239, 
          r241 = m3^2 + 3*m4^2 - m5^2, 
          r242 = -3*m3^2 - m4^2 + m5^2, 
          r243 = 
           m5^4 - m5^2*r228 + m1^2*r241 + m2^2*r242 + 
            2*PP*r32*r33 + 2*m4^2*r9, 
          r244 = 2*m1^2 + m2^2 + m4^2, 
          r245 = -2*m1^2 - m3^2 - m4^2 + 3*m5^2, 
          r246 = 
           m1^4 - 3*m2^2*m3^2 + m5^4 + PP^2 + 
            m4^2*r173 - m5^2*r244 + PP*r245, 
          r247 = -2*m3^2 + m4^2 + m5^2, 
          r248 = 
           -m2^4 - m4^2*m5^2 + PP*r235 + m2^2*r247 + 
            m1^2*r46, r249 = m2^2*r153 + m1^2*r5, 
          r250 = 2*m1^2 + m2^2*r4, 
          r251 = m2^2*r13 + m1^2*r139, 
          r252 = r107*r108*r250 + m3^2*r251 - m3^4*r4, 
          r253 = m1^2*r140 + 2*m3^2*r4 + m2^2*r72, 
          r254 = 
           m3^4*r249 + m4^2*r252 + m4^4*r253 - m4^6*r4 + 
            m2^2*m3^2*r100*r99, r255 = 2*m3^2 + m4^2*r13, 
          r256 = -2*m3^2*r5 + m2^2*r99 - m4^2*r99, 
          r257 = -11 + 2*d, 
          r258 = 
           -(m4^4*r106) + 2*m3^2*m4^2*r23 - m3^4*r257, 
          r259 = 4*m4^2 + m3^2*r13, 
          r260 = 
           -2*m2^4 + 6*m3^2*m4^2 + 2*m2^2*r259 + 
            m4^4*r37 + m3^4*r5, 
          r261 = 
           -5*m3^4*m4^2 + m2^4*r255 + m1^4*r256 + 
            m2^2*r258 + m1^2*r260 - m2^6*r4 + m3^6*r5 + 
            m4^6*r6 - 3*m3^2*m4^4*r72, 
          r262 = m3^4 + m3^2*m4^2 + m4^4, 
          r263 = 
           m1^4 + m2^4 - 2*r262 + m1^2*r81 + m2^2*r81, 
          r264 = m1^2 + m2^2 - m3^2 - m4^2, 
          r265 = -(r13*r32*r33) + m2^2*r99, 
          r266 = 
           -2*m1^2*r265 - r32^2*r33^2*r4 + 
            2*m2^2*r13*r9 + m1^4*r99 + m2^4*r99, 
          r267 = m3^2*r13 + m4^2*r4, r268 = -16 + 3*d, 
          r269 = m3^2*r13 - m4^2*r268 + m2^2*r4, 
          r270 = 
           2*m2^2*r267 + 2*m1^2*r269 + 
            r13*r228*r32*r33 + m2^4*r61 + m1^4*r99, 
          r271 = 2*m2^2 + m4^2*r23 - m3^2*r72, 
          r272 = 
           -(m5^2*r270) - 2*m5^4*r271 + r266*r32*r33, 
          r273 = 2*m3^2 - 2*m4^2 - m5^2, 
          r274 = m1^2*r5 - m2^2*r5 + r273*r72, 
          r275 = 
           r200*r254 + m5^2*r261 + PP*r272 + 
            2*m5^2*PP^2*r274 + m5^4*r263*r5 - 
            m5^6*r264*r5, 
          r276 = 
           -2*m2^2*m3^2 - 2*m3^4 + 3*m3^2*m4^2 - m4^4 + 
            3*m3^2*m5^2 + 2*m4^2*m5^2 - m5^4 + 
            m1^2*r177 + PP*r77, 
          r277 = -(m4^2*r106) + m2^2*r4 - m3^2*r5, 
          r278 = 
           -2*m3^4 - m3^2*m4^2*r13 + m2^2*r196 + 
            m2^4*r4 + m4^4*r4, 
          r279 = m4^2*r131 + m3^2*r8, 
          r280 = 
           m2^2*r279 - 2*m2^4*r4 + m4^2*r32*r33*r99, 
          r281 = m1^4*r277 + m2^2*r278 + m1^2*r280, 
          r282 = 11*m3^2 - 3*d*m3^2 + 9*m4^2 - 2*d*m4^2, 
          r283 = 15 - 4*d, 
          r284 = m4^2*r106 + m2^2*r283 + m3^2*r5, 
          r285 = m4^4*r13 + 4*m3^2*m4^2*r5 - m3^4*r99, 
          r286 = -m4^2 + m3^2*r5, 
          r287 = -(m4^2*r5) + m3^2*r6, 
          r288 = 2*m2^2*r287 + m2^4*r72 - 2*r286*r81, 
          r289 = 
           -2*m3^2*m4^4 + m2^4*r282 + m1^4*r284 + 
            m2^2*r285 + m1^2*r288 - m4^6*r4 + m1^6*r5 + 
            m2^6*r6 + m3^4*m4^2*r99, 
          r290 = 2*m2^2 - m3^2 - m4^2, 
          r291 = 
           2*m1^4 + 2*m2^4 - m3^4 - m4^4 + m1^2*r290 - 
            m2^2*r81, r292 = m2^2 - m3^2 + m4^2, 
          r293 = 
           m1^4*r4 + m2^4*r4 - 2*m1^2*r292*r4 + 
            2*m2^2*r4*r9 - r32^2*r33^2*r99, 
          r294 = m3^2*r13 - m4^2*r131, 
          r295 = m2^2*r13 - r4*r81, 
          r296 = 
           m1^4*r131 + 2*m2^2*r294 - 2*m1^2*r295 - 
            2*m3^2*m4^2*r4 + m4^4*r8 - m2^4*r99 - 
            m3^4*r99, r297 = 2*m4^2 - m2^2*r5 + m1^2*r72, 
          r298 = 
           r107*r108*r293 - m5^2*r296 + 2*m5^4*r297, 
          r299 = -2*m5^2*r32*r33*r5 + 2*m5^4*r72, 
          r300 = 
           r130*r281 - m5^2*r289 + PP*r298 + PP^2*r299 - 
            m5^6*r264*r5 + m5^4*r291*r5, 
          r301 = 
           -2*m2^2*m3^2 - m3^2*m4^2 + m4^4 + m5^4 - 
            m5^2*r17 + m1^2*r177 + PP*r77, 
          r302 = m2^2 + 2*m3^2 - 3*m4^2, 
          r303 = -m1^2 + m2^2 - 3*m5^2, 
          r304 = 
           m1^4 - m2^2*m4^2 - m5^4 + m5^2*r149 - 
            m1^2*r302 + PP*r303, 
          r305 = 
           -((r45*TBI[d, PP, {{1, m4}, {1, m2}}]*
                 TBI[d, PP, {{2, m1}, {1, m3}}])/(4*r3)) 
+ (r76*TBI[d, PP, {{1, m3}, {1, m1}}]*
               TBI[d, PP, {{2, m2}, {1, m4}}])/(4*r3) - 
            (r80*TBI[d, PP, {{2, m1}, {1, m3}}]*
               TBI[d, PP, {{2, m2}, {1, m4}}])/2 - 
            (r45*TBI[d, PP, {{1, m4}, {1, m2}}]*
               TBI[d, PP, {{2, m3}, {1, m1}}])/(4*r3) - 
            (r84*TBI[d, PP, {{2, m2}, {1, m4}}]*
               TBI[d, PP, {{2, m3}, {1, m1}}])/2 - 
            r86*r88*TBI[d, PP, {{1, m4}, {1, m2}}]*
             TBI[d, PP, {{2, m3}, {2, m1}}] + 
            (r76*TBI[d, PP, {{1, m3}, {1, m1}}]*
               TBI[d, PP, {{2, m4}, {1, m2}}])/(4*r3) - 
            (r92*TBI[d, PP, {{2, m1}, {1, m3}}]*
               TBI[d, PP, {{2, m4}, {1, m2}}])/2 - 
            (r98*TBI[d, PP, {{2, m3}, {1, m1}}]*
               TBI[d, PP, {{2, m4}, {1, m2}}])/2 - 
            r86*r88*TBI[d, PP, {{1, m4}, {1, m2}}]*
             TBI[d, PP, {{3, m1}, {1, m3}}] - 
            r86*r88*TBI[d, PP, {{1, m4}, {1, m2}}]*
             TBI[d, PP, {{3, m3}, {1, m1}}] - 
            r3*TFI[r99, PP, 
              {{1, m1}, {1, m2}, {1, m3}, {1, m4}, 
               {1, m5}}] - 
            (r134*TVI[d, PP, 
                {{1, m5}, {1, m1}, {2, m4}, {1, m3}}])/
             (4*r3) - (r135*r136*r137*r138*
               TVI[d, PP, 
                {{1, m5}, {1, m1}, {2, m4}, {2, m3}}])/2 
- (r176*TVI[d, PP, {{1, m5}, {1, m2}, {2, m3}, 
                 {1, m4}}])/(4*r3) + 
            (r180*TVI[d, PP, 
                {{1, m5}, {1, m2}, {2, m3}, {2, m4}}])/2 
- r182*TVI[d, PP, {{1, m5}, {1, m2}, {3, m3}, 
               {1, m4}}] + 
            (r203*TVI[d, PP, 
                {{1, m5}, {1, m3}, {2, m2}, {1, m1}}])/
             (4*r3) + (r232*
               TVI[d, PP, 
                {{1, m5}, {1, m4}, {2, m1}, {1, m2}}])/
             (4*r3) + (r236*
               TVI[d, PP, 
                {{1, m5}, {1, m4}, {2, m1}, {2, m2}}])/2 
+ r240*TVI[d, PP, {{1, m5}, {1, m4}, {3, m1}, 
               {1, m2}}] + 
            (r45*TVI[d, PP, 
                {{1, m5}, {2, m1}, {1, m4}, {1, m3}}])/
             (4*r3) + (r86*r88*
               TVI[d, PP, 
                {{1, m5}, {2, m1}, {1, m4}, {2, m3}}])/2 
+ (r92*TVI[d, PP, {{1, m5}, {2, m1}, {2, m4}, 
                 {1, m3}}])/2 - 
            (r76*TVI[d, PP, 
                {{1, m5}, {2, m2}, {1, m3}, {1, m4}}])/
             (4*r3) - (r243*
               TVI[d, PP, 
                {{1, m5}, {2, m2}, {2, m3}, {1, m4}}])/2 
+ (r45*TVI[d, PP, {{1, m5}, {2, m3}, {1, m2}, 
                 {1, m1}}])/(4*r3) + 
            (r86*r88*TVI[d, PP, 
                {{1, m5}, {2, m3}, {1, m2}, {2, m1}}])/2 
+ (r246*TVI[d, PP, {{1, m5}, {2, m3}, {2, m2}, 
                 {1, m1}}])/2 - 
            (r76*TVI[d, PP, 
                {{1, m5}, {2, m4}, {1, m1}, {1, m2}}])/
             (4*r3) + (r248*
               TVI[d, PP, 
                {{1, m5}, {2, m4}, {2, m1}, {1, m2}}])/2 
+ r86*r88*TVI[d, PP, {{1, m5}, {3, m1}, {1, m4}, 
               {1, m3}}] + 
            r86*r88*TVI[d, PP, 
              {{1, m5}, {3, m3}, {1, m2}, {1, m1}}] + 
            (r275*TVI[d, PP, 
                {{2, m5}, {1, m1}, {1, m4}, {1, m3}}])/
             (4*r3) + (r276*
               TVI[d, PP, 
                {{2, m5}, {1, m1}, {1, m4}, {2, m3}}])/2 
- (r180*TVI[d, PP, {{2, m5}, {1, m1}, {2, m4}, 
                 {1, m3}}])/2 - 
            (r275*TVI[d, PP, 
                {{2, m5}, {1, m2}, {1, m3}, {1, m4}}])/
             (4*r3) + (r180*
               TVI[d, PP, 
                {{2, m5}, {1, m2}, {1, m3}, {2, m4}}])/2 
- (r276*TVI[d, PP, {{2, m5}, {1, m2}, {2, m3}, 
                 {1, m4}}])/2 - 
            (r300*TVI[d, PP, 
                {{2, m5}, {1, m3}, {1, m2}, {1, m1}}])/
             (4*r3) - (r240*
               TVI[d, PP, 
                {{2, m5}, {1, m3}, {1, m2}, {2, m1}}])/2 
- (r236*TVI[d, PP, {{2, m5}, {1, m3}, {2, m2}, 
                 {1, m1}}])/2 + 
            (r300*TVI[d, PP, 
                {{2, m5}, {1, m4}, {1, m1}, {1, m2}}])/
             (4*r3) + (r236*
               TVI[d, PP, 
                {{2, m5}, {1, m4}, {1, m1}, {2, m2}}])/2 
+ (r240*TVI[d, PP, {{2, m5}, {1, m4}, {2, m1}, 
                 {1, m2}}])/2 + 
            (r182*TVI[d, PP, 
                {{2, m5}, {2, m1}, {1, m4}, {1, m3}}])/2 
- (r301*TVI[d, PP, {{2, m5}, {2, m2}, {1, m3}, 
                 {1, m4}}])/2 - 
            (r304*TVI[d, PP, 
                {{2, m5}, {2, m3}, {1, m2}, {1, m1}}])/2 
+ (r248*TVI[d, PP, {{2, m5}, {2, m4}, {1, m1}, 
                 {1, m2}}])/2, r306}, 
         r306 = Factor /@ r305/(PP*r1*r2); r306]) /; 
         n1 === 1 && n2 === 1 && n3 === 1 && n4 === 1 && n5 === 1 &&  
       MatchQ[d, _Symbol + _Integer?Positive] && PP =!= 0 &&
       Cayley[m1, m2, m3, m4, m5, Sqrt[PP]] =!= 0]; \
\>", "Input",
  Editable->False]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (51)", "Subsection",
  CellTags->"T1.6.16.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_51\), "=", 
        RowBox[{
          RowBox[{\(\[CapitalDelta]\_\(1, 3, 4\)\), \(\[Nu]\_1\), 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{"2", \(m\_3\%2\), \(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True], 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(1\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 1, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(4\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 4, "-"],
                        Editable->True]}], ")"}]}], "+", 
                RowBox[{\(u\_\(4, 1, 3\)\), \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True], 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(3\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 3, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(4\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 4, "-"],
                        Editable->True]}], ")"}]}], 
                "+", \(\(u\_\(1, 3, 4\)\) \((d - \[Nu]\_1 - 2  \[Nu]\_3)\)\), 
                "+", \(2 \( m\_3\%2\) \((\[Nu]\_1 - \[Nu]\_3)\)\)}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_51 = OperatorApplyV[e\_51]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (51)", "Subsection",
  CellTags->"T1.6.17.1"],

Cell[BoxData[
    \(s\_51\)], "Input"],

Cell[BoxData[
    \(h\_51 = s\_51 /. \[Nu]\_1 \[Rule] \[Nu]\_1 - 1\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_51\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_51\), ",", "\"\<(51)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], 
            "[", \(\[Nu]\_1 > 1 && \[CapitalDelta]\_\(1, 3, 4\) =!= 0\), 
            "]"}]}], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTVIRecursion[f\_51]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`"Off"\)], "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (52)", "Subsection",
  CellTags->"T1.6.18.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_52\), "=", 
        RowBox[{
          RowBox[{\(\[CapitalDelta]\_\(2, 4, 6\)\), \(\[Nu]\_2\), 
            InterpretationBox[
              StyleBox[\(2\^+\),
                FontWeight->"Bold"],
              Schiebe[ 2, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{"2", \(m\_1\%2\), \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True], 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(4\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 4, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(2\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 2, "-"],
                        Editable->True]}], ")"}]}], "+", 
                RowBox[{"2", \(m\_3\%2\), \(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True], 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(4\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 4, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(2\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 2, "-"],
                        Editable->True]}], ")"}]}], "-", 
                RowBox[{"2", \(m\_4\%2\), \(\[Nu]\_4\), 
                  InterpretationBox[
                    StyleBox[\(4\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 4, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(2\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "-"],
                    Editable->True]}], "+", 
                RowBox[{\((2  m\_2\%2 - u\_\(4, 2, 6\))\), \(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(4\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 4, "-"],
                    Editable->True]}], "-", 
                RowBox[{\((\[CapitalSigma]\_4 + 2)\), 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(4\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 4, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(2\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 2, "-"],
                        Editable->True]}], ")"}]}], 
                "+", \(\(u\_\(2, 4, 6\)\) \((d - 3  \[Nu]\_2)\)\), 
                "+", \(2 \( m\_4\%2\) \((\[Nu]\_4 - \[Nu]\_2)\)\)}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_52 = OperatorApplyV[e\_52]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (52)", "Subsection",
  CellTags->"T1.6.19.1"],

Cell[BoxData[
    \(h\_52 = \[Nu]explicit[
          s\_52] /. \[Nu]\_2 \[Rule] \[Nu]\_2 - 1\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_52\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_52\), ",", "\"\<(52)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], 
            "[", \(\[Nu]\_2 > 1 && \[CapitalDelta]\_\(2, 4, 6\) =!= 0\), 
            "]"}]}], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTVIRecursion[f\_52]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`"Off"\)], "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (53)", "Subsection",
  CellTags->"T1.6.20.1"],

Cell[BoxData[
    \(h\_53 = 
      Interchange[s\_51, 
          1 \[LeftRightArrow] 3] /. \[Nu]\_3 \[Rule] \[Nu]\_3 - 1\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_53\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_53\), ",", "\"\<(53)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], 
            "[", \(\[Nu]\_3 > 1 && \[CapitalDelta]\_\(3, 1, 4\) =!= 0\), 
            "]"}]}], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTVIRecursion[f\_53]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`"Off"\)], "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (54)", "Subsection",
  CellTags->"T1.6.21.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_54\), "=", 
        RowBox[{
          RowBox[{"2", \(m\_3\%2\), \(\[Nu]\_3\), 
            InterpretationBox[
              StyleBox[\(3\^+\),
                FontWeight->"Bold"],
              Schiebe[ 3, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{\(u\_\(4, 1, 3\)\), \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True]}], "+", 
                RowBox[{\(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True], 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(4\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 4, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(3\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 3, "-"],
                        Editable->True]}], ")"}]}], "+", "\n", 
                "\t\t\t\t", \((d - \[Nu]\_1 - 2  \[Nu]\_3)\)}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_54 = OperatorApplyV[e\_54]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (54)", "Subsection",
  CellTags->"T1.6.22.1"],

Cell[BoxData[
    \(h\_54 = s\_54 /. \[Nu]\_3 \[Rule] \[Nu]\_3 - 1\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_54\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_54\), ",", "\"\<(54)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", 
            " ", \(\[Nu]\_3 > 1 && \ m3 =!= 0\), "]"}]}], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTVIRecursion[f\_54]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`"Off"\)], "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (55)", "Subsection",
  CellTags->"T1.6.23.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_55\), "=", 
        RowBox[{
          RowBox[{"2", " ", \(m\_4\%2\), \(\[Nu]\_4\), 
            InterpretationBox[
              StyleBox[\(4\^+\),
                FontWeight->"Bold"],
              Schiebe[ 4, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{\(u\_\(3, 1, 4\)\), \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True]}], "+", 
                RowBox[{\(u\_\(6, 2, 4\)\), \(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True]}], "+", "\n", "\t\t\t\t", 
                RowBox[{\(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True], 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(3\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 3, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(4\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 4, "-"],
                        Editable->True]}], ")"}]}], "-", 
                RowBox[{\(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(4\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 4, "-"],
                    Editable->True]}], 
                "+", \((d - \[Nu]\_1 - \[Nu]\_2 - 2  \[Nu]\_4)\)}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_55 = OperatorApplyV[e\_55]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (55)", "Subsection",
  CellTags->"T1.6.24.1"],

Cell[BoxData[
    \(h\_55 = s\_55 /. \[Nu]\_4 \[Rule] \[Nu]\_4 - 1\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_55\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_55\), ",", "\"\<(55)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], 
            "[", \(\[Nu]\_4 > 1 && \ m4 =!= 0\  && \ \[Nu]\_3 > 1\), "]"}]}], 
        "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTVIRecursion[f\_55]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`"Off"\)], "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (56)", "Subsection",
  CellTags->"T1.6.25.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_56\), "=", 
        RowBox[{
          RowBox[{"2", " ", \(m\_4\%2\), \(\[Nu]\_4\), 
            InterpretationBox[
              StyleBox[\(4\^+\),
                FontWeight->"Bold"],
              Schiebe[ 4, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{\(-2\), \(m\_1\%2\), \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True]}], "+", 
                RowBox[{\(u\_\(6, 2, 4\)\), \(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True]}], "-", 
                RowBox[{"2", \(m\_3\%2\), \(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True]}], "-", "\n", "\t\t\t\t", 
                RowBox[{\(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(4\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 4, "-"],
                    Editable->True]}], 
                "+", \((2  d - 2  \[Nu]\_1 - \[Nu]\_2 - 2  \[Nu]\_3 - 
                    2  \[Nu]\_4)\)}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_56 = OperatorApplyV[e\_56]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (56)", "Subsection",
  CellTags->"T1.6.26.1"],

Cell[BoxData[
    \(h\_56 = s\_56 /. \[Nu]\_4 \[Rule] \[Nu]\_4 - 1\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_56\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_56\), ",", "\"\<(56)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], 
            "[", \(\[Nu]\_4 > 1 && \ m4 =!= 0\  && \ \[Nu]\_3 === 1\), 
            "]"}]}], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTVIRecursion[f\_56]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`"Off"\)], "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (59)", "Subsection",
  CellTags->"T1.6.27.1"],

Cell[TextData[{
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalDelta]\_\(1, 3, 4\)\)]],
  "== 0 but all  ",
  Cell[BoxData[
      \(m\_i\%2\)]],
  " != 0"
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_59\), "=", 
        RowBox[{
          
          RowBox[{\((2 \( m\_1\%2\) \(u\_\(1, 3, 4\)\) \((d - \[Nu]\_1 - 
                      2  \[Nu]\_3)\) + \((\[Nu]\_3 - 1)\) 
                  u\_\(4, 1, 3\)\%2 + \n\t
                4 \( m\_1\%2\) \(m\_3\%2\) \((\[Nu]\_1 - \[Nu]\_3)\))\), 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{\(u\_\(4, 1, 3\)\), \(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(4\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 4, "-"],
                    Editable->True], 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(4\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 4, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(1\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 1, "-"],
                        Editable->True]}], ")"}]}], "-", "\n", "\t", 
                RowBox[{"4", \(m\_1\%2\), \(m\_3\%2\), \(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(1\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "-"],
                    Editable->True]}], "+", 
                
                RowBox[{\((4 \( m\_1\%2\) m\_3\%2 + 
                      u\_\(4, 1, 3\)\%2)\), \(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(4\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 4, "-"],
                    Editable->True]}], "+", 
                RowBox[{\(u\_\(4, 1, 3\)\), \((\[Nu]\_3 - 1)\), 
                  InterpretationBox[
                    StyleBox[\(1\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "-"],
                    Editable->True]}], "-", "\n", "\t\t", 
                
                RowBox[{\((d - 2  \[Nu]\_1 - \[Nu]\_3 + 1)\), \(u\_\(4, 1, 
                      3\)\), 
                  InterpretationBox[
                    StyleBox[\(3\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "-"],
                    Editable->True]}], "+", 
                
                RowBox[{\((d - 2  \[Nu]\_1 - 2  \[Nu]\_3 + 1)\), \(u\_\(4, 1, 
                      3\)\), 
                  InterpretationBox[
                    StyleBox[\(4\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 4, "-"],
                    Editable->True]}]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_59 = OperatorApplyV[e\_59]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (59)", "Subsection",
  CellTags->"T1.6.28.1"],

Cell[BoxData[
    \(h\_59 = s\_59\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_59\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_59\), ",", "\"\<(59)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], 
            "[", \(\[CapitalDelta]\_\(1, 3, 4\) == 
                0 && \((4\ m\_3\%2\ m\_1\%2 - 4\ m\_3\%2\ \[Nu]\_3\ m\_1\%2 + 
                    2\ d\ u\_\(1, 3, 4\)\ m\_1\%2 - 
                    4\ \[Nu]\_3\ u\_\(1, 3, 4\)\ m\_1\%2 - 
                    2\ u\_\(1, 3, 4\)\ m\_1\%2 + \[Nu]\_3\ u\_\(4, 1, 3\)\%2 \
- u\_\(4, 1, 3\)\%2)\)\ \  =!= 0 && n1\  === \ 1 && \ n2\  === \ 1 && \ 
              n4\  === \ 1 && MatchQ[d, _Integer]\), "]"}]}], 
        "]"}]}]], "Input"],

Cell["CheckTVIRecursion not applicable .", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (60) ", "Subsection",
  CellTags->"T1.6.29.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_60\), "=", 
        RowBox[{
          RowBox[{"2", \(m\_1\%2\), \(\[Nu]\_1\), 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{\(u\_\(4, 1, 3\)\), \(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True]}], "+", 
                RowBox[{\(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True], 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(4\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 4, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(1\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 1, "-"],
                        Editable->True]}], ")"}]}], "+", "\n", 
                "\t\t\t\t", \((d - \[Nu]\_3 - 2  \[Nu]\_1)\)}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_60 = OperatorApplyV[e\_60]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (60)", "Subsection",
  CellTags->"T1.6.30.1"],

Cell[BoxData[
    \(h\_60 = s\_60 /. \[Nu]\_1 \[Rule] \[Nu]\_1 - 1\)], "Input"],

Cell[BoxData[
    RowBox[{\(notnecessaryf\_60\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_60\), ",", "\"\<(60)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], 
            "[", \(\[Nu]\_1 > 1 && \ m1 =!= 0\), "]"}]}], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTVIRecursion[notnecessaryf\_60]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`"Off"\)], "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (61) ", "Subsection",
  CellTags->"T1.6.31.1"],

Cell[TextData[{
  " ( ",
  Cell[BoxData[
      \(m\_1\%2\)]],
  "== 0 && ",
  Cell[BoxData[
      \(m\_4\%2\)]],
  "== ",
  Cell[BoxData[
      \(m\_3\%2\)]],
  " )  \[DoubleLongRightArrow] ( ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalDelta]\_\(1, 3, 4\)\)]],
  "== 0 )"
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_61\), "=", 
        RowBox[{
          RowBox[{\((d - 2  \[Nu]\_1 - \[Nu]\_3)\), 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}], "==", 
          RowBox[{\(\[Nu]\_3\), 
            InterpretationBox[
              StyleBox[\(3\^+\),
                FontWeight->"Bold"],
              Schiebe[ 3, "+"],
              Editable->True], 
            RowBox[{"(", 
              RowBox[{
                InterpretationBox[
                  StyleBox[\(1\^-\),
                    FontWeight->"Bold"],
                  Schiebe[ 1, "-"],
                  Editable->True], "-", 
                InterpretationBox[
                  StyleBox[\(4\^-\),
                    FontWeight->"Bold"],
                  Schiebe[ 4, "-"],
                  Editable->True]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_61 = OperatorApplyV[e\_61]\)], "Input"],

Cell[TextData[{
  "For  ( ",
  Cell[BoxData[
      \(m\_3\%2\)]],
  "== 0 && ",
  Cell[BoxData[
      \(m\_4\%2\)]],
  "== ",
  Cell[BoxData[
      \(m\_1\%2\)]],
  " )  \[DoubleLongRightArrow] ( ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalDelta]\_\(1, 3, 4\)\)]],
  "== 0 )\nuse ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              InterpretationBox[
                SubsuperscriptBox[
                  StyleBox["V",
                    SingleLetterItalics->False,
                    FontWeight->"Bold"], 
                  RowBox[{
                    FormBox[\(\[Nu]\_1\),
                      "TraditionalForm"], 
                    FormBox[\(\[Nu]\_2\),
                      "TraditionalForm"], 
                    FormBox[\(\[Nu]\_3\),
                      "TraditionalForm"], 
                    FormBox[\(\[Nu]\_4\),
                      "TraditionalForm"]}], 
                  RowBox[{"(", 
                    FormBox["d",
                      "TraditionalForm"], ")"}]],
                TVI[ d, {
                  Subscript[ \[Nu], 1], 
                  Subscript[ \[Nu], 2], 
                  Subscript[ \[Nu], 3], 
                  Subscript[ \[Nu], 4]}],
                Editable->True], "==", 
              InterpretationBox[
                SubsuperscriptBox[
                  StyleBox["V",
                    SingleLetterItalics->False,
                    FontWeight->"Bold"], 
                  RowBox[{
                    FormBox[\(\[Nu]\_3\),
                      "TraditionalForm"], 
                    FormBox[\(\[Nu]\_2\),
                      "TraditionalForm"], 
                    FormBox[\(\[Nu]\_1\),
                      "TraditionalForm"], 
                    FormBox[\(\[Nu]\_4\),
                      "TraditionalForm"]}], 
                  RowBox[{"(", 
                    FormBox["d",
                      "TraditionalForm"], ")"}]],
                TVI[ d, {
                  Subscript[ \[Nu], 1], 
                  Subscript[ \[Nu], 2], 
                  Subscript[ \[Nu], 3], 
                  Subscript[ \[Nu], 4]}],
                Editable->True]}], ")"}], " "}], TextForm]]],
  " and switch the masses accordingly."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (61)", "Subsection",
  CellTags->"T1.6.32.1"],

Cell[TextData[{
  "( ",
  Cell[BoxData[
      \(m\_1\%2\)]],
  "== 0 && ",
  Cell[BoxData[
      \(m\_4\%2\)]],
  "== ",
  Cell[BoxData[
      \(m\_3\%2\)]],
  " )  \[DoubleLongRightArrow] ( ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalDelta]\_\(1, 3, 4\)\)]],
  "== 0 )"
}], "Text"],

Cell[BoxData[
    \(h\_61 = s\_61\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_61\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_61\), ",", "\"\<(61)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", 
            RowBox[{
              RowBox[{
                InterpretationBox[\(m\_1\),
                  m1], "===", "0"}], "&&", 
              RowBox[{
                InterpretationBox[\(m\_3\),
                  m3], "===", 
                InterpretationBox[\(m\_4\),
                  m4]}]}], "]"}], ",", \({n1 :> 1, n4 :> 1}\)}], 
        "]"}]}]], "Input"],

Cell["CheckTVIRecursion not applicable .", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (6153)", "Subsection",
  CellTags->"T1.6.33.1"],

Cell[TextData[{
  " ( ",
  Cell[BoxData[
      \(m\_3\%2\)]],
  "== 0 && ",
  Cell[BoxData[
      \(m\_4\%2\)]],
  "== ",
  Cell[BoxData[
      \(m\_1\%2\)]],
  ")  \[DoubleLongRightArrow] ( ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalDelta]\_\(1, 3, 4\)\)]],
  "== 0 )"
}], "Text"],

Cell[BoxData[
    \(h\_6153 = Interchange[s\_61, 1 \[LeftRightArrow] 3]\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_6153\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_6153\), ",", "\"\<(6153)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", 
            RowBox[{
              RowBox[{
                InterpretationBox[\(m\_3\),
                  m3], "===", "0"}], "&&", 
              RowBox[{
                InterpretationBox[\(m\_1\),
                  m1], "===", 
                InterpretationBox[\(m\_4\),
                  m4]}]}], "]"}], ",", \({n3 :> 1, n4 :> 1}\)}], 
        "]"}]}]], "Input"],

Cell["CheckTVIRecursion not applicable .", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (62) ", "Subsection",
  CellTags->"T1.6.34.1"],

Cell[TextData[{
  "( ",
  Cell[BoxData[
      \(m\_4\%2\)]],
  "== 0 && ",
  Cell[BoxData[
      \(m\_1\%2\)]],
  "== ",
  Cell[BoxData[
      \(m\_3\%2\)]],
  " ) \[DoubleLongRightArrow] (",
  Cell[BoxData[
      \(TraditionalForm\`\(\(\ \)\(\[CapitalDelta]\_\(1, 3, 4\)\)\)\)]],
  "== 0 )\nlower ",
  Cell[BoxData[
      \(TraditionalForm\`\[Nu]\_2\)]],
  " via (52) to one first"
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_62\), "=", 
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                InterpretationBox[\(p\^2\),
                  PP], "-", \(m\_2\%2\)}], 
              ")"}], \((\[Nu]\_1 - 2  \[Nu]\_2 + 2  \[Nu]\_4 + 2)\), 
            InterpretationBox[
              StyleBox[\(4\^+\),
                FontWeight->"Bold"],
              Schiebe[ 4, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}], "==", "\n", "\t\t", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{\((m\_1\%2 - u\_\(6, 1, 2\))\), \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True]}], "+", 
                RowBox[{"4", \(m\_2\%2\), \(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True]}], "+", 
                RowBox[{"2", \(m\_1\%2\), \(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True]}], "+", "\n", "\t\t\t\t", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{\(\[CapitalSigma]\_4\), "-", 
                      RowBox[{"2", \(m\_1\%2\), \(\[Nu]\_1\), 
                        InterpretationBox[
                          StyleBox[\(1\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 1, "+"],
                          Editable->True]}], "-", 
                      RowBox[{"2", \(m\_1\%2\), \(\[Nu]\_3\), 
                        InterpretationBox[
                          StyleBox[\(3\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 3, "+"],
                          Editable->True]}]}], ")"}], 
                  InterpretationBox[
                    StyleBox[\(2\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "-"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(4\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 4, "+"],
                    Editable->True]}], "+", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[\(p\^2\),
                        PP], "-", \(m\_2\%2\)}], ")"}], \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(4\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 4, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(3\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "-"],
                    Editable->True]}], "-", "\n", 
                "\t\t\t\t", \(\[CapitalSigma]\_4\)}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_62 = OperatorApplyV[e\_62]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (62)", "Subsection",
  CellTags->"T1.6.35.1"],

Cell[BoxData[
    \(h\_62 = \[Nu]explicit[
          s\_62] /. \[Nu]\_4 \[Rule] \[Nu]\_4 - 1\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_62\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_62\), ",", "\"\<(62)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", 
            RowBox[{
              RowBox[{
                InterpretationBox[\(p\^2\),
                  PP], "=!=", 
                InterpretationBox[\(m\_2\%2\),
                  Power[ m2, 2]]}], 
              "&&", \(\[Nu]\_1 - 2\ \[Nu]\_2 + 2\ \[Nu]\_4 =!= 0\)}], "]"}], 
          ",", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{
                InterpretationBox[\(m\_4\),
                  m4], "\[RuleDelayed]", "0"}], ",", 
              RowBox[{
                InterpretationBox[\(m\_3\),
                  m3], "\[RuleDelayed]", 
                InterpretationBox[\(m\_1\),
                  m1]}], ",", \(n2 \[RuleDelayed] 1\)}], "}"}]}], 
        "]"}]}]], "Input"],

Cell["CheckTVIRecursion not applicable .", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (63) ", "Subsection",
  CellTags->"T1.6.36.1"],

Cell[TextData[{
  Cell[BoxData[
      \(m\_4\%2\)]],
  "== 0  && ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(\ \)\(\[CapitalDelta]\_\(1, 3, 4\)\)\)\)]],
  " != 0\nreplace (55) by the following"
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_63\), "=", 
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                InterpretationBox[\(p\^2\),
                  PP], "-", \(m\_2\%2\)}], 
              ")"}], \((\((m\_3\%2 - m\_1\%2)\) \((d - 2  \[Nu]\_2 - 
                      2  \[Nu]\_3 + 2  \[Nu]\_4)\) + 
                2 \( m\_3\%2\) \((\[Nu]\_3 - \[Nu]\_1)\))\), 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}], "==", "\n", "\t\t", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{\((m\_1\%2 - m\_3\%2)\), 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{"2", \(m\_1\%2\), \(\[Nu]\_1\), 
                              InterpretationBox[
                                StyleBox[\(1\^+\),
                                  FontWeight->"Bold"],
                                Schiebe[ 1, "+"],
                                Editable->True]}], "+", 
                            RowBox[{"2", \(m\_3\%2\), \(\[Nu]\_3\), 
                              InterpretationBox[
                                StyleBox[\(3\^+\),
                                  FontWeight->"Bold"],
                                Schiebe[ 3, "+"],
                                Editable->True]}], 
                            "-", \(\[CapitalSigma]\_4\), "-", "2"}], ")"}], 
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[
                              StyleBox[\(2\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 2, "-"],
                              Editable->True], "-", 
                            InterpretationBox[
                              StyleBox[\(4\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 4, "-"],
                              Editable->True]}], ")"}]}], "-", 
                      RowBox[{"4", \(m\_2\%2\), \(\[Nu]\_2\), 
                        InterpretationBox[
                          StyleBox[\(2\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 2, "+"],
                          Editable->True], 
                        InterpretationBox[
                          StyleBox[\(4\^-\),
                            FontWeight->"Bold"],
                          Schiebe[ 4, "-"],
                          Editable->True]}]}], ")"}]}], "+", "\n", "\t\t\t\t",
                 
                RowBox[{"2", 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[\(p\^2\),
                        PP], "-", \(m\_2\%2\)}], ")"}], 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{\(m\_3\%2\), \(\[Nu]\_3\), 
                        InterpretationBox[
                          StyleBox[\(3\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 3, "+"],
                          Editable->True], 
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[
                              StyleBox[\(1\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 1, "-"],
                              Editable->True], "-", 
                            InterpretationBox[
                              StyleBox[\(4\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 4, "-"],
                              Editable->True]}], ")"}]}], "+", 
                      RowBox[{\(m\_1\%2\), \(\[Nu]\_1\), 
                        InterpretationBox[
                          StyleBox[\(1\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 1, "+"],
                          Editable->True], 
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[
                              StyleBox[\(4\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 4, "-"],
                              Editable->True], "-", 
                            InterpretationBox[
                              StyleBox[\(3\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 3, "-"],
                              Editable->True]}], ")"}]}]}], ")"}]}]}], ")"}], 
            
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_63 = OperatorApplyV[e\_63]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (63)", "Subsection",
  CellTags->"T1.6.37.1"],

Cell[TextData[{
  Cell[BoxData[
      \(m\_4\%2\)]],
  "== 0  && ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(\ \)\(\[CapitalDelta]\_\(1, 3, 4\)\)\)\)]],
  " != 0"
}], "Text"],

Cell[BoxData[
    \(h\_63 = \[Nu]explicit[s\_63]\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_63\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_63\), ",", "\"\<(63)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], 
            "[", \(PP\  =!= \ m2^2 && \(Cayley[1, 3, 4]\)[{m1, m2, m3, m4}] =!= 
                0\), "]"}], ",", 
          RowBox[{"{", 
            RowBox[{
              InterpretationBox[\(m\_4\),
                m4], "\[RuleDelayed]", "0"}], "}"}]}], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTVIRecursion[f\_63]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`"Off"\)], "Print"]
}, Closed]],

Cell[BoxData[
    RowBox[{\(f\_63\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_63\), ",", "\"\<(63)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], 
            "[", \(PP\  =!= \ m2^2 && \(Cayley[1, 3, 4]\)[{m1, m2, m3, m4}] =!= 
                0\), "]"}], ",", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{
                InterpretationBox[\(m\_4\),
                  m4], "\[RuleDelayed]", "0"}], ",", \(n1 :> 1\), 
              ",", \(n3 :> 1\)}], "}"}]}], "]"}]}]], "Input"],

Cell["\<\

CheckTVIRecursion not applicable for special indices since A's and 
B's appear .
\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (6552)", "Subsection",
  CellTags->"T1.6.38.1"],

Cell[BoxData[
    \(r\_6552 = \[Nu]explicit[e\_52] /. \[CapitalDelta]\_\(2, 4, 6\) -> 
          0\)], "Input"],

Cell[BoxData[
    RowBox[{"formal", "=", 
      RowBox[{
        RowBox[{
          RowBox[{"Solve", "[", 
            RowBox[{\(e\_55 /. TVI[__] -> 1\), ",", 
              InterpretationBox[
                StyleBox[\(4\^+\),
                  FontWeight->"Bold"],
                Schiebe[ 4, "+"],
                Editable->True]}], "]"}], "[", \([1, 1]\), "]"}], 
        "/.", \(\[Nu]\_2 -> \[Nu]\_2 - 1\)}]}]], "Input"],

Cell[BoxData[
    \(r\_6552 /. formal\)], "Input"],

Cell[BoxData[
    \(n\_6552 = OperatorApplyV[r\_6552 /. formal]\)], "Input"],

Cell[BoxData[
    \(n\_6552[\([2, 4]\)]\)], "Input"],

Cell[BoxData[
    \(s\_6552 = \(-n\_6552[\([2, 4]\)]\) == \((n\_6552[\([2]\)] - 
            n\_6552[\([2, 4]\)])\)\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_6552\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(s\_6552\), ",", "\"\<(6552)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], "[", 
            RowBox[{\(\[CapitalDelta]\_\(2, 4, 6\) === 0\), "&&", 
              RowBox[{
                RowBox[{"Expand", "[", 
                  RowBox[{
                    RowBox[{"2", " ", \((\[Nu]\_2 - \[Nu]\_4)\), " ", 
                      InterpretationBox[\(m\_4\%2\),
                        Power[ m4, 2]]}], 
                    "+", \(\((3\ \[Nu]\_2 - d)\)\ u\_\(2, 4, 6\)\), 
                    "+", \(\[Nu]\_2\ u\_\(6, 2, 4\)\)}], "]"}], "=!=", 
                "0"}]}], "]"}]}], "]"}]}]], "Input"],

Cell["CheckTVIRecursion not applicable .", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (6552)", "Subsection",
  CellTags->"T1.6.39.1"],

Cell[BoxData[
    \(r\_6552 = \[Nu]explicit[e\_52] /. \[CapitalDelta]\_\(2, 4, 6\) -> 
          0\)], "Input"],

Cell[BoxData[
    \(n\_6552 = OperatorApplyV[r\_6552]\)], "Input"],

Cell[BoxData[
    \(n\_6552[\([2, 5]\)]\)], "Input"],

Cell[BoxData[
    \(s\_6552 = \(-n\_6552[\([2, 5]\)]\) == \((n\_6552[\([2]\)] - 
            n\_6552[\([2, 5]\)])\)\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_6552\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(s\_6552\), ",", "\"\<(6552)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], 
            "[", \(\[CapitalDelta]\_\(2, 4, 6\) === 0 && 
              Expand[\((\(-2\)\ \[Nu]\_2\ m\_4\%2 + 2\ \[Nu]\_4\ m\_4\%2 + 
                      d\ u\_\(2, 4, 6\) - 3\ \[Nu]\_2\ u\_\(2, 4, 6\))\)] =!= 
                0\), "]"}]}], "]"}]}]], "Input"],

Cell["CheckTVIRecursion not applicable .", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (6553)", "Subsection",
  CellTags->"T1.6.40.1"],

Cell[BoxData[
    \(r\_6553 = \(\(\[Nu]explicit[s\_55] /. u\_\(3, 1, 4\) -> 0\) /. 
          u\_\(6, 2, 4\) -> 0\) /. m\_4 -> 0\)], "Input"],

Cell[BoxData[
    \(h\_6553 = \(-r\_6553[\([2, 1]\)]\) == \((r\_6553[\([2]\)] - 
            r\_6553[\([2, 1]\)])\)\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_6553\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_6553\), ",", "\"\<(6553)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], 
            "[", \(u\_\(6, 2, 4\) === 0 && u\_\(1, 3, 4\) === 0\), "]"}], 
          ",", \(m4 :> 0\)}], "]"}]}]], "Input"],

Cell["CheckTVIRecursion not applicable .", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (66) ", "Subsection",
  CellTags->"T1.6.41.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_66\), "=", 
        RowBox[{
          RowBox[{"4", 
            InterpretationBox[\(p\^2\),
              PP], \(\[Nu]\_1\), 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], \(\[Nu]\_2\), 
            InterpretationBox[
              StyleBox[\(2\^+\),
                FontWeight->"Bold"],
              Schiebe[ 2, "+"],
              Editable->True], \(\[Nu]\_3\), 
            InterpretationBox[
              StyleBox[\(3\^+\),
                FontWeight->"Bold"],
              Schiebe[ 3, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox[\(d + 2\),
                    "TraditionalForm"], ")"}]],
              TVI[ 
                Plus[ 2, d], {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"-", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"(", 
                    
                    RowBox[{\(6  d\), "-", \(6  \[Nu]\_1\), 
                      "-", \(5  \[Nu]\_2\), "-", \(6  \[Nu]\_3\), 
                      "-", \(4  \[Nu]\_4\), "-", 
                      RowBox[{"6", \(m\_1\%2\), \(\[Nu]\_1\), 
                        InterpretationBox[
                          StyleBox[\(1\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 1, "+"],
                          Editable->True]}]}], ")"}], "+", 
                  RowBox[{\((u\_\(6, 2, 4\) - 4  m\_2\%2)\), \(\[Nu]\_2\), 
                    InterpretationBox[
                      StyleBox[\(2\^+\),
                        FontWeight->"Bold"],
                      Schiebe[ 2, "+"],
                      Editable->True]}], "-", 
                  RowBox[{"6", \(m\_3\%2\), \(\[Nu]\_3\), 
                    InterpretationBox[
                      StyleBox[\(3\^+\),
                        FontWeight->"Bold"],
                      Schiebe[ 3, "+"],
                      Editable->True]}], "+", 
                  RowBox[{"2", \((u\_\(6, 2, 4\) - m\_4\%2)\), \(\[Nu]\_4\), 
                    InterpretationBox[
                      StyleBox[\(4\^+\),
                        FontWeight->"Bold"],
                      Schiebe[ 4, "+"],
                      Editable->True]}], "-", 
                  RowBox[{\(\[Nu]\_2\), 
                    InterpretationBox[
                      StyleBox[\(2\^+\),
                        FontWeight->"Bold"],
                      Schiebe[ 2, "+"],
                      Editable->True], 
                    InterpretationBox[
                      StyleBox[\(4\^-\),
                        FontWeight->"Bold"],
                      Schiebe[ 4, "-"],
                      Editable->True]}], "-", 
                  RowBox[{"2", \(\[Nu]\_4\), 
                    InterpretationBox[
                      StyleBox[\(4\^+\),
                        FontWeight->"Bold"],
                      Schiebe[ 4, "+"],
                      Editable->True], 
                    InterpretationBox[
                      StyleBox[\(2\^-\),
                        FontWeight->"Bold"],
                      Schiebe[ 2, "-"],
                      Editable->True]}]}], ")"}]}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_66 = OperatorApplyV[e\_66]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (66)", "Subsection",
  CellTags->"T1.6.42.1"],

Cell[BoxData[
    \(h\_66 = 
      s\_66 /. {\[Nu]\_1 :> \[Nu]\_1 - 1, \[Nu]\_2 :> \[Nu]\_2 - 
              1, \[Nu]\_3 :> \[Nu]\_3 - 1, d :> d - 2}\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_66\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_66\), ",", "\"\<(66)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], 
            "[", \(PP =!= 0 && \[Nu]\_1 > 1 && \[Nu]\_2 > 1 && \[Nu]\_3 > 1 && 
              MatchQ[d, _Symbol + _Integer?Positive]\), "]"}]}], 
        "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTVIRecursion[f\_66]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`"Off"\)], "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (alternative67)", "Subsection",
  CellTags->"T1.6.43.1"],

Cell["\<\
we dereived an alternative formula for Tarasov's (67), but do not \
save it by default.\
\>", "Text"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_67\), "=", 
        RowBox[{
          RowBox[{"4", 
            InterpretationBox[\(p\^2\),
              PP], \(\[Nu]\_1\), \((\[Nu]\_1 + 1)\), 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], \(\[Nu]\_2\), 
            InterpretationBox[
              StyleBox[\(2\^+\),
                FontWeight->"Bold"],
              Schiebe[ 2, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox[\(d + 2\),
                    "TraditionalForm"], ")"}]],
              TVI[ 
                Plus[ 2, d], {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}], " ", "==", 
          RowBox[{
            RowBox[{"(", 
              
              RowBox[{\((6  d - 6  \[Nu]\_1 - 5  \[Nu]\_2 - 6  \[Nu]\_3 - 
                    4  \[Nu]\_4)\), "-", 
                RowBox[{
                "2", \((u\_\(2, 4, 6\) + 3  m\_1\%2)\), \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True]}], "+", 
                RowBox[{\((u\_\(6, 2, 4\) - 4  m\_2\%2)\), \(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True]}], "-", 
                RowBox[{"6", \(m\_3\%2\), \(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True]}], "+", 
                RowBox[{"2", \((u\_\(6, 2, 4\) - m\_4\%2)\), \(\[Nu]\_4\), 
                  InterpretationBox[
                    StyleBox[\(4\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 4, "+"],
                    Editable->True]}], "+", 
                RowBox[{"2", \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True], 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(4\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 4, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(2\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 2, "-"],
                        Editable->True]}], ")"}]}], "-", 
                RowBox[{\(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(4\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 4, "-"],
                    Editable->True]}], "-", 
                RowBox[{"2", \(\[Nu]\_4\), 
                  InterpretationBox[
                    StyleBox[\(4\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 4, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(2\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "-"],
                    Editable->True]}]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_67 = OperatorApplyV[e\_67]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (67)", "Subsection",
  CellTags->"T1.6.44.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_67\), "=", 
        RowBox[{
          RowBox[{"4", 
            InterpretationBox[\(p\^2\),
              PP], \(\[Nu]\_1\), \((\[Nu]\_1 + 1)\), 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], \(\[Nu]\_2\), 
            InterpretationBox[
              StyleBox[\(2\^+\),
                FontWeight->"Bold"],
              Schiebe[ 2, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox[\(d + 2\),
                    "TraditionalForm"], ")"}]],
              TVI[ 
                Plus[ 2, d], {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}], " ", "==", 
          RowBox[{
            RowBox[{"(", 
              
              RowBox[{\(2 \((2  d - 2  \[Nu]\_1 - 2  \[Nu]\_2 - 
                      2  \[Nu]\_3 - \[Nu]\_4)\)\), "-", 
                RowBox[{
                "2", \((u\_\(2, 4, 6\) + 2  m\_1\%2)\), \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True]}], "+", 
                RowBox[{\(-4\), \(m\_2\%2\), \(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True]}], "-", 
                RowBox[{"4", \(m\_3\%2\), \(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True]}], "+", 
                RowBox[{"2", \(u\_\(6, 2, 4\)\), \(\[Nu]\_4\), 
                  InterpretationBox[
                    StyleBox[\(4\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 4, "+"],
                    Editable->True]}], "+", 
                RowBox[{"2", \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True], 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(4\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 4, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(2\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 2, "-"],
                        Editable->True]}], ")"}]}], "-", 
                RowBox[{"2", \(\[Nu]\_4\), 
                  InterpretationBox[
                    StyleBox[\(4\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 4, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(2\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "-"],
                    Editable->True]}]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[TextData[Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{\(-4\), " ", \(\[Nu]\_2\), " ", 
          InterpretationBox[
            SubsuperscriptBox[
              StyleBox["V",
                SingleLetterItalics->False,
                FontWeight->"Bold"], 
              RowBox[{
                FormBox[\(\[Nu]\_1\),
                  "TraditionalForm"], 
                StyleBox[
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"],
                  FontColor->RGBColor[0, 0, 1]], 
                FormBox[\(\[Nu]\_3\),
                  "TraditionalForm"], 
                FormBox[\(\[Nu]\_4\),
                  "TraditionalForm"]}], 
              RowBox[{"(", 
                FormBox["d",
                  "TraditionalForm"], ")"}]],
            TVI[ d, {
              Subscript[ \[Nu], 1], 
              Plus[ 1, 
                Subscript[ \[Nu], 2]], 
              Subscript[ \[Nu], 3], 
              Subscript[ \[Nu], 4]}],
            Editable->True], " ", \(m\_2\%2\)}], "-", 
        RowBox[{"2", " ", \(\[Nu]\_4\), " ", 
          InterpretationBox[
            SubsuperscriptBox[
              StyleBox["V",
                SingleLetterItalics->False,
                FontWeight->"Bold"], 
              RowBox[{
                FormBox[\(\[Nu]\_1\),
                  "TraditionalForm"], 
                StyleBox[
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"],
                  FontColor->RGBColor[1, 0, 0]], 
                FormBox[\(\[Nu]\_3\),
                  "TraditionalForm"], 
                StyleBox[
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"],
                  FontColor->RGBColor[0, 0, 1]]}], 
              RowBox[{"(", 
                FormBox["d",
                  "TraditionalForm"], ")"}]],
            TVI[ d, {
              Subscript[ \[Nu], 1], 
              Plus[ -1, 
                Subscript[ \[Nu], 2]], 
              Subscript[ \[Nu], 3], 
              Plus[ 1, 
                Subscript[ \[Nu], 4]]}],
            Editable->True]}], "+", 
        RowBox[{
        "2", " ", \((2\ d - 2\ \[Nu]\_1 - 2\ \[Nu]\_2 - 
              2\ \[Nu]\_3 - \[Nu]\_4)\), " ", 
          InterpretationBox[
            SubsuperscriptBox[
              StyleBox["V",
                SingleLetterItalics->False,
                FontWeight->"Bold"], 
              RowBox[{
                FormBox[\(\[Nu]\_1\),
                  "TraditionalForm"], 
                FormBox[\(\[Nu]\_2\),
                  "TraditionalForm"], 
                FormBox[\(\[Nu]\_3\),
                  "TraditionalForm"], 
                FormBox[\(\[Nu]\_4\),
                  "TraditionalForm"]}], 
              RowBox[{"(", 
                FormBox["d",
                  "TraditionalForm"], ")"}]],
            TVI[ d, {
              Subscript[ \[Nu], 1], 
              Subscript[ \[Nu], 2], 
              Subscript[ \[Nu], 3], 
              Subscript[ \[Nu], 4]}],
            Editable->True]}], "-", 
        RowBox[{
        "2", " ", \((m\_2\%2 + m\_4\%2 - p\^2)\), " ", \(\[Nu]\_4\), " ", 
          InterpretationBox[
            SubsuperscriptBox[
              StyleBox["V",
                SingleLetterItalics->False,
                FontWeight->"Bold"], 
              RowBox[{
                FormBox[\(\[Nu]\_1\),
                  "TraditionalForm"], 
                FormBox[\(\[Nu]\_2\),
                  "TraditionalForm"], 
                FormBox[\(\[Nu]\_3\),
                  "TraditionalForm"], 
                StyleBox[
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"],
                  FontColor->RGBColor[0, 0, 1]]}], 
              RowBox[{"(", 
                FormBox["d",
                  "TraditionalForm"], ")"}]],
            TVI[ d, {
              Subscript[ \[Nu], 1], 
              Subscript[ \[Nu], 2], 
              Subscript[ \[Nu], 3], 
              Plus[ 1, 
                Subscript[ \[Nu], 4]]}],
            Editable->True]}], "-", 
        RowBox[{"4", " ", \(m\_3\%2\), " ", \(\[Nu]\_3\), " ", 
          InterpretationBox[
            SubsuperscriptBox[
              StyleBox["V",
                SingleLetterItalics->False,
                FontWeight->"Bold"], 
              RowBox[{
                FormBox[\(\[Nu]\_1\),
                  "TraditionalForm"], 
                FormBox[\(\[Nu]\_2\),
                  "TraditionalForm"], 
                StyleBox[
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"],
                  FontColor->RGBColor[0, 0, 1]], 
                FormBox[\(\[Nu]\_4\),
                  "TraditionalForm"]}], 
              RowBox[{"(", 
                FormBox["d",
                  "TraditionalForm"], ")"}]],
            TVI[ d, {
              Subscript[ \[Nu], 1], 
              Subscript[ \[Nu], 2], 
              Plus[ 1, 
                Subscript[ \[Nu], 3]], 
              Subscript[ \[Nu], 4]}],
            Editable->True]}], "-", 
        RowBox[{"2", " ", \(\[Nu]\_1\), " ", 
          InterpretationBox[
            SubsuperscriptBox[
              StyleBox["V",
                SingleLetterItalics->False,
                FontWeight->"Bold"], 
              RowBox[{
                StyleBox[
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"],
                  FontColor->RGBColor[0, 0, 1]], 
                StyleBox[
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"],
                  FontColor->RGBColor[1, 0, 0]], 
                FormBox[\(\[Nu]\_3\),
                  "TraditionalForm"], 
                FormBox[\(\[Nu]\_4\),
                  "TraditionalForm"]}], 
              RowBox[{"(", 
                FormBox["d",
                  "TraditionalForm"], ")"}]],
            TVI[ d, {
              Plus[ 1, 
                Subscript[ \[Nu], 1]], 
              Plus[ -1, 
                Subscript[ \[Nu], 2]], 
              Subscript[ \[Nu], 3], 
              Subscript[ \[Nu], 4]}],
            Editable->True]}], "+", 
        RowBox[{"2", " ", \(\[Nu]\_1\), " ", 
          InterpretationBox[
            SubsuperscriptBox[
              StyleBox["V",
                SingleLetterItalics->False,
                FontWeight->"Bold"], 
              RowBox[{
                StyleBox[
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"],
                  FontColor->RGBColor[0, 0, 1]], 
                FormBox[\(\[Nu]\_2\),
                  "TraditionalForm"], 
                FormBox[\(\[Nu]\_3\),
                  "TraditionalForm"], 
                StyleBox[
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"],
                  FontColor->RGBColor[1, 0, 0]]}], 
              RowBox[{"(", 
                FormBox["d",
                  "TraditionalForm"], ")"}]],
            TVI[ d, {
              Plus[ 1, 
                Subscript[ \[Nu], 1]], 
              Subscript[ \[Nu], 2], 
              Subscript[ \[Nu], 3], 
              Plus[ -1, 
                Subscript[ \[Nu], 4]]}],
            Editable->True]}], "-", 
        RowBox[{
        "2", " ", \((2\ m\_1\%2 + m\_2\%2 - m\_4\%2 - p\^2)\), 
          " ", \(\[Nu]\_1\), " ", 
          InterpretationBox[
            SubsuperscriptBox[
              StyleBox["V",
                SingleLetterItalics->False,
                FontWeight->"Bold"], 
              RowBox[{
                StyleBox[
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"],
                  FontColor->RGBColor[0, 0, 1]], 
                FormBox[\(\[Nu]\_2\),
                  "TraditionalForm"], 
                FormBox[\(\[Nu]\_3\),
                  "TraditionalForm"], 
                FormBox[\(\[Nu]\_4\),
                  "TraditionalForm"]}], 
              RowBox[{"(", 
                FormBox["d",
                  "TraditionalForm"], ")"}]],
            TVI[ d, {
              Plus[ 1, 
                Subscript[ \[Nu], 1]], 
              Subscript[ \[Nu], 2], 
              Subscript[ \[Nu], 3], 
              Subscript[ \[Nu], 4]}],
            Editable->True]}], "-", 
        RowBox[{
        "4", " ", \(p\^2\), " ", \(\[Nu]\_1\), " ", \((\[Nu]\_1 + 1)\), 
          " ", \(\[Nu]\_2\), " ", 
          InterpretationBox[
            SubsuperscriptBox[
              StyleBox["V",
                SingleLetterItalics->False,
                FontWeight->"Bold"], 
              RowBox[{
                StyleBox[
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"],
                  FontWeight->"Bold",
                  FontColor->RGBColor[0, 0, 1]], 
                StyleBox[
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"],
                  FontColor->RGBColor[0, 0, 1]], 
                FormBox[\(\[Nu]\_3\),
                  "TraditionalForm"], 
                FormBox[\(\[Nu]\_4\),
                  "TraditionalForm"]}], 
              RowBox[{"(", 
                FormBox[\(d + 2\),
                  "TraditionalForm"], ")"}]],
            TVI[ 
              Plus[ 2, d], {
              Plus[ 2, 
                Subscript[ \[Nu], 1]], 
              Plus[ 1, 
                Subscript[ \[Nu], 2]], 
              Subscript[ \[Nu], 3], 
              Subscript[ \[Nu], 4]}],
            Editable->True]}]}], TraditionalForm]]]], "Text"],

Cell[BoxData[
    \(s\_67 = OperatorApplyV[e\_67]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (67)", "Subsection",
  CellTags->"T1.6.45.1"],

Cell[BoxData[
    \(h\_67 = 
      s\_67 /. {\[Nu]\_1 :> \[Nu]\_1 - 2, \[Nu]\_2 :> \[Nu]\_2 - 1, 
          d :> d - 2}\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_67\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_67\), ",", "\"\<(67)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], 
            "[", \(PP =!= 0 && \[Nu]\_1 > 2 && \[Nu]\_2 > 1 && 
              MatchQ[d, _Symbol + _Integer?Positive]\), "]"}]}], 
        "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTVIRecursion[f\_67]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`"Off"\)], "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (6713)", "Subsection",
  CellTags->"T1.6.46.1"],

Cell[BoxData[
    \(h\_6713 = 
      Interchange[s\_67, 
          1 \[LeftRightArrow] 
            3] /. {\[Nu]\_3 :> \[Nu]\_3 - 2, \[Nu]\_2 :> \[Nu]\_2 - 1, 
          d :> d - 2}\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_6713\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_6713\), ",", "\"\<(6713)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], 
            "[", \(PP =!= 0 && \[Nu]\_3 > 2 && \[Nu]\_2 > 1 && 
              MatchQ[d, _Symbol + _Integer?Positive]\), "]"}]}], 
        "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTVIRecursion[f\_6713]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`"Off"\)], "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (68)", "Subsection",
  CellTags->"T1.6.47.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_68\), "=", 
        RowBox[{
          RowBox[{"4", 
            InterpretationBox[\(p\^2\),
              PP], \(m\_4\%2\), \(\[Nu]\_1\), 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], \(\[Nu]\_2\), \((\[Nu]\_2 + 1)\), 
            InterpretationBox[
              StyleBox[\(2\^+\),
                FontWeight->"Bold"],
              Schiebe[ 2, "+"],
              Editable->True], 
            InterpretationBox[
              StyleBox[\(2\^+\),
                FontWeight->"Bold"],
              Schiebe[ 2, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox[\(d + 2\),
                    "TraditionalForm"], ")"}]],
              TVI[ 
                Plus[ 2, d], {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"-", 
              RowBox[{"(", 
                
                RowBox[{\((\((m\_1\%2 - m\_3\%2)\) \((2  d - 
                            2  \[Nu]\_1 - \[Nu]\_2 - 2  \[Nu]\_3 - 
                            2  \[Nu]\_4)\) + \(m\_4\%2\) \((2  \[Nu]\_1 + \
\[Nu]\_2 - 2  \[Nu]\_3)\))\), "-", 
                  RowBox[{\(u\_\(1, 3, 4\)\), 
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{"2", \(m\_1\%2\), \(\[Nu]\_1\), 
                          InterpretationBox[
                            StyleBox[\(1\^+\),
                              FontWeight->"Bold"],
                            Schiebe[ 1, "+"],
                            Editable->True]}], "+", 
                        RowBox[{\(u\_\(2, 4, 6\)\), \(\[Nu]\_2\), 
                          InterpretationBox[
                            StyleBox[\(2\^+\),
                              FontWeight->"Bold"],
                            Schiebe[ 2, "+"],
                            Editable->True]}]}], ")"}]}], "+", 
                  RowBox[{\(u\_\(3, 1, 4\)\), 
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{"2", \(m\_3\%2\), \(\[Nu]\_3\), 
                          InterpretationBox[
                            StyleBox[\(3\^+\),
                              FontWeight->"Bold"],
                            Schiebe[ 3, "+"],
                            Editable->True]}], "+", 
                        RowBox[{\(\[Nu]\_2\), 
                          InterpretationBox[
                            StyleBox[\(2\^+\),
                              FontWeight->"Bold"],
                            Schiebe[ 2, "+"],
                            Editable->True], 
                          InterpretationBox[
                            StyleBox[\(4\^-\),
                              FontWeight->"Bold"],
                            Schiebe[ 4, "-"],
                            Editable->True]}]}], ")"}]}], "-", 
                  RowBox[{"2", \(m\_4\%2\), 
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{\(\[Nu]\_2\), 
                          InterpretationBox[
                            StyleBox[\(2\^+\),
                              FontWeight->"Bold"],
                            Schiebe[ 2, "+"],
                            Editable->True]}], "+", 
                        RowBox[{\(\[Nu]\_4\), 
                          InterpretationBox[
                            StyleBox[\(4\^+\),
                              FontWeight->"Bold"],
                            Schiebe[ 4, "+"],
                            Editable->True]}]}], ")"}], 
                    RowBox[{"(", 
                      RowBox[{
                        InterpretationBox[
                          StyleBox[\(3\^-\),
                            FontWeight->"Bold"],
                          Schiebe[ 3, "-"],
                          Editable->True], "-", 
                        InterpretationBox[
                          StyleBox[\(1\^-\),
                            FontWeight->"Bold"],
                          Schiebe[ 1, "-"],
                          Editable->True]}], ")"}]}]}], ")"}]}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_4\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3], 
                Subscript[ \[Nu], 4]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_68 = OperatorApplyV[e\_68]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (68)", "Subsection",
  CellTags->"T1.6.48.1"],

Cell[BoxData[
    \(h\_68 = 
      s\_68 /. {\[Nu]\_1 :> \[Nu]\_1 - 1, \[Nu]\_2 :> \[Nu]\_2 - 2, 
          d :> d - 2}\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_68\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_68\), ",", "\"\<(68)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], 
            "[", \(PP =!= 0 && m4 =!= 0 && \[Nu]\_1 > 1 && \[Nu]\_2 > 2 && 
              MatchQ[d, _Symbol + _Integer?Positive]\), "]"}]}], 
        "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTVIRecursion[f\_68]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`"Off"\)], "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (6813)", "Subsection",
  CellTags->"T1.6.49.1"],

Cell[BoxData[
    \(h\_6813 = 
      Interchange[s\_68, 
          1 \[LeftRightArrow] 
            3] /. {\[Nu]\_2 :> \[Nu]\_2 - 2, \[Nu]\_3 :> \[Nu]\_3 - 1, 
          d :> d - 2}\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_6813\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_6813\), ",", "\"\<(6813)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], 
            "[", \(\(PP =!= 0 && m4 =!= 0\) && \[Nu]\_2 > 2 && \[Nu]\_3 > 1 && 
              MatchQ[d, _Symbol + _Integer?Positive]\), "]"}]}], 
        "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTVIRecursion[f\_6813]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`"Off"\)], "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (70)", "Subsection",
  CellTags->"T1.6.50.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_70\), "=", 
        RowBox[{
          RowBox[{\((3  d - 10)\), \(\((d - 3)\)\^2\), \(m\_4\%2\), 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["V",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox["1",
                    "TraditionalForm"], 
                  FormBox["1",
                    "TraditionalForm"], 
                  FormBox["1",
                    "TraditionalForm"], 
                  FormBox["1",
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TVI[ d, {1, 1, 1, 1}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{
              FractionBox["1", 
                RowBox[{"4", 
                  InterpretationBox[\(p\^2\),
                    PP]}]], 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{\((3  d - 10)\), \(\[CapitalDelta]\_\(1, 3, 4\)\), 
                    RowBox[{"(", 
                      RowBox[{\(\[CapitalDelta]\_\(2, 4, 6\)\), "+", 
                        RowBox[{\(u\_\(2, 4, 6\)\), 
                          RowBox[{"(", 
                            RowBox[{
                              InterpretationBox[
                                StyleBox[\(2\^-\),
                                  FontWeight->"Bold"],
                                Schiebe[ 2, "-"],
                                Editable->True], "-", 
                              InterpretationBox[
                                StyleBox[\(4\^-\),
                                  FontWeight->"Bold"],
                                Schiebe[ 4, "-"],
                                Editable->True]}], ")"}]}]}], ")"}]}], "-", 
                  RowBox[{"8", \(\((d - 4)\)\^2\), 
                    InterpretationBox[\(p\^2\),
                      PP], \(m\_3\%2\), \(m\_4\%2\), 
                    InterpretationBox[
                      StyleBox[\(4\^-\),
                        FontWeight->"Bold"],
                      Schiebe[ 4, "-"],
                      Editable->True]}]}], ")"}], 
              InterpretationBox[
                SubsuperscriptBox[
                  StyleBox["V",
                    SingleLetterItalics->False,
                    FontWeight->"Bold"], 
                  RowBox[{
                    FormBox["1",
                      "TraditionalForm"], 
                    FormBox["1",
                      "TraditionalForm"], 
                    FormBox["1",
                      "TraditionalForm"], 
                    FormBox["1",
                      "TraditionalForm"]}], 
                  RowBox[{"(", 
                    FormBox[\(d - 2\),
                      "TraditionalForm"], ")"}]],
                TVI[ 
                  Plus[ -2, d], {1, 1, 1, 1}],
                Editable->True]}], "+", "\n", "\t\t\t", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{\((d - 3)\), \(m\_4\%2\), 
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{"4", \(m\_1\%2\), \((d - 4)\), 
                          InterpretationBox[
                            StyleBox[\(1\^+\),
                              FontWeight->"Bold"],
                            Schiebe[ 1, "+"],
                            Editable->True], 
                          InterpretationBox[
                            StyleBox[\(1\^+\),
                              FontWeight->"Bold"],
                            Schiebe[ 1, "+"],
                            Editable->True]}], "-", 
                        RowBox[{"8", \((d - 3)\), \(m\_2\%2\), 
                          InterpretationBox[
                            StyleBox[\(2\^+\),
                              FontWeight->"Bold"],
                            Schiebe[ 2, "+"],
                            Editable->True], 
                          InterpretationBox[
                            StyleBox[\(2\^+\),
                              FontWeight->"Bold"],
                            Schiebe[ 2, "+"],
                            Editable->True]}], "+", 
                        RowBox[{\((3  d - 10)\), \((d - 3)\), 
                          InterpretationBox[
                            StyleBox[\(2\^+\),
                              FontWeight->"Bold"],
                            Schiebe[ 2, "+"],
                            Editable->True]}]}], ")"}]}], "+", 
                  RowBox[{\(u\_\(6, 2, 4\)\), 
                    RowBox[{"(", 
                      RowBox[{
                        
                        RowBox[{\(1\/2\), \((3  d - 10)\), \(u\_\(1, 3, 4\)\),
                           
                          RowBox[{"(", 
                            RowBox[{
                              InterpretationBox[
                                StyleBox[\(1\^+\),
                                  FontWeight->"Bold"],
                                Schiebe[ 1, "+"],
                                Editable->True], "-", 
                              InterpretationBox[
                                StyleBox[\(3\^+\),
                                  FontWeight->"Bold"],
                                Schiebe[ 3, "+"],
                                Editable->True]}], ")"}]}], "+", 
                        RowBox[{\((d - 2)\), \((d - 3)\), \(m\_4\%2\), 
                          InterpretationBox[
                            StyleBox[\(1\^+\),
                              FontWeight->"Bold"],
                            Schiebe[ 1, "+"],
                            Editable->True]}]}], ")"}], 
                    InterpretationBox[
                      StyleBox[\(2\^+\),
                        FontWeight->"Bold"],
                      Schiebe[ 2, "+"],
                      Editable->True]}], "-", 
                  RowBox[{\(m\_4\%2\), 
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{\(\((d - 4)\)\^2\), \(u\_\(1, 3, 4\)\), 
                          InterpretationBox[
                            StyleBox[\(2\^+\),
                              FontWeight->"Bold"],
                            Schiebe[ 2, "+"],
                            Editable->True]}], "+", 
                        RowBox[{\((3  d - 10)\), \(u\_\(4, 1, 3\)\), 
                          InterpretationBox[
                            StyleBox[\(3\^+\),
                              FontWeight->"Bold"],
                            Schiebe[ 3, "+"],
                            Editable->True]}]}], ")"}], 
                    InterpretationBox[
                      StyleBox[\(1\^+\),
                        FontWeight->"Bold"],
                      Schiebe[ 1, "+"],
                      Editable->True]}]}], ")"}], 
              InterpretationBox[
                StyleBox[\(4\^-\),
                  FontWeight->"Bold"],
                Schiebe[ 4, "-"],
                Editable->True], 
              InterpretationBox[
                SubsuperscriptBox[
                  StyleBox["V",
                    SingleLetterItalics->False,
                    FontWeight->"Bold"], 
                  RowBox[{
                    FormBox["1",
                      "TraditionalForm"], 
                    FormBox["1",
                      "TraditionalForm"], 
                    FormBox["1",
                      "TraditionalForm"], 
                    FormBox["1",
                      "TraditionalForm"]}], 
                  RowBox[{"(", 
                    FormBox["d",
                      "TraditionalForm"], ")"}]],
                TVI[ d, {1, 1, 1, 1}],
                Editable->True]}], "-", 
            RowBox[{\((d - 3)\), \(m\_4\%2\), 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"4", \((d - 4)\), \(m\_1\%2\), 
                    InterpretationBox[
                      StyleBox[\(1\^+\),
                        FontWeight->"Bold"],
                      Schiebe[ 1, "+"],
                      Editable->True]}], "+", 
                  RowBox[{\((d - 2)\), 
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{\(u\_\(6, 2, 4\)\), 
                          InterpretationBox[
                            StyleBox[\(2\^+\),
                              FontWeight->"Bold"],
                            Schiebe[ 2, "+"],
                            Editable->True]}], "-", 
                        RowBox[{"2", \(m\_4\%2\), 
                          InterpretationBox[
                            StyleBox[\(4\^+\),
                              FontWeight->"Bold"],
                            Schiebe[ 4, "+"],
                            Editable->True]}]}], ")"}]}]}], ")"}], 
              InterpretationBox[
                StyleBox[\(1\^+\),
                  FontWeight->"Bold"],
                Schiebe[ 1, "+"],
                Editable->True], 
              InterpretationBox[
                StyleBox[\(3\^-\),
                  FontWeight->"Bold"],
                Schiebe[ 3, "-"],
                Editable->True], 
              InterpretationBox[
                SubsuperscriptBox[
                  StyleBox["V",
                    SingleLetterItalics->False,
                    FontWeight->"Bold"], 
                  RowBox[{
                    FormBox["1",
                      "TraditionalForm"], 
                    FormBox["1",
                      "TraditionalForm"], 
                    FormBox["1",
                      "TraditionalForm"], 
                    FormBox["1",
                      "TraditionalForm"]}], 
                  RowBox[{"(", 
                    FormBox["d",
                      "TraditionalForm"], ")"}]],
                TVI[ d, {1, 1, 1, 1}],
                Editable->True]}], "+", 
            RowBox[{\(1\/2\), \((3  d - 10)\), \(u\_\(1, 3, 4\)\), 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{\(u\_\(6, 2, 4\)\), 
                    InterpretationBox[
                      StyleBox[\(2\^+\),
                        FontWeight->"Bold"],
                      Schiebe[ 2, "+"],
                      Editable->True]}], "-", 
                  RowBox[{"2", \(m\_4\%2\), 
                    InterpretationBox[
                      StyleBox[\(4\^+\),
                        FontWeight->"Bold"],
                      Schiebe[ 4, "+"],
                      Editable->True]}]}], ")"}], 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    InterpretationBox[
                      StyleBox[\(3\^+\),
                        FontWeight->"Bold"],
                      Schiebe[ 3, "+"],
                      Editable->True], 
                    InterpretationBox[
                      StyleBox[\(1\^-\),
                        FontWeight->"Bold"],
                      Schiebe[ 1, "-"],
                      Editable->True]}], "-", 
                  RowBox[{
                    InterpretationBox[
                      StyleBox[\(1\^+\),
                        FontWeight->"Bold"],
                      Schiebe[ 1, "+"],
                      Editable->True], 
                    InterpretationBox[
                      StyleBox[\(3\^-\),
                        FontWeight->"Bold"],
                      Schiebe[ 3, "-"],
                      Editable->True]}]}], ")"}], 
              InterpretationBox[
                SubsuperscriptBox[
                  StyleBox["V",
                    SingleLetterItalics->False,
                    FontWeight->"Bold"], 
                  RowBox[{
                    FormBox["1",
                      "TraditionalForm"], 
                    FormBox["1",
                      "TraditionalForm"], 
                    FormBox["1",
                      "TraditionalForm"], 
                    FormBox["1",
                      "TraditionalForm"]}], 
                  RowBox[{"(", 
                    FormBox["d",
                      "TraditionalForm"], ")"}]],
                TVI[ d, {1, 1, 1, 1}],
                Editable->True]}]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_70 = OperatorApplyV[e\_70]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (70)", "Subsection",
  CellTags->"T1.6.51.1"],

Cell[BoxData[
    \(h\_70 = s\_70\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_70\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_70\), ",", "\"\<(70)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], 
            "[", \(\(m4 =!= 0\  && \ PP =!= 0\) && 
              MatchQ[d, _Symbol + _Integer?Positive]\), "]"}]}], 
        "]"}]}]], "Input"],

Cell["\<\
CheckTVIRecursion not applicable for special indices since A's and 
B's appear .\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (71)", "Subsection",
  CellTags->"T1.6.52.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`\[Nu]\_1, \[Nu]\_2 > 0\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_71\), "=", 
        RowBox[{
          RowBox[{"2", \(D\_\(1, 2, 3\)\), \(\[Nu]\_1\), 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], \(\[Nu]\_2\), 
            InterpretationBox[
              StyleBox[\(2\^+\),
                FontWeight->"Bold"],
              Schiebe[ 2, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TJI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{"2", \(h\_\(1, 2, 3\)\), \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True]}], "+", 
                RowBox[{"2", \(h\_\(2, 1, 3\)\), \(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True]}], "+", 
                RowBox[{
                "4", \(m\_3\%2\), \(\[Sigma]\_\(1, 2, 3\)\), \(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True]}], "+", 
                RowBox[{\(m\_3\%2\), \(\[Phi]\_\(2, 1, 3\)\), \(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True], \(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(1\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "-"],
                    Editable->True]}], "+", 
                RowBox[{\(m\_3\%2\), \(\[Phi]\_\(1, 2, 3\)\), \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True], \(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(2\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "-"],
                    Editable->True]}], "-", 
                RowBox[{"2", \(\[Rho]\_\(1, 2, 3\)\), \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True], \(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(3\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "-"],
                    Editable->True]}], 
                "+", \(\(1\/2\) \(\[CapitalSigma]\_3\) \((d - \[Nu]\_1 - \
\[Nu]\_2 - \[Nu]\_3)\) \[Phi]\_\(3, 2, 1\)\)}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TJI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_71 = OperatorApplyJ[e\_71]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (71)", "Subsection",
  CellTags->"T1.6.53.1"],

Cell[BoxData[
    \(h\_71 = \(\[Nu]explicit[
            s\_71] /. \[Nu]\_1 -> \[Nu]\_1 - 1\) /. \[Nu]\_2 -> \[Nu]\_2 - 
            1\)], "Input"],

Cell[BoxData[
    \(f\_71 = 
      MakeFun[h\_71, "\<(71)\>", 
        IFF[\[Nu]\_1 > 1 && \[Nu]\_2 > 1 && D\_\(1, 2, 3\) =!= 0]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTJIRecursion[f\_71]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`"Off"\)], "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (7123)", "Subsection",
  CellTags->"T1.6.54.1"],

Cell[BoxData[
    \(h\_7123 = \(Interchange[s\_71, 
            2 \[LeftRightArrow] 3] /. \[Nu]\_1 -> \[Nu]\_1 - 
              1\) /. \[Nu]\_3 -> \[Nu]\_3 - 1\)], "Input"],

Cell[BoxData[
    \(f\_7123 = 
      MakeFun[h\_7123, "\<(7123)\>", 
        IFF[\[Nu]\_1 > 1 && \[Nu]\_3 > 1 && D\_\(1, 3, 2\) =!= 0]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTJIRecursion[f\_7123]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`"Off"\)], "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (7113)", "Subsection",
  CellTags->"T1.6.55.1"],

Cell[BoxData[
    \(h\_7113 = \(Interchange[s\_71, 
            1 \[LeftRightArrow] 3] /. \[Nu]\_2 -> \[Nu]\_2 - 
              1\) /. \[Nu]\_3 -> \[Nu]\_3 - 1\)], "Input"],

Cell[BoxData[
    \(f\_7113 = 
      MakeFun[h\_7113, "\<(7113)\>", 
        IFF[\[Nu]\_2 > 1 && \[Nu]\_3 > 1 && D\_\(3, 2, 1\) =!= 0]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTJIRecursion[f\_7113]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`"Off"\)], "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (78)", "Subsection",
  CellTags->"T1.6.56.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`\[Nu]\_1 > 0\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_78\), "=", 
        RowBox[{
          RowBox[{
          "2", \(m\_1\%2\), 
            " ", \(D\_\(1, 2, 3\)\), \(\[Nu]\_1\), \((\[Nu]\_1 + 1)\), 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TJI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", 
              
              RowBox[{\(\(-\[CapitalSigma]\_3\) \((d - \[Nu]\_1 - \[Nu]\_2 - \
\[Nu]\_3)\) \[Rho]\_\(1, 2, 3\)\), "+", 
                
                RowBox[{\(m\_2\%2\), \(m\_3\%2\), \(\[Phi]\_\(1, 2, 
                      3\)\), \(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True], \(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(1\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "-"],
                    Editable->True]}], "+", 
                
                RowBox[{\(m\_1\%2\), \(m\_3\%2\), \(\[Phi]\_\(2, 1, 
                      3\)\), \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True], \(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(2\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "-"],
                    Editable->True]}], "+", 
                
                RowBox[{\(m\_1\%2\), \(m\_2\%2\), \(\[Phi]\_\(3, 1, 
                      2\)\), \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True], \(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(3\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "-"],
                    Editable->True]}], "+", 
                
                RowBox[{\((d - 2 - 2  \[Nu]\_1)\), \(D\_\(1, 2, 
                      3\)\), \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True]}], "+", 
                RowBox[{\(m\_1\%2\), \(S\_\(1, 2, 3\)\), \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True]}], "+", 
                RowBox[{\(m\_2\%2\), \(S\_\(2, 1, 3\)\), \(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True]}], "+", 
                RowBox[{\(m\_3\%2\), \(S\_\(3, 1, 2\)\), \(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True]}]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TJI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_78 = OperatorApplyJ[e\_78]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (78)", "Subsection",
  CellTags->"T1.6.57.1"],

Cell[BoxData[
    \(h\_78 = \[Nu]explicit[s\_78] /. \[Nu]\_1 -> \[Nu]\_1 - 2\)], "Input"],

Cell[BoxData[
    \(f\_78 = 
      MakeFun[h\_78, "\<(78)\>", 
        IFF[\(\[Nu]\_1 > 2 && m1 =!= 0\) && D\_\(1, 2, 3\) =!= 0]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTJIRecursion[f\_78]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`"Off"\)], "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (80)", "Subsection",
  CellTags->"T1.6.58.1"],

Cell[TextData[{
  Cell[BoxData[
      \(m\_1\%2 == 0\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`\[Nu]\_1 > 0\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_80\), "=", 
        RowBox[{
          
          RowBox[{\(\[Nu]\_1\), 
            " ", \((d - 2\ \[Nu]\_1 - 2)\), \(\[CapitalDelta]\_\(2, 3, 6\)\), 
            
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TJI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{\(-4\), \(m\_2\%2\), \(m\_3\%2\), \(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True], \(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(1\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "-"],
                    Editable->True]}], "+", "\n", "\t\t\t\t", 
                RowBox[{"2", \(m\_2\%2\), 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[\(p\^2\),
                              PP], "-", \(m\_2\%2\)}], 
                          ")"}], \((2  d - 2  \[Nu]\_1 - \[Nu]\_2 - 
                            2  \[Nu]\_3 - 1)\)}], 
                      "-", \(\(m\_3\%2\) \((2  \[Nu]\_1 - \[Nu]\_2 - 1)\)\)}],
                     ")"}], \(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True]}], "+", "\n", "\t\t\t  ", 
                RowBox[{"2", " ", \(m\_3\%2\), 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[\(p\^2\),
                              PP], "-", \(m\_3\%2\)}], 
                          ")"}], \((2  d - 2  \[Nu]\_1 - 
                            2  \[Nu]\_2 - \[Nu]\_3 - 1)\)}], 
                      "-", \(\(m\_2\%2\) \((2  \[Nu]\_1 - \[Nu]\_3 - 1)\)\)}],
                     ")"}], \(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True]}], "-", "\n", 
                "\t\t\t\t", \(\(\[CapitalSigma]\_3\) \((d - \[Nu]\_1 - \
\[Nu]\_2 - \[Nu]\_3)\) u\_\(6, 2, 3\)\)}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TJI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_80 = OperatorApplyJ[e\_80]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (80)", "Subsection",
  CellTags->"T1.6.59.1"],

Cell[BoxData[
    \(h\_80 = \[Nu]explicit[
          s\_80] /. \[Nu]\_1 \[Rule] \[Nu]\_1 - 1\)], "Input"],

Cell[BoxData[
    \(f\_80 = 
      MakeFun[h\_80, "\<(80)\>", 
        IFF[\[Nu]\_1 > 1 && \[CapitalDelta]\_\(2, 3, 6\) =!= 0], 
        m1 :> 0]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTJIRecursion[f\_80]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`"Off"\)], "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (81)", "Subsection",
  CellTags->"T1.6.60.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]],
  "and ",
  Cell[BoxData[
      \(TraditionalForm\`\[Nu]\_1, \[Nu]\_2 > 0\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_81\), "=", 
        RowBox[{
          RowBox[{
            InterpretationBox[\(p\^2\),
              PP], \((\[CapitalSigma]\_3 + 2)\), \(\[Nu]\_1\), 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], \(\[Nu]\_2\), 
            InterpretationBox[
              StyleBox[\(2\^+\),
                FontWeight->"Bold"],
              Schiebe[ 2, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox[\(d + 2\),
                    "TraditionalForm"], ")"}]],
              TJI[ 
                Plus[ 2, d], {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  InterpretationBox[\(p\^2\),
                    PP], \((d - 2  \[Nu]\_3)\)}], 
                "+", \(\(m\_1\%2\) \((d - \[Nu]\_1 - 2  \[Nu]\_2)\)\), 
                "+", \(\(m\_2\%2\) \((d - 2  \[Nu]\_1 - \[Nu]\_2)\)\), 
                "-", \(2 \( 
                      m\_3\%2\) \((d - \[Nu]\_1 - \[Nu]\_2 - \[Nu]\_3)\)\), 
                "-", 
                RowBox[{"2", \(m\_3\%2\), 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[\(p\^2\),
                        PP], "-", \(m\_3\%2\)}], ")"}], \(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True]}], "+", 
                RowBox[{\(m\_1\%2\), 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[\(p\^2\),
                        PP], "-", \(m\_1\%2\), "-", \(3  m\_2\%2\), 
                      "+", \(3  m\_3\%2\), "-", 
                      InterpretationBox[
                        StyleBox[\(2\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 2, "-"],
                        Editable->True], "+", 
                      InterpretationBox[
                        StyleBox[\(3\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 3, "-"],
                        Editable->True]}], ")"}], \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True]}], "+", 
                RowBox[{\(m\_2\%2\), 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[\(p\^2\),
                        PP], "-", \(3  m\_1\%2\), "-", \(m\_2\%2\), 
                      "+", \(3  m\_3\%2\), "-", 
                      InterpretationBox[
                        StyleBox[\(1\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 1, "-"],
                        Editable->True], "+", 
                      InterpretationBox[
                        StyleBox[\(3\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 3, "-"],
                        Editable->True]}], ")"}], \(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True]}]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TJI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_81 = OperatorApplyJ[e\_81]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (81)", "Subsection",
  CellTags->"T1.6.61.1"],

Cell[BoxData[
    \(h\_81 = \(\(\[Nu]explicit[
              s\_81] /. \[Nu]\_1 -> \[Nu]\_1 - 1\) /. \[Nu]\_2 -> \[Nu]\_2 - 
              1\) /. d -> d - 2\)], "Input"],

Cell[BoxData[
    \(f\_81 = 
      MakeFun[h\_81, "\<(81)\>", 
        IFF[PP =!= 0 && \[Nu]\_1 > 1 && \[Nu]\_2 > 1 && 
            MatchQ[d, _Symbol + _Integer?Positive]]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTJIRecursion[f\_81]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`"Off"\)], "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (8123)", "Subsection",
  CellTags->"T1.6.62.1"],

Cell[BoxData[
    \(h\_8123 = \(\(Interchange[s\_81, 
              2 \[LeftRightArrow] 3] /. \[Nu]\_1 -> \[Nu]\_1 - 
                1\) /. \[Nu]\_3 -> \[Nu]\_3 - 1\) /. d -> d - 2\)], "Input"],

Cell[BoxData[
    \(f\_8123 = 
      MakeFun[h\_8123, "\<(8123)\>", 
        IFF[PP =!= 0 && \[Nu]\_1 > 1 && \[Nu]\_3 > 1 && 
            MatchQ[d, _Symbol + _Integer?Positive]]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTJIRecursion[f\_8123]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`"Off"\)], "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (8113)", "Subsection",
  CellTags->"T1.6.63.1"],

Cell[BoxData[
    \(h\_8113 = \(\(Interchange[s\_81, 
              1 \[LeftRightArrow] 3] /. \[Nu]\_2 \[Rule] \[Nu]\_2 - 
                1\) /. \[Nu]\_3 \[Rule] \[Nu]\_3 - 1\) /. 
        d \[Rule] d - 2\)], "Input"],

Cell[BoxData[
    \(f\_8113 = 
      MakeFun[h\_8113, "\<(8113)\>", 
        IFF[PP =!= 0 && \[Nu]\_2 > 1 && \[Nu]\_3 > 1 && 
            MatchQ[d, _Symbol + _Integer?Positive]]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTJIRecursion[f\_8113]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`"Off"\)], "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (82)", "Subsection",
  CellTags->"T1.6.64.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]],
  "and ",
  Cell[BoxData[
      \(TraditionalForm\`\[Nu]\_1 > 0\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_82\), "=", 
        RowBox[{
          RowBox[{
            RowBox[{
              RowBox[{"-", 
                InterpretationBox[\(p\^2\),
                  PP]}], \(m\_1\%2\), \((\[CapitalSigma]\_3 + 
                  2)\), \(\[Nu]\_1\), \((\[Nu]\_1 + 1)\), 
              InterpretationBox[
                StyleBox[\(1\^+\),
                  FontWeight->"Bold"],
                Schiebe[ 1, "+"],
                Editable->True], 
              InterpretationBox[
                StyleBox[\(1\^+\),
                  FontWeight->"Bold"],
                Schiebe[ 1, "+"],
                Editable->True], 
              InterpretationBox[
                SubsuperscriptBox[
                  StyleBox["J",
                    SingleLetterItalics->False,
                    FontWeight->"Bold"], 
                  RowBox[{
                    FormBox[\(\[Nu]\_1\),
                      "TraditionalForm"], 
                    FormBox[\(\[Nu]\_2\),
                      "TraditionalForm"], 
                    FormBox[\(\[Nu]\_3\),
                      "TraditionalForm"]}], 
                  RowBox[{"(", 
                    FormBox[\(d + 2\),
                      "TraditionalForm"], ")"}]],
                TJI[ 
                  Plus[ 2, d], {
                  Subscript[ \[Nu], 1], 
                  Subscript[ \[Nu], 2], 
                  Subscript[ \[Nu], 3]}],
                Editable->True]}], "+", 
            RowBox[{\(1\/2\), 
              InterpretationBox[\(p\^2\),
                
                PP], \((\[CapitalSigma]\_3 + 2)\), \((\[CapitalSigma]\_3 + 
                  4)\), \(\[Nu]\_1\), 
              InterpretationBox[
                StyleBox[\(1\^+\),
                  FontWeight->"Bold"],
                Schiebe[ 1, "+"],
                Editable->True], 
              InterpretationBox[
                SubsuperscriptBox[
                  StyleBox["J",
                    SingleLetterItalics->False,
                    FontWeight->"Bold"], 
                  RowBox[{
                    FormBox[\(\[Nu]\_1\),
                      "TraditionalForm"], 
                    FormBox[\(\[Nu]\_2\),
                      "TraditionalForm"], 
                    FormBox[\(\[Nu]\_3\),
                      "TraditionalForm"]}], 
                  RowBox[{"(", 
                    FormBox[\(d + 2\),
                      "TraditionalForm"], ")"}]],
                TJI[ 
                  Plus[ 2, d], {
                  Subscript[ \[Nu], 1], 
                  Subscript[ \[Nu], 2], 
                  Subscript[ \[Nu], 3]}],
                Editable->True]}]}], "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{\(-\(1\/2\)\), 
                  SuperscriptBox[
                    RowBox[{"(", 
                      InterpretationBox[\(p\^2\),
                        PP], ")"}], 
                    "2"], \((d + 2  \[Nu]\_1 - 2  \[Nu]\_2 - 2  \[Nu]\_3)\)}],
                 "+", 
                RowBox[{\(1\/2\), 
                  InterpretationBox[\(p\^2\),
                    
                    PP], \((\(m\_1\%2\) \((6  \[Nu]\_1 + 6  \[Nu]\_2 + 
                            6  \[Nu]\_3 - 7  d)\) + \(m\_2\%2\) \((7  d - 
                            2  \[Nu]\_1 - 4  \[Nu]\_2 - 
                            10  \[Nu]\_3)\) + \(m\_3\%2\) \((7  d - 
                            2  \[Nu]\_1 - 10  \[Nu]\_2 - 4  \[Nu]\_3)\))\)}], 
                "+", \(\(\((m\_2\%2)\)\^2\) \((d - 
                      2  \[Nu]\_1 - \[Nu]\_2)\)\), 
                "+", \(\(\((m\_3\%2)\)\^2\) \((d - 
                      2  \[Nu]\_1 - \[Nu]\_3)\)\), 
                "+", \(\(m\_1\%2\) \(m\_2\%2\) \((d - \[Nu]\_1 - 
                      2  \[Nu]\_2)\)\), 
                "+", \(\(m\_1\%2\) \(m\_3\%2\) \((d - \[Nu]\_1 - 
                      2  \[Nu]\_3)\)\), 
                "-", \(4 \( 
                      m\_2\%2\) \(m\_3\%2\) \((d - \[Nu]\_1 - \[Nu]\_2 - \
\[Nu]\_3)\)\), "-", 
                RowBox[{\(m\_1\%2\), 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"2", 
                        SuperscriptBox[
                          RowBox[{"(", 
                            InterpretationBox[\(p\^2\),
                              PP], ")"}], "2"]}], "-", 
                      RowBox[{
                        InterpretationBox[\(p\^2\),
                          PP], \((2  m\_1\%2 + m\_2\%2 + m\_3\%2)\)}], 
                      "+", \(\(m\_1\%2\) \((m\_2\%2 + m\_3\%2)\)\), 
                      "+", \(3 \((m\_2\%2 - m\_3\%2)\)\^2\)}], 
                    ")"}], \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True]}], "+", 
                RowBox[{\(m\_2\%2\), 
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox[
                        RowBox[{"(", 
                          InterpretationBox[\(p\^2\),
                            PP], ")"}], "2"], "+", 
                      RowBox[{
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[\(p\^2\),
                              PP], "-", \(m\_2\%2\)}], ")"}], 
                        RowBox[{"(", 
                          RowBox[{\(3  m\_1\%2\), "-", \(5  m\_3\%2\), "+", 
                            InterpretationBox[
                              StyleBox[\(1\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 1, "-"],
                              Editable->True], "-", 
                            InterpretationBox[
                              StyleBox[\(3\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 3, "-"],
                              Editable->True]}], ")"}]}], 
                      "-", \(\((m\_2\%2)\)\^2\)}], ")"}], \(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True]}], "+", 
                RowBox[{\(m\_3\%2\), 
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox[
                        RowBox[{"(", 
                          InterpretationBox[\(p\^2\),
                            PP], ")"}], "2"], "+", 
                      RowBox[{
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[\(p\^2\),
                              PP], "-", \(m\_3\%2\)}], ")"}], 
                        RowBox[{"(", 
                          RowBox[{\(3  m\_1\%2\), "-", \(5  m\_2\%2\), "+", 
                            InterpretationBox[
                              StyleBox[\(1\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 1, "-"],
                              Editable->True], "-", 
                            InterpretationBox[
                              StyleBox[\(2\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 2, "-"],
                              Editable->True]}], ")"}]}], 
                      "-", \(\((m\_3\%2)\)\^2\)}], ")"}], \(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True]}], "-", 
                RowBox[{\(1\/2\), 
                  InterpretationBox[\(p\^2\),
                    PP], \((\[CapitalSigma]\_3 + 2)\), 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(1\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 1, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(2\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 2, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(3\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 3, "-"],
                        Editable->True]}], ")"}]}], "-", 
                RowBox[{\(m\_1\%2\), \((m\_2\%2 - m\_3\%2)\), \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True], 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(2\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 2, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(3\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 3, "-"],
                        Editable->True]}], ")"}]}]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TJI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(r\_82 = OperatorApplyJ[e\_82]\)], "Input"],

Cell[BoxData[
    \(r\_82[\([1, 1]\)]\)], "Input"],

Cell[BoxData[
    \(\(s\_82 = \((r\_82[\([1, 2]\)] == \(-r\_82[\([1, 1]\)]\) + 
              r\_82[\([2]\)])\);\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (82)", "Subsection",
  CellTags->"T1.6.65.1"],

Cell[BoxData[
    \(h\_82 = \(\[Nu]explicit[s\_82] /. \[Nu]\_1 -> \[Nu]\_1 - 2\) /. 
        d -> d - 2\)], "Input"],

Cell[BoxData[
    \(f\_82 = 
      MakeFun[h\_82, "\<(82)\>", 
        IFF[\(PP =!= 0 && \[Nu]\_1 > 2 && m1 =!= 0\) && 
            MatchQ[d, _Symbol + _Integer?Positive]]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTJIRecursion[f\_82]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`"Off"\)], "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (8213)", "Subsection",
  CellTags->"T1.6.66.1"],

Cell[BoxData[
    \(h\_8213 = \(Interchange[s\_82, 
            1 \[LeftRightArrow] 3] /. \[Nu]\_3 -> \[Nu]\_3 - 2\) /. 
        d -> d - 2\)], "Input"],

Cell[BoxData[
    \(f\_8213 = 
      MakeFun[h\_8213, "\<(8213)\>", 
        IFF[\(PP =!= 0 && m3 =!= 0\) && \[Nu]\_3 > 2 && 
            MatchQ[d, _Symbol + _Integer?Positive]]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTJIRecursion[f\_8213]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`"Off"\)], "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (8212)", "Subsection",
  CellTags->"T1.6.67.1"],

Cell[BoxData[
    \(h\_8212 = \(Interchange[s\_82, 
            1 \[LeftRightArrow] 2] /. \[Nu]\_2 -> \[Nu]\_2 - 2\) /. 
        d -> d - 2\)], "Input"],

Cell[BoxData[
    \(f\_8212 = 
      MakeFun[h\_8212, "\<(8212)\>", 
        IFF[\(PP =!= 0 && \[Nu]\_2 > 2 && m2 =!= 0\) && 
            MatchQ[d, _Symbol + _Integer?Positive]]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTJIRecursion[f\_8212]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`"Off"\)], "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (83)", "Subsection",
  CellTags->"T1.6.68.1"],

Cell[TextData[{
  Cell[BoxData[
      \(m\_1\%2 == 0\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`\[Nu]\_1 > 0\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_83\), "=", 
        RowBox[{
          RowBox[{
            InterpretationBox[\(p\^2\),
              
              PP], \((\[CapitalSigma]\_3 + 2)\), \((\[CapitalSigma]\_3 + 
                4)\), \(\[Nu]\_1\), 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox[\(d + 2\),
                    "TraditionalForm"], ")"}]],
              TJI[ 
                Plus[ 2, d], {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  RowBox[{"-", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        InterpretationBox[\(p\^2\),
                          PP], ")"}], "2"]}], \((d + 2  \[Nu]\_1 - 
                      2  \[Nu]\_2 - 2  \[Nu]\_3)\)}], "+", 
                RowBox[{
                  InterpretationBox[\(p\^2\),
                    
                    PP], \((\(m\_2\%2\) \((7  d - 2  \[Nu]\_1 - 4  \[Nu]\_2 - 
                            10  \[Nu]\_3)\) + \(m\_3\%2\) \((7  d - 
                            2  \[Nu]\_1 - 10  \[Nu]\_2 - 4  \[Nu]\_3)\))\)}], 
                "+", \(2 \(\((m\_2\%2)\)\^2\) \((d - 
                      2  \[Nu]\_1 - \[Nu]\_2)\)\), 
                "+", \(2 \(\((m\_3\%2)\)\^2\) \((d - 
                      2  \[Nu]\_1 - \[Nu]\_3)\)\), 
                "-", \(8 \( 
                      m\_2\%2\) \(m\_3\%2\) \((d - \[Nu]\_1 - \[Nu]\_2 - \
\[Nu]\_3)\)\), "+", 
                RowBox[{"2", \(m\_2\%2\), 
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox[
                        RowBox[{"(", 
                          InterpretationBox[\(p\^2\),
                            PP], ")"}], "2"], "-", 
                      RowBox[{
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[\(p\^2\),
                              PP], "-", \(m\_2\%2\)}], ")"}], 
                        RowBox[{"(", 
                          RowBox[{\(5  m\_3\%2\), "-", 
                            InterpretationBox[
                              StyleBox[\(1\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 1, "-"],
                              Editable->True], "+", 
                            InterpretationBox[
                              StyleBox[\(3\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 3, "-"],
                              Editable->True]}], ")"}]}], 
                      "-", \(\((m\_2\%2)\)\^2\)}], ")"}], \(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True]}], "+", 
                RowBox[{"2", \(m\_3\%2\), 
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox[
                        RowBox[{"(", 
                          InterpretationBox[\(p\^2\),
                            PP], ")"}], "2"], "-", 
                      RowBox[{
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[\(p\^2\),
                              PP], "-", \(m\_3\%2\)}], ")"}], 
                        RowBox[{"(", 
                          RowBox[{\(5  m\_2\%2\), "-", 
                            InterpretationBox[
                              StyleBox[\(1\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 1, "-"],
                              Editable->True], "+", 
                            InterpretationBox[
                              StyleBox[\(2\^-\),
                                FontWeight->"Bold"],
                              Schiebe[ 2, "-"],
                              Editable->True]}], ")"}]}], 
                      "-", \(\((m\_3\%2)\)\^2\)}], ")"}], \(\[Nu]\_3\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True]}], "-", 
                RowBox[{
                  InterpretationBox[\(p\^2\),
                    PP], \((\[CapitalSigma]\_3 + 2)\), 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(1\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 1, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(2\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 2, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(3\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 3, "-"],
                        Editable->True]}], ")"}]}]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TJI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_83 = OperatorApplyJ[e\_83]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (83)", "Subsection",
  CellTags->"T1.6.69.1"],

Cell[BoxData[
    \(h\_83 = \(\[Nu]explicit[s\_83] /. \[Nu]\_1 -> \[Nu]\_1 - 1\) /. 
        d -> d - 2\)], "Input"],

Cell[BoxData[
    \(f\_83 = 
      MakeFun[h\_83, "\<(83)\>", 
        IFF[PP =!= 0 && \[Nu]\_1 > 1 && 
            MatchQ[d, _Symbol + _Integer?Positive]], m1 :> 0]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTJIRecursion[f\_83]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`"Off"\)], "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (87)", "Subsection",
  CellTags->"T1.6.70.1"],

Cell[CellGroupData[{

Cell["(88)", "Subsubsection",
  CellTags->"T1.6.70.1.1"],

Cell[BoxData[
    \(\(t\_1 = m\_1\%2 + m\_2\%2 + m\_3\%2;\)\)], "Input"],

Cell[BoxData[
    \(\(t\_2 = 
        3 \((m\_1\%4 + m\_2\%4 + m\_3\%4)\) + 
          2 \((\(m\_1\%2\) m\_2\%2 + \(m\_1\%2\) m\_3\%2 + \(m\_2\%2\) 
                  m\_3\%2)\);\)\)], "Input"],

Cell[BoxData[
    \(\(t\_3 = \(m\_1\%2\) \((m\_1\%4 - m\_2\%4 - 
                m\_3\%4)\) + \(m\_2\%2\) \((m\_2\%4 - m\_1\%4 - 
                m\_3\%4)\) + \(m\_3\%2\) \((m\_3\%4 - m\_1\%4 - m\_2\%4)\) + 
          10 \( m\_1\%2\) \(m\_2\%2\) m\_3\%2;\)\)], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(f[\[Mu]1_, \[Mu]2_, \ \[Mu]3_]\), ":=", 
        RowBox[{"\[Mu]1", 
          RowBox[{"(", 
            RowBox[{
              InterpretationBox[\(p\^2\),
                PP], "-", "\[Mu]1"}], ")"}], 
          RowBox[{"(", 
            RowBox[{
              RowBox[{\(-2\), \((d - 4)\), 
                SuperscriptBox[
                  RowBox[{"(", 
                    InterpretationBox[\(p\^2\),
                      PP], ")"}], "2"]}], "+", 
              RowBox[{
                InterpretationBox[\(p\^2\),
                  
                  PP], \((4 \( t\_1\) \((5  d - 18)\) - 
                    24  \[Mu]1 \((2  d - 7)\))\)}], 
              "-", \(2 \((4  d - 13)\) t\_1\^2\), 
              "+", \(2 \((9  d - 31)\) t\_2\), 
              "-", \(24  \[Mu]2\ \[Mu]3 \((4  d - 13)\)\), 
              "-", \(24\ \(\[Mu]1\^2\) \((2  d - 7)\)\)}], ")"}]}]}], 
      ";"}]], "Input"],

Cell[BoxData[
    \(\(f\_\(1, 2, 3\) = f[m\_1\%2, m\_2\%2, m\_3\%2];\)\)], "Input"],

Cell[BoxData[
    \(\(f\_\(2, 1, 3\) = f[m\_2\%2, m\_1\%2, m\_3\%2];\)\)], "Input"],

Cell[BoxData[
    \(\(f\_\(3, 2, 1\) = f[m\_3\%2, m\_2\%2, m\_1\%2];\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["(89)", "Subsubsection",
  CellTags->"T1.6.70.2.1"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(g[\[Mu]1_, \[Mu]2_, \ \[Mu]3_]\), ":=", 
        RowBox[{\(\(\[Mu]1\ \[Mu]2\)\/\(d - 4\)\), 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"4", \((d - 4)\), 
                SuperscriptBox[
                  RowBox[{"(", 
                    InterpretationBox[\(p\^2\),
                      PP], ")"}], "2"]}], "-", 
              RowBox[{"4", \((7  d - 24)\), 
                InterpretationBox[\(p\^2\),
                  PP], \((3  \[Mu]3 - 2  t\_1)\)}], 
              "-", \(\(t\_1\^2\) \((23  d - 80)\)\), 
              "+", \(\(t\_2\) \((9  d - 32)\)\), 
              "-", \(12 \( \[Mu]3\^2\) \((d - 4)\)\), 
              "+", \(12  \[Mu]1\ \[Mu]2\ \((7  d - 24)\)\)}], ")"}]}]}], 
      ";"}]], "Input"],

Cell[BoxData[
    \(\(g\_\(1, 2, 3\) = g[m\_1\%2, m\_2\%2, m\_3\%2];\)\)], "Input"],

Cell[BoxData[
    \(\(g\_\(1, 3, 2\) = g[m\_1\%2, m\_3\%2, m\_2\%2];\)\)], "Input"],

Cell[BoxData[
    \(\(g\_\(2, 3, 1\) = g[m\_2\%2, m\_3\%2, m\_1\%2];\)\)], "Input"],

Cell[BoxData[
    \(g\_\(1, 2, 3\)\)], "Input"],

Cell[TextData[{
  "general  ",
  Cell[BoxData[
      \(m\_i\%2\)]],
  " "
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_87\), "=", 
        RowBox[{
          RowBox[{"3", 
            InterpretationBox[\(p\^2\),
              PP], \((d - 3)\), \((d - 4)\), \((3  d - 8)\), \((3  d - 10)\), 
            
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox["1",
                    "TraditionalForm"], 
                  FormBox["1",
                    "TraditionalForm"], 
                  FormBox["1",
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TJI[ d, {1, 1, 1}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{\(\((d - 4)\)\^2\), 
                  SuperscriptBox[
                    RowBox[{"(", 
                      InterpretationBox[\(p\^2\),
                        PP], ")"}], "3"]}], "-", 
                RowBox[{"2", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      InterpretationBox[\(p\^2\),
                        PP], ")"}], 
                    "2"], \(t\_1\), \((d - 4)\), \((6  d - 23)\)}], "+", 
                RowBox[{
                  InterpretationBox[\(p\^2\),
                    
                    PP], \((5 \( t\_1\^2\) \((15  d\^2 - 117  d + 
                            224)\) - \(t\_2\) \((42  d\^2 - 331  d + 
                            640)\))\)}], 
                "-", \(\(1\/4\) \((d - 
                      5)\) \((\(t\_3\) \((27  d - 
                            90)\) - \(t\_1\) \(t\_2\) \((3  d - 2)\) - 
                      2 \( t\_1\^3\) \((5  d - 26)\))\)\), "+", 
                RowBox[{\(f\_\(1, 2, 3\)\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True]}], "+", 
                RowBox[{\(f\_\(2, 1, 3\)\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True]}], "+", 
                RowBox[{\(f\_\(3, 2, 1\)\), 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True]}], "+", 
                RowBox[{\(g\_\(1, 2, 3\)\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(3\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "-"],
                    Editable->True]}], "+", 
                RowBox[{\(g\_\(1, 3, 2\)\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(2\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "-"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True]}], "+", 
                RowBox[{\(g\_\(2, 3, 1\)\), 
                  InterpretationBox[
                    StyleBox[\(1\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "-"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True], 
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True]}]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox["1",
                    "TraditionalForm"], 
                  FormBox["1",
                    "TraditionalForm"], 
                  FormBox["1",
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox[\(d - 2\),
                    "TraditionalForm"], ")"}]],
              TJI[ 
                Plus[ -2, d], {1, 1, 1}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_87 = OperatorApplyJ[e\_87]\)], "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (87)", "Subsection",
  CellTags->"T1.6.71.1"],

Cell[BoxData[
    \(h\_87 = s\_87\)], "Input"],

Cell[BoxData[
    \(\(f\_87 = 
        MakeFun[h\_87, "\<(87)\>", 
          IFF[PP =!= 0 && 
              MatchQ[d, _Symbol + _Integer?Positive]]];\)\)], "Input"],

Cell["CheckTJIRecursion not applicable.", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (90)", "Subsection",
  CellTags->"T1.6.72.1"],

Cell[TextData[{
  "general  ",
  Cell[BoxData[
      \(m\_i\%2\)]],
  " "
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_90\), "=", 
        RowBox[{
          RowBox[{"3", 
            InterpretationBox[\(p\^2\),
              PP], \((d - 3)\), \((d - 4)\), \((3  d - 10)\), 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox["1",
                    "TraditionalForm"], 
                  FormBox["1",
                    "TraditionalForm"], 
                  FormBox["1",
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TJI[ d, {1, 1, 1}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{\(-m\_1\%2\), 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                        SuperscriptBox[
                          RowBox[{"(", 
                            InterpretationBox[\(p\^2\),
                              PP], ")"}], "2"], \((7  d - 24)\)}], "-", 
                      RowBox[{"2", 
                        InterpretationBox[\(p\^2\),
                          
                          PP], \((\((4  d - 15)\) t\_1 - \((d - 5)\) 
                              m\_1\%2)\)}], 
                      "+", \(\(3\/2\) \((d - 5)\) t\_1\^2\), 
                      "+", \(\(5\/2\) \((d - 3)\) t\_2\), 
                      "-", \(2 \((5  d - 17)\) \((m\_1\%2)\)\^2\), 
                      "-", \(2 \((13  d - 45)\) \(m\_2\%2\) m\_3\%2\)}], 
                    ")"}], 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True]}], "+", "\n", "\t\t\t\t", 
                RowBox[{"2", \(m\_2\%2\), 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[\(p\^2\),
                        PP], "-", \(m\_2\%2\)}], ")"}], 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{\((d - 3)\), 
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[\(p\^2\),
                              PP], "+", \(m\_2\%2\), "-", \(5  m\_3\%2\)}], 
                          ")"}]}], "+", \(\((7  d - 25)\) m\_1\%2\)}], ")"}], 
                  
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True]}], "+", "\n", "\t\t\t\t", 
                RowBox[{"2", \(m\_3\%2\), 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[\(p\^2\),
                        PP], "-", \(m\_3\%2\)}], ")"}], 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{\((d - 3)\), 
                        RowBox[{"(", 
                          RowBox[{
                            InterpretationBox[\(p\^2\),
                              PP], "-", \(5  m\_2\%2\), "+", \(m\_3\%2\)}], 
                          ")"}]}], "+", \(\((7  d - 25)\) m\_1\%2\)}], ")"}], 
                  
                  InterpretationBox[
                    StyleBox[\(3\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 3, "+"],
                    Editable->True]}], "-", "\n", "\t\t\t\t", 
                RowBox[{\((d - 3)\), \((d - 4)\), 
                  SuperscriptBox[
                    RowBox[{"(", 
                      InterpretationBox[\(p\^2\),
                        PP], ")"}], "2"]}], "+", 
                RowBox[{
                  InterpretationBox[\(p\^2\),
                    
                    PP], \((\((7  d - 30)\) \((d - 3)\) 
                        t\_1 - \((7  d - 31)\) \((3  d - 10)\) m\_1\%2)\)}], 
                "+", \(\(1\/4\) \((d - 
                      5)\) \((\((17  d - 66)\) t\_1\^2 - \((3  d - 14)\) 
                        t\_2 - 
                      4 \((3  d - 10)\) \((\((m\_1\%2)\)\^2 + 
                            5 \( m\_2\%2\) m\_3\%2)\))\)\), "+", "\t", 
                RowBox[{\(2\/\(d - 4\)\), 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              RowBox[{"(", 
                                RowBox[{
                                  InterpretationBox[\(p\^2\),
                                    PP], "+", \(m\_2\%2\)}], 
                                ")"}], \((7  d - 24)\)}], 
                            "+", \(\((d - 4)\) m\_1\%2\), 
                            "-", \(\((5  d - 18)\) m\_3\%2\)}], 
                          ")"}], \(m\_1\%2\), \(m\_2\%2\), 
                        InterpretationBox[
                          StyleBox[\(1\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 1, "+"],
                          Editable->True], 
                        InterpretationBox[
                          StyleBox[\(2\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 2, "+"],
                          Editable->True], 
                        InterpretationBox[
                          StyleBox[\(3\^-\),
                            FontWeight->"Bold"],
                          Schiebe[ 3, "-"],
                          Editable->True]}], "+", "\n", "\t\t\t\t\t\t", 
                      RowBox[{
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              RowBox[{"(", 
                                RowBox[{
                                  InterpretationBox[\(p\^2\),
                                    PP], "+", \(m\_3\%2\)}], 
                                ")"}], \((7  d - 24)\)}], 
                            "+", \(\((d - 4)\) m\_1\%2\), 
                            "-", \(\((5  d - 18)\) m\_2\%2\)}], 
                          ")"}], \(m\_1\%2\), \(m\_3\%2\), 
                        InterpretationBox[
                          StyleBox[\(1\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 1, "+"],
                          Editable->True], 
                        InterpretationBox[
                          StyleBox[\(2\^-\),
                            FontWeight->"Bold"],
                          Schiebe[ 2, "-"],
                          Editable->True], 
                        InterpretationBox[
                          StyleBox[\(3\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 3, "+"],
                          Editable->True]}], "-", "\n", "\t\t\t\t\t\t", 
                      RowBox[{"2", 
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              RowBox[{"(", 
                                RowBox[{
                                  InterpretationBox[\(p\^2\),
                                    PP], "+", \(m\_2\%2\), "+", \(m\_3\%2\)}],
                                 ")"}], \((d - 3)\)}], 
                            "+", \(\((d - 4)\) m\_1\%2\)}], 
                          ")"}], \(m\_2\%2\), \(m\_3\%2\), 
                        InterpretationBox[
                          StyleBox[\(1\^-\),
                            FontWeight->"Bold"],
                          Schiebe[ 1, "-"],
                          Editable->True], 
                        InterpretationBox[
                          StyleBox[\(2\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 2, "+"],
                          Editable->True], 
                        InterpretationBox[
                          StyleBox[\(3\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 3, "+"],
                          Editable->True]}]}], ")"}]}]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox["1",
                    "TraditionalForm"], 
                  FormBox["1",
                    "TraditionalForm"], 
                  FormBox["1",
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox[\(d - 2\),
                    "TraditionalForm"], ")"}]],
              TJI[ 
                Plus[ -2, d], {1, 1, 1}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_90 = OperatorApplyJ[e\_90]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (90)", "Subsection",
  CellTags->"T1.6.73.1"],

Cell[BoxData[
    \(\(h\_90 = s\_90;\)\)], "Input"],

Cell[BoxData[
    \(f\_90 = 
      MakeFun[h\_90, "\<(90)\>", 
        IFF[PP =!= 0 && MatchQ[d, _Symbol + _Integer?Positive]]]\)], "Input"],

Cell["CheckTJIRecursion not applicable.", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (91)", "Subsection",
  CellTags->"T1.6.74.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_91.1\), "=", " ", 
        RowBox[{
          RowBox[{\((d - 2)\), \(\[Nu]\_1\), 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["K",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TKI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{\(-u\_\(1, 2, 3\)\), "-", 
                InterpretationBox[
                  StyleBox[\(1\^-\),
                    FontWeight->"Bold"],
                  Schiebe[ 1, "-"],
                  Editable->True], "+", 
                InterpretationBox[
                  StyleBox[\(2\^-\),
                    FontWeight->"Bold"],
                  Schiebe[ 2, "-"],
                  Editable->True], "+", 
                InterpretationBox[
                  StyleBox[\(3\^-\),
                    FontWeight->"Bold"],
                  Schiebe[ 3, "-"],
                  Editable->True]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["K",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox[\(d - 2\),
                    "TraditionalForm"], ")"}]],
              TKI[ 
                Plus[ -2, d], {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_91.1 = OperatorApplyK[e\_91.1]\)], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_91.2\), "=", 
        RowBox[{
          RowBox[{\((d - 2)\), \(\[Nu]\_2\), 
            InterpretationBox[
              StyleBox[\(2\^+\),
                FontWeight->"Bold"],
              Schiebe[ 2, "+"],
              Editable->True], \(\[Nu]\_3\), 
            InterpretationBox[
              StyleBox[\(3\^+\),
                FontWeight->"Bold"],
              Schiebe[ 3, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["K",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TKI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{\(-2\), \(m\_1\%2\), \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True]}], "+", \((d - 2 - 2  \[Nu]\_1)\)}], 
              ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["K",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox[\(d - 2\),
                    "TraditionalForm"], ")"}]],
              TKI[ 
                Plus[ -2, d], {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_91.2 = OperatorApplyK[e\_91.2]\)], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_91.3\), "=", 
        RowBox[{
          RowBox[{\((d - 2)\), \((d - \[Nu]\_1 - \[Nu]\_2 - \[Nu]\_3)\), 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["K",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TKI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"-", 
              RowBox[{"(", 
                RowBox[{\(\[CapitalDelta]\_\(1, 2, 3\)\), "+", 
                  RowBox[{\(u\_\(1, 2, 3\)\), 
                    InterpretationBox[
                      StyleBox[\(1\^-\),
                        FontWeight->"Bold"],
                      Schiebe[ 1, "-"],
                      Editable->True]}], "+", 
                  RowBox[{\(u\_\(2, 1, 3\)\), 
                    InterpretationBox[
                      StyleBox[\(2\^-\),
                        FontWeight->"Bold"],
                      Schiebe[ 2, "-"],
                      Editable->True]}], "+", 
                  RowBox[{\(u\_\(3, 1, 2\)\), 
                    InterpretationBox[
                      StyleBox[\(3\^-\),
                        FontWeight->"Bold"],
                      Schiebe[ 3, "-"],
                      Editable->True]}]}], ")"}]}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["K",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox[\(d - 2\),
                    "TraditionalForm"], ")"}]],
              TKI[ 
                Plus[ -2, d], {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_91.3 = OperatorApplyK[e\_91.3]\)], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_91.4\), "=", 
        RowBox[{
          RowBox[{\(\[CapitalDelta]\_\(1, 2, 3\)\), \(\[Nu]\_1\), 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["K",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TKI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", 
              
              RowBox[{\(\(u\_\(1, 2, 3\)\) \((d - \[Nu]\_1 - 2  \[Nu]\_2)\)\),
                 "+", \(2 \( m\_2\%2\) \((\[Nu]\_1 - \[Nu]\_2)\)\), "+", 
                RowBox[{\(u\_\(3, 1, 2\)\), \(\[Nu]\_1\), 
                  InterpretationBox[
                    StyleBox[\(1\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "+"],
                    Editable->True], 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(2\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 2, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(3\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 3, "-"],
                        Editable->True]}], ")"}]}], "+", 
                RowBox[{"2", \(m\_2\%2\), \(\[Nu]\_2\), 
                  InterpretationBox[
                    StyleBox[\(2\^+\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "+"],
                    Editable->True], 
                  RowBox[{"(", 
                    RowBox[{
                      InterpretationBox[
                        StyleBox[\(1\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 1, "-"],
                        Editable->True], "-", 
                      InterpretationBox[
                        StyleBox[\(3\^-\),
                          FontWeight->"Bold"],
                        Schiebe[ 3, "-"],
                        Editable->True]}], ")"}]}]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["K",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TKI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_91.4 = OperatorApplyK[e\_91.4]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (911)", "Subsection",
  CellTags->"T1.6.75.1"],

Cell[BoxData[
    \(h\_911 = s\_91.1 /. \[Nu]\_1 -> \[Nu]\_1 - 1\)], "Input"],

Cell[BoxData[
    \(f\_911 = 
      MakeFun[h\_911, "\<(911)\>", 
        IFF[\[Nu]\_1 > 1 && 
            MatchQ[d, _Symbol + _Integer?Positive]]]\)], "Input"],

Cell["CheckTJIRecursion not applicable.", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (912)", "Subsection",
  CellTags->"T1.6.76.1"],

Cell[BoxData[
    \(h\_912 = \(s\_91.2 /. \[Nu]\_2 -> \[Nu]\_2 - 
              1\) /. \[Nu]\_3 -> \[Nu]\_3 - 1\)], "Input"],

Cell[BoxData[
    \(f\_912 = 
      MakeFun[h\_912, "\<(912)\>", 
        IFF[\[Nu]\_2 > 1 && \[Nu]\_3 > 1 && 
            MatchQ[d, _Symbol + _Integer?Positive]]]\)], "Input"],

Cell["CheckTJIRecursion not applicable.", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (913)", "Subsection",
  CellTags->"T1.6.77.1"],

Cell[BoxData[
    \(h\_913 = s\_91.3\)], "Input"],

Cell[BoxData[
    \(f\_913 = 
      MakeFun[h\_913, "\<(913)\>", 
        IFF[\((n2 === 1\  || \ n3 === 1)\) && 
            MatchQ[d, _Symbol + _Integer?Positive]], n1 :> 1]\)], "Input"],

Cell["CheckTJIRecursion not applicable.", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (914)", "Subsection",
  CellTags->"T1.6.78.1"],

Cell[BoxData[
    \(h\_914 = s\_91.4 /. \[Nu]\_1 -> \[Nu]\_1 - 1\)], "Input"],

Cell[BoxData[
    \(f\_914 = 
      MakeFun[h\_914, "\<(914)\>", 
        IFF[\[CapitalDelta]\_\(1, 2, 3\) =!= 0 && \((\[Nu]\_1\  > \ 1\ )\) && 
            MatchQ[d, _Symbol]]]\)], "Input"],

Cell["CheckTJIRecursion not applicable.", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (92)", "Subsection",
  CellTags->"T1.6.79.1"],

Cell[TextData[{
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalDelta]\_\(1, 2, 3\)\)]],
  " = Cayley[1,2,3][{m1,m2,m3}]"
}], "Text"],

Cell[BoxData[
    \(\(Cayley[1, 2, 3]\)[{m1, m2, m3}]\)], "Input"],

Cell[TextData[Cell[BoxData[
    \(TraditionalForm\`\((\ 
        m\_1\%2 == \((m\_2 + m\_3)\)\^2\ )\)\ \[DoubleLongRightArrow]\ \((\ \
\[CapitalDelta]\_\(1, 2, 3\) == 0\ )\)\)]]], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_92\), "=", 
        RowBox[{
          RowBox[{
          "2", \(m\_2\), \(m\_3\), \((m\_2 + m\_3)\), \((d - 2  \[Nu]\_1 - 
                2  \[Nu]\_2 - 2  \[Nu]\_3 - 1)\), 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["K",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TKI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{\(m\_2\), 
                  RowBox[{"(", 
                    RowBox[{
                      
                      RowBox[{\((d - \[Nu]\_1 - \[Nu]\_2 - 2  \[Nu]\_3 - 1)\),
                         
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              InterpretationBox[
                                StyleBox[\(1\^+\),
                                  FontWeight->"Bold"],
                                Schiebe[ 1, "+"],
                                Editable->True], 
                              InterpretationBox[
                                StyleBox[\(2\^-\),
                                  FontWeight->"Bold"],
                                Schiebe[ 2, "-"],
                                Editable->True]}], "-", "1"}], ")"}]}], "+", 
                      RowBox[{\((\[Nu]\_1 - \[Nu]\_2 + 1)\), 
                        InterpretationBox[
                          StyleBox[\(1\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 1, "+"],
                          Editable->True], 
                        InterpretationBox[
                          StyleBox[\(3\^-\),
                            FontWeight->"Bold"],
                          Schiebe[ 3, "-"],
                          Editable->True]}]}], ")"}]}], "+", "\n", "\t\t\t", 
                RowBox[{\(m\_3\), 
                  RowBox[{"(", 
                    RowBox[{
                      
                      RowBox[{\((d - \[Nu]\_1 - 2  \[Nu]\_2 - \[Nu]\_3 - 1)\),
                         
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              InterpretationBox[
                                StyleBox[\(1\^+\),
                                  FontWeight->"Bold"],
                                Schiebe[ 1, "+"],
                                Editable->True], 
                              InterpretationBox[
                                StyleBox[\(3\^-\),
                                  FontWeight->"Bold"],
                                Schiebe[ 3, "-"],
                                Editable->True]}], "-", "1"}], ")"}]}], "+", 
                      RowBox[{\((\[Nu]\_1 - \[Nu]\_3 + 1)\), 
                        InterpretationBox[
                          StyleBox[\(1\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 1, "+"],
                          Editable->True], 
                        InterpretationBox[
                          StyleBox[\(2\^-\),
                            FontWeight->"Bold"],
                          Schiebe[ 2, "-"],
                          Editable->True]}]}], ")"}]}]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["K",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TKI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_92 = OperatorApplyK[e\_92]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (93)", "Subsection",
  CellTags->"T1.6.80.1"],

Cell[TextData[{
  "( ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(\((m\_1 == 0)\)\  && \ \((m\_2 == 
                m\_3)\)\)\(\ \)\(\ \)\()\)\)\ \[DoubleLongRightArrow]\ \((\ \
\[CapitalDelta]\_\(1, 2, 3\) == 0\ )\)\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_93\), "=", 
        RowBox[{
          RowBox[{
          "2", \(m\_2\%2\), \((d - 2  \[Nu]\_1 - 2)\), \((d - 
                2  \[Nu]\_1 - \[Nu]\_2 - \[Nu]\_3 - 1)\), \((d - 
                2  \[Nu]\_1 - \[Nu]\_2 - \[Nu]\_3)\), 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["K",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TKI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}], "==", 
          
          RowBox[{\((d - 2  \[Nu]\_1 - 2  \[Nu]\_3)\), \((d - 2  \[Nu]\_1 - 
                2  \[Nu]\_2)\), \((d - \[Nu]\_1 - \[Nu]\_2 - \[Nu]\_3)\), 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["K",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TKI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_93 = OperatorApplyK[e\_93]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (93)", "Subsection",
  CellTags->"T1.6.81.1"],

Cell[BoxData[
    \(h\_93 = s\_93 /. \[Nu]\_1 -> \[Nu]\_1 - 1\)], "Input"],

Cell[BoxData[
    \(f\_93 = 
      MakeFun[h\_93, "\<(93)\>", 
        IFF[\[Nu]\_1 > 1 && m\_2 =!= 0], {m1 :> 0, m3 :> m2}]\)], "Input"],

Cell["CheckTJIRecursion not applicable.", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (94)", "Subsection",
  CellTags->"T1.6.82.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_94\), "=", 
        RowBox[{
          RowBox[{\(\[CapitalDelta]\_\(1, 2, 6\)\), \(\[Nu]\_1\), 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["B",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TBI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{\(u\_\(6, 1, 2\)\), 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{\(\[Nu]\_1\), 
                        InterpretationBox[
                          StyleBox[\(1\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 1, "+"],
                          Editable->True], 
                        InterpretationBox[
                          StyleBox[\(2\^-\),
                            FontWeight->"Bold"],
                          Schiebe[ 2, "-"],
                          Editable->True]}], "-", "d", "+", \(\[Nu]\_1\), 
                      "+", \(2  \[Nu]\_2\)}], ")"}]}], "+", 
                RowBox[{"2", \(m\_2\%2\), 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{\(\[Nu]\_2\), 
                        InterpretationBox[
                          StyleBox[\(2\^+\),
                            FontWeight->"Bold"],
                          Schiebe[ 2, "+"],
                          Editable->True], 
                        InterpretationBox[
                          StyleBox[\(1\^-\),
                            FontWeight->"Bold"],
                          Schiebe[ 1, "-"],
                          Editable->True]}], "-", "d", "+", \(2  \[Nu]\_1\), 
                      "+", \(\[Nu]\_2\)}], ")"}]}]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["B",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TBI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_94 = OperatorApplyB[e\_94]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (94)", "Subsection",
  CellTags->"T1.6.83.1"],

Cell[BoxData[
    \(h\_94 = s\_94 /. \[Nu]\_1 -> \[Nu]\_1 - 1\)], "Input"],

Cell[BoxData[
    \(f\_94 = 
      MakeFun[h\_94, "\<(94)\>", 
        IFF[\[CapitalDelta]\_\(1, 2, 6\) =!= 0 && \[Nu]\_1 > 1]]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (9401)", "Subsection",
  CellTags->"T1.6.84.1"],

Cell[BoxData[
    \(n\_9401 = \(s\_94 /. \[CapitalDelta]\_\(1, 2, 6\) -> 0\) /. 
        u\_\(6, 1, 2\) -> 0\)], "Input"],

Cell[BoxData[
    \(h\_9401 = \(-n\_9401[\([2, 2]\)]\) == n\_9401[\([2, 1]\)]\)], "Input"],

Cell[BoxData[
    \(f\_9401 = 
      MakeFun[h\_9401, "\<(9401)\>", 
        IFF[\[CapitalDelta]\_\(1, 2, 6\) === 0 && u\_\(6, 1, 2\) === 0 && 
            m2 =!= 0]]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (9402)", "Subsection",
  CellTags->"T1.6.85.1"],

Cell[BoxData[
    \(n\_9402 = 
      Interchange[\(s\_94 /. \[CapitalDelta]\_\(1, 2, 6\) -> 0\) /. 
          u\_\(6, 1, 2\) -> 0, 1 \[LeftRightArrow] 2]\)], "Input"],

Cell[BoxData[
    \(h\_9402 = \(-n\_9402[\([2, 1]\)]\) == n\_9402[\([2, 2]\)]\)], "Input"],

Cell[BoxData[
    \(f\_9402 = 
      MakeFun[h\_9402, "\<(9402)\>", 
        IFF[\[CapitalDelta]\_\(1, 2, 6\) === 0 && u\_\(6, 1, 2\) === 0 && 
            m1 =!= 0]]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (95)", "Subsection",
  CellTags->"T1.6.86.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_95\), "=", 
        RowBox[{
          RowBox[{"2", 
            InterpretationBox[\(p\^2\),
              PP], \(\[Nu]\_1\), 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["B",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TBI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{\(u\_\(1, 2, 6\)\), "+", 
                InterpretationBox[
                  StyleBox[\(1\^-\),
                    FontWeight->"Bold"],
                  Schiebe[ 1, "-"],
                  Editable->True], "-", 
                InterpretationBox[
                  StyleBox[\(2\^-\),
                    FontWeight->"Bold"],
                  Schiebe[ 2, "-"],
                  Editable->True]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["B",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox[\(d - 2\),
                    "TraditionalForm"], ")"}]],
              TBI[ 
                Plus[ -2, d], {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_95 = OperatorApplyB[e\_95]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (95)", "Subsection",
  CellTags->"T1.6.87.1"],

Cell[BoxData[
    \(h\_95 = s\_95 /. \[Nu]\_1 -> \[Nu]\_1 - 1\)], "Input"],

Cell[BoxData[
    \(f\_95 = 
      MakeFun[h\_95, "\<(95)\>", 
        IFF[\[Nu]\_1 > 1\  && \ 
            MatchQ[d, _Symbol + _Integer?Positive]]]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (96)", "Subsection",
  CellTags->"T1.6.88.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_96\), "=", 
        RowBox[{
          RowBox[{"2", 
            InterpretationBox[\(p\^2\),
              PP], \((d - \[Nu]\_1 - \[Nu]\_2 - 1)\), 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["B",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TBI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2]}],
              Editable->True]}], "==", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{\(\[CapitalDelta]\_\(1, 2, 6\)\), "+", 
                RowBox[{\(u\_\(1, 2, 6\)\), 
                  InterpretationBox[
                    StyleBox[\(1\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 1, "-"],
                    Editable->True]}], "+", 
                RowBox[{\(u\_\(2, 1, 6\)\), 
                  InterpretationBox[
                    StyleBox[\(2\^-\),
                      FontWeight->"Bold"],
                    Schiebe[ 2, "-"],
                    Editable->True]}]}], ")"}], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["B",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox[\(d - 2\),
                    "TraditionalForm"], ")"}]],
              TBI[ 
                Plus[ -2, d], {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_96 = OperatorApplyB[e\_96]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (96)", "Subsection",
  CellTags->"T1.6.89.1"],

Cell[BoxData[
    \(h\_96 = s\_96\)], "Input"],

Cell[BoxData[
    \(f\_96 = 
      MakeFun[h\_96, "\<(96)\>", 
        IFF[\ MatchQ[d, _Symbol + _Integer\ ?Positive]], n1 :> 1]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (97)", "Subsection",
  CellTags->"T1.6.90.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_97\), "=", 
        RowBox[{
          RowBox[{"2", \(m\_1\%2\), \(\[Nu]\_1\), 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["A",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                FormBox[\(\[Nu]\_1\),
                  "TraditionalForm"], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TAI[ d, {
                Subscript[ \[Nu], 1]}],
              Editable->True]}], "==", 
          RowBox[{\((d - 2  \[Nu]\_1)\), 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["A",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                FormBox[\(\[Nu]\_1\),
                  "TraditionalForm"], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TAI[ d, {
                Subscript[ \[Nu], 1]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_97 = OperatorApplyA[e\_97]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (97)", "Subsection",
  CellTags->"T1.6.91.1"],

Cell[BoxData[
    \(h\_97 = s\_97 /. \[Nu]\_1 -> \[Nu]\_1 - 1\)], "Input"],

Cell[BoxData[
    \(f\_97 = 
      MakeFun[h\_97, "\<(97)\>", 
        IFF[\[Nu]\_1 > 1 && \ MatchQ[d, _Symbol\ ]]]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (98)", "Subsection",
  CellTags->"T1.6.92.1"],

Cell[TextData[{
  "general ",
  Cell[BoxData[
      \(m\_i\%2\)]]
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(e\_98\), "=", 
        RowBox[{
          RowBox[{\((d - 2  \[Nu]\_1)\), 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["A",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                FormBox[\(\[Nu]\_1\),
                  "TraditionalForm"], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TAI[ d, {
                Subscript[ \[Nu], 1]}],
              Editable->True]}], "==", 
          RowBox[{\(-2\), \(m\_1\%2\), 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["A",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                FormBox[\(\[Nu]\_1\),
                  "TraditionalForm"], 
                RowBox[{"(", 
                  FormBox[\(d - 2\),
                    "TraditionalForm"], ")"}]],
              TAI[ 
                Plus[ -2, d], {
                Subscript[ \[Nu], 1]}],
              Editable->True]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(s\_98 = OperatorApplyA[e\_98]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (98)", "Subsection",
  CellTags->"T1.6.93.1"],

Cell[BoxData[
    \(h\_98 = s\_98\)], "Input"],

Cell[BoxData[
    \(f\_98 = 
      MakeFun[h\_98, "\<(98)\>", 
        IFF[MatchQ[d, _Symbol\  + _Integer?Positive]]]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (99)", "Subsection",
  CellTags->"T1.6.94.1"],

Cell[TextData[Cell[BoxData[
    \(TraditionalForm\`m\_1 = M, \ p\^2 = M\^2, 
    m\_2 = \(m\_3 = 0\)\)]]], "Text"],

Cell[TextData[{
  "Herleitung in three.nb ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          
          RowBox[{\(-\[Nu]\_1\), " ", \((d - 2\ \[Nu]\_2 - 2\ \[Nu]\_3)\), 
            " ", \((d - \[Nu]\_2 - \[Nu]\_3 - 1)\), " ", 
            SubsuperscriptBox[
              StyleBox["J",
                SingleLetterItalics->False,
                FontWeight->"Bold"], 
              RowBox[{
                StyleBox[
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"],
                  FontColor->RGBColor[0, 0, 1]], 
                FormBox[\(\[Nu]\_2\),
                  "TraditionalForm"], 
                FormBox[\(\[Nu]\_3\),
                  "TraditionalForm"]}], 
              RowBox[{"(", 
                FormBox["d",
                  "TraditionalForm"], ")"}]]}], "==", 
          
          RowBox[{\(\[Nu]\_2\), " ", \((\(-d\) + 2\ \[Nu]\_2 + 2)\), 
            " ", \((\(-2\)\ d + \[Nu]\_1 + 2\ \[Nu]\_2 + 2\ \[Nu]\_3 + 2)\), 
            " ", 
            SubsuperscriptBox[
              StyleBox["J",
                SingleLetterItalics->False,
                FontWeight->"Bold"], 
              RowBox[{
                FormBox[\(\[Nu]\_1\),
                  "TraditionalForm"], 
                StyleBox[
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"],
                  FontColor->RGBColor[0, 0, 1]], 
                FormBox[\(\[Nu]\_3\),
                  "TraditionalForm"]}], 
              RowBox[{"(", 
                FormBox["d",
                  "TraditionalForm"], ")"}]]}]}], TraditionalForm]]]
}], "Text"],

Cell[BoxData[
    RowBox[{\(e\_99\), "=", 
      RowBox[{
        RowBox[{\(\[Nu]\_1\), " ", \((d - 2\ \[Nu]\_2 - 2\ \[Nu]\_3)\), 
          " ", \((d - \[Nu]\_2 - \[Nu]\_3 - 1)\), " ", 
          InterpretationBox[
            StyleBox[\(1\^+\),
              FontWeight->"Bold"],
            Schiebe[ 1, "+"],
            Editable->True], 
          InterpretationBox[
            SubsuperscriptBox[
              StyleBox["J",
                SingleLetterItalics->False,
                FontWeight->"Bold"], 
              RowBox[{
                FormBox[\(\[Nu]\_1\),
                  "TraditionalForm"], 
                FormBox[\(\[Nu]\_2\),
                  "TraditionalForm"], 
                FormBox[\(\[Nu]\_3\),
                  "TraditionalForm"]}], 
              RowBox[{"(", 
                FormBox["d",
                  "TraditionalForm"], ")"}]],
            TJI[ d, {
              Subscript[ \[Nu], 1], 
              Subscript[ \[Nu], 2], 
              Subscript[ \[Nu], 3]}],
            Editable->True]}], "==", 
        RowBox[{\(-\[Nu]\_2\), " ", \((\(-d\) + 2\ \[Nu]\_2 + 2)\), 
          " ", \((\(-2\)\ d + \[Nu]\_1 + 2\ \[Nu]\_2 + 2\ \[Nu]\_3 + 2)\), 
          InterpretationBox[
            StyleBox[\(2\^+\),
              FontWeight->"Bold"],
            Schiebe[ 2, "+"],
            Editable->True], 
          InterpretationBox[
            SubsuperscriptBox[
              StyleBox["J",
                SingleLetterItalics->False,
                FontWeight->"Bold"], 
              RowBox[{
                FormBox[\(\[Nu]\_1\),
                  "TraditionalForm"], 
                FormBox[\(\[Nu]\_2\),
                  "TraditionalForm"], 
                FormBox[\(\[Nu]\_3\),
                  "TraditionalForm"]}], 
              RowBox[{"(", 
                FormBox["d",
                  "TraditionalForm"], ")"}]],
            TJI[ d, {
              Subscript[ \[Nu], 1], 
              Subscript[ \[Nu], 2], 
              Subscript[ \[Nu], 3]}],
            Editable->True]}]}]}]], "Input"],

Cell[BoxData[
    \(s\_99 = OperatorApplyJ[e\_99]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (99)", "Subsection",
  CellTags->"T1.6.95.1"],

Cell[BoxData[
    \(h\_99 = s\_99 /. \[Nu]\_1 -> \[Nu]\_1 - 1\)], "Input"],

Cell[BoxData[
    \(f\_99 = 
      MakeFun[h\_99, "\<(99)\>", 
        IFF[m1^2 === PP && \[Nu]\_1 > 1], {m2 :> 0, m3 :> 0}]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (9912)", "Subsection",
  CellTags->"T1.6.96.1"],

Cell[BoxData[
    \(h\_9912 = 
      Interchange[s\_99 /. \[Nu]\_1 -> \[Nu]\_1 - 1, 
        1 \[LeftRightArrow] 2]\)], "Input"],

Cell[BoxData[
    \(f\_9912 = 
      MakeFun[h\_9912, "\<(9912)\>", 
        IFF[m2^2 === PP && \[Nu]\_2 > 1], {m1 :> 0, m3 :> 0}]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (9913)", "Subsection",
  CellTags->"T1.6.97.1"],

Cell[BoxData[
    \(h\_9913 = 
      Interchange[s\_99 /. \[Nu]\_1 -> \[Nu]\_1 - 1, 
        1 \[LeftRightArrow] 3]\)], "Input"],

Cell[BoxData[
    \(f\_9913 = 
      MakeFun[h\_9913, "\<(9913)\>", 
        IFF[m3^2 === PP && \[Nu]\_3 > 1], {m1 :> 0, m2 :> 0}]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (100)", "Subsection",
  CellTags->"T1.6.98.1"],

Cell["Herleitung in three.nb ", "Text"],

Cell[TextData[Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
        "2", " ", \(m\_2\%2\), " ", \((1 - \[Nu]\_2)\), " ", \(\[Nu]\_2\), 
          " ", 
          InterpretationBox[
            SubsuperscriptBox[
              StyleBox["J",
                SingleLetterItalics->False,
                FontWeight->"Bold"], 
              RowBox[{
                FormBox[\(\[Nu]\_1\),
                  "TraditionalForm"], 
                StyleBox[
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"],
                  FontColor->RGBColor[0, 0, 1]], 
                FormBox[\(\[Nu]\_3\),
                  "TraditionalForm"]}], 
              RowBox[{"(", 
                FormBox["d",
                  "TraditionalForm"], ")"}]],
            TJI[ d, {
              Subscript[ \[Nu], 1], 
              Plus[ 1, 
                Subscript[ \[Nu], 2]], 
              Subscript[ \[Nu], 3]}],
            Editable->True]}], "==", 
        RowBox[{
          RowBox[{\((d - 2\ \[Nu]\_2)\), " ", \((1 - \[Nu]\_2)\), " ", 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TJI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}], "-", 
          RowBox[{\(\[Nu]\_1\), " ", \((\(-d\) + 2\ \[Nu]\_1 + 2)\), " ", 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  StyleBox[
                    FormBox[\(\[Nu]\_1\),
                      "TraditionalForm"],
                    FontColor->RGBColor[0, 0, 1]], 
                  StyleBox[
                    FormBox[\(\[Nu]\_2\),
                      "TraditionalForm"],
                    FontColor->RGBColor[1, 0, 0]], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TJI[ d, {
                Plus[ 1, 
                  Subscript[ \[Nu], 1]], 
                Plus[ -1, 
                  Subscript[ \[Nu], 2]], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}]}]}], TraditionalForm]], "Input"]], "Text"],

Cell[TextData[Cell[BoxData[
    \(TraditionalForm\`m\_1 = 0\)]]], "Text"],

Cell[BoxData[
    RowBox[{\(e\_100\), "=", 
      RowBox[{
        RowBox[{
        "2", " ", \(m\_2\%2\), " ", \((1 - \[Nu]\_2)\), " ", \(\[Nu]\_2\), 
          " ", 
          InterpretationBox[
            StyleBox[\(2\^+\),
              FontWeight->"Bold"],
            Schiebe[ 2, "+"],
            Editable->True], 
          InterpretationBox[
            SubsuperscriptBox[
              StyleBox["J",
                SingleLetterItalics->False,
                FontWeight->"Bold"], 
              RowBox[{
                FormBox[\(\[Nu]\_1\),
                  "TraditionalForm"], 
                FormBox[\(\[Nu]\_2\),
                  "TraditionalForm"], 
                FormBox[\(\[Nu]\_3\),
                  "TraditionalForm"]}], 
              RowBox[{"(", 
                FormBox["d",
                  "TraditionalForm"], ")"}]],
            TJI[ d, {
              Subscript[ \[Nu], 1], 
              Subscript[ \[Nu], 2], 
              Subscript[ \[Nu], 3]}],
            Editable->True]}], "==", 
        RowBox[{
          RowBox[{\((d - 2\ \[Nu]\_2)\), " ", \((1 - \[Nu]\_2)\), " ", 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TJI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}], "-", 
          RowBox[{\(\[Nu]\_1\), " ", \((\(-d\) + 2\ \[Nu]\_1 + 2)\), " ", 
            InterpretationBox[
              StyleBox[\(1\^+\),
                FontWeight->"Bold"],
              Schiebe[ 1, "+"],
              Editable->True], 
            InterpretationBox[
              StyleBox[\(2\^-\),
                FontWeight->"Bold"],
              Schiebe[ 2, "-"],
              Editable->True], 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\(\[Nu]\_1\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TJI[ d, {
                Subscript[ \[Nu], 1], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}]}]}]}]], "Input"],

Cell[BoxData[
    \(s\_100 = OperatorApplyJ[e\_100]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (100)", "Subsection",
  CellTags->"T1.6.99.1"],

Cell[BoxData[
    \(h\_100 = s\_100\)], "Input"],

Cell[BoxData[
    \(f\_100 = 
      MakeFun[h\_100 /. \[Nu]\_2 -> \[Nu]\_2 - 1, "\<(100)\>", 
        IFF[\[Nu]\_2 > 2 && m2 =!= 0], m1 :> 0]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTJIRecursion[f\_100]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`"Off"\)], "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (1002)", "Subsection",
  CellTags->"T1.6.100.1"],

Cell[BoxData[
    \(h\_1002 = Interchange[s\_100, 1 \[LeftRightArrow] 3]\)], "Input"],

Cell[BoxData[
    \(f\_1002 = 
      MakeFun[h\_1002 /. \[Nu]\_2 -> \[Nu]\_2 - 1, "\<(1002)\>", 
        IFF[\[Nu]\_2 > 2 && m2 =!= 0], m3 :> 0]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTJIRecursion[f\_1002]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`"Off"\)], "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (1003)", "Subsection",
  CellTags->"T1.6.101.1"],

Cell[BoxData[
    \(h\_1003 = 
      Interchange[s\_100, 1 \[LeftRightArrow] 3, 
        1 \[LeftRightArrow] 2]\)], "Input"],

Cell[BoxData[
    \(f\_1003 = 
      MakeFun[h\_1003 /. \[Nu]\_1 -> \[Nu]\_1 - 1, "\<(1003)\>", 
        IFF[\[Nu]\_1 > 2 && m1 =!= 0], m3 :> 0]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTJIRecursion[f\_1003]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`"Off"\)], "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (1004)", "Subsection",
  CellTags->"T1.6.102.1"],

Cell[BoxData[
    \(h\_1004 = Interchange[s\_100, 1 \[LeftRightArrow] 2]\)], "Input"],

Cell[BoxData[
    \(f\_1004 = 
      MakeFun[h\_1004 /. \[Nu]\_1 -> \[Nu]\_1 - 1, "\<(1004)\>", 
        IFF[\[Nu]\_1 > 2 && m1 =!= 0], m2 :> 0]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTJIRecursion[f\_1004]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`"Off"\)], "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (1005)", "Subsection",
  CellTags->"T1.6.103.1"],

Cell[BoxData[
    \(h\_1005 = Interchange[s\_100, 2 \[LeftRightArrow] 3]\)], "Input"],

Cell[BoxData[
    \(f\_1005 = 
      MakeFun[h\_1005 /. \[Nu]\_3 -> \[Nu]\_3 - 1, "\<(1005)\>", 
        IFF[\[Nu]\_3 > 2 && m3 =!= 0], m1 :> 0]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTJIRecursion[f\_1005]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`"Off"\)], "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (1006)", "Subsection",
  CellTags->"T1.6.104.1"],

Cell[BoxData[
    \(h\_1006 = 
      Interchange[s\_100, 2 \[LeftRightArrow] 3, 
        1 \[LeftRightArrow] 2]\)], "Input"],

Cell[BoxData[
    \(f\_1006 = 
      MakeFun[h\_1006 /. \[Nu]\_3 -> \[Nu]\_3 - 1, "\<(1006)\>", 
        IFF[\[Nu]\_3 > 2 && m3 =!= 0], m2 :> 0]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CheckTJIRecursion[f\_1006]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`"Off"\)], "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (150)", "Subsection",
  CellTags->"T1.6.105.1"],

Cell[BoxData[
    \(TextForm\`\[Nu]\_4 = \(-1\)\)], "Text"],

Cell[BoxData[
    \(TextForm\`\((\(q\_2\)\(-\)\(p\)\(\ \))\)\^2 - m\_4\%2 = \ q\_2\%2 - 
      m\_2\%2 - 2  p\ q\_2 + p\^2 + m\_2\%2 - m\_4\%2\)], "Text"],

Cell[BoxData[
    \(s\_150 = 
      TFI[d, PP, {{n1, m1}, {n2, m2}, {n3, m3}, {\(-1\), m4}, {n5, m5}}] == 
        TFI[d, PP, {{n1, m1}, {n2 - 1, m2}, {n3, m3}, {0, 0}, {n5, m5}}] - 
          2  TFI[d, 
              PP, {0, 0, 0, 1, 
                0}, {{n1, m1}, {n2, m2}, {n3, m3}, {0, 0}, {n5, 
                  m5}}] + \((PP + m2^2 - m4^2)\) 
            TFI[d, PP, {{n1, m1}, {n2, m2}, {n3, m3}, {0, 0}, {n5, 
                  m5}}]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (150)", "Subsection",
  CellTags->"T1.6.106.1"],

Cell[BoxData[
    \(h\_150 = s\_150\)], "Input"],

Cell[BoxData[
    \(f\_150 = MakeFun[h\_150, "\<(150)\>"]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (151)", "Subsection",
  CellTags->"T1.6.107.1"],

Cell[BoxData[
    \(TextForm\`\[Nu]\_4 = \(-1\)\)], "Text"],

Cell[BoxData[
    \(TextForm\`\((\(q\_2\)\(-\)\(p\)\(\ \))\)\^2 - m\_4\%2 = \ q\_2\%2 - 
      m\_2\%2 - 2  p\ q\_2 + p\^2 + m\_2\%2 - m\_4\%2\)], "Text"],

Cell[BoxData[
    \(s\_151 = 
      TFI[d, PP, 
          DP, {a, b}, {{n1, m1}, {n2, m2}, {n3, m3}, {\(-1\), m4}, {n5, 
              m5}}] == TFI[d, PP, 
            DP, {a, b}, {{n1, m1}, {n2 - 1, m2}, {n3, m3}, {0, 0}, {n5, 
                m5}}] - 
          2  TFI[d, PP, 
              DP, {a, b}, {0, 0, 0, 1, 
                0}, {{n1, m1}, {n2, m2}, {n3, m3}, {0, 0}, {n5, 
                  m5}}] + \((PP + m2^2 - m4^2)\) 
            TFI[d, PP, 
              DP, {a, b}, {{n1, m1}, {n2, m2}, {n3, m3}, {0, 0}, {n5, 
                  m5}}]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (151)", "Subsection",
  CellTags->"T1.6.108.1"],

Cell[BoxData[
    \(h\_151 = s\_151\)], "Input"],

Cell[BoxData[
    \(f\_151 = 
      ReplacePart[
        MakeFun[h\_151, "\<(151)\>"], {a_, b_}, {1, 1, 4}]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (152)", "Subsection",
  CellTags->"T1.6.109.1"],

Cell[BoxData[
    \(TextForm\`\[Nu]\_4 = \(-1\)\)], "Text"],

Cell[BoxData[
    \(TextForm\`\((\(q\_2\)\(-\)\(p\)\(\ \))\)\^2 - m\_4\%2 = \ q\_2\%2 - 
      m\_2\%2 - 2  p\ q\_2 + p\^2 + m\_2\%2 - m\_4\%2\)], "Text"],

Cell[BoxData[
    \(s\_152 = 
      TVI[d, PP, {{n1, m1}, {n2, m2}, {n3, m3}, {\(-1\), m4}}] == 
        TVI[d, PP, {{n1, m1}, {n2 - 1, m2}, {n3, m3}, {0, 0}}] - 
          2  TFI[d, 
              PP, {0, 0, 0, 1, 
                0}, {{0, 0}, {n2, m2}, {n3, m3}, {0, 0}, {n1, m1}}] + \((PP + 
                m2^2 - m4^2)\) 
            TVI[d, PP, {{n1, m1}, {n2, m2}, {n3, m3}, {0, 0}}]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (152)", "Subsection",
  CellTags->"T1.6.110.1"],

Cell[BoxData[
    \(h\_152 = s\_152\)], "Input"],

Cell[BoxData[
    \(f\_152 = MakeFun[h\_152, "\<(152)\>"]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (153)", "Subsection",
  CellTags->"T1.6.111.1"],

Cell[BoxData[
    \(TextForm\`\[Nu]\_2 = \(-1\)\)], "Text"],

Cell[BoxData[
    \(s\_153 = 
      TVI[d, PP, {{n1, m1}, {\(-1\), m2}, {n3, m3}, {n4, m4}}] == \n\t\tTFI[
            d, \ PP, {0, 1, 0, 0, 
              0}, \ {{0, 0}, {0, 0}, {n3, m3}, {n4, m4}, {n1, m1}}] + 
          m2^2\ TVI[d, \ 
              PP, \ {{n1, m1}, {0, 0}, {n3, m3}, {n4, m4}}]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (153)", "Subsection",
  CellTags->"T1.6.112.1"],

Cell[BoxData[
    \(h\_153 = s\_153\)], "Input"],

Cell[BoxData[
    \(f\_153 = MakeFun[h\_153, "\<(153)\>"]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (200)", "Subsection",
  CellTags->"T1.6.113.1"],

Cell["Derivation in TJRspecial.nb", "Text"],

Cell[TextData[Cell[BoxData[
    \(TraditionalForm\`m\_3 = \(m\_2 = m\_1\), \ p\^2 = m\_1\%2\)]]], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(s\_200\), "=", 
        RowBox[{
          RowBox[{
          "16", " ", \(m\_1\%2\), " ", \(\[Nu]\_1\), 
            " ", \((d - \[Nu]\_1 - \[Nu]\_2 - \[Nu]\_3)\), " ", 
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  StyleBox[
                    FormBox[\(\[Nu]\_1\),
                      "TraditionalForm"],
                    FontColor->RGBColor[0, 0, 1]], 
                  FormBox[\(\[Nu]\_2\),
                    "TraditionalForm"], 
                  FormBox[\(\[Nu]\_3\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d",
                    "TraditionalForm"], ")"}]],
              TJI[ d, {
                Plus[ 1, 
                  Subscript[ \[Nu], 1]], 
                Subscript[ \[Nu], 2], 
                Subscript[ \[Nu], 3]}],
              Editable->True]}], "==", 
          RowBox[{
            
            RowBox[{\(-\((1 + 3\ d - 3\ \[Nu]\_1 - 4\ \[Nu]\_2)\)\), 
              " ", \(\[Nu]\_3\), " ", 
              InterpretationBox[
                SubsuperscriptBox[
                  StyleBox["J",
                    SingleLetterItalics->False,
                    FontWeight->"Bold"], 
                  RowBox[{
                    StyleBox[
                      FormBox[\(\[Nu]\_1\),
                        "TraditionalForm"],
                      FontColor->RGBColor[1, 0, 0]], 
                    FormBox[\(\[Nu]\_2\),
                      "TraditionalForm"], 
                    StyleBox[
                      FormBox[\(\[Nu]\_3\),
                        "TraditionalForm"],
                      FontColor->RGBColor[0, 0, 1]]}], 
                  RowBox[{"(", 
                    FormBox["d",
                      "TraditionalForm"], ")"}]],
                TJI[ d, {
                  Plus[ -1, 
                    Subscript[ \[Nu], 1]], 
                  Subscript[ \[Nu], 2], 
                  Plus[ 1, 
                    Subscript[ \[Nu], 3]]}],
                Editable->True]}], "-", 
            
            RowBox[{\(\[Nu]\_2\), 
              " ", \((1 + 3\ d - 3\ \[Nu]\_1 - 4\ \[Nu]\_3)\), " ", 
              InterpretationBox[
                SubsuperscriptBox[
                  StyleBox["J",
                    SingleLetterItalics->False,
                    FontWeight->"Bold"], 
                  RowBox[{
                    StyleBox[
                      FormBox[\(\[Nu]\_1\),
                        "TraditionalForm"],
                      FontColor->RGBColor[1, 0, 0]], 
                    StyleBox[
                      FormBox[\(\[Nu]\_2\),
                        "TraditionalForm"],
                      FontColor->RGBColor[0, 0, 1]], 
                    FormBox[\(\[Nu]\_3\),
                      "TraditionalForm"]}], 
                  RowBox[{"(", 
                    FormBox["d",
                      "TraditionalForm"], ")"}]],
                TJI[ d, {
                  Plus[ -1, 
                    Subscript[ \[Nu], 1]], 
                  Plus[ 1, 
                    Subscript[ \[Nu], 2]], 
                  Subscript[ \[Nu], 3]}],
                Editable->True]}], "+", 
            
            RowBox[{\((\(-1\) + 2\ d - \[Nu]\_1 - 2\ \[Nu]\_2)\), 
              " ", \(\[Nu]\_3\), " ", 
              InterpretationBox[
                SubsuperscriptBox[
                  StyleBox["J",
                    SingleLetterItalics->False,
                    FontWeight->"Bold"], 
                  RowBox[{
                    FormBox[\(\[Nu]\_1\),
                      "TraditionalForm"], 
                    StyleBox[
                      FormBox[\(\[Nu]\_2\),
                        "TraditionalForm"],
                      FontColor->RGBColor[1, 0, 0]], 
                    StyleBox[
                      FormBox[\(\[Nu]\_3\),
                        "TraditionalForm"],
                      FontColor->RGBColor[0, 0, 1]]}], 
                  RowBox[{"(", 
                    FormBox["d",
                      "TraditionalForm"], ")"}]],
                TJI[ d, {
                  Subscript[ \[Nu], 1], 
                  Plus[ -1, 
                    Subscript[ \[Nu], 2]], 
                  Plus[ 1, 
                    Subscript[ \[Nu], 3]]}],
                Editable->True]}], "-", 
            
            RowBox[{\((\(-6\)\ d\^2 + 22\ d\ \[Nu]\_1 - 
                  16\ \[Nu]\_1\%2 - \[Nu]\_2 + 6\ d\ \[Nu]\_2 - 
                  13\ \[Nu]\_1\ \[Nu]\_2 - \[Nu]\_3 + 6\ d\ \[Nu]\_3 - 
                  13\ \[Nu]\_1\ \[Nu]\_3 - 4\ \[Nu]\_2\ \[Nu]\_3)\), " ", 
              InterpretationBox[
                SubsuperscriptBox[
                  StyleBox["J",
                    SingleLetterItalics->False,
                    FontWeight->"Bold"], 
                  RowBox[{
                    FormBox[\(\[Nu]\_1\),
                      "TraditionalForm"], 
                    FormBox[\(\[Nu]\_2\),
                      "TraditionalForm"], 
                    FormBox[\(\[Nu]\_3\),
                      "TraditionalForm"]}], 
                  RowBox[{"(", 
                    FormBox["d",
                      "TraditionalForm"], ")"}]],
                TJI[ d, {
                  Subscript[ \[Nu], 1], 
                  Subscript[ \[Nu], 2], 
                  Subscript[ \[Nu], 3]}],
                Editable->True]}], "+", 
            
            RowBox[{\(\[Nu]\_2\), 
              " ", \((\(-1\) + 2\ d - \[Nu]\_1 - 2\ \[Nu]\_3)\), " ", 
              InterpretationBox[
                SubsuperscriptBox[
                  StyleBox["J",
                    SingleLetterItalics->False,
                    FontWeight->"Bold"], 
                  RowBox[{
                    FormBox[\(\[Nu]\_1\),
                      "TraditionalForm"], 
                    StyleBox[
                      FormBox[\(\[Nu]\_2\),
                        "TraditionalForm"],
                      FontColor->RGBColor[0, 0, 1]], 
                    StyleBox[
                      FormBox[\(\[Nu]\_3\),
                        "TraditionalForm"],
                      FontColor->RGBColor[1, 0, 0]]}], 
                  RowBox[{"(", 
                    FormBox["d",
                      "TraditionalForm"], ")"}]],
                TJI[ d, {
                  Subscript[ \[Nu], 1], 
                  Plus[ 1, 
                    Subscript[ \[Nu], 2]], 
                  Plus[ -1, 
                    Subscript[ \[Nu], 3]]}],
                Editable->True]}]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    \(TextForm\`Due\ to\ sorting\ \ \[Nu]\_1 \[GreaterSlantEqual] \[Nu]\_2 \
\[GreaterSlantEqual] \[Nu]\_3\ \ by\ TJI\ only\ this\ rule\ is\ \(\(necessary\
\)\(.\)\)\)], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (200)", "Subsection",
  CellTags->"T1.6.114.1"],

Cell[BoxData[
    \(h\_200 = s\_200 /. \[Nu]\_1 -> \[Nu]\_1 - 1\)], "Input"],

Cell[BoxData[
    RowBox[{\(f\_200\), "=", 
      RowBox[{
        StyleBox["MakeFun",
          FontColor->RGBColor[1, 0.6, 0.6]], "[", 
        RowBox[{\(h\_200\), ",", "\"\<(200)\>\"", ",", 
          RowBox[{
            StyleBox["IFF",
              FontColor->RGBColor[0.6, 0.4, 1]], 
            "[", \(\[Nu]\_1 > 1 && m1 =!= 0 && PP === m1^2\), "]"}], 
          ",", \({m2 :> m1, m3 :> m1}\)}], "]"}]}]], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (201)", "Subsection",
  CellTags->"T1.6.115.1"],

Cell["\<\
Subscript[f, 201] = 
  HoldForm[TJR[d_, PP_, 
     {{(n1_)?PQ, 0}, 
      {(n2_)?PQ, 0}, 
      {(n3_)?PQ, m3_}}] := 
    (TComment[\"(201)\", 
        TJIC[d, 
         {{n1, 0}, {n2, 0}, 
          {n3, m3}}]]; 
       -(((d - n1 - n2)*
          (-2 - d + 
         2*n1 + 2*n2)*
          (3*d - 2*n1 - 
         2*n2 - 2*n3)*
          (-1 - d + n1 + 
          n2 + n3)*
          TJI[d, PP, 
          {{-1 + n1, 0}, 
          {n2, 0}, {n3, m3}}]
)/(2*m3^2*(d - 2*n1)*
          (-1 + n1)*
          (2*d - 2*n1 - 
          2*n2 - n3)*
          (1 + 2*d - 2*n1 - 
          2*n2 - n3)))) /; 
     n1 > 1 && m3 =!= 0 && 
      PP === m3^2]\
\>", "Input",
  Editable->False]
}, Closed]],

Cell[CellGroupData[{

Cell["MakeFun (202)", "Subsection",
  CellTags->"T1.6.116.1"],

Cell["\<\
Subscript[f, 202] = 
  HoldForm[TJR[d_, PP_, 
     {{(n1_)?PQ, 0}, 
      {(n3_)?PQ, m3_}, 
      {(n2_)?PQ, 0}}] := 
    (TComment[\"(202)\", 
        TJIC[d, 
         {{n1, 0}, {n2, 0}, 
          {n3, m3}}]]; 
       -(((d - n1 - n2)*
          (-2 - d + 2*n1 + 
          2*n2)*
          (3*d - 2*n1 - 
          2*n2 - 2*n3)*
          (-1 - d + n1 + 
          n2 + n3)*
          TJI[d, m3^2, 
          {{-1 + n1, 0}, 
          {n2, 0}, {n3, m3}}
])/(2*m3^2*(d - 2*n1)*
          (-1 + n1)*
          (2*d - 2*n1 - 
          2*n2 - n3)*
          (1 + 2*d - 2*n1 - 
          2*n2 - n3)))) /; 
     n1 > 1 && m3 =!= 0 && 
      PP === m3^2]\
\>", "Input",
  Editable->False]
}, Closed]],

Cell[CellGroupData[{

Cell[" MakeFun (203)", "Subsection",
  CellTags->"T1.6.117.1"],

Cell[BoxData[
    \(Subscript[f, \ 203]\  = 
      HoldForm[\n\t\t\ TJR[d_, \ 
            0, \ {{\((n1_)\)?PQ, \ 0}, \ {\((n2_)\)?PQ, \ 
                m2_}, \ \n\ \ \ \ {\((n3_)\)?PQ, \ 
                m2_}}]\  := \((\n\t\t\t\t\ TComment["\<(203)\>", \ \n\ \ \ \ \
\ \ \ \ TJIC[d, 
                0, \ {{n1, \ 0}, \ {n2, \ m2}, \ {n3, \ 
                    m2}}]]; \ \n\t\t\t\t\((\((d\  - \ 
                      2*\((\(-1\)\  + \ n1\  + \ 
                            n2)\))\)*\n\ \ \ \ \ \((1\  + \ d\  - \ n1\  - \ 
                      n2\  - \ n3)\)*\n\ \ \ \ \ \((d\  - \ 
                      2*\((\(-1\)\  + \ n1\  + \ n3)\))\)*\n\ \ \ \ \ TJR[
                    d, \ 0, \ {{\(-1\)\  + \ n1, \ 0}, \ {n2, \ 
                        m2}, \ \n\ \ \ \ \ \ \ {n3, \ m2}}])\)/\n\ \ \ \((2*
                  m2^2*\((d\  - \ 2*n1)\)*\n\ \ \ \ \ \((1\  + \ d\  - \ 
                      2*n1\  - \ n2\  - \ n3)\)*\n\ \ \ \ \ \((2\  + \ 
                      d\  - \ 2*n1\  - \ n2\  - \ 
                      n3)\))\)\n\t\t\t\n\t\t\t)\)]\)], "Input",
  Editable->False]
}, Closed]],

Cell[CellGroupData[{

Cell[" MakeFun (204)", "Subsection",
  CellTags->"T1.6.118.1"],

Cell[BoxData[
    \(Subscript[f, \ 204]\  = 
      HoldForm[\n\t\t\ TJR[d_, \ 
            0, \ {\ {\((n2_)\)?PQ, \ m2_}, \ {\((n1_)\)?PQ, \ 
                0}, \n\ \ \ \ {\((n3_)\)?PQ, \ 
                m2_}}]\  := \((\n\t\ TComment["\<(204)\>", \ \n\ \ \ \ \ \ \ \
\ TJIC[d, 0, \ {\ {n2, \ m2}, \ {n1, \ 0}, {n3, \ 
                    m2}}]]; \ \n\t\((\((d\  - \ 
                      2*\((\(-1\)\  + \ n1\  + \ 
                            n2)\))\)*\n\ \ \ \ \ \((1\  + \ d\  - \ n1\  - \ 
                      n2\  - \ n3)\)*\n\ \ \ \ \ \((d\  - \ 
                      2*\((\(-1\)\  + \ n1\  + \ n3)\))\)*\n\ \ \ \ \ TJR[
                    d, \ 0, \ {{\(-1\)\  + \ n1, \ 0}, \ {n2, \ 
                        m2}, \ \n\ \ \ \ \ \ \ {n3, \ m2}}])\)/\n\ \ \ \((2*
                  m2^2*\((d\  - \ 2*n1)\)*\n\ \ \ \ \ \((1\  + \ d\  - \ 
                      2*n1\  - \ n2\  - \ n3)\)*\n\ \ \ \ \ \((2\  + \ 
                      d\  - \ 2*n1\  - \ n2\  - \ 
                      n3)\))\)\n\t\t\t)\)]\)], "Input",
  Editable->False]
}, Closed]],

Cell[CellGroupData[{

Cell[" MakeFun (205)", "Subsection",
  CellTags->"T1.6.119.1"],

Cell[BoxData[
    \(Subscript[f, \ 205]\  = 
      HoldForm[\n\t\t\ TJR[d_, \ 
            0, \ {\ {\((n2_)\)?PQ, \ m2_}, \n\ \ \ \ {\((n3_)\)?PQ, \ 
                m2_}, \ {\((n1_)\)?PQ, \ 
                0}}]\  := \((\n\t\ TComment["\<(205)\>", \ \n\ \ \ \ \ \ \ \ \
TJIC[d, 0, \ {\ {n2, \ m2}, {n3, \ m2}, \ {n1, \ 
                    0}}]]; \ \n\((\((d\  - \ 
                      2*\((\(-1\)\  + \ n1\  + \ 
                            n2)\))\)*\n\ \ \ \ \ \((1\  + \ d\  - \ n1\  - \ 
                      n2\  - \ n3)\)*\n\ \ \ \ \ \((d\  - \ 
                      2*\((\(-1\)\  + \ n1\  + \ n3)\))\)*\n\ \ \ \ \ TJR[
                    d, \ 0, \ {{\(-1\)\  + \ n1, \ 0}, \ {n2, \ 
                        m2}, \ \n\ \ \ \ \ \ \ {n3, \ m2}}])\)/\n\ \ \ \((2*
                  m2^2*\((d\  - \ 2*n1)\)*\n\ \ \ \ \ \((1\  + \ d\  - \ 
                      2*n1\  - \ n2\  - \ n3)\)*\n\ \ \ \ \ \((2\  + \ 
                      d\  - \ 2*n1\  - \ n2\  - \ 
                      n3)\))\)\n\t\t\t)\)]\)], "Input",
  Editable->False]
}, Closed]],

Cell[CellGroupData[{

Cell["Input (1xxxx)", "Subsection",
  CellTags->"T1.6.120.1"],

Cell["\<\
If[MatchQ[$RankLimit, {_Integer?NonNegative, 
     _Integer?NonNegative}], 
   $TarasovTdeltaplimit = $RankLimit[[1]]; 
    ($TarasovTplimit = $RankLimit[[2]]; ), 
   $TarasovTdeltaplimit = 2; $TarasovTplimit = 2]; \
\>", "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"tarti", "=", 
      RowBox[{"Timing", "[", 
        RowBox[{"Do", "[", 
          RowBox[{
            RowBox[{"If", "[", 
              RowBox[{\(0 < a + b + r + es\), ",", 
                RowBox[{\(WriteString["\<stdout\>", {a, b, r, es}]\), ";", 
                  RowBox[{"Set", "@@", 
                    RowBox[{"{", 
                      
                      RowBox[{\(s\_\(10\^4 + 1000\ a + 100\ b + 10\ r + 
                              es\)\), ",", 
                        RowBox[{
                          RowBox[{
                            RowBox[{
                              StyleBox["TFI",
                                FontColor->RGBColor[1, 0.6, 0]], "[", 
                              RowBox[{"d", ",", 
                                InterpretationBox[\(p\^2\),
                                  PP], ",", "DP", ",", \({a, b}\), 
                                ",", \({0, 0, r, es, 0}\), ",", 
                                RowBox[{"{", 
                                  RowBox[{
                                    RowBox[{"{", 
                                      RowBox[{"n1", ",", 
                                        InterpretationBox[\(m\_1\),
                                        m1]}], "}"}], ",", 
                                    RowBox[{"{", 
                                      RowBox[{"n2", ",", 
                                        InterpretationBox[\(m\_2\),
                                        m2]}], "}"}], ",", 
                                    RowBox[{"{", 
                                      RowBox[{"n3", ",", 
                                        InterpretationBox[\(m\_3\),
                                        m3]}], "}"}], ",", 
                                    RowBox[{"{", 
                                      RowBox[{"n4", ",", 
                                        InterpretationBox[\(m\_4\),
                                        m4]}], "}"}], ",", 
                                    RowBox[{"{", 
                                      RowBox[{"n5", ",", 
                                        InterpretationBox[\(m\_5\),
                                        m5]}], "}"}]}], "}"}]}], "]"}], "==", 
                            
                            RowBox[{"ApplyTarasovT", "[", 
                              RowBox[{
                                RowBox[{"TarasovT", "[", 
                                  RowBox[{
                                  "a", ",", "b", ",", "r", ",", "es", ",", 
                                    InterpretationBox[\(p\^2\),
                                      PP], ",", "DP"}], "]"}], ",", 
                                RowBox[{"STLI", "[", 
                                  RowBox[{"d", ",", 
                                    InterpretationBox[\(p\^2\),
                                      PP], ",", 
                                    RowBox[{"{", 
                                      RowBox[{
                                        RowBox[{"{", 
                                        RowBox[{"n1", ",", 
                                        InterpretationBox[\(m\_1\),
                                        m1]}], "}"}], ",", 
                                        RowBox[{"{", 
                                        RowBox[{"n2", ",", 
                                        InterpretationBox[\(m\_2\),
                                        m2]}], "}"}], ",", 
                                        RowBox[{"{", 
                                        RowBox[{"n3", ",", 
                                        InterpretationBox[\(m\_3\),
                                        m3]}], "}"}], ",", 
                                        RowBox[{"{", 
                                        RowBox[{"n4", ",", 
                                        InterpretationBox[\(m\_4\),
                                        m4]}], "}"}], ",", 
                                        RowBox[{"{", 
                                        RowBox[{"n5", ",", 
                                        InterpretationBox[\(m\_5\),
                                        m5]}], "}"}]}], "}"}]}], "]"}]}], 
                              "]"}]}], "/.", 
                          RowBox[{"STLI", "\[Rule]", 
                            StyleBox["TFI",
                              FontColor->RGBColor[1, 0.6, 0]]}]}]}], 
                      "}"}]}]}]}], "]"}], 
            ",", \({a, 0, $TarasovTdeltaplimit}\), 
            ",", \({b, 0, $TarasovTdeltaplimit - a}\), 
            ",", \({r, 0, $TarasovTplimit}\), 
            ",", \({es, 0, $TarasovTplimit - r}\)}], "]"}], "]"}]}]], "Input"],

Cell["\<\
{0, 0, 0, 1}{0, 0, 0, 2}{0, 0, 0, 3}{0, 0, 0, 4}{0, 0, 0, 5}{0, 0, \
0, 6}{0, 0, 1, 0}{0, 0, 1, 1}{0, 0, 1, 2}{0, 0, 1, 3}{0, 0, 1, 4}{0, 0, 1, \
5}{0, 0, 2, 0}{0, 0, 2, 1}{0, 0, 2, 2}{0, 0, 2, 3}{0, 0, 2, 4}{0, 0, 3, 0}{0, \
0, 3, 1}{0, 0, 3, 2}{0, 0, 3, 3}{0, 0, 4, 0}{0, 0, 4, 1}{0, 0, 4, 2}{0, 0, 5, \
0}{0, 0, 5, 1}{0, 0, 6, 0}\
\>", "Print"]
}, Closed]],

Cell[BoxData[
    \(Share[]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(matarti = 
      Timing[Do[
          If[0 < ia + ib + ir + es, 
            WriteString["\<stdout\>", {ia, ib, ir, es}]; 
            Set[f\_\(10\^4 + 1000\ ia + 100\ ib + 10\ ir + es\), 
              MakeFun[s\_\(10\^4 + 1000\ ia + 100\ ib + 10\ ir + es\), \
"\<(1\>" <> ToString[ia] <> ToString[ib] <> ToString[ir] <> 
                    ToString[es] <> "\<)\>"] /. PQ :> PNQ]], {ia, 
            0, \ $TarasovTdeltaplimit}, {ib, 
            0, $TarasovTdeltaplimit - ia}, {ir, 0, \ \ $TarasovTplimit}, {es, 
            0, $TarasovTplimit - ir}]]\)], "Input"],

Cell["\<\
{0, 0, 0, 1}{0, 0, 0, 2}{0, 0, 0, 3}{0, 0, 0, 4}{0, 0, 0, 5}{0, 0, \
0, 6}{0, 0, 1, 0}{0, 0, 1, 1}{0, 0, 1, 2}{0, 0, 1, 3}{0, 0, 1, 4}{0, 0, 1, \
5}{0, 0, 2, 0}{0, 0, 2, 1}{0, 0, 2, 2}{0, 0, 2, 3}{0, 0, 2, 4}{0, 0, 3, 0}{0, \
0, 3, 1}{0, 0, 3, 2}{0, 0, 3, 3}{0, 0, 4, 0}{0, 0, 4, 1}{0, 0, 4, 2}{0, 0, 5, \
0}{0, 0, 5, 1}{0, 0, 6, 0}\
\>", "Print"]
}, Closed]],

Cell[BoxData[
    \(Share[]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["symmetry", "Subsection",
  CellTags->"Tsymmetry"],

Cell["\<\
TVR[d_, pp_, {a_, b_, c_, z_}] := 
  TVR[d, pp, {c, b, a, z}] /;  !OrderedQ[{a, c}]\
\>", "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["special TVR reductions", "Subsection",
  CellTags->"specialTVR"],

Cell["\<\
TVR[2 + d_Symbol, pp_, {{1, m1_}, {1, m1_}, {2, 0}, {1, m1_}}] := 
   ((-6*m1^2 + 2*d*m1^2 + 3*pp)*TAI[d, 0, {{1, m1}}]^2)/
      (12*(-3 + d)*(-1 + d)*m1^2*pp) + 
     ((4*m1^2 - pp)*TAI[d, 0, {{1, m1}}]*TBI[d, pp, {{1, m1}, {1, m1}}])/
      (4*(-3 + d)*(-1 + d)*m1^2) - 
     ((6*m1^2 - 2*d*m1^2 - 7*pp + 2*d*pp)*
        TJI[d, pp, {{1, m1}, {1, m1}, {1, 0}}])/(6*(-2 + d)*(-1 + d)*pp) + 
     ((-4*m1^2 + pp)*(-6*m1^2 + 2*d*m1^2 + 3*pp)*
        TJI[d, pp, {{2, m1}, {1, m1}, {1, 0}}])/
      (6*(-3 + d)*(-2 + d)*(-1 + d)*pp) /; (m1=!=0)&&(pp=!=0);\
\>", "Input"],

Cell["\<\
TVR[4 + d_Symbol, pp_, {{1, m1_}, {1, m1_}, {2, 0}, {1, m1_}}] := 
   -((-96*d*m1^6 + 352*d^2*m1^6 - 384*d^3*m1^6 + 128*d^4*m1^6 - 
           192*m1^4*pp + 1008*d*m1^4*pp - 1632*d^2*m1^4*pp + 
           912*d^3*m1^4*pp - 144*d^4*m1^4*pp + 288*m1^2*pp^2 - 
           1536*d*m1^2*pp^2 + 2316*d^2*m1^2*pp^2 - 1086*d^3*m1^2*pp^2 + 
           66*d^4*m1^2*pp^2 + 72*d*pp^3 - 166*d^2*pp^3 + 81*d^3*pp^3 + 
           4*d^4*pp^3)*TAI[d, 0, {{1, m1}}]^2)/
      (36*(-1 + d)^2*d^2*(1 + d)*(-4 + 3*d)*(-2 + 3*d)*pp^2) + 
     ((4*m1^2 - pp)^2*TAI[d, 0, {{1, m1}}]*TBI[d, pp, {{1, m1}, {1, m1}}])/
      (4*(-1 + d)^2*d*(1 + d)) - 
     ((288*m1^8 - 1152*d*m1^8 + 1504*d^2*m1^8 - 768*d^3*m1^8 + 
          128*d^4*m1^8 - 816*m1^6*pp + 2480*d*m1^6*pp - 
          2224*d^2*m1^6*pp + 736*d^3*m1^6*pp - 80*d^4*m1^6*pp + 
          864*m1^4*pp^2 - 2196*d*m1^4*pp^2 + 1446*d^2*m1^4*pp^2 - 
          252*d^3*m1^4*pp^2 - 6*d^4*m1^4*pp^2 - 432*m1^2*pp^3 + 
          1086*d*m1^2*pp^3 - 895*d^2*m1^2*pp^3 + 315*d^3*m1^2*pp^3 - 
          44*d^4*m1^2*pp^3 + 4*d*pp^4 - 2*d^2*pp^4 - 4*d^3*pp^4 + 
          2*d^4*pp^4)*TJI[d, pp, {{1, m1}, {1, m1}, {1, 0}}])/
      (18*(-2 + d)*(-1 + d)^2*d*(1 + d)*(-4 + 3*d)*(-2 + 3*d)*pp^2) + 
     (m1^2*(4*m1^2 - pp)*(-96*m1^6 + 352*d*m1^6 - 384*d^2*m1^6 + 
          128*d^3*m1^6 + 240*m1^4*pp - 624*d*m1^4*pp + 384*d^2*m1^4*pp - 
          48*d^3*m1^4*pp - 192*m1^2*pp^2 + 420*d*m1^2*pp^2 - 
          162*d^2*m1^2*pp^2 - 18*d^3*m1^2*pp^2 + 72*pp^3 - 154*d*pp^3 + 
          81*d^2*pp^3 - 8*d^3*pp^3)*TJI[d, pp, {{2, m1}, {1, m1}, {1, 0}}])/
      (18*(-2 + d)*(-1 + d)^2*d*(1 + d)*(-4 + 3*d)*(-2 + 3*d)*pp^2) /; \
(m1=!=0)&&(pp=!=0);\
\>", "Input"],

Cell["\<\
TVR[4 + d_Symbol, pp_, {{1, m1_}, {1, m1_}, {3, 0}, {1, m1_}}] := 
   ((-72*d^2*m1^6 + 144*d^3*m1^6 - 88*d^4*m1^6 + 16*d^5*m1^6 - 
          240*d*m1^4*pp + 572*d^2*m1^4*pp - 398*d^3*m1^4*pp + 
          84*d^4*m1^4*pp - 2*d^5*m1^4*pp - 96*m1^2*pp^2 + 
          480*d*m1^2*pp^2 - 744*d^2*m1^2*pp^2 + 390*d^3*m1^2*pp^2 - 
          50*d^4*m1^2*pp^2 + 4*d^5*m1^2*pp^2 - 24*d*pp^3 + 54*d^2*pp^3 - 
          27*d^3*pp^3)*TAI[d, 0, {{1, m1}}]^2)/
      (24*(-3 + d)*(-1 + d)^2*d*(1 + d)*(-4 + 3*d)*(-2 + 3*d)*m1^2*pp^2) + 
     ((4*m1^2 - pp)^2*TAI[d, 0, {{1, m1}}]*TBI[d, pp, {{1, m1}, {1, m1}}])/
      (8*(-3 + d)*(-1 + d)^2*(1 + d)*m1^2) + 
     ((-72*d*m1^6 + 144*d^2*m1^6 - 88*d^3*m1^6 + 16*d^4*m1^6 - 
          96*m1^4*pp + 260*d*m1^4*pp - 166*d^2*m1^4*pp + 
          12*d^3*m1^4*pp + 6*d^4*m1^4*pp + 160*m1^2*pp^2 - 
          300*d*m1^2*pp^2 + 50*d^2*m1^2*pp^2 + 90*d^3*m1^2*pp^2 - 
          24*d^4*m1^2*pp^2 - 48*pp^3 + 114*d*pp^3 - 67*d^2*pp^3 + 
          4*d^3*pp^3 + 2*d^4*pp^3)*TJI[d, pp, {{1, m1}, {1, m1}, {1, 0}}])/
      (12*(-2 + d)*(-1 + d)^2*(1 + d)*(-4 + 3*d)*(-2 + 3*d)*pp^2) - 
     ((-4*m1^2 + pp)*(72*d*m1^6 - 144*d^2*m1^6 + 88*d^3*m1^6 - 
          16*d^4*m1^6 + 96*m1^4*pp - 236*d*m1^4*pp + 122*d^2*m1^4*pp + 
          12*d^3*m1^4*pp - 10*d^4*m1^4*pp - 96*m1^2*pp^2 + 
          180*d*m1^2*pp^2 - 30*d^2*m1^2*pp^2 - 46*d^3*m1^2*pp^2 + 
          8*d^4*m1^2*pp^2 + 24*pp^3 - 54*d*pp^3 + 27*d^2*pp^3)*
        TJI[d, pp, {{2, m1}, {1, m1}, {1, 0}}])/
(12*(-3 + d)*(-2 + d)*(-1 + d)^2*(1 + d)*(-4 + 3*d)*(-2 + 3*d)*pp^2) /; \
(m1=!=0)&&(pp=!=0);\
\>", "Input"],

Cell["\<\
TVR[6 + d_Symbol, pp_, 
    {{1, m1_}, {1, m1_}, {4, 0}, {1, m1_}}] := 
   ((2304*d*m1^10 - 4608*d^2*m1^10 - 3520*d^3*m1^10 + 6976*d^4*m1^10 + 
          1472*d^5*m1^10 - 2624*d^6*m1^10 - 256*d^7*m1^10 + 
          256*d^8*m1^10 + 4608*m1^8*pp - 9600*d*m1^8*pp - 
          7680*d^2*m1^8*pp + 9312*d^3*m1^8*pp + 11680*d^4*m1^8*pp - 
          6560*d^5*m1^8*pp - 2880*d^6*m1^8*pp + 1088*d^7*m1^8*pp + 
          32*d^8*m1^8*pp + 6912*m1^6*pp^2 - 35712*d*m1^6*pp^2 + 
          1056*d^2*m1^6*pp^2 + 85296*d^3*m1^6*pp^2 - 
          37032*d^4*m1^6*pp^2 - 20484*d^5*m1^6*pp^2 + 
          8700*d^6*m1^6*pp^2 - 12*d^7*m1^6*pp^2 - 84*d^8*m1^6*pp^2 - 
          20736*m1^4*pp^3 + 58752*d*m1^4*pp^3 + 40560*d^2*m1^4*pp^3 - 
          150536*d^3*m1^4*pp^3 + 18980*d^4*m1^4*pp^3 + 
          48946*d^5*m1^4*pp^3 - 6748*d^6*m1^4*pp^3 - 146*d^7*m1^4*pp^3 + 
          128*d^8*m1^4*pp^3 + 3456*m1^2*pp^4 - 13824*d*m1^2*pp^4 - 
          4176*d^2*m1^2*pp^4 + 35784*d^3*m1^2*pp^4 - 
          8332*d^4*m1^2*pp^4 - 10162*d^5*m1^2*pp^4 + 
          1122*d^6*m1^2*pp^4 - 80*d^7*m1^2*pp^4 - 8*d^8*m1^2*pp^4 + 
          864*d*pp^5 - 216*d^2*pp^5 - 2268*d^3*pp^5 + 486*d^4*pp^5 + 
          729*d^5*pp^5)*TAI[d, 0, {{1, m1}}]^2)/
      (432*(-3 + d)*(-1 + d)^2*d^2*(1 + d)*(2 + d)*(3 + d)*(-4 + 3*d)*
        (-2 + 3*d)*(2 + 3*d)*m1^2*pp^3) + 
     ((4*m1^2 - pp)^3*TAI[d, 0, {{1, m1}}]*TBI[d, pp, {{1, m1}, {1, m1}}])/
      (16*(-3 + d)*(-1 + d)^2*d*(1 + d)*(3 + d)*m1^2) - 
     ((-1152*m1^10 + 2880*d*m1^10 + 320*d^2*m1^10 - 3648*d^3*m1^10 + 
          1088*d^4*m1^10 + 768*d^5*m1^10 - 256*d^6*m1^10 - 192*m1^8*pp + 
          1312*d*m1^8*pp + 2080*d^2*m1^8*pp - 6880*d^3*m1^8*pp + 
          4032*d^4*m1^8*pp - 192*d^5*m1^8*pp - 160*d^6*m1^8*pp + 
          6336*m1^6*pp^2 - 6960*d*m1^6*pp^2 - 16960*d^2*m1^6*pp^2 + 
          17292*d^3*m1^6*pp^2 - 1828*d^4*m1^6*pp^2 - 828*d^5*m1^6*pp^2 + 
          68*d^6*m1^6*pp^2 - 9504*m1^4*pp^3 + 3624*d*m1^4*pp^3 + 
          26104*d^2*m1^4*pp^3 - 10938*d^3*m1^4*pp^3 - 
          4976*d^4*m1^4*pp^3 + 330*d^5*m1^4*pp^3 + 400*d^6*m1^4*pp^3 + 
          6336*m1^2*pp^4 - 3640*d*m1^2*pp^4 - 14584*d^2*m1^2*pp^4 + 
          9190*d^3*m1^2*pp^4 + 1374*d^4*m1^2*pp^4 - 600*d^5*m1^2*pp^4 - 
          56*d^6*m1^2*pp^4 - 864*pp^5 + 732*d*pp^5 + 1864*d^2*pp^5 - 
          1695*d^3*pp^5 + 148*d^4*pp^5 + 36*d^5*pp^5 + 4*d^6*pp^5)*
        TJI[d, pp, {{1, m1}, {1, m1}, {1, 0}}])/
      (216*(-2 + d)*(-1 + d)^2*d*(1 + d)*(3 + d)*(-4 + 3*d)*(-2 + 3*d)*
        (2 + 3*d)*pp^3) + ((-4*m1^2 + pp)*
        (1152*m1^10 - 2880*d*m1^10 - 320*d^2*m1^10 + 3648*d^3*m1^10 - 
          1088*d^4*m1^10 - 768*d^5*m1^10 + 256*d^6*m1^10 + 576*m1^8*pp - 
          2208*d*m1^8*pp - 2304*d^2*m1^8*pp + 8000*d^3*m1^8*pp - 
          4256*d^4*m1^8*pp - 32*d^5*m1^8*pp + 224*d^6*m1^8*pp - 
          6336*m1^6*pp^2 + 6288*d*m1^6*pp^2 + 16320*d^2*m1^6*pp^2 - 
          14868*d^3*m1^6*pp^2 + 684*d^4*m1^6*pp^2 + 804*d^5*m1^6*pp^2 - 
          12*d^6*m1^6*pp^2 + 6912*m1^4*pp^3 - 3192*d*m1^4*pp^3 - 
          18160*d^2*m1^4*pp^3 + 7950*d^3*m1^4*pp^3 + 3128*d^4*m1^4*pp^3 - 
          78*d^5*m1^4*pp^3 - 160*d^6*m1^4*pp^3 - 3456*m1^2*pp^4 + 
          1800*d*m1^2*pp^4 + 8136*d^2*m1^2*pp^4 - 4322*d^3*m1^2*pp^4 - 
          1042*d^4*m1^2*pp^4 + 128*d^5*m1^2*pp^4 + 16*d^6*m1^2*pp^4 + 
          432*pp^5 - 324*d*pp^5 - 972*d^2*pp^5 + 729*d^3*pp^5)*
        TJI[d, pp, {{2, m1}, {1, m1}, {1, 0}}])/
      (216*(-3 + d)*(-2 + d)*(-1 + d)^2*d*(1 + d)*(3 + d)*(-4 + 3*d)*
        (-2 + 3*d)*(2 + 3*d)*pp^3) /; (m1=!=0)&&(pp=!=0);\
\>", "Input"],

Cell["\<\
 TVR[6 + d_Symbol, pp_, {{2, m1_}, {1, m1_}, {3, 0}, {1, m1_}}] := 
   -((-1536*d*m1^10 + 2560*d^2*m1^10 + 3200*d^3*m1^10 - 3584*d^4*m1^10 - 
           2176*d^5*m1^10 + 1024*d^6*m1^10 + 512*d^7*m1^10 - 
           3072*m1^8*pp + 6528*d*m1^8*pp + 5376*d^2*m1^8*pp - 
           7520*d^3*m1^8*pp - 5568*d^4*m1^8*pp + 3232*d^5*m1^8*pp + 
           1344*d^6*m1^8*pp - 320*d^7*m1^8*pp - 2304*m1^6*pp^2 + 
           13824*d*m1^6*pp^2 + 192*d^2*m1^6*pp^2 - 31584*d^3*m1^6*pp^2 + 
           11232*d^4*m1^6*pp^2 + 8088*d^5*m1^6*pp^2 - 
           2304*d^6*m1^6*pp^2 - 24*d^7*m1^6*pp^2 + 6912*m1^4*pp^3 - 
           19584*d*m1^4*pp^3 - 12688*d^2*m1^4*pp^3 + 
           50584*d^3*m1^4*pp^3 - 7516*d^4*m1^4*pp^3 - 
           16982*d^5*m1^4*pp^3 + 2564*d^6*m1^4*pp^3 + 310*d^7*m1^4*pp^3 - 
           1152*m1^2*pp^4 + 4608*d*m1^2*pp^4 + 1392*d^2*m1^2*pp^4 - 
           11896*d^3*m1^2*pp^4 + 2820*d^4*m1^2*pp^4 + 
           3374*d^5*m1^2*pp^4 - 414*d^6*m1^2*pp^4 + 8*d^7*m1^2*pp^4 - 
           288*d*pp^5 + 72*d^2*pp^5 + 756*d^3*pp^5 - 162*d^4*pp^5 - 
           243*d^5*pp^5)*TAI[d, 0, {{1, m1}}]^2)/
      (144*(-1 + d)^2*d^2*(1 + d)*(2 + d)*(3 + d)*(-4 + 3*d)*(-2 + 3*d)*
        (2 + 3*d)*m1^2*pp^3) + 
     ((4*m1^2 - pp)^3*TAI[d, 0, {{1, m1}}]*TBI[d, pp, {{1, m1}, {1, m1}}])/
      (16*(-1 + d)^2*d*(1 + d)*(3 + d)*m1^2) - 
     ((2304*m1^10 - 5760*d*m1^10 - 640*d^2*m1^10 + 7296*d^3*m1^10 - 
          2176*d^4*m1^10 - 1536*d^5*m1^10 + 512*d^6*m1^10 - 
          1344*m1^8*pp + 1120*d*m1^8*pp - 1376*d^2*m1^8*pp + 
          5280*d^3*m1^8*pp - 4896*d^4*m1^8*pp + 1280*d^5*m1^8*pp - 
          64*d^6*m1^8*pp - 5184*m1^6*pp^2 + 7584*d*m1^6*pp^2 + 
          14800*d^2*m1^6*pp^2 - 21496*d^3*m1^6*pp^2 + 
          6120*d^4*m1^6*pp^2 + 280*d^5*m1^6*pp^2 - 184*d^6*m1^6*pp^2 + 
          9504*m1^4*pp^3 - 3720*d*m1^4*pp^3 - 27296*d^2*m1^4*pp^3 + 
          15378*d^3*m1^4*pp^3 + 4114*d^4*m1^4*pp^3 - 666*d^5*m1^4*pp^3 - 
          674*d^6*m1^4*pp^3 - 6336*m1^2*pp^4 + 5720*d*m1^2*pp^4 + 
          13712*d^2*m1^2*pp^4 - 13926*d^3*m1^2*pp^4 + 
          1284*d^4*m1^2*pp^4 + 946*d^5*m1^2*pp^4 - 80*d^6*m1^2*pp^4 + 
          864*pp^5 - 1020*d*pp^5 - 1636*d^2*pp^5 + 2311*d^3*pp^5 - 
          693*d^4*pp^5 + 20*d^5*pp^5 + 4*d^6*pp^5)*
        TJI[d, pp, {{1, m1}, {1, m1}, {1, 0}}])/
      (72*(-2 + d)*(-1 + d)^2*d*(1 + d)*(3 + d)*(-4 + 3*d)*(-2 + 3*d)*
        (2 + 3*d)*pp^3) + ((4*m1^2 - pp)*
        (-768*m1^10 + 1664*d*m1^10 + 768*d^2*m1^10 - 2176*d^3*m1^10 + 
          512*d^5*m1^10 + 192*m1^8*pp + 288*d*m1^8*pp + 704*d^2*m1^8*pp - 
          2272*d^3*m1^8*pp + 1024*d^4*m1^8*pp + 64*d^5*m1^8*pp + 
          1920*m1^6*pp^2 - 1824*d*m1^6*pp^2 - 5472*d^2*m1^6*pp^2 + 
          4584*d^3*m1^6*pp^2 - 168*d^5*m1^6*pp^2 - 2304*m1^4*pp^3 + 
          616*d*m1^4*pp^3 + 6192*d^2*m1^4*pp^3 - 1946*d^3*m1^4*pp^3 - 
          1128*d^4*m1^4*pp^3 - 230*d^5*m1^4*pp^3 + 1152*m1^2*pp^4 - 
          600*d*m1^2*pp^4 - 2744*d^2*m1^2*pp^4 + 1414*d^3*m1^2*pp^4 + 
          374*d^4*m1^2*pp^4 - 16*d^5*m1^2*pp^4 - 144*pp^5 + 108*d*pp^5 + 
          324*d^2*pp^5 - 243*d^3*pp^5)*
        TJI[d, pp, {{2, m1}, {1, m1}, {1, 0}}])/
      (72*(-2 + d)*(-1 + d)^2*d*(1 + d)*(3 + d)*(-4 + 3*d)*(-2 + 3*d)*
        (2 + 3*d)*pp^3) /; (m1=!=0)&&(pp=!=0);\
\>", "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["five -> four", "Subsection",
  CellTags->"T1.6.121.1"],

Cell[BoxData[
    \(TFI[d_, \ 
        PP_ /; Head[PP] =!= List, {\ {0, \ m1_}, \ {n2_, \ 
            m2_}, \ \n\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ \ \ {n3_, \ m3_}, \ {n4_, \ 
            m4_}, \ \n\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ \ \ {n5_, \ m5_}\ }]\  := \ \n
      TVI[d, PP, \ {{n5, m5}, \ {n2, m2}, \ {n3, m3}, \ {n4, m4}}]\)], "Input"],

Cell[BoxData[
    \(TFI[
        d_, \ \ PP_ /; 
          Head[PP] =!= List, {\ {\((n2_)\), \ m2_}, {0, \ 
            m1_}, \n\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ \ {\((n4_)\), \ m4_}, \ {\((n3_)\), \ 
            m3_}, \ \n\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ \ \ {\((n5_)\), \ m5_}\ }]\  := \ \n
      TVI[d, PP, \ {{n5, m5}, \ {n2, m2}, \ {n3, m3}, \ {n4, m4}}]\)], "Input"],

Cell[BoxData[
    \(TFI[
        d_, \ \ \ PP_ /; 
          Head[PP] =!= List, {\ {\((n3_)\), \ m3_}, \ {\((n4_)\), \ 
            m4_}, \n\t\t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
{0, \ m1_}, \ {\((n2_)\), \ 
            m2_}, \ \n\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ \ \ {\((n5_)\), \ m5_}\ }]\  := \ \n
      TVI[d, PP, \ {{n5, m5}, \ {n2, m2}, \ {n3, m3}, \ {n4, m4}}]\)], "Input"],

Cell[BoxData[
    \(TFI[
        d_, \ \ \ PP_ /; 
          Head[PP] =!= List, {\ {\((n4_)\), \ m4_}, {\((n3_)\), \ 
            m3_}, \n\t\t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
{\((n2_)\), \ m2_}, \ {0, \ 
            m1_}, \ \n\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ \ \ {\((n5_)\), \ m5_}\ }]\  := \ \n
      TVI[d, PP, \ {{n5, m5}, \ {n2, m2}, \ {n3, m3}, \ {n4, m4}}]\)], "Input"],

Cell["\<\

TFI[d_,  PP_/;Head[PP]=!=List,{{(n1_), m1_}, 

    {(n2_), m2_}, 

    {(n3_), m3_}, 

    {(n4_), m4_}, {0, m5_}}] := 

  TBI[d, PP, {{n1, m1}, {n3, m3}}]*

   TBI[d, PP, {{n2, m2}, {n4, m4}}]
\
\>", "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["four -> three", "Subsection",
  CellTags->"T1.6.122.1"],

Cell[BoxData[
    \(TVI[d_, \ 
        PP_, {\ {0, \ m1_}, \ {n2_, \ 
            m2_}, \ \n\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ \ \ {n3_, \ m3_}, \ {n4_, \ m4_}\ }]\  := \ \n
      TAI[d, 0, {{n3, m3}}]\ TBI[d, PP, {{n2, m2}, {n4, m4}}]\)], "Input"],

Cell[BoxData[
    \(TVI[d_, \ 
        PP_, {\ {n1_, \ m1_}, \ {n2_, \ 
            m2_}, \ \n\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ \ \ {0, \ m3_}, \ {n4_, \ m4_}\ }]\  := \ \n
      TAI[d, 0, {{n1, m1}}]\ TBI[d, PP, {{n2, m2}, {n4, m4}}]\)], "Input"],

Cell["\<\
TVI[d_, PP_, {{n1_, m1_}, {0, m2_}, {n3_, m3_}, {n4_, m4_}}] := 
  TJI[d, 0, {{n1, m1}, {n3, m3}, {n4, m4}}]\
\>", "Input"],

Cell[BoxData[
    \(TVI[d_, \ 
        PP_, {\ {n1_, \ m1_}, \ {n2_, \ 
            m2_}, \ \n\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ \ \ {n3_, \ m3_}, \ {0, \ m4_}\ }]\  := \ \n
      TJI[d, PP, \ {{n2, m2}, {n1, m1}, {n3, m3}\ }]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["three -> two", "Subsection",
  CellTags->"T1.6.123.1"],

Cell[BoxData[
    \(TJI[d_, \ 
        PP_, {\ {0, \ m1_}, \ {\((n2_)\), \ m2_}, \ {\((n3_)\), \ 
            m3_}\ }]\  := \ \n
      TAI[d, 0, {{n2, m2}}]\ \ TAI[d, 0, {{n3, m3}}]\)], "Input"],

Cell[BoxData[
    \(TJI[d_, \ 
        PP_, {\ {\((n1_)\), \ m1_}, \ {0, \ m2_}, \ {\((n3_)\), \ 
            m3_}\ }]\  := \ \n
      TAI[d, 0, {{n1, m1}}]\ \ TAI[d, 0, {{n3, m3}}]\)], "Input"],

Cell[BoxData[
    \(TJI[d_, \ 
        PP_, {\ {\((n1_)\), \ m1_}, \ {\((n2_)\), \ m2_}, \ {0, \ 
            m3_}\ }]\  := \ \n
      TAI[d, 0, {{n1, m1}}]\ \ TAI[d, 0, {{n2, m2}}]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["two -> one", "Subsection",
  CellTags->"T1.6.124.1"],

Cell[BoxData[
    \(\(\(\(TBI[d_, \ 
        PP_, {\ {0, \ m1_}, \ {\((n2_)\), \ 
            m2_}\ }]\)\(\ \)\)\(:=\)\(\ \)\(\n\)\(TAI[d, 
        0, {{n2, m2}}]\)\(\ \)\)\)], "Input"],

Cell[BoxData[
    \(\(\(\(TBI[d_, \ 
        PP_, {\ {\((n1_)\), \ m1_}, \ {0, \ 
            m2_}\ }]\)\(\ \)\)\(:=\)\(\ \)\(\n\)\(TAI[d, 
        0, {{n1, m1}}]\)\(\ \)\)\)], "Input"],

Cell[BoxData[
    \(\(TAI[d_, {{n_Integer, m_}}] := TAI[d, 0, {{n, m}}];\)\)], "Input"],

Cell[BoxData[
    \(\(TAI[d_, {n_Integer, m_}] := TAI[d, 0, {{n, m}}];\)\)], "Input"],

Cell[BoxData[
    \(\(TAI[d_, 0, {n_Integer, m_}] := TAI[d, 0, {{n, m}}];\)\)], "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["TarcerRecurse and TFIRecurse", "Section",
  CellDingbat->"\[FilledSmallCircle]",
  CellTags->"T1.7.1"],

Cell[CellGroupData[{

Cell["TFIRecurse & TFISimplify", "Subsection",
  CellTags->"T1.7.1.1"],

Cell["This cancels scalar products.", "Text"],

Cell[BoxData[
    \(\($TLRComment = False;\)\)], "Input"],

Cell[BoxData[
    \(\(TLRComment[s_String] := 
        WriteString["\<stdout\>", s, "\< \>"] /; $TLRComment === 
            True;\)\)], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(applytlrules[y_, fun_]\), " ", ":=", 
        RowBox[{"fun", "[", 
          RowBox[{
            RowBox[{
              RowBox[{"y", " ", "/.", 
                RowBox[{
                  StyleBox["TFI",
                    FontColor->RGBColor[1, 0.6, 0]], "->", "TLR"}]}], "/.", 
              "TLRules"}], "/.", 
            RowBox[{"TLR", "->", 
              StyleBox["TFI",
                FontColor->RGBColor[1, 0.6, 0]]}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
    \(\(TFIRecurse[z_, f_ : Identity]\  := \ 
        FixedPoint[\ applytlrules[#, f] &, z, 1000];\)\)], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(applytlrules2[y_, fun_]\), " ", ":=", 
        RowBox[{"fun", "[", 
          RowBox[{
            RowBox[{
              RowBox[{"y", " ", "/.", 
                RowBox[{
                  StyleBox["TFI",
                    FontColor->RGBColor[1, 0.6, 0]], "->", "TLR"}]}], "/.", 
              "TLRules2"}], "/.", 
            RowBox[{"TLR", "->", 
              StyleBox["TFI",
                FontColor->RGBColor[1, 0.6, 0]]}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
    \(\(TFISimplify[z_, f_ : Identity]\  := \ 
        FixedPoint[\ applytlrules2[#, f] &, z, 1000];\)\)], "Input"],

Cell["\<\
TFIRecurse[z_]:=z/.TFI\[Rule]TLR//.TLRules/.TLR\[Rule]TFI;\
\>", \
"Text"],

Cell[BoxData[
    \(\(ExpandMaybe = Identity;\)\)], "Input"],

Cell[BoxData[
    \(\(fN[li_List] := \(fN[li] = 
          li /. {0, _} \[RuleDelayed] {0, 0}\);\)\)], "Input"],

Cell["\<\
RM: new Feb. 1999  
need TLR[ ...]:= rule;
(copy from TFI definition) (in order to get e.g. 
TarcerRecurse[TFI[D,pp,{1,1,1,1,1},{{1,0},{1,M},{0,0},{1,m},{1,M}}]]
to work)\
\>", "Text",
  CellTags->"TLRdef"],

Cell[BoxData[
    \(\(TLR[depp__, {a___, b_Integer, c___}] := 
        TLR[depp, {a, {b, 0}, c}];\)\)], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(tlrule[1]\), "=", 
        RowBox[{\(TLR[d_, pp_, {0, 0, 0, 0, 0}, pr_]\), "\[RuleDelayed]", 
          RowBox[{"(", 
            RowBox[{\(NoTLRComment["\<tlrule1\>"]\), ";", 
              RowBox[{
                StyleBox["TFI",
                  FontColor->RGBColor[1, 0.6, 0]], "[", \(d, pp, 
                pr /. {0, _} \[RuleDelayed] {0, 0}\), "]"}]}], ")"}]}]}], 
      ";"}]], "Input"],

Cell["(* q1^2 *)", "Text"],

Cell[BoxData[
    \(\(tlrule[2] = 
        TLR[dpp__, {v_?PQ, w_, x_, y_, z_}, {{n1_?PQ, m1_}, nm2_, nm3_, nm4_, 
              nm5_}] \[RuleDelayed] \((TLRComment["\<tlrule2\>"]; 
            ExpandMaybe[
              If[v \[LessEqual] n1, 
                If[m1 === 0, 
                  TLR[dpp, {0, w, x, y, z}, {{n1 - v, 0}, nm2, nm3, nm4, 
                      nm5}], \[Sum]\+\(i = 0\)\%v Binomial[v, i]\ TLR[
                        dpp, \ {0, w, x, y, z}, {fN[{n1 - i, m1}], nm2, nm3, 
                          nm4, nm5}]\ \((m1\^2)\)\^\(v - i\)], 
                If[m1 === 0, 
                  TLR[dpp, {v - n1, w, x, y, z}, {{0, 0}, nm2, nm3, nm4, 
                      nm5}], \[Sum]\+\(i = 0\)\%n1 Binomial[n1, i]\ TLR[
                        dpp, {v - n1, w, x, y, z}, {fN[{n1 - i, m1}], nm2, 
                          nm3, nm4, 
                          nm5}]\ \((m1\^2)\)\^\(n1 - i\)]]])\);\)\)], "Input"],

Cell[TextData[Cell[BoxData[
    \(TraditionalForm\`q\_2\%2\)]]], "Text"],

Cell[BoxData[
    \(\(tlrule[3] = 
        TLR[dpp__, {v_, w_?PQ, x_, y_, z_}, {nm1_, {n2_?PQ, m2_}, nm3_, nm4_, 
              nm5_}] \[RuleDelayed] \((TLRComment["\<tlrule3\>"]; 
            ExpandMaybe[
              If[w \[LessEqual] n2, 
                If[m2 === 0, 
                  TLR[dpp, {v, 0, x, y, z}, {nm1, {n2 - w, 0}, nm3, nm4, 
                      nm5}], \[Sum]\+\(i = 0\)\%w Binomial[w, i]\ TLR[
                        dpp, {v, 0, x, y, z}, {nm1, fN[{n2 - i, m2}], nm3, 
                          nm4, nm5}]\ \((m2\^2)\)\^\(w - i\)], 
                If[m2 === 0, 
                  TLR[dpp, {v, w - n2, x, y, z}, {nm1, {0, 0}, nm3, nm4, 
                      nm5}], \[Sum]\+\(i = 0\)\%n2 Binomial[n2, i]\ TLR[
                        dpp, {v, w - n2, x, y, z}, {nm1, fN[{n2 - i, m2}], 
                          nm3, nm4, 
                          nm5}]\ \((m2\^2)\)\^\(n2 - i\)]]])\);\)\)], "Input"],

Cell[TextData[Cell[BoxData[
    \(TraditionalForm\`p\[CenterDot]q\_1\)]]], "Text"],

Cell[BoxData[
    \(\(tlrule[4] = 
        TLR[dpp__, {v_, w_, x_?PQ, y_, z_}, {{n1_?PQ, m1_}, 
              nm2_, {n3_?PQ, m3_}, nm4_, 
              nm5_}] \[RuleDelayed] \((TLRComment["\<tlrule4\>"]; 
            ExpandMaybe[
              1\/2\ TLR[
                    dpp, {v + 1, w, x - 1, y, z}, {{n1, m1}, nm2, {n3, m3}, 
                      nm4, nm5}] - 
                1\/2\ TLR[
                    dpp, {v, w, x - 1, y, z}, {{n1, m1}, nm2, 
                      fN[{n3 - 1, m3}], nm4, nm5}] + 
                1\/2\ \(({dpp}\[LeftDoubleBracket]2\[RightDoubleBracket] - 
                      m3\^2)\)\ TLR[
                    dpp, {v, w, x - 1, y, z}, {{n1, m1}, nm2, {n3, m3}, nm4, 
                      nm5}]])\);\)\)], "Input"],

Cell[TextData[Cell[BoxData[
    \(TraditionalForm\`p\[CenterDot]q\_2\)]]], "Text"],

Cell[BoxData[
    \(\(tlrule[5] = 
        TLR[dpp__, {v_, w_, x_, y_?PQ, z_}, {{n1_, m1_}, {n2_?PQ, m2_}, 
              nm3_, {n4_?PQ, m4_}, 
              nm5_}] \[RuleDelayed] \((TLRComment["\<tlrule5\>"]; 
            ExpandMaybe[
              1\/2\ TLR[
                    dpp, {v, w + 1, x, y - 1, z}, {{n1, m1}, {n2, m2}, 
                      nm3, {n4, m4}, nm5}] - 
                1\/2\ TLR[
                    dpp, {v, w, x, y - 1, z}, {{n1, m1}, {n2, m2}, nm3, 
                      fN[{n4 - 1, m4}], nm5}] + 
                1\/2\ \(({dpp}\[LeftDoubleBracket]2\[RightDoubleBracket] - 
                      m4\^2)\)\ TLR[
                    dpp, {v, w, x, y - 1, z}, {{n1, m1}, {n2, m2}, 
                      nm3, {n4, m4}, nm5}]])\);\)\)], "Input"],

Cell[TextData[Cell[BoxData[
    \(TraditionalForm\`q\_1\[CenterDot]q\_2\)]]], "Text"],

Cell[BoxData[
    \(\(tlrule[6] = 
        TLR[dpp__, {v_, w_, x_, y_, z_?PQ}, {{n1_, m1_}, {n2_, m2_}, nm3_, 
              nm4_, {n5_?PQ, 
                m5_}}] \[RuleDelayed] \((TLRComment["\<tlrule6\>"]; 
            ExpandMaybe[
              1\/2\ TLR[
                    dpp, {v + 1, w, x, y, z - 1}, {{n1, m1}, {n2, m2}, nm3, 
                      nm4, {n5, m5}}] - 
                1\/2\ TLR[
                    dpp, {v, w, x, y, z - 1}, {{n1, m1}, {n2, m2}, nm3, nm4, 
                      fN[{n5 - 1, m5}]}] + 
                1\/2\ TLR[
                    dpp, {v, w + 1, x, y, z - 1}, {{n1, m1}, {n2, m2}, nm3, 
                      nm4, {n5, m5}}] - 
                1\/2\ m5\^2\ TLR[
                    dpp, {v, w, x, y, z - 1}, {{n1, m1}, {n2, m2}, nm3, 
                      nm4, {n5, m5}}]])\);\)\)], "Input"],

Cell["(* q1.q1  --> q1.p *)", "Text"],

Cell[BoxData[
    \(\(tlrule[7] = 
        TLR[d_, pp_, 
            dp___, {v_?PQ, w_, x_, y_, z_}, {{0, _}, nm2_, {n3_?PQ, m3_}, 
              nm4_, nm5_}] \[RuleDelayed] \((TLRComment["\<tlrule7\>"]; 
            ExpandMaybe[
              TLR[d, pp, 
                  dp, {v - 1, w, x, y, z}, {{0, 0}, nm2, fN[{n3 - 1, m3}], 
                    nm4, nm5}] + 
                2\ TLR[d, pp, 
                    dp, {v - 1, w, x + 1, y, z}, {{0, 0}, nm2, {n3, m3}, nm4, 
                      nm5}] - \((pp - m3\^2)\)\ TLR[d, pp, 
                    dp, {v - 1, w, x, y, z}, {{0, 0}, nm2, {n3, m3}, nm4, 
                      nm5}]])\);\)\)], "Input"],

Cell["(* q2.q2  --> q2.p *)", "Text"],

Cell[BoxData[
    \(\(tlrule[8] = 
        TLR[d_, pp_, 
            dp___, {v_, w_?PQ, x_, y_, z_}, {nm1_, {0, _}, 
              nm3_, {n4_?PQ, m4_}, 
              nm5_}] \[RuleDelayed] \((TLRComment["\<tlrule8\>"]; 
            ExpandMaybe[
              TLR[d, pp, 
                  dp, {v, w - 1, x, y, z}, {nm1, {0, 0}, nm3, 
                    fN[{n4 - 1, m4}], nm5}] + 
                2\ TLR[d, pp, 
                    dp, {v, w - 1, x, y + 1, z}, {nm1, {0, 0}, nm3, {n4, m4}, 
                      nm5}] - \((pp - m4\^2)\)\ TLR[d, pp, 
                    dp, {v, w - 1, x, y, z}, {nm1, {0, 0}, nm3, {n4, m4}, 
                      nm5}]])\);\)\)], "Input"],

Cell["(* eq. (3) - (5) *)", "Text"],

Cell[BoxData[
    \(\(tlrule[9] = 
        TLR[d_, pp_, {v_, w_, x_, y_, z_?PQ}, {nm1_, nm2_, nm3_, 
              nm4_, {0, _}}] \[RuleDelayed] \((TLRComment["\<tlrule9\>"]; 
            FunctionExpand[
              ExpandMaybe[\[Sum]\+\(k = 0\)\%\(Floor[z\/2]\)\(\[Sum]\+\(i = 0\
\)\%k\(\[Sum]\+\(j = 0\)\%k\((Gamma[k + 1\/2]\ Gamma[d\/2 - 1\/2]\ Binomial[
                              z, 2\ k]\ Binomial[k, i]\ Binomial[k, 
                              j]\ \((\(-1\))\)\^\(i + j\)\ pp\^\(i + j - z\)\ \
TLR[d, pp, {v + i, w + j, x + z - 2\ i, y + z - 2\ j, 0}, {nm1, nm2, nm3, 
                                nm4, {0, 0}}])\)/\((Gamma[1\/2]\ Gamma[
                              k + d\/2 - 1\/2])\)\)\)]])\);\)\)], "Input"],

Cell[" OLD trule[10]:", "Text"],

Cell["\<\
tlrule[10] = 
(* (9) and (10) *)
TLR[d_, pp_, {v_,w_,x_?PQ,y_?PQ,z_}, {{n1_?PQ,m1_},{n2_?PQ,m2_}, 
                                {0,_},{0,_},nm5_}] :>  
( TLRComment[\"tlrule10\"]; 0 /; OddQ[x+y] );\
\>", "Text"],

Cell["New tlrule[10]:", "Text"],

Cell["\<\
tlrule[10] = 
   TLR[d_, pp_, {v_, w_, (x_)?PNQ, (y_)?PNQ, z_}, 
     {{(n1_)?PQ, m1_}, {(n2_)?PQ, m2_}, {0, _}, {0, _}, nm5_}] :> \
(TLRComment[\"tlrule10\"]; 0 /; OddQ[x + y]); \
\>", "Input"],

Cell["\<\
Bug in  tlrule[11] and tlrule[12] fixed on Feb. 26th 1998 by \
R.Scharf 
with thanks to York Schroeder for locating the bug.\
\>", "Text"],

Cell["\<\

tlrule[11] = 
TLR[d_, pp_, {v_,w_,x_?PQ,y_?PQ,z_}, {nm1_,nm2_,{0,_},{0,_},{(n5_)?PQ, m5_}}] \
:> 
(
TLRComment[\"tlrule11\"];
 ExpandMaybe[
 pp/(d+x+y-2)( (x-1) TLR[d, pp, {v+1,w,x-2,y,z}, 
                         {{n1,m1},{n2,m2},{0,0},{0,0},{n5,m5}}] +
                   y TLR[d, pp, {v,w,x-1,y-1,z+1}, 
                         {nm1,nm2,{0,0},{0,0},{n5,m5}}]
             )
           ] /; EvenQ[x+y] && x > 1
);\
\>", "Input"],

Cell["\<\

tlrule[12] = 
TLR[d_, pp_, {v_,w_,x_?PQ,y_?PQ,z_}, {nm1_,nm2_,{0,_},{0,_},{(n5_)?PQ, m5_}}] \
:> 
(
TLRComment[\"tlrule12\"];
 ExpandMaybe[
 pp/(d+x+y-2)( (y-1) TLR[d, pp, {v,w+1,x,y-2,z}, 
                         {{n1,m1},{n2,m2},{0,0},{0,0},{n5,m5}}] +
                   x TLR[d, pp, {v,w,x-1,y-1,z+1}, 
                         {nm1,nm2,{0,0},{0,0},{n5,m5}}]
             )
           ] /; EvenQ[x+y] && y > 1
);\
\>", "Input"],

Cell["(*symm.   1 <-> 3  &&  2 <-> 4 *)", "Text"],

Cell[BoxData[
    \(\(tlrule[13] = 
        TLR[d_, pp_, {0, 0, 0, y_?PQ, 0}, {nm1_, {0, 0}, nm3_, {n4_?PQ, m4_}, 
              nm5_}] \[RuleDelayed] \((TLRComment["\<tlrule13\>"]; \
\[Sum]\+\(i = 0\)\%y Binomial[y, i]\ pp\^\(y - i\)\ \((\(-1\))\)\^i\ TLR[d, 
                  pp, {0, 0, 0, i, 0}, {nm3, {n4, m4}, nm1, {0, 0}, 
                    nm5}])\);\)\)], "Input"],

Cell["(*symm.   1 <-> 3  &&  2 <-> 4 *)", "Text"],

Cell[BoxData[
    \(\(tlrule[14] = 
        TLR[d_, pp_, {0, 0, x_?PQ, 0, 0}, {{0, 0}, nm2_, {n3_?PQ, m3_}, nm4_, 
              nm5_}] \[RuleDelayed] \((TLRComment["\<tlrule14\>"]; \
\[Sum]\+\(i = 0\)\%x Binomial[x, i]\ pp\^\(x - i\)\ \((\(-1\))\)\^i\ TLR[d, 
                  pp, {0, 0, i, 0, 0}, {{n3, m3}, nm4, {0, 0}, nm2, 
                    nm5}])\);\)\)], "Input"],

Cell["(* 0's *)", "Text"],

Cell[BoxData[
    \(\(tlrule[15] = 
        HoldPattern[
            TLR[d_, pp_, {v_, w_, x_, y_, 
                z_}, {{n1_, 0}, {n2_, 
                  0}, {0, _}, {0, _}, {0, _}}]] \[RuleDelayed] \
\((TLRComment["\<tlrule15\>"]; 0)\);\)\)], "Input"],

Cell[BoxData[
    \(\(tlrule[16] = 
        TLR[d_, pp_, {0, 0, x_?PQ, y_?PQ, 
              0}, {{n1_, _}, {n2_, _}, {0, _}, {0, _}, {0, _}}] \
\[RuleDelayed] \((TLRComment["\<tlrule16\>"]; 
            0 /; OddQ[x] || OddQ[y])\);\)\)], "Input"],

Cell["bug in tlrule[17] fixedApril 23rd 1999", "Text",
  CellTags->"tlrule17"],

Cell[BoxData[
    \(\(tlrule[17] = 
        TLR[d_, pp_, {a_?PQ, b_, 0, de_, 
              0}, {{0, _}, {n2_, m2_}, {0, _}, {n4_, m4_}, {n5_, 
                m5_}}] \[RuleDelayed] \((TLRComment["\<tlrule17\>"]; \[Sum]\+\
\(i = 0\)\%a\(\[Sum]\+\(j = 0\)\%i Binomial[a, i]\ Binomial[i, 
                    j]\ \((1 + \((\(-1\))\)\^j)\)\ 2\^\(j - 1\)\ TLR[d, 
                    pp, {a - i, b + i - j, 0, de, 
                      j}, {{n5, m5}, {n2, m2}, {0, 0}, {n4, m4}, {0, 
                        0}}]\))\);\)\)], "Input"],

Cell[BoxData[
    \(\(tlrule[18] = 
        TLR[d_, pp_, 
            dp_, {a_?PNQ, b_?PNQ}, {1, 0, 0, 0, 
              0}, {{0, 0}, {n2_, m2_}, {n3_, m3_}, {n4_, m4_}, {n5_, 
                m5_}}] \[RuleDelayed] \((TLRComment["\<tlrule18\>"]; \[Sum]\+\
\(i = 0\)\%a\(\[Sum]\+\(j = 0\)\%b Binomial[a, i]\ Binomial[b, 
                    j]\ \((\((pp + m3\^2)\)\ TLR[d, pp, 
                          dp, {i, 
                            j}, {{n3, m3}, {n4, m4}, {0, 0}, {n2, m2}, {n5, 
                              m5}}] + 
                      TLR[d, pp, 
                        dp, {i, 
                          j}, {{n3 - 1, m3}, {n4, m4}, {0, 0}, {n2, m2}, {n5, 
                            m5}}] - 
                      2\ TLR[d, pp, 
                          dp, {i, j}, {0, 0, 1, 0, 
                            0}, {{n3, m3}, {n4, m4}, {0, 0}, {n2, m2}, {n5, 
                              m5}}])\)\ \((\(-1\))\)\^\(i + j\)\ dp\^\(a + b \
- i - j\)\))\);\)\)], "Input"],

Cell[BoxData[
    \(\(tlrule[19] = 
        TLR[d_, pp_, 
            dp_, {a_?PNQ, b_?PNQ}, {0, 1, 0, 0, 
              0}, {{n1_, m1_}, {0, 0}, {n3_, m3_}, {n4_, m4_}, {n5_, 
                m5_}}] \[RuleDelayed] \((TLRComment["\<tlrule18\>"]; \[Sum]\+\
\(i = 0\)\%a\(\[Sum]\+\(j = 0\)\%b Binomial[a, i]\ Binomial[b, 
                    j]\ \((\((pp + m4\^2)\)\ TLR[d, pp, 
                          dp, {i, 
                            j}, {{n3, m3}, {n4, m4}, {n1, m1}, {0, 0}, {n5, 
                              m5}}] + 
                      TLR[d, pp, 
                        dp, {i, 
                          j}, {{n3, m3}, {n4 - 1, m4}, {n1, m1}, {0, 0}, {n5, 
                            m5}}] - 
                      2\ TLR[d, pp, 
                          dp, {i, j}, {0, 0, 0, 1, 
                            0}, {{n3, m3}, {n4, m4}, {n1, m1}, {0, 0}, {n5, 
                              m5}}])\)\ \((\(-1\))\)\^\(i + j\)\ dp\^\(a + b \
- i - j\)\))\);\)\)], "Input"],

Cell["(* q1.q1  --> q1.p *)", "Text"],

Cell[BoxData[
    \(\(tlrule[20] = 
        TLR[d_, pp_, {v_?PQ, w_, x_, y_, 0}, {{0, _}, nm2_, {0, _}, 
              nm4_, {n5_?PQ, 
                m5_}}] \[RuleDelayed] \((TLRComment["\<tlrule20\>"]; \[Sum]\+\
\(j = 0\)\%x\(\[Sum]\+\(i = 0\)\%v\(\[Sum]\+\(r = 0\)\%i Binomial[i, 
                      r]\ Binomial[v, i]\ Binomial[x, j]\ 2\^r\ TLR[d, 
                      pp, {v - i, \(-r\) + i + w, j, x - j + y, 
                        r}, {{n5, m5}, nm2, 0, nm4, 0}]\)\))\);\)\)], "Input"],

Cell[BoxData[
    \(\(tlrule[21] = 
        TLR[d_, pp_, {v_, w_?PQ, x_, y_, 0}, {nm1_, {0, _}, nm3_, {0, _}, 
              nm5_}] \[RuleDelayed] \((TLRComment["\<tlrule21\>"]; 
            TLR[d, pp, {w, v, y, x, 0}, {{0, 0}, nm1, {0, 0}, nm3, 
                nm5}])\);\)\)], "Input"],

Cell[BoxData[
    \(\(tlrule[22] = 
        TLR[d_, pp_, 
            dp_, {0, b_?PNQ}, {0, w_?PNQ, 0, y_?PNQ, 1}, {nm1_, nm2_, nm3_, 
              nm4_, {0, 
                0}}] \[RuleDelayed] \((TLRComment["\<tlrule22\>"]; \(1\/pp\) \
\((TLR[d, pp, dp, {0, b}, {0, w, 1, y + 1, 0}, {nm1, nm2, nm3, 
                    nm4, {0, 0}}])\))\);\)\)], "Input"],

Cell[TextData[{
  "Using  FeynCalc : \n\n",
  Cell[BoxData[
      \(TIDL[{{k1, \[Mu]}, {k1, \[Nu]}}, {p}]\)], "Input"],
  "\n",
  Cell[BoxData[
      FormBox[
        RowBox[{
          FractionBox[
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      FormBox[
                        RowBox[{
                          FormBox["k1",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}],
                        "TraditionalForm"], ")"}], "2"], "-", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["k1",
                        "TraditionalForm"], "2"], " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], "2"]}]}], ")"}]}], 
            RowBox[{\((1 - D)\), " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "2"]}]], "-", 
          FractionBox[
            RowBox[{
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"D", " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        FormBox[
                          RowBox[{
                            FormBox["k1",
                              "TraditionalForm"], "\[NoBreak]", 
                            "\[CenterDot]", "\[NoBreak]", 
                            FormBox["p",
                              "TraditionalForm"]}],
                          "TraditionalForm"], ")"}], "2"]}], "-", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["k1",
                        "TraditionalForm"], "2"], " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], "2"]}]}], ")"}]}], 
            RowBox[{\((1 - D)\), " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "4"]}]]}], TraditionalForm]], 
    "Output"],
  "\n",
  Cell[BoxData[
      \(Collect2[Calc[\ FVD[OPEDelta, \[Mu]] FVD[k2, \[Nu]] %], {k1, k2}]\)], 
    "Input"],
  "\n",
  Cell[BoxData[
      FormBox[
        RowBox[{
          FractionBox[
            RowBox[{
              RowBox[{
                FormBox["k2",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"]}], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  FormBox[
                    RowBox[{
                      FormBox["k1",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["p",
                        "TraditionalForm"]}],
                    "TraditionalForm"], ")"}], "2"]}], 
            RowBox[{\((1 - D)\), " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "2"]}]], "-", 
          FractionBox[
            RowBox[{"D", " ", 
              RowBox[{
                FormBox["k2",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], " ", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  FormBox[
                    RowBox[{
                      FormBox["k1",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["p",
                        "TraditionalForm"]}],
                    "TraditionalForm"], ")"}], "2"]}], 
            RowBox[{\((1 - D)\), " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "4"]}]], "-", 
          FractionBox[
            RowBox[{
              SuperscriptBox[
                FormBox["k1",
                  "TraditionalForm"], "2"], " ", 
              RowBox[{
                FormBox["k2",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"]}]}], \(1 - D\)], "+", 
          FractionBox[
            RowBox[{
              SuperscriptBox[
                FormBox["k1",
                  "TraditionalForm"], "2"], " ", 
              RowBox[{
                FormBox["k2",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], " ", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}]}], 
            RowBox[{\((1 - D)\), " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "2"]}]]}], TraditionalForm]], 
    "Output"],
  "\n"
}], "Text"],

Cell[BoxData[
    \(\(tlrule[23] = 
        TLR[d_, pp_, 
            dp_, {1, b_?PNQ}, {0, w_?PNQ, 0, y_?PNQ, 1}, {nm1_, nm2_, nm3_, 
              nm4_, {0, 
                0}}] \[RuleDelayed] \((TLRComment["\<tlrule23\>"]; \
\(1\/\(\((1 - d)\) pp\)\) \((TLR[d, pp, 
                    dp, {0, b + 1}, {0, w, 2, y, 0}, {nm1, nm2, nm3, 
                      nm4, {0, 0}}])\) - \(\(d\ \ dp\)\/\(\((1 - d)\) 
                      pp\^2\)\) \((TLR[d, pp, 
                    dp, {0, b}, {0, w, 2, y + 1, 0}, {nm1, nm2, nm3, 
                      nm4, {0, 0}}])\) - \(1\/\(1 - d\)\) 
                TLR[d, pp, 
                  dp, {0, b + 1}, {1, w, 0, y, 0}, {nm1, nm2, nm3, 
                    nm4, {0, 0}}] + \(dp\/\(\((1 - d)\) pp\)\) 
                TLR[d, pp, 
                  dp, {0, b}, {1, w, 0, y + 1, 0}, {nm1, nm2, nm3, 
                    nm4, {0, 0}}])\);\)\)], "Input"],

Cell[TextData[{
  Cell[BoxData[
      \(TIDL[{{k1, \[Mu]}, {k1, \[Nu]}, {k1, \[Rho]}}, {p}]\)], "Input"],
  "\n",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"-", 
            FractionBox[
              RowBox[{
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]], " ", 
                RowBox[{
                  FormBox["k1",
                    "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                  "\[NoBreak]", 
                  FormBox["p",
                    "TraditionalForm"]}], " ", 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{\((D + 2)\), " ", 
                      SuperscriptBox[
                        RowBox[{"(", 
                          FormBox[
                            RowBox[{
                              FormBox["k1",
                                "TraditionalForm"], "\[NoBreak]", 
                              "\[CenterDot]", "\[NoBreak]", 
                              FormBox["p",
                                "TraditionalForm"]}],
                            "TraditionalForm"], ")"}], "2"]}], "-", 
                    RowBox[{"3", " ", 
                      SuperscriptBox[
                        FormBox["k1",
                          "TraditionalForm"], "2"], " ", 
                      SuperscriptBox[
                        FormBox["p",
                          "TraditionalForm"], "2"]}]}], ")"}]}], 
              RowBox[{\((1 - D)\), " ", 
                SuperscriptBox[
                  FormBox["p",
                    "TraditionalForm"], "6"]}]]}], "+", 
          FractionBox[
            RowBox[{
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Mu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              RowBox[{
                FormBox["k1",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], " ", 
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      FormBox[
                        RowBox[{
                          FormBox["k1",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}],
                        "TraditionalForm"], ")"}], "2"], "-", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["k1",
                        "TraditionalForm"], "2"], " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], "2"]}]}], ")"}]}], 
            RowBox[{\((1 - D)\), " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "4"]}]], "+", 
          FractionBox[
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Rho]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Nu]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{
                FormBox["k1",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], " ", 
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      FormBox[
                        RowBox[{
                          FormBox["k1",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}],
                        "TraditionalForm"], ")"}], "2"], "-", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["k1",
                        "TraditionalForm"], "2"], " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], "2"]}]}], ")"}]}], 
            RowBox[{\((1 - D)\), " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "4"]}]], "+", 
          FractionBox[
            RowBox[{
              SuperscriptBox["g", 
                RowBox[{
                  FormBox[
                    FormBox["\[Mu]",
                      "TraditionalForm"],
                    "TraditionalForm"], "\[NoBreak]", 
                  FormBox[
                    FormBox["\[Nu]",
                      "TraditionalForm"],
                    "TraditionalForm"]}]], " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], 
                FormBox[
                  FormBox["\[Rho]",
                    "TraditionalForm"],
                  "TraditionalForm"]], " ", 
              RowBox[{
                FormBox["k1",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], " ", 
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox[
                    RowBox[{"(", 
                      FormBox[
                        RowBox[{
                          FormBox["k1",
                            "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                          "\[NoBreak]", 
                          FormBox["p",
                            "TraditionalForm"]}],
                        "TraditionalForm"], ")"}], "2"], "-", 
                  RowBox[{
                    SuperscriptBox[
                      FormBox["k1",
                        "TraditionalForm"], "2"], " ", 
                    SuperscriptBox[
                      FormBox["p",
                        "TraditionalForm"], "2"]}]}], ")"}]}], 
            RowBox[{\((1 - D)\), " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "4"]}]]}], TraditionalForm]], 
    "Output"],
  "\n",
  Cell[BoxData[
      \(Collect2[
        Calc[\ FVD[OPEDelta, \[Mu]] FVD[OPEDelta, \[Nu]] 
            FVD[k2, \[Rho]] %], {k1, k2}]\)], "Input"],
  "\n",
  Cell[BoxData[
      FormBox[
        RowBox[{
          FractionBox[
            RowBox[{"2", " ", 
              RowBox[{
                FormBox["k2",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"]}], " ", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  FormBox[
                    RowBox[{
                      FormBox["k1",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["p",
                        "TraditionalForm"]}],
                    "TraditionalForm"], ")"}], "3"]}], 
            RowBox[{\((1 - D)\), " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "4"]}]], "-", 
          FractionBox[
            RowBox[{\((D + 2)\), " ", 
              RowBox[{
                FormBox["k2",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  FormBox[
                    RowBox[{
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["p",
                        "TraditionalForm"]}],
                    "TraditionalForm"], ")"}], "2"], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  FormBox[
                    RowBox[{
                      FormBox["k1",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["p",
                        "TraditionalForm"]}],
                    "TraditionalForm"], ")"}], "3"]}], 
            RowBox[{\((1 - D)\), " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "6"]}]], "-", 
          FractionBox[
            RowBox[{"2", " ", 
              SuperscriptBox[
                FormBox["k1",
                  "TraditionalForm"], "2"], " ", 
              RowBox[{
                FormBox["k2",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"]}], " ", 
              RowBox[{
                FormBox["\[CapitalDelta]",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], " ", 
              RowBox[{
                FormBox["k1",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}]}], 
            RowBox[{\((1 - D)\), " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "2"]}]], "+", 
          FractionBox[
            RowBox[{"3", " ", 
              SuperscriptBox[
                FormBox["k1",
                  "TraditionalForm"], "2"], " ", 
              RowBox[{
                FormBox["k2",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}], " ", 
              SuperscriptBox[
                RowBox[{"(", 
                  FormBox[
                    RowBox[{
                      FormBox["\[CapitalDelta]",
                        "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                      "\[NoBreak]", 
                      FormBox["p",
                        "TraditionalForm"]}],
                    "TraditionalForm"], ")"}], "2"], " ", 
              RowBox[{
                FormBox["k1",
                  "TraditionalForm"], "\[NoBreak]", "\[CenterDot]", 
                "\[NoBreak]", 
                FormBox["p",
                  "TraditionalForm"]}]}], 
            RowBox[{\((1 - D)\), " ", 
              SuperscriptBox[
                FormBox["p",
                  "TraditionalForm"], "4"]}]]}], TraditionalForm]], "Output"]
}], "Text"],

Cell[BoxData[
    \(\(tlrule[24] = 
        TLR[d_, pp_, 
            dp_, {2, b_?PNQ}, {0, w_?PNQ, 0, y_?PNQ, 1}, {nm1_, nm2_, nm3_, 
              nm4_, {0, 0}}] \[RuleDelayed] \((TLRComment["\<tlrule24\>"]; 
            2  dp \(\( 1\/\(\((1 - d)\) pp\^2\)\) \((TLR[d, pp, 
                      dp, {0, b + 1}, {0, w, 3, y, 0}, {nm1, nm2, nm3, 
                        nm4, {0, 0}}])\)\) - \(\(\((d + 
                          2)\)\ \ dp\^2\)\/\(\((1 - d)\) pp\^3\)\) \((TLR[d, 
                    pp, dp, {0, b}, {0, w, 3, y + 1, 0}, {nm1, nm2, nm3, 
                      nm4, {0, 0}}])\) - \(\(2  dp\)\/\(\((1 - d)\) pp\)\) 
                TLR[d, pp, 
                  dp, {0, b + 1}, {1, w, 1, y, 0}, {nm1, nm2, nm3, 
                    nm4, {0, 0}}] + \(\(3  dp\^2\)\/\(\((1 - d)\) pp\^2\)\) 
                TLR[d, pp, 
                  dp, {0, b}, {1, w, 1, y + 1, 0}, {nm1, nm2, nm3, 
                    nm4, {0, 0}}])\);\)\)], "Input"],

Cell["Symmetry 1 <--> 2; added February 21 1999 by R.M.", "Text"],

Cell[BoxData[
    \(\(\(tlrule[25] = 
        TLR[dim_, 
            pp_, \ {a_, \ b_, \ \((c_)\)?PNQ, \ \((d_)\)?PQ, \ 
              0}, \ \(\({nm1_, \ nm2_, \ nm3_, \ {0, \ _}, \ 
              nm5_}\)\(\n\)\)\ \ \ ]\  :> \((TLRComment["\<tlrule25\>"]; \ 
              TLR[dim, \ 
                pp, \ {b, a, \ d, \ c, \ 0}, \ {nm2, \ nm1, \ {0, 0}, \ 
                  nm3, \ nm5}])\)\  /; \ d\  > \ c;\)\(\n\)\)\)], "Input"],

Cell[BoxData[
    \(\(TLRules = Array[tlrule, 25];\)\)], "Input"],

Cell[BoxData[
    \(\(TLRules2 = Drop[TLRules, {9, 9}];\)\)], "Input"],

Cell[BoxData[
    \(\(TLR[__, {{_, _}, {0, _}, {_, _}, {0, _}, {_, 0}}] := 0;\)\)], "Input"],

Cell[BoxData[
    \(\(TLR[__, {{0, _}, {_, _}, {0, _}, {_, _}, {_, 0}}] := 0;\)\)], "Input"],

Cell[BoxData[
    \(\(TLR[__, {{0, _}, {_, _}, {_, 0}, {_, _}, {0, _}}] := 0;\)\)], "Input"],

Cell[BoxData[
    \(\(TLR[__, {{_, 0}, {_, _}, {0, _}, {_, _}, {0, _}}] := 0;\)\)], "Input"],

Cell[BoxData[
    \(\(TLR[__, {{_, _}, {_, 0}, {_, _}, {0, _}, {0, _}}] := 0;\)\)], "Input"],

Cell[BoxData[
    \(\(TLR[__, {{_, _}, {0, _}, {_, _}, {_, 0}, {0, _}}] := 0;\)\)], "Input"],

Cell[BoxData[
    \(\(SetAttributes[seeetDelayed, HoldAll];\)\)], "Input"],

Cell[BoxData[
    \(seeetDelayed[TLR[a__], b_] := SetDelayed @@ {TLR[a], b}\)], "Input"],

Cell[BoxData[
    \(\(lis1 = 
        Permutations[{{0, _}, {0, _}, {0, _}, {_, _}, {_, 0}}];\)\)], "Input"],

Cell[BoxData[
    \(\(lis2 = 
        Table[TLR[__, 
                lis1\[LeftDoubleBracket]
                  i\[RightDoubleBracket]] \[RuleDelayed] 0, {i, 
              Length[lis1]}] /. 
          RuleDelayed \[Rule] seeetDelayed;\)\)], "Input"],

Cell[BoxData[
    \(\(Clear[lis1, lis2];\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["T*IS", "Subsection",
  CellTags->"T1.7.2.1"],

Cell["\<\
These functions below are never used.
They can serve as tests.\
\>", "Text"],

Cell[BoxData[
    \(TJIS[d_, 
        Power[M_, \ 
          2], \ {{\[Alpha]_, 0}, {\[Beta]_, 0}, {\[Gamma]_, 
            0}}] := \(\(SMu\^2\) \((\(\((\(-1\))\)\^\(\[Alpha] + \[Beta] + \
\[Gamma] + 1\)\/\((\(-\ M\^2\))\)\^\(\[Alpha] + \[Beta] + \[Gamma] - 4\)\) 
            Exp[\(-I\)\ Pi\ \((d - 
                    4)\)]\ \(Gamma[\[Alpha] + \[Beta] + \[Gamma] - 
                    d]\/Gamma[\(3\/2\) 
                      d - \[Alpha] - \[Beta] - \[Gamma]]\) \(Gamma[
                  d\/2 - \[Alpha]]\/Gamma[\[Alpha]]\) \(Gamma[
                  d\/2 - \[Beta]]\/Gamma[\[Beta]]\) 
            Gamma[d\/2 - \[Gamma]]\/Gamma[\[Gamma]])\)\(\n\)\)\)], "Input"],

Cell[BoxData[
    \(TJIS[d_, 
        Power[M_, \ 
          2], \ {{\[Beta]_, 0}, {\[Gamma]_, 0}, {\[Alpha]_, M_}}] := \n\t
      TJIS[d, M^2, {{\[Alpha], M}, {\[Beta], 0}, {\[Gamma], 0}}]\)], "Input"],

Cell[BoxData[
    \(TJIS[d_, 
        Power[M_, \ 
          2], \ {{\[Beta]_, 0}, {\[Alpha]_, M_}, {\[Gamma]_, 0}}] := \n\t
      TJIS[d, M^2, {{\[Alpha], M}, {\[Beta], 0}, {\[Gamma], 0}}]\)], "Input"],

Cell[BoxData[
    \(TJIS[d_, 
        Power[M_, \ 
          2], \ {{\[Alpha]_, M_}, {\[Beta]_, 0}, {\[Gamma]_, 
            0}}] := \n\(SMu\^2\/\((M\^2)\)\^\(\[Alpha] + \[Beta] + \[Gamma] - \
4\)\) \(\((\(-1\))\)\^\(\[Alpha] + \[Beta] + \[Gamma] + 
              1\)\) \(Gamma[\[Alpha] + \[Beta] + \[Gamma] - 
                d]\/\(Gamma[\[Alpha]] Gamma[\[Beta]] 
              Gamma[\[Gamma]]\)\) \(\(Gamma[d\/2 - \[Gamma]] 
              Gamma[d\/2 - \[Beta]]\)\/Gamma[
              d - \[Beta] - \[Gamma]]\) \(Gamma[\[Beta] + \[Gamma] - d\/2] \
Gamma[2\ d - \[Alpha] - 2  \[Beta] - 2  \[Gamma]]\)\/Gamma[\(3  d\)\/2 - \
\[Alpha] - \[Beta] - \[Gamma]]\)], "Input"],

Cell[BoxData[
    \(TJIS[d_, 0, \ {{\[Beta]_, 0}, {\[Alpha]_, M_}, {\[Gamma]_, 0}}] := 
      TJIS[d, 0, {{\[Alpha], M}, {\[Beta], 0}, {\[Gamma], 0}}]\)], "Input"],

Cell[BoxData[
    \(TJIS[d_, 0, \ {{\[Beta]_, 0}, {\[Gamma]_, 0}, {\[Alpha]_, M_}}] := 
      TJIS[d, 0, {{\[Alpha], M}, {\[Beta], 0}, {\[Gamma], 0}}]\)], "Input"],

Cell[BoxData[
    \(TJIS[d_, 
        0, \ {{\[Alpha]_, M_}, {\[Beta]_, 0}, {\[Gamma]_, 
            0}}] := \n\(SMu\^2\/\((M\^2)\)\^\(\[Alpha] + \[Beta] + \[Gamma] - \
4\)\) \(\((\(-1\))\)\^\(\[Alpha] + \[Beta] + \[Gamma] + 
              1\)\) \(Gamma[\[Alpha] + \[Beta] + \[Gamma] - 
                d]\/\(Gamma[\[Alpha]] Gamma[\[Beta]] 
              Gamma[\[Gamma]]\)\) \((Gamma[\[Beta] + \[Gamma] - d/2] 
              Gamma[d/2 - \[Beta]] Gamma[d/2 - \[Gamma]])\)/
          Gamma[d/2]\)], "Input"],

Cell[BoxData[
    \(TJIS[d_, 0, \ {{\[Beta]_, M_}, {\[Alpha]_, 0}, {\[Gamma]_, M_}}] := 
      TJIS[d, 0, {{\[Alpha], 0}, {\[Beta], M}, {\[Gamma], M}}]\)], "Input"],

Cell[BoxData[
    \(TJIS[d_, 0, \ {{\[Beta]_, M_}, {\[Gamma]_, M_}, {\[Alpha]_, 0}}] := 
      TJIS[d, 0, {{\[Alpha], 0}, {\[Beta], M}, {\[Gamma], M}}]\)], "Input"],

Cell[BoxData[
    \(TJIS[d_, 
        0, \ {{\[Alpha]_, 0}, {\[Beta]_, M_}, {\[Gamma]_, 
            M_}}] := \n\(SMu\^2\/\((M\^2)\)\^\(\[Alpha] + \[Beta] + \[Gamma] \
- 4\)\) \(\((\(-1\))\)\^\(\[Alpha] + \[Beta] + \[Gamma] + 
              1\)\) \(Gamma[\[Alpha] + \[Beta] + \[Gamma] - 
                d]\/\(Gamma[\[Beta]] Gamma[\[Gamma]]\)\) \(Gamma[
              d\/2 - \[Alpha]]\/Gamma[
              d\/2]\) \(Gamma[\[Alpha] + \[Gamma] - d\/2] Gamma[\[Alpha] + \
\[Beta] - d\/2]\)\/Gamma[2  \[Alpha] + \[Beta] + \[Gamma] - d]\)], "Input"],

Cell[BoxData[
    \(TJIS[d_, 
        0, {{\[Alpha]_, 0}, {\[Beta]_, M_}, {\[Gamma]_, 
            M_}}] := \((SMu\^2\ \((\(-1\))\)\^\(\[Alpha] + \[Beta] + \[Gamma] \
+ 1\)\ Gamma[\[Alpha] + \[Beta] + \[Gamma] - d]\ Gamma[
              d\/2 - \[Alpha]]\ \((Gamma[\[Alpha] + \[Gamma] - 
                    d\/2]\ Gamma[\[Alpha] + \[Beta] - 
                    d\/2])\))\)/\((\((M\^2)\)\^\(\[Alpha] + \[Beta] + \
\[Gamma] - 4\)\ \((Gamma[\[Beta]]\ Gamma[\[Gamma]])\)\ Gamma[d\/2]\ Gamma[
              2\ \[Alpha] + \[Beta] + \[Gamma] - d])\)\)], "Input"],

Cell["(2.65)", "Text"],

Cell[BoxData[
    \(TAIS[d_, 
        0, {{1, M_}}] := \ \(-I\)\ \ M\^2\ \ SMu\ Gamma[\((4 - d)\)/2 - 
            1]\)], "Input"],

Cell[BoxData[
    \(\(TBIS[d_, 0, {{\[Alpha]_, 0}, {\[Beta]_, 0}}] := 0;\)\)], "Input"],

Cell[BoxData[
    \(TBIS[d_, 
        Power[M_, \ 
          2], {{\[Alpha]_, 0}, {\[Beta]_, 
            0}}] := \(\(\(I\)\(\ \)\) \(SMu\)\(\ \)\(*\)\((\(\((\(-1\))\)\^\(\
\[Alpha] + \[Beta]\)\/\((\(-\ M\^2\))\)\^\(\[Alpha] + \[Beta] - 2\)\) 
            Exp[\(-I\)\ Pi\ \((d - 4)\)/
                  2]\ \(Gamma[\[Alpha] + \[Beta] - d/2]\/\(Gamma[\[Alpha]] 
                  Gamma[\[Beta]]\)\) \(Gamma[d/2 - \[Alpha]] Gamma[d/2 - \
\[Beta]]\)\/Gamma[d - \[Alpha] - \[Beta]])\)\(\n\)\)\)], "Input"],

Cell[BoxData[
    \(TJIS[d_Symbol, M_\^2, {{1, M_}, {1, M_}, {1, M_}}] := 
      M\^2\ SMu\^2\ \((\(-\(6\/\((\(-4\) + d)\)\^2\)\) + 
            17\/\(2\ \((\(-4\) + d)\)\) + 
            1\/8\ \((\(-59\) - 
                  2\ \[Pi]\^2)\) + \((\(-4\) + d)\)\^3\ TJI111e + \(1\/1920\) \
\((\((\(-4\) + d)\)\^2\ \((16755 - 4750\ \[Pi]\^2 - 14\ \[Pi]\^4 + 
                      3840\ \[Pi]\^2\ Log[2] - 12080\ Zeta[3])\))\) + 
            1\/96\ \((\(-4\) + d)\)\ \((195 + 98\ \[Pi]\^2 - 
                  48\ Zeta[3])\))\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["TComment", "Subsection",
  CellTags->"T1.7.3.1"],

Cell[BoxData[
    \(\($CommentNotebook = True;\)\)], "Input"],

Cell[BoxData[
    \(TComment[s_String, b_] := 
      If[If[\(\(\[InvisibleSpace]\)\(! \((ValueQ[$TarcerRecursed])\)\)\), \
$TarcerRecursed = {}]; \(\(\[InvisibleSpace]\)\(! \((MemberQ[$TarcerRecursed, \
{s, b}])\)\)\), AppendTo[$TarcerRecursed, {s, b}]; 
          If[$Notebooks && $CommentNotebook === True, 
            If[$commentnb === False, $commentnb = True; $cnb = 
                NotebookCreate[
                  StyleDefinitions \[Rule] "\<NaturalColor.nb\>", 
                  Magnification \[Rule] 1.5]; 
              SetOptions[$cnb, 
                WindowMargins \[Rule] {{0, Automatic}, {Automatic, 0}}]; 
              SetOptions[$cnb, WindowSize \[Rule] {500, 300}]; 
              NotebookWrite[$cnb, 
                Cell[BoxData[
                    ToBoxes[b /. TFIC \[Rule] TFI, 
                      StandardForm]], "\<Input\>", 
                  CellDingbat \[Rule] s <> "\< \>"]], 
              NotebookWrite[$cnb, 
                Cell[BoxData[
                    ToBoxes[b /. TFIC \[Rule] TFI, 
                      StandardForm]], "\<Input\>", 
                  CellDingbat \[Rule] s <> "\< \>"]]], 
            Print[s, 
              b /. {TFIC \[RuleDelayed] TFI, TVIC \[RuleDelayed] TVI, 
                  TJIC \[RuleDelayed] TJI, TBIC \[RuleDelayed] TBI, 
                  TAIC \[RuleDelayed] TAI}]]] /; $Comment === True\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["$TTable", "Subsection",
  CellTags->"$TTable"],

Cell[BoxData[
    \(\($TTable = {};\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["TarcerRecurse", "Subsection",
  CellTags->"T1.7.4.1"],

Cell[BoxData[
    \(\(FEPrint[a__String] := 
        Print[a]\  /; \ $Notebooks === False;\)\)], "Input"],

Cell[BoxData[
    \(\(FEPrint[a__String] := 
        If[Head[$cnb] === NotebookObject, \n\t\tNotebookWrite[$cnb, 
              Cell[StringJoin @@ {a}, "\<SmallText\>"]]; \n\t
            SelectionMove[$cnb, After, Cell]] /; $Notebooks === 
            True;\)\)], "Input"],

Cell[BoxData[
    \(\(Options[TarcerRecurse] = {Factor \[Rule] Factor, 
          TimeConstraint -> Infinity, Table :> $TTable, Together -> Together, 
          Replace -> {}};\)\)], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{\(TarcerRecurse[z_, opts___]\), ":=", 
        RowBox[{"Catch", "[", 
          RowBox[{
            StyleBox["Block",
              FontSize->16,
              FontWeight->"Bold",
              FontColor->RGBColor[1, 0.2, 0.2]], "[", 
            
            RowBox[{\({basisdone, factorfun, fertig, dummm1, dummm2, getvars, 
                te, time, cutoff, tog, ttable, facfun, nexp, oldexpr, rec, 
                special, sti, tl, tcheck, tjrd, tvrd, together, 
                vars, $cc = 0, $cv}\), ",", "\n", "\t\t\t", 
              
              RowBox[{\(time\  = \ AbsoluteTime[]\), ";", "\n", 
                "\t\t\t", \(ttable = \(Table /. {opts}\) /. 
                    Options[TarcerRecurse]\), ";", "\n", 
                "\t\t\t", \(If[\(! MatchQ[ttable, {__RuleDelayed}]\), 
                  ttable = {}]\), ";", "\n", 
                "\t\t\t", \(special = \(Replace /. {opts}\) /. 
                    Options[TarcerRecurse]\), ";", "\n", 
                "\t\t\t", \(cutoff = \(TimeConstraint /. {opts}\) /. 
                    Options[TarcerRecurse]\), ";", "\n", 
                "\t\t\t", \(together = \(Together /. {opts}\) /. 
                    Options[TarcerRecurse]\), 
                ";", \(factorfun = \(Factor /. {opts}\) /. 
                    Options[TarcerRecurse]\), ";", \($commentnb = False\), 
                ";", \(tog = \((\((If[#1 > 1001, 
                              If[$Comment, 
                                FEPrint["\<coefficient # \>" <> 
                                    If[ValueQ[$cv], ToString[$cc], "\<\>"] <> 
                                    If[ValueQ[$cv], "\< (\>" <> 
                                        ToString[$cv] <> "\<)\>", "\<\>"] <> \
"\< simplified, \>", ToString[#1]]]]; #2)\) &)\)[LeafCount[#1], \($cc++\); 
                      together[#1]] &\), 
                ";", \(If[\(\(\[InvisibleSpace]\)\(! \
\((ValueQ[$TarcerRecursed])\)\)\), $TarcerRecursed = {}]\), 
                ";", \($TarcerRecursed = {}\), ";", 
                RowBox[{"If", "[", "\[InvisibleSpace]", 
                  
                  RowBox[{\(! \((MemberQ[$ContextPath, \
"\<HighEnergyPhysics`fcloops`ToTFi`\>"])\)\), ",", \(t0 = z\), ",", 
                    RowBox[{"t0", "=", 
                      
                      RowBox[{\(HighEnergyPhysics`fcloops`ToTFi`ToTFi[z]\), "/.",
                         
                        RowBox[{"{", 
                          RowBox[{
                            RowBox[{
                            "HighEnergyPhysics`fcloops`TFi`TFi", 
                              "\[RuleDelayed]", 
                              StyleBox["TFI",
                                FontColor->RGBColor[1, 0.6, 0]]}], 
                            ",", \(HighEnergyPhysics`fcloops`ToTFi`ToTFi \
\[RuleDelayed] Identity\)}], "}"}]}]}]}], "]"}], ";", "\n", 
                "\t\t\t", \(If[ttable =!= {}, t0 = t0 /. ttable]\), ";", "\n",
                 "\t\t\t", 
                RowBox[{"getvars", "=", 
                  RowBox[{
                    RowBox[{"Select", "[", 
                      RowBox[{\(Variables[#1]\), ",", 
                        RowBox[{
                          RowBox[{"MatchQ", "[", 
                            RowBox[{"#1", ",", 
                              RowBox[{
                                RowBox[{
                                  StyleBox["TFI",
                                    FontColor->RGBColor[1, 0.6, 0]], "[", 
                                  "__", "]"}], "|", 
                                RowBox[{
                                  StyleBox["TVI",
                                    FontColor->RGBColor[0.2, 0.4, 0.8]], "[", 
                                  "__", "]"}], "|", \(TJI[__]\), 
                                "|", \(TBI[__]\), "|", \(TAI[__]\)}]}], "]"}],
                           "&"}]}], "]"}], "&"}]}], ";", "\n", 
                "\t\t\t", \(If[\(! FreeQ[t0, TFI]\), 
                  t0 = Collect[t0 = TFIRecurse[t0] /. ttable, getvars[t0], 
                      tog]]\), ";", "\n", 
                "\t\t\t", \(If[\(! FreeQ[t0, 
                      TVI[_Plus, __]]\), \n\t\t\ttvrd = 
                    Select[DownValues[
                          TVR], \(! FreeQ[#, TVI[_ - 2, __]]\) &] /. 
                      TVR -> TVI; \n\t\t\t\tt0 = 
                    Collect[
                      t0 = \(\(\(t0 /. tvrd\) /. tvrd\) /. tvrd\) /. tvrd, 
                      getvars[t0], 
                      tog]; \n\t\t\t\tIf[\(! FreeQ[t0, TVI[_Plus, __]]\), \t
                    t0 = Collect[t0 = t0 //. tvrd, getvars[t0], tog]]]\), ";",
                 "\n", "\t\t\t\t", "\n", 
                "\t\t\t", \(tjrd = 
                  Select[DownValues[TJR], \(! FreeQ[#, TJI[_ - 2, __]]\) &] /. 
                    TJR -> TJI\), ";", "\n", "\t\t\t", "\n", 
                "\t\t\t", \(If[\(! FreeQ[t0, 
                      TJI[_Plus, __]]\), \n\t\t\ttjrd = 
                    Select[DownValues[
                          TJR], \(! FreeQ[#, TJI[_ - 2, __]]\) &] /. 
                      TJR -> TJI; \n\t\t\t\tt0 = 
                    Collect[
                      t0 = \(\(\(t0 /. tjrd\) /. tjrd\) /. tjrd\) /. tjrd, 
                      getvars[t0], 
                      tog]; \n\t\t\t\tIf[\(! FreeQ[t0, TJI[_Plus, __]]\), \t
                    t0 = Collect[t0 = t0 //. tjrd, getvars[t0], tog]]]\), ";",
                 "\n", "\t\t\t", "\n", 
                "\t\t\t", \(rec[expr_, {ti_, tr_}] := \((sti = ToString[ti]; 
                    tcheck = \(\(\[InvisibleSpace]\)\(! \((FreeQ[expr, 
                            ti])\)\)\); oldexpr =  . ; nexp = expr; 
                    While[tcheck === True && oldexpr =!= nexp, 
                      oldexpr = nexp; 
                      If[$Comment, \((If[#1 === 1, 
                                FEPrint[
                                  sti <> "\< level, there is 1 \>" <> sti], 
                                FEPrint[sti <> "\< level, there are \>", 
                                  ToString[#1], "\< \>" <> 
                                    sti <> "\<'s \>"]] &)\)[
                          Count[nexp, 
                            ti[__], \(-1\)]]]; \n\t\t\t\t\n\t\t\t\t\t\tnexp = \
\(nexp /. ti \[Rule] tr\) /. 
                          tr \[Rule] ti; \n\t\t\t\t\t\t\tIf[\(! FreeQ[nexp, 
                            TJI[_Plus, __]]\), 
                        nexp = 
                          nexp //. tjrd]; \n\t\t\t\t\t\t\n\t\t\t\t\t\ttvars = 
                        getvars[nexp]; $cc = 0; $cv = Length[tvars]; 
                      nexp = Collect[nexp, tvars, tog]; \n\t\t\t\t\t\tIf[
                        cutuff =!= Infinity, 
                        If[AbsoluteTime[] - time > cutoff, 
                          Throw[$Failed]]]; \n\t\t\t\t\t\t\n\t\t\t\t\t\t\
tcheck = \[InvisibleSpace]\(! \((FreeQ[nexp, ti])\)\);]; nexp)\)\), 
                ";", \(basisdone = 0\), ";", 
                RowBox[{"tl", "=", 
                  RowBox[{"{", 
                    RowBox[{
                      RowBox[{"{", 
                        RowBox[{
                          StyleBox["TFI",
                            FontColor->RGBColor[1, 0.6, 0]], ",", 
                          StyleBox["TFR",
                            FontColor->RGBColor[0, 0.6, 0]]}], "}"}], ",", 
                      RowBox[{"{", 
                        RowBox[{
                          StyleBox["TVI",
                            FontColor->RGBColor[0.2, 0.4, 0.8]], ",", 
                          StyleBox["TVR",
                            FontColor->RGBColor[0.2, 0.8, 1]]}], "}"}], 
                      ",", \({TJI, TJR}\), ",", \({TBI, TBR}\), 
                      ",", \({TAI, TAR}\)}], "}"}]}], 
                ";", \(Do[
                  t0 = rec[t0, tl\[LeftDoubleBracket]i\[RightDoubleBracket]]; 
                  If[Head[t0] === Plus, 
                    fertig = 
                      Select[t0, \(\(\[InvisibleSpace]\)\(! \((FreeQ[#1, 
                              tl\[LeftDoubleBracket]i, 
                                1\[RightDoubleBracket]])\)\)\)]; 
                    basisdone = basisdone + fertig; t0 = t0 - fertig], {i, 
                    Length[tl]}]\), ";", \(t0 = t0 + basisdone\), 
                ";", \($cv =  . \), 
                ";", \(t0 = 
                  Collect[
                    t0, {TAI[__], TAI[a1__]\ TAI[a2__], TAI[__], TBI[__], 
                      TJI[__], TVI[__], TFI[__]}, factorfun]\), 
                ";", \(If[$Comment, 
                  FEPrint["\<the result has \>", 
                    ToString[
                      Length[t0 + dummm1 + dummm2] - 2], "\< terms\>"]]\), 
                ";", \(If[$Notebooks, 
                  SetSelectedNotebook[EvaluationNotebook[]]]\), ";", "t0"}]}],
             "]"}], "]"}]}], ";"}]], "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["TarcerExpand", "Section",
  CellDingbat->"\[FilledSmallCircle]",
  CellTags->"T1.8.1"],

Cell[BoxData[
    \(\(Options[TarcerExpand] = {\n\t\tTarcerReduce :> $BasisIntegrals, \n\t
          TarcerRecurse -> False, Zeta -> True};\)\)], "Input"],

Cell[BoxData[
    \(\(TarcerExpand[expr_, d_Symbol -> fe_ /; Length[Variables[fe]] === 1, 
          ru___Rule] := TarcerExpand[expr, d\  -> fe, 0, ru];\)\)], "Input"],

Cell[BoxData[
    RowBox[{\(TarcerExpand[expr_, 
        d_Symbol \[Rule] fe_ /; Length[Variables[fe]] === 1, 
        order_Integer?NonNegative, opts___Rule]\), ":=", 
      RowBox[{
        StyleBox["Block",
          FontSize->16,
          FontWeight->"Bold",
          FontColor->RGBColor[1, 0.2, 0.2]], "[", 
        RowBox[{\({}\), ",", 
          RowBox[{"Catch", "[", 
            
            RowBox[{\(e = First[Variables[fe]]\), 
              ";", \({ints, recurse, 
                  zeta2replace} = {TarcerReduce, TarcerRecurse, Zeta} /. 
                  Flatten[{opts, Options[TarcerExpand]}]\), 
              ";", \(If[
                zeta2replace =!= True && 
                  MemberQ[$ContextPath, \
"\<HighEnergyPhysics`general`Zeta2`\>"], z2rule = {}, 
                z2rule = \[Pi]\^2 \[RuleDelayed] 
                    6\ ToExpression["\<HighEnergyPhysics`general`Zeta2`Zeta2\>\
"]]\), ";", \(ints = Flatten[ints]\), 
              ";", \(If[recurse =!= True, ta0 = expr, 
                ta0 = TarcerRecurse[expr]]\), ";", 
              RowBox[{"t1", "=", 
                RowBox[{\(ta0 /. ints\), "/.", 
                  RowBox[{
                    InterpretationBox["any_",
                      PreFactor[ 
                        Pattern[ FE`Conversion`Private`any, 
                          Blank[ ]]],
                      Editable->False], "\[RuleDelayed]", 
                    InterpretationBox["any",
                      PreFactor[ FE`Conversion`Private`any],
                      Editable->False]}]}]}], 
              ";", \(vars = Select[Variables[t1], Length[#1] > 0 &]\), ";", 
              RowBox[{"If", "[", 
                RowBox[{
                  RowBox[{"\[InvisibleSpace]", "!", 
                    RowBox[{"(", 
                      RowBox[{"FreeQ", "[", 
                        RowBox[{"vars", ",", 
                          RowBox[{
                            StyleBox["TFI",
                              FontColor->RGBColor[1, 0.6, 0]], "|", 
                            StyleBox["TVI",
                              FontColor->RGBColor[0.2, 0.4, 0.8]], "|", "TJI",
                             "|", "TKI", "|", "TBI", "|", "TAI"}]}], "]"}], 
                      ")"}]}], ",", 
                  
                  RowBox[{\(Print["\<Returning the list of remaining \
integrals.\>"]\), ";", 
                    RowBox[{"Throw", "[", 
                      RowBox[{"Select", "[", 
                        RowBox[{"vars", ",", 
                          RowBox[{
                            RowBox[{"\[InvisibleSpace]", "!", 
                              RowBox[{"(", 
                                RowBox[{"FreeQ", "[", 
                                  RowBox[{"#1", ",", 
                                    RowBox[{
                                      StyleBox["TFI",
                                        FontColor->RGBColor[1, 0.6, 0]], "|", 
                                      
                                      StyleBox["TVI",
                                        FontColor->RGBColor[0.2, 0.4, 0.8]], 
                                      "|", "TJI", "|", "TKI", "|", "TBI", "|",
                                       "TAI"}]}], "]"}], ")"}]}], "&"}]}], 
                        "]"}], "]"}]}]}], "]"}], 
              ";", \(t1 = t1 /. d \[Rule] fe\), 
              ";", \(polygafu[2, 1] = \(-2\)\ Zeta[3]\), 
              ";", \(polygafu[2, 2] = 2 - 2\ Zeta[3]\), 
              ";", \(polygafu[x_, y_] := \(polygafu[x, y] = 
                  FunctionExpand[PolyGamma[x, y]]\)\), 
              ";", \(t2 = 
                Normal[t1 + O[e]\^\(order + 1\)] /. 
                  PolyGamma \[Rule] polygafu\), ";", 
              RowBox[{"t3", "=", 
                RowBox[{
                  RowBox[{\(Collect[t2, e, Expand[#1] /. z2rule &]\), "//.", 
                    RowBox[{"{", 
                      RowBox[{
                        RowBox[{
                          InterpretationBox["a_",
                            PreFactor[ 
                              Pattern[ FE`Conversion`Private`a, 
                                Blank[ ]]],
                            Editable->False], " ", 
                          InterpretationBox["b_",
                            PreFactor[ 
                              Pattern[ FE`Conversion`Private`b, 
                                Blank[ ]]],
                            Editable->False]}], "\[RuleDelayed]", 
                        InterpretationBox[\(a\ b\),
                          PreFactor[ 
                            Times[ 
                            FE`Conversion`Private`a, 
                              FE`Conversion`Private`b]],
                          Editable->False]}], "}"}]}], "/.", 
                  RowBox[{
                    InterpretationBox["c_",
                      PreFactor[ 
                        Pattern[ FE`Conversion`Private`c, 
                          Blank[ ]]],
                      Editable->False], "\[RuleDelayed]", 
                    InterpretationBox["c",
                      PreFactor[ FE`Conversion`Private`c],
                      Editable->False]}]}]}], ";", 
              RowBox[{"prec", "=", 
                RowBox[{"Union", "[", 
                  RowBox[{"Cases", "[", 
                    RowBox[{"t3", ",", 
                      InterpretationBox["_",
                        PreFactor[ 
                          Blank[ ]],
                        Editable->False], ",", \(-1\)}], "]"}], "]"}]}], ";", 
              
              RowBox[{"If", "[", 
                RowBox[{\(Length[prec] === 1\), ",", 
                  RowBox[{"If", "[", 
                    RowBox[{\(Head[t3] === Plus\), ",", 
                      
                      RowBox[{\(mass = 
                          Select[
                            Variables[
                              prec\[LeftDoubleBracket]1, 
                                1\[RightDoubleBracket]], 
                            FreeQ[#1, SEpsilon] &]\), 
                        ";", \(If[Length[mass] === 1, 
                          If[FreeQ[First[t3], First[mass]], massfactor = 1, 
                            massfactor = 
                              First[mass]\^Exponent[First[t3], First[mass]]], 
                          massfactor = 1]\), ";", \(delpfactor = 1\), ";", 
                        RowBox[{"delp", "=", 
                          RowBox[{"Variables", "[", 
                            RowBox[{\(First[t3]\/massfactor\), "/.", 
                              RowBox[{"{", 
                                RowBox[{
                                  RowBox[{
                                    InterpretationBox["__",
                                      PreFactor[ 
                                        BlankSequence[ ]],
                                      Editable->False], "\[RuleDelayed]", 
                                    "1"}], 
                                  ",", \(ToExpression["\<HighEnergyPhysics`\
general`Zeta2`Zeta2\>"] \[Rule] Zeta[2]\)}], "}"}]}], "]"}]}], 
                        ";", \(If[Length[delp] === 1, delp = First[delp]; 
                          If[Length[t3] === 
                              Length[
                                Select[
                                  t3, \(\(\[InvisibleSpace]\)\(! \((FreeQ[#1, 
                                        delp])\)\)\) &]], 
                            delpfactor = delp\^Exponent[First[t3], delp]]]\), 
                        ";", \(t3 = \((\(massfactor\ delpfactor\ First[prec] /. 
                                  Prefactor1 \[Rule] 
                                    Identity\) /. {\((m_\^2)\)\^n_\ \
m_\^j_?EvenQ \[RuleDelayed] \((m\^2)\)\^\(n + j\/2\)})\)\ \
Collect[\((#1\/\(First[prec]\ massfactor\ delpfactor\) &)\) /@ t3, e, 
                              Expand]\)}]}], "]"}]}], "]"}], 
              ";", \(t3 /. Prefactor1 \[Rule] Identity\)}], "]"}]}], 
        "]"}]}]], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Construct T's", "Section",
  CellDingbat->"\[FilledSmallCircle]",
  CellTags->"T1.9.1"],

Cell[CellGroupData[{

Cell["remember", "Subsection",
  CellTags->"T1.9.1.1"],

Cell["The function remember is not used by default.", "Text"],

Cell[BoxData[
    \(\(tfch = {TFR :> Hold[TFR], TVR :> Hold[TVR], TJR :> Hold[TJR], 
          TBR :> Hold[TBR], TAR :> Hold[TAR], 
          TComment :> Hold[TComment]};\)\)], "Input"],

Cell[BoxData[
    \(\(rtfch = Map[Reverse, tfch];\)\)], "Input"],

Cell[BoxData[
    \(\(SetAttributes[condrest, HoldRest];\)\)], "Input"],

Cell[BoxData[
    \(\(Clear[remember];\)\)], "Input"],

Cell[BoxData[
    \(\(remember[zi_HoldForm] := 
        Block[{z = zi /. tfch}, 
          If[\(\(\[InvisibleSpace]\)\(! \((FreeQ[z, 
                  Condition])\)\)\), \(\(\(\(Hold[
                        HoldForm]\)[\(Hold[SetDelayed]\)[
                        z\[LeftDoubleBracket]1, 1\[RightDoubleBracket], 
                        ReplacePart[
                          z\[LeftDoubleBracket]1, 2\[RightDoubleBracket], 
                          Prepend[
                            Extract[z, {1, 2, 1}, 
                              Hold], \(\(\(z\[LeftDoubleBracket]1, 
                                      1\[RightDoubleBracket] /. 
                                    PatternTest \[Rule] pt\) /. 
                                  pt[a_, b_] \[RuleDelayed] a\) /. 
                                Pattern \[Rule] pat\) /. 
                              pat[a_, b_] \[RuleDelayed] a], 1]]] /. 
                    Hold[HoldForm] \[Rule] HoldForm\) /. rtfch\) /. 
                Hold[SetDelayed] \[Rule] SetDelayed\) /. 
              Hold \[Rule] 
                Set, \(\(\(\(Hold[HoldForm]\)[\(Hold[SetDelayed]\)[
                        z\[LeftDoubleBracket]1, 1\[RightDoubleBracket], 
                        Prepend[
                          Extract[z, {1, 2}, 
                            Hold], \(\(\(z\[LeftDoubleBracket]1, 
                                    1\[RightDoubleBracket] /. 
                                  PatternTest \[Rule] pt\) /. 
                                pt[a_, b_] \[RuleDelayed] a\) /. 
                              Pattern \[Rule] pat\) /. 
                            pat[a_, b_] \[RuleDelayed] a]]] /. 
                    Hold[HoldForm] \[Rule] HoldForm\) /. rtfch\) /. 
                Hold[SetDelayed] \[Rule] SetDelayed\) /. 
              Hold \[Rule] Set]];\)\)], "Input"],

Cell[BoxData[
    \($TarasovTdeltaplimit\)], "Input"],

Cell[BoxData[
    \(\(aa2\  = \ 
        Array[f\_# &, \ 
          10000 + \(($TarasovTdeltaplimit + 1)\)\ 1000];\)\)], "Input"],

Cell[BoxData[
    \(\(bb12 = Select[aa2, Head[#] =!= Subscript &];\)\)], "Input"],

Cell[BoxData[
    \(\(Clear[aa2];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Print["\<there are \>", 
      Length[bb12], "\< recursion equations for $RankLimit = \>", $RankLimit]\
\)], "Input"],

Cell[BoxData[
    FormBox[
      InterpretationBox[\("there are "\[InvisibleSpace]103\[InvisibleSpace]" \
recursion equations for $RankLimit = "\[InvisibleSpace]{0, 6}\),
        SequenceForm[ 
        "there are ", 103, " recursion equations for $RankLimit = ", {0, 6}],
        Editable->False], TraditionalForm]], "Print"]
}, Closed]],

Cell[BoxData[
    \(\(If[Global`$Remember\  === \ True, Global`RCHECK = True; \ 
        nb\  = Map[remember, bb12], nb = bb12];\)\)], "Input"],

Cell[BoxData[
    \(\(Clear[bb12];\)\)], "Input"],

Cell[BoxData[
    \(\(ReleaseHold[nb];\)\)], "Input"],

Cell[BoxData[
    \(Unset[$Post]\)], "Input"],

Cell[BoxData[
    \(\($Comment = True;\)\)], "Input"],

Cell[BoxData[
    \(\(End[];\)\)], "Input"],

Cell[BoxData[
    \(\(EndPackage[];\)\)], "Input"],

Cell[BoxData[
    \(\(If[\(! ValueQ[HighEnergyPhysics`FeynCalc`$FeynCalcDirectory]\), 
        BeginPackage["\<HighEnergyPhysics`general`Zeta2`\>"]; \n
        Zeta2::usage = \n"\<Zeta2 denotes Zeta[2].\>"; \n
        Begin["\<`Private`\>"]; \nZeta2\  /: \ N[Zeta2]\  = \ N[Zeta[2]]; \n
        Zeta2 /: MakeBoxes[Zeta2, fmt_] := 
          InterpretationBox @@ {\ \ RowBox[{"\<\[Zeta]\>", "\<(\>", 
                    2, "\<)\>"}], Zeta2, Editable -> False}\nEnd[]; \n\ 
        EndPackage[];];\)\)], "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["$BasisIntegrals", "Section",
  CellDingbat->"\[EmptySmallCircle]",
  CellTags->"T1.10.1"],

Cell["\<\
This a (incomplete) set of basis integrals. You can add your \
favorite integrals here.\
\>", "Text"],

Cell[CellGroupData[{

Cell["\<\
$BasisIntegrals = 
  {TAI[d_, 0, {{1, M_}}] :> 
    -I*E^(1/2*(4 - d)*EulerGamma)*M^2*
     Gamma[-1 + (4 - d)/2]*
     Prefactor1[(M^2)^(1/2*(-4 + d))*
       SEpsilon[d]], 
       
       TBI[d_, pp_, {{1, 0},{1, 0}}] :> 
   (I*E^(((4 - d)*EulerGamma)/2 - 
         I/2*(-4 + d)*Pi)*Gamma[2 - d/2]*
      Gamma[-1 + d/2]^2*
      Prefactor1[(pp)^((-4 + d)/2)*
        SEpsilon[d]])/Gamma[-2 + d],
        
        TJI[d_, pp_, {{1,0},{1,0},{1,0}}] :>-((pp*Gamma[3 - d]*Gamma[-1 + \
d/2]^3*
    E^((4 - d)*EulerGamma)*  Prefactor1[
        (pp)^(-4 + d)*SEpsilon[d]^2])/
    (E^(I*(-4 + d)*Pi)*Gamma[-3 + (3*d)/2])
),
    
          TJI[d_, M_^2, {{1,M_},{1,0},{1,0}}] :>(M^2*Gamma[3 - d]*Gamma[2 - \
d/2]*
    Gamma[-1 + d/2]^2*Gamma[-5 + 2*d]*E^((4 - d)*EulerGamma)*
    Prefactor1[
      (M^2)^(-4 + d)*SEpsilon[d]^2])/
  (Gamma[-2 + d]*Gamma[-3 + (3*d)/2]),
  
  TJI[d_Symbol, (M_)^2, 
  {{1, M_}, {1, M_}, {1, M_}}]:>M^2*Prefactor1[
    (M^2)^(-4 + d)*SEpsilon[d]^2]*
  (-6/(-4 + d)^2 + 17/(2*(-4 + d)) + 
    (-59 - 2*Pi^2)/8 + 
    (-4 + d)^3*TJI111e + 
    ((-4 + d)^2*(16755 - 4750*Pi^2 - 
         14*Pi^4 + 3840*Pi^2*Log[2] - 
         12080*Zeta[3]))/1920 + 
    ((-4 + d)*(195 + 98*Pi^2 - 
         48*Zeta[3]))/96),
         
         TJI[d_, 0, {{1, M_}, {1, M_}, {1, 0}}]:>(M^2*Gamma[3 - d]*Gamma[2 - \
d/2]^2*
    Gamma[-1 + d/2]*E^((4 - d)*EulerGamma)*
    Prefactor1[
      (M^2)^(-4 + d)*SEpsilon[d]^2])/
  (Gamma[4 - d]*Gamma[d/2]),
  
  TJI[d_, 0, {{1,M_},{1,0},{1,0}}] :>(M^2*Gamma[3 - d]*Gamma[2 - d/2]*
    Gamma[-1 + d/2]^2*E^((4 - d)*EulerGamma)*
    Prefactor1[
      (M^2)^(-4 + d)*SEpsilon[d]^2])/
  Gamma[d/2]
        }\
\>", "Input"],

Cell[BoxData[
    FormBox[
      RowBox[{"{", 
        RowBox[{
          RowBox[{
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["A",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                FormBox[\({1, M_}\),
                  "TraditionalForm"], 
                RowBox[{"(", 
                  FormBox["d_",
                    "TraditionalForm"], ")"}]],
              TAI[ 
                Pattern[ d, 
                  Blank[ ]], 0, {{1, 
                Pattern[ M, 
                  Blank[ ]]}}],
              Editable->True], "\[RuleDelayed]", 
            RowBox[{\(-\[ImaginaryI]\), " ", 
              SuperscriptBox["\[ExponentialE]", 
                RowBox[{\(1\/2\), " ", \((4 - d)\), " ", 
                  TagBox["\[DoubledGamma]",
                    (EulerGamma&)]}]], " ", \(M\^2\), 
              " ", \(\[CapitalGamma](\(4 - d\)\/2 - 1)\), " ", 
              RowBox[{"(", 
                InterpretationBox[
                  RowBox[{\(\((M\^2)\)\^\(\(d - 4\)\/2\)\), " ", 
                    InterpretationBox[
                      StyleBox[\(S\_d\),
                        FontWeight->"Bold"],
                      SEpsilon[ d],
                      Editable->False]}],
                  Prefactor1[ 
                    Times[ 
                      Power[ 
                        Power[ M, 2], 
                        Times[ 
                          Rational[ 1, 2], 
                          Plus[ -4, d]]], 
                      SEpsilon[ d]]],
                  Editable->False], ")"}]}]}], ",", 
          RowBox[{
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["B",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\({1, 0}\),
                    "TraditionalForm"], 
                  FormBox[\({1, 0}\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d_",
                    "TraditionalForm"], ")"}]],
              TBI[ 
                Pattern[ d, 
                  Blank[ ]], 
                Pattern[ pp, 
                  Blank[ ]], {{1, 0}, {1, 0}}],
              Editable->True], "\[RuleDelayed]", 
            FractionBox[
              RowBox[{"\[ImaginaryI]", " ", 
                SuperscriptBox["\[ExponentialE]", 
                  RowBox[{
                    RowBox[{\(1\/2\), " ", \((4 - d)\), " ", 
                      TagBox["\[DoubledGamma]",
                        (EulerGamma&)]}], 
                    "-", \(1\/2\ \[ImaginaryI]\ \((d - 4)\)\ \[Pi]\)}]], 
                " ", \(\[CapitalGamma](2 - d\/2)\), 
                " ", \(\(\[CapitalGamma](d\/2 - 1)\)\^2\), " ", 
                RowBox[{"(", 
                  InterpretationBox[
                    RowBox[{\(pp\^\(\(d - 4\)\/2\)\), " ", 
                      InterpretationBox[
                        StyleBox[\(S\_d\),
                          FontWeight->"Bold"],
                        SEpsilon[ d],
                        Editable->False]}],
                    Prefactor1[ 
                      Times[ 
                        Power[ pp, 
                          Times[ 
                            Rational[ 1, 2], 
                            Plus[ -4, d]]], 
                        SEpsilon[ d]]],
                    Editable->False], ")"}]}], \(\[CapitalGamma](d - 2)\)]}], 
          ",", 
          RowBox[{
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\({1, 0}\),
                    "TraditionalForm"], 
                  FormBox[\({1, 0}\),
                    "TraditionalForm"], 
                  FormBox[\({1, 0}\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d_",
                    "TraditionalForm"], ")"}]],
              TJI[ 
                Pattern[ d, 
                  Blank[ ]], 
                Pattern[ pp, 
                  Blank[ ]], {{1, 0}, {1, 0}, {1, 0}}],
              Editable->True], "\[RuleDelayed]", 
            RowBox[{"-", 
              FractionBox[
                RowBox[{
                "pp", " ", \(\[CapitalGamma](3 - d)\), 
                  " ", \(\(\[CapitalGamma](d\/2 - 1)\)\^3\), " ", 
                  SuperscriptBox["\[ExponentialE]", 
                    RowBox[{\((4 - d)\), " ", 
                      TagBox["\[DoubledGamma]",
                        (EulerGamma&)]}]], " ", 
                  RowBox[{"(", 
                    InterpretationBox[
                      RowBox[{\(pp\^\(d - 4\)\), " ", 
                        InterpretationBox[
                          StyleBox[\(S\_d\%2\),
                            FontWeight->"Bold"],
                          SEpsilon[ d],
                          Editable->False]}],
                      Prefactor1[ 
                        Times[ 
                          Power[ pp, 
                            Plus[ -4, d]], 
                          Power[ 
                            SEpsilon[ d], 2]]],
                      Editable->False], 
                    ")"}]}], \(\[ExponentialE]\^\(\[ImaginaryI]\ \((d - 4)\)\ \
\[Pi]\)\ \(\[CapitalGamma](\(3\ d\)\/2 - 3)\)\)]}]}], ",", 
          RowBox[{
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\({1, M_}\),
                    "TraditionalForm"], 
                  FormBox[\({1, 0}\),
                    "TraditionalForm"], 
                  FormBox[\({1, 0}\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d_",
                    "TraditionalForm"], ")"}]],
              TJI[ 
                Pattern[ d, 
                  Blank[ ]], 
                Power[ 
                  Pattern[ M, 
                    Blank[ ]], 2], {{1, 
                Pattern[ M, 
                  Blank[ ]]}, {1, 0}, {1, 0}}],
              Editable->True], "\[RuleDelayed]", 
            FractionBox[
              
              RowBox[{\(M\^2\), " ", \(\[CapitalGamma](3 - d)\), 
                " ", \(\[CapitalGamma](2 - d\/2)\), 
                " ", \(\(\[CapitalGamma](d\/2 - 1)\)\^2\), 
                " ", \(\[CapitalGamma](2\ d - 5)\), " ", 
                SuperscriptBox["\[ExponentialE]", 
                  RowBox[{\((4 - d)\), " ", 
                    TagBox["\[DoubledGamma]",
                      (EulerGamma&)]}]], " ", 
                RowBox[{"(", 
                  InterpretationBox[
                    RowBox[{\(\((M\^2)\)\^\(d - 4\)\), " ", 
                      InterpretationBox[
                        StyleBox[\(S\_d\%2\),
                          FontWeight->"Bold"],
                        SEpsilon[ d],
                        Editable->False]}],
                    Prefactor1[ 
                      Times[ 
                        Power[ 
                          Power[ M, 2], 
                          Plus[ -4, d]], 
                        Power[ 
                          SEpsilon[ d], 2]]],
                    Editable->False], ")"}]}], \(\(\[CapitalGamma](
                  d - 2)\)\ \(\[CapitalGamma](\(3\ d\)\/2 - 3)\)\)]}], ",", 
          RowBox[{
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["J",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\({1, M_}\),
                    "TraditionalForm"], 
                  FormBox[\({1, M_}\),
                    "TraditionalForm"], 
                  FormBox[\({1, M_}\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d_Symbol",
                    "TraditionalForm"], ")"}]],
              TJI[ 
                Pattern[ d, 
                  Blank[ Symbol]], 
                Power[ 
                  Pattern[ M, 
                    Blank[ ]], 2], {{1, 
                Pattern[ M, 
                  Blank[ ]]}, {1, 
                Pattern[ M, 
                  Blank[ ]]}, {1, 
                Pattern[ M, 
                  Blank[ ]]}}],
              Editable->True], "\[RuleDelayed]", 
            RowBox[{\(M\^2\), " ", 
              RowBox[{"(", 
                InterpretationBox[
                  RowBox[{\(\((M\^2)\)\^\(d - 4\)\), " ", 
                    InterpretationBox[
                      StyleBox[\(S\_d\%2\),
                        FontWeight->"Bold"],
                      SEpsilon[ d],
                      Editable->False]}],
                  Prefactor1[ 
                    Times[ 
                      Power[ 
                        Power[ M, 2], 
                        Plus[ -4, d]], 
                      Power[ 
                        SEpsilon[ d], 2]]],
                  Editable->False], ")"}], " ", 
              RowBox[{"(", 
                
                RowBox[{\(TJI111e\ \((d - 4)\)\^3\), 
                  "+", \(1\/8\ \((\(-59\) - 2\ \[Pi]\^2)\)\), "+", 
                  RowBox[{\(1\/96\), " ", \((d - 4)\), " ", 
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{\(-48\), " ", 
                          TagBox[
                            RowBox[{"\[Zeta]", "(", 
                              TagBox["3",
                                (Editable -> True)], ")"}],
                            InterpretTemplate[ Zeta[ #]&]]}], 
                        "+", \(98\ \[Pi]\^2\), "+", "195"}], ")"}]}], "+", 
                  FractionBox[
                    RowBox[{\(\((d - 4)\)\^2\), " ", 
                      RowBox[{"(", 
                        
                        RowBox[{\(3840\ \[Pi]\^2\ \(log(2)\)\), 
                          "-", \(14\ \[Pi]\^4\), "-", \(4750\ \[Pi]\^2\), "-",
                           
                          RowBox[{"12080", " ", 
                            TagBox[
                              RowBox[{"\[Zeta]", "(", 
                                TagBox["3",
                                  (Editable -> True)], ")"}],
                              InterpretTemplate[ Zeta[ #]&]]}], "+", 
                          "16755"}], ")"}]}], "1920"], 
                  "+", \(17\/\(2\ \((d - 4)\)\)\), 
                  "-", \(6\/\((d - 4)\)\^2\)}], ")"}]}]}], ",", 
          RowBox[{
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["K",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\({1, M_}\),
                    "TraditionalForm"], 
                  FormBox[\({1, M_}\),
                    "TraditionalForm"], 
                  FormBox[\({1, 0}\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d_",
                    "TraditionalForm"], ")"}]],
              TJI[ 
                Pattern[ d, 
                  Blank[ ]], 0, {{1, 
                Pattern[ M, 
                  Blank[ ]]}, {1, 
                Pattern[ M, 
                  Blank[ ]]}, {1, 0}}],
              Editable->True], "\[RuleDelayed]", 
            FractionBox[
              
              RowBox[{\(M\^2\), " ", \(\[CapitalGamma](3 - d)\), 
                " ", \(\(\[CapitalGamma](2 - d\/2)\)\^2\), 
                " ", \(\[CapitalGamma](d\/2 - 1)\), " ", 
                SuperscriptBox["\[ExponentialE]", 
                  RowBox[{\((4 - d)\), " ", 
                    TagBox["\[DoubledGamma]",
                      (EulerGamma&)]}]], " ", 
                RowBox[{"(", 
                  InterpretationBox[
                    RowBox[{\(\((M\^2)\)\^\(d - 4\)\), " ", 
                      InterpretationBox[
                        StyleBox[\(S\_d\%2\),
                          FontWeight->"Bold"],
                        SEpsilon[ d],
                        Editable->False]}],
                    Prefactor1[ 
                      Times[ 
                        Power[ 
                          Power[ M, 2], 
                          Plus[ -4, d]], 
                        Power[ 
                          SEpsilon[ d], 2]]],
                    Editable->False], ")"}]}], \(\(\[CapitalGamma](
                  4 - d)\)\ \(\[CapitalGamma](d\/2)\)\)]}], ",", 
          RowBox[{
            InterpretationBox[
              SubsuperscriptBox[
                StyleBox["K",
                  SingleLetterItalics->False,
                  FontWeight->"Bold"], 
                RowBox[{
                  FormBox[\({1, M_}\),
                    "TraditionalForm"], 
                  FormBox[\({1, 0}\),
                    "TraditionalForm"], 
                  FormBox[\({1, 0}\),
                    "TraditionalForm"]}], 
                RowBox[{"(", 
                  FormBox["d_",
                    "TraditionalForm"], ")"}]],
              TJI[ 
                Pattern[ d, 
                  Blank[ ]], 0, {{1, 
                Pattern[ M, 
                  Blank[ ]]}, {1, 0}, {1, 0}}],
              Editable->True], "\[RuleDelayed]", 
            FractionBox[
              
              RowBox[{\(M\^2\), " ", \(\[CapitalGamma](3 - d)\), 
                " ", \(\[CapitalGamma](2 - d\/2)\), 
                " ", \(\(\[CapitalGamma](d\/2 - 1)\)\^2\), " ", 
                SuperscriptBox["\[ExponentialE]", 
                  RowBox[{\((4 - d)\), " ", 
                    TagBox["\[DoubledGamma]",
                      (EulerGamma&)]}]], " ", 
                RowBox[{"(", 
                  InterpretationBox[
                    RowBox[{\(\((M\^2)\)\^\(d - 4\)\), " ", 
                      InterpretationBox[
                        StyleBox[\(S\_d\%2\),
                          FontWeight->"Bold"],
                        SEpsilon[ d],
                        Editable->False]}],
                    Prefactor1[ 
                      Times[ 
                        Power[ 
                          Power[ M, 2], 
                          Plus[ -4, d]], 
                        Power[ 
                          SEpsilon[ d], 2]]],
                    Editable->False], ")"}]}], \(\[CapitalGamma](d\/2)\)]}]}],
         "}"}], TraditionalForm]], "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Save tarcer*.mx file", "Section",
  CellDingbat->"\[FilledSmallCircle]",
  CellTags->"T1.11.1"],

Cell[BoxData[
    \(\(Share[];\)\)], "Input"],

Cell["\<\
DumpSave[StringJoin[\"tarcer\", StringJoin @@ 
     ToString /@ $RankLimit, \".mx\"], 
   {$ContextPath, HighEnergyPhysics`Tarcer`Cayley, 
    HighEnergyPhysics`Tarcer`CayleyD, 
    HighEnergyPhysics`Tarcer`Cayleyu, 
    HighEnergyPhysics`Tarcer`Prefactor1, 
    HighEnergyPhysics`Tarcer`TarcerRecurse, 
    HighEnergyPhysics`Tarcer`TFIRecurse, 
    HighEnergyPhysics`Tarcer`TFISimplify, 
    HighEnergyPhysics`Tarcer`SEpsilon, 
    HighEnergyPhysics`Tarcer`TarcerExpand, 
    HighEnergyPhysics`Tarcer`TarcerReduce, 
    HighEnergyPhysics`Tarcer`TAI, 
    HighEnergyPhysics`Tarcer`TAR, 
    HighEnergyPhysics`Tarcer`TBI, 
    HighEnergyPhysics`Tarcer`TBR, 
    HighEnergyPhysics`Tarcer`TFI, 
    HighEnergyPhysics`Tarcer`TFR, 
    HighEnergyPhysics`Tarcer`TJI, 
    HighEnergyPhysics`Tarcer`TJR, 
    HighEnergyPhysics`Tarcer`TKI, 
    HighEnergyPhysics`Tarcer`TVI, 
    HighEnergyPhysics`Tarcer`TVR, 
    HighEnergyPhysics`Tarcer`$BasisIntegrals, 
    HighEnergyPhysics`Tarcer`$Comment, 
    HighEnergyPhysics`Tarcer`$CommentNotebook, 
    HighEnergyPhysics`Tarcer`$RankLimit, 
    HighEnergyPhysics`Tarcer`$TTable, 
    HighEnergyPhysics`Tarcer`Private`applytlrules, 
    HighEnergyPhysics`Tarcer`Private`applytlrules2,
    HighEnergyPhysics`Tarcer`Private`ExpandMaybe, 
    HighEnergyPhysics`Tarcer`Private`FactorC, 
    HighEnergyPhysics`Tarcer`Private`FEPrint, 
    HighEnergyPhysics`Tarcer`Private`fN, 
    HighEnergyPhysics`Tarcer`Private`mbt, 
    HighEnergyPhysics`Tarcer`Private`PNQ, 
    HighEnergyPhysics`Tarcer`Private`PQ, 
    HighEnergyPhysics`Tarcer`Private`ta, 
    HighEnergyPhysics`Tarcer`Private`TA, 
    HighEnergyPhysics`Tarcer`Private`TAIC, 
    HighEnergyPhysics`Tarcer`Private`TBIC, 
    HighEnergyPhysics`Tarcer`Private`TComment, 
    HighEnergyPhysics`Tarcer`Private`TLRComment, 
    HighEnergyPhysics`Tarcer`Private`TFIC, 
    HighEnergyPhysics`Tarcer`Private`TJIC, 
    HighEnergyPhysics`Tarcer`Private`TLR, 
    HighEnergyPhysics`Tarcer`Private`tlrule, 
    HighEnergyPhysics`Tarcer`Private`redblue, 
    HighEnergyPhysics`Tarcer`Private`TLRules, 
    HighEnergyPhysics`Tarcer`Private`TLRules2, 
    HighEnergyPhysics`Tarcer`Private`TVIC, 
    HighEnergyPhysics`Tarcer`Private`$TLRComment, 
    HighEnergyPhysics`general`Zeta2`Zeta2}]; \
\>", "Input",
  Editable->True],

Cell[BoxData[
    \(\($HistoryLength = \[Infinity];\)\)], "Input"],

Cell["\<\
This gives the absolute time (in minutes) to run this \
notebook.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Round[AbsoluteTime[] - HighEnergyPhysics`Tarcer`Private`atime]/
        60. \ min\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`0.9166666666666666`\ min\)], "Output"]
}, Closed]],

Cell["Display the size of the generated file:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(filename\  = \ "\<tarcer\>" <> \((Apply[StringJoin, 
            ToString /@ $RankLimit])\) <> "\<.mx\>"\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`"tarcer06.mx"\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FileByteCount[filename]\ Byte\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`1674536\ Byte\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Directory[]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`"F:\\fjob\\Mathematica\\4.0\\AddOns\\Applications\\\
HighEnergyPhysics\\tarcer"\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(MaxMemoryUsed[]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`11855232\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(MemoryInUse[]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`7725504\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(TimeUsed[]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`54.19700000000001`\)], "Output"]
}, Closed]],

Cell[BoxData[
    \(\ \)], "Input"]
}, Closed]]
}, Open  ]]
},
FrontEndVersion->"4.1 for Microsoft Windows",
ScreenRectangle->{{0, 1024}, {0, 723}},
WindowToolbars->{},
WindowSize->{845, 605},
WindowMargins->{{71, Automatic}, {Automatic, 6}},
CellLabelAutoDelete->True,
Magnification->1
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "T1.1"->{
    Cell[1727, 52, 73, 2, 115, "Title",
      CellTags->"T1.1"]},
  "T1.2"->{
    Cell[1983, 64, 434, 14, 109, "Text",
      CellTags->"T1.2"]},
  "T1.1.1"->{
    Cell[30070, 1070, 93, 2, 39, "Section",
      CellTags->"T1.1.1"]},
  "T1.2.1.1"->{
    Cell[33715, 1189, 78, 1, 43, "Subsubsection",
      CellTags->"T1.2.1.1"]},
  "T1.2.1"->{
    Cell[34357, 1209, 86, 2, 39, "Section",
      CellTags->"T1.2.1"]},
  "T1.3.1"->{
    Cell[40269, 1360, 112, 2, 39, "Section",
      CellTags->"T1.3.1"]},
  "T1.3.1.1"->{
    Cell[42343, 1434, 63, 1, 47, "Subsection",
      CellTags->"T1.3.1.1"]},
  "T1.3.2.1"->{
    Cell[67018, 2030, 58, 1, 47, "Subsection",
      CellTags->"T1.3.2.1"]},
  "T1.3.3.1"->{
    Cell[71650, 2116, 59, 1, 47, "Subsection",
      CellTags->"T1.3.3.1"]},
  "T1.3.4.1"->{
    Cell[75884, 2206, 60, 1, 47, "Subsection",
      CellTags->"T1.3.4.1"]},
  "T1.3.5.1"->{
    Cell[76669, 2228, 54, 1, 47, "Subsection",
      CellTags->"T1.3.5.1"]},
  "T1.3.6.1"->{
    Cell[81516, 2330, 59, 1, 47, "Subsection",
      CellTags->"T1.3.6.1"]},
  "T1.3.7.1"->{
    Cell[82101, 2347, 50, 1, 47, "Subsection",
      CellTags->"T1.3.7.1"]},
  "T1.3.7.1.1"->{
    Cell[82266, 2355, 57, 1, 43, "Subsubsection",
      CellTags->"T1.3.7.1.1"]},
  "T1.3.7.2.1"->{
    Cell[84906, 2446, 53, 1, 43, "Subsubsection",
      CellTags->"T1.3.7.2.1"]},
  "T1.3.7.3.1"->{
    Cell[85070, 2455, 54, 1, 43, "Subsubsection",
      CellTags->"T1.3.7.3.1"]},
  "T1.3.7.4.1"->{
    Cell[85290, 2467, 58, 1, 43, "Subsubsection",
      CellTags->"T1.3.7.4.1"]},
  "T1.3.7.5.1"->{
    Cell[85462, 2476, 54, 1, 43, "Subsubsection",
      CellTags->"T1.3.7.5.1"]},
  "T1.3.7.6.1"->{
    Cell[88298, 2567, 54, 1, 43, "Subsubsection",
      CellTags->"T1.3.7.6.1"]},
  "T1.3.7.7.1"->{
    Cell[88919, 2588, 54, 1, 43, "Subsubsection",
      CellTags->"T1.3.7.7.1"]},
  "T1.3.7.8.1"->{
    Cell[89672, 2613, 54, 1, 43, "Subsubsection",
      CellTags->"T1.3.7.8.1"]},
  "T1.4.1"->{
    Cell[90005, 2629, 104, 2, 39, "Section",
      CellTags->"T1.4.1"]},
  "T1.4.1.1"->{
    Cell[91990, 2678, 116, 3, 46, "Subsection",
      CellTags->"T1.4.1.1"]},
  "T1.4.2.1"->{
    Cell[92434, 2697, 114, 3, 46, "Subsection",
      CellTags->"T1.4.2.1"]},
  "T1.4.3.1"->{
    Cell[94656, 2766, 205, 6, 46, "Subsection",
      CellTags->"T1.4.3.1"]},
  "T1.4.4.1"->{
    Cell[99343, 2889, 223, 6, 59, "Subsection",
      CellTags->"T1.4.4.1"]},
  "T1.5.1"->{
    Cell[111729, 3210, 97, 2, 39, "Section",
      CellTags->"T1.5.1"]},
  "T1.5.1.1"->{
    Cell[111851, 3216, 65, 1, 47, "Subsection",
      CellTags->"T1.5.1.1"]},
  "T1.5.1.1.1"->{
    Cell[172258, 4382, 62, 1, 43, "Subsubsection",
      CellTags->"T1.5.1.1.1"]},
  "T1.5.2.1"->{
    Cell[175714, 4470, 59, 1, 47, "Subsection",
      CellTags->"T1.5.2.1"]},
  "T1.5.2.1.1"->{
    Cell[177053, 4497, 53, 1, 43, "Subsubsection",
      CellTags->"T1.5.2.1.1"]},
  "T1.5.3.1"->{
    Cell[183054, 4647, 60, 1, 47, "Subsection",
      CellTags->"T1.5.3.1"]},
  "T1.5.3.1.1"->{
    Cell[183460, 4663, 68, 1, 43, "Subsubsection",
      CellTags->"T1.5.3.1.1"]},
  "T1.5.3.2.1"->{
    Cell[186530, 4728, 68, 1, 43, "Subsubsection",
      CellTags->"T1.5.3.2.1"]},
  "T1.6.1"->{
    Cell[189002, 4787, 140, 2, 39, "Section",
      CellTags->"T1.6.1"]},
  "T1.6.1.1"->{
    Cell[189442, 4802, 56, 1, 47, "Subsection",
      CellTags->"T1.6.1.1"]},
  "T1.6.2.1"->{
    Cell[197051, 4990, 58, 1, 47, "Subsection",
      CellTags->"T1.6.2.1"]},
  "T1.6.2.1.1"->{
    Cell[197218, 4998, 139, 5, 43, "Subsubsection",
      CellTags->"T1.6.2.1.1"]},
  "T1.6.2.2.1"->{
    Cell[198485, 5038, 139, 5, 43, "Subsubsection",
      CellTags->"T1.6.2.2.1"]},
  "T1.6.2.3.1"->{
    Cell[199824, 5079, 139, 5, 43, "Subsubsection",
      CellTags->"T1.6.2.3.1"]},
  "T1.6.2.4.1"->{
    Cell[201163, 5120, 139, 5, 43, "Subsubsection",
      CellTags->"T1.6.2.4.1"]},
  "T1.6.3.1"->{
    Cell[202514, 5162, 58, 1, 47, "Subsection",
      CellTags->"T1.6.3.1"]},
  "T1.6.4.1"->{
    Cell[206307, 5269, 159, 5, 47, "Subsection",
      CellTags->"T1.6.4.1"]},
  "T1.6.5.1"->{
    Cell[206952, 5293, 58, 1, 47, "Subsection",
      CellTags->"T1.6.5.1"]},
  "T1.6.6.1"->{
    Cell[210745, 5400, 159, 5, 47, "Subsection",
      CellTags->"T1.6.6.1"]},
  "T1.6.7.1"->{
    Cell[211390, 5424, 159, 5, 47, "Subsection",
      CellTags->"T1.6.7.1"]},
  "T1.6.8.1"->{
    Cell[212177, 5453, 159, 5, 47, "Subsection",
      CellTags->"T1.6.8.1"]},
  "T1.6.9.1"->{
    Cell[212964, 5482, 56, 1, 47, "Subsection",
      CellTags->"T1.6.9.1"]},
  "T1.6.10.1"->{
    Cell[219817, 5651, 59, 1, 47, "Subsection",
      CellTags->"T1.6.10.1"]},
  "T1.6.11.1"->{
    Cell[221082, 5690, 57, 1, 47, "Subsection",
      CellTags->"T1.6.11.1"]},
  "T1.6.12.1"->{
    Cell[225385, 5818, 57, 1, 47, "Subsection",
      CellTags->"T1.6.12.1"]},
  "T1.6.13.1"->{
    Cell[229688, 5946, 57, 1, 47, "Subsection",
      CellTags->"T1.6.13.1"]},
  "T1.6.14.1"->{
    Cell[237163, 6127, 68, 1, 47, "Subsection",
      CellTags->"T1.6.14.1"]},
  "T1.6.15.1"->{
    Cell[237580, 6143, 59, 1, 47, "Subsection",
      CellTags->"T1.6.15.1"]},
  "T1.6.16.1"->{
    Cell[260334, 6687, 57, 1, 47, "Subsection",
      CellTags->"T1.6.16.1"]},
  "T1.6.17.1"->{
    Cell[264367, 6800, 59, 1, 47, "Subsection",
      CellTags->"T1.6.17.1"]},
  "T1.6.18.1"->{
    Cell[265142, 6833, 57, 1, 47, "Subsection",
      CellTags->"T1.6.18.1"]},
  "T1.6.19.1"->{
    Cell[270654, 6981, 59, 1, 47, "Subsection",
      CellTags->"T1.6.19.1"]},
  "T1.6.20.1"->{
    Cell[271414, 7012, 59, 1, 47, "Subsection",
      CellTags->"T1.6.20.1"]},
  "T1.6.21.1"->{
    Cell[272202, 7044, 57, 1, 47, "Subsection",
      CellTags->"T1.6.21.1"]},
  "T1.6.22.1"->{
    Cell[275623, 7144, 59, 1, 47, "Subsection",
      CellTags->"T1.6.22.1"]},
  "T1.6.23.1"->{
    Cell[276325, 7173, 57, 1, 47, "Subsection",
      CellTags->"T1.6.23.1"]},
  "T1.6.24.1"->{
    Cell[280459, 7290, 59, 1, 47, "Subsection",
      CellTags->"T1.6.24.1"]},
  "T1.6.25.1"->{
    Cell[281185, 7320, 57, 1, 47, "Subsection",
      CellTags->"T1.6.25.1"]},
  "T1.6.26.1"->{
    Cell[284874, 7426, 59, 1, 47, "Subsection",
      CellTags->"T1.6.26.1"]},
  "T1.6.27.1"->{
    Cell[285606, 7456, 57, 1, 47, "Subsection",
      CellTags->"T1.6.27.1"]},
  "T1.6.28.1"->{
    Cell[290983, 7602, 59, 1, 47, "Subsection",
      CellTags->"T1.6.28.1"]},
  "T1.6.29.1"->{
    Cell[291957, 7631, 58, 1, 47, "Subsection",
      CellTags->"T1.6.29.1"]},
  "T1.6.30.1"->{
    Cell[295379, 7731, 59, 1, 47, "Subsection",
      CellTags->"T1.6.30.1"]},
  "T1.6.31.1"->{
    Cell[296100, 7760, 58, 1, 47, "Subsection",
      CellTags->"T1.6.31.1"]},
  "T1.6.32.1"->{
    Cell[301323, 7923, 59, 1, 47, "Subsection",
      CellTags->"T1.6.32.1"]},
  "T1.6.33.1"->{
    Cell[302477, 7970, 61, 1, 47, "Subsection",
      CellTags->"T1.6.33.1"]},
  "T1.6.34.1"->{
    Cell[303677, 8017, 58, 1, 47, "Subsection",
      CellTags->"T1.6.34.1"]},
  "T1.6.35.1"->{
    Cell[309302, 8174, 59, 1, 47, "Subsection",
      CellTags->"T1.6.35.1"]},
  "T1.6.36.1"->{
    Cell[310564, 8215, 58, 1, 47, "Subsection",
      CellTags->"T1.6.36.1"]},
  "T1.6.37.1"->{
    Cell[317293, 8382, 59, 1, 47, "Subsection",
      CellTags->"T1.6.37.1"]},
  "T1.6.38.1"->{
    Cell[319107, 8450, 61, 1, 47, "Subsection",
      CellTags->"T1.6.38.1"]},
  "T1.6.39.1"->{
    Cell[320937, 8508, 61, 1, 47, "Subsection",
      CellTags->"T1.6.39.1"]},
  "T1.6.40.1"->{
    Cell[322015, 8544, 61, 1, 47, "Subsection",
      CellTags->"T1.6.40.1"]},
  "T1.6.41.1"->{
    Cell[322861, 8572, 58, 1, 47, "Subsection",
      CellTags->"T1.6.41.1"]},
  "T1.6.42.1"->{
    Cell[327977, 8712, 59, 1, 47, "Subsection",
      CellTags->"T1.6.42.1"]},
  "T1.6.43.1"->{
    Cell[328853, 8745, 68, 1, 47, "Subsection",
      CellTags->"T1.6.43.1"]},
  "T1.6.44.1"->{
    Cell[334615, 8905, 57, 1, 47, "Subsection",
      CellTags->"T1.6.44.1"]},
  "T1.6.45.1"->{
    Cell[349271, 9314, 59, 1, 47, "Subsection",
      CellTags->"T1.6.45.1"]},
  "T1.6.46.1"->{
    Cell[350101, 9347, 61, 1, 47, "Subsection",
      CellTags->"T1.6.46.1"]},
  "T1.6.47.1"->{
    Cell[351003, 9382, 57, 1, 47, "Subsection",
      CellTags->"T1.6.47.1"]},
  "T1.6.48.1"->{
    Cell[356987, 9537, 59, 1, 47, "Subsection",
      CellTags->"T1.6.48.1"]},
  "T1.6.49.1"->{
    Cell[357829, 9570, 61, 1, 47, "Subsection",
      CellTags->"T1.6.49.1"]},
  "T1.6.50.1"->{
    Cell[358747, 9605, 57, 1, 47, "Subsection",
      CellTags->"T1.6.50.1"]},
  "T1.6.51.1"->{
    Cell[371321, 9915, 59, 1, 47, "Subsection",
      CellTags->"T1.6.51.1"]},
  "T1.6.52.1"->{
    Cell[372008, 9942, 57, 1, 47, "Subsection",
      CellTags->"T1.6.52.1"]},
  "T1.6.53.1"->{
    Cell[377372, 10088, 59, 1, 47, "Subsection",
      CellTags->"T1.6.53.1"]},
  "T1.6.54.1"->{
    Cell[377915, 10113, 61, 1, 47, "Subsection",
      CellTags->"T1.6.54.1"]},
  "T1.6.55.1"->{
    Cell[378493, 10138, 61, 1, 47, "Subsection",
      CellTags->"T1.6.55.1"]},
  "T1.6.56.1"->{
    Cell[379071, 10163, 57, 1, 47, "Subsection",
      CellTags->"T1.6.56.1"]},
  "T1.6.57.1"->{
    Cell[384945, 10324, 59, 1, 47, "Subsection",
      CellTags->"T1.6.57.1"]},
  "T1.6.58.1"->{
    Cell[385430, 10347, 57, 1, 47, "Subsection",
      CellTags->"T1.6.58.1"]},
  "T1.6.59.1"->{
    Cell[390050, 10473, 59, 1, 47, "Subsection",
      CellTags->"T1.6.59.1"]},
  "T1.6.60.1"->{
    Cell[390567, 10498, 57, 1, 47, "Subsection",
      CellTags->"T1.6.60.1"]},
  "T1.6.61.1"->{
    Cell[395705, 10638, 59, 1, 47, "Subsection",
      CellTags->"T1.6.61.1"]},
  "T1.6.62.1"->{
    Cell[396313, 10664, 61, 1, 47, "Subsection",
      CellTags->"T1.6.62.1"]},
  "T1.6.63.1"->{
    Cell[396956, 10690, 61, 1, 47, "Subsection",
      CellTags->"T1.6.63.1"]},
  "T1.6.64.1"->{
    Cell[397623, 10717, 57, 1, 47, "Subsection",
      CellTags->"T1.6.64.1"]},
  "T1.6.65.1"->{
    Cell[408225, 10981, 59, 1, 47, "Subsection",
      CellTags->"T1.6.65.1"]},
  "T1.6.66.1"->{
    Cell[408780, 11006, 61, 1, 47, "Subsection",
      CellTags->"T1.6.66.1"]},
  "T1.6.67.1"->{
    Cell[409381, 11032, 61, 1, 47, "Subsection",
      CellTags->"T1.6.67.1"]},
  "T1.6.68.1"->{
    Cell[409982, 11058, 57, 1, 47, "Subsection",
      CellTags->"T1.6.68.1"]},
  "T1.6.69.1"->{
    Cell[416902, 11234, 59, 1, 47, "Subsection",
      CellTags->"T1.6.69.1"]},
  "T1.6.70.1"->{
    Cell[417450, 11259, 57, 1, 47, "Subsection",
      CellTags->"T1.6.70.1"]},
  "T1.6.70.1.1"->{
    Cell[417532, 11264, 56, 1, 43, "Subsubsection",
      CellTags->"T1.6.70.1.1"]},
  "T1.6.70.2.1"->{
    Cell[419370, 11320, 56, 1, 43, "Subsubsection",
      CellTags->"T1.6.70.2.1"]},
  "T1.6.71.1"->{
    Cell[425906, 11503, 59, 1, 47, "Subsection",
      CellTags->"T1.6.71.1"]},
  "T1.6.72.1"->{
    Cell[426270, 11520, 57, 1, 47, "Subsection",
      CellTags->"T1.6.72.1"]},
  "T1.6.73.1"->{
    Cell[435646, 11746, 59, 1, 47, "Subsection",
      CellTags->"T1.6.73.1"]},
  "T1.6.74.1"->{
    Cell[435991, 11762, 57, 1, 47, "Subsection",
      CellTags->"T1.6.74.1"]},
  "T1.6.75.1"->{
    Cell[447471, 12084, 60, 1, 47, "Subsection",
      CellTags->"T1.6.75.1"]},
  "T1.6.76.1"->{
    Cell[447863, 12101, 60, 1, 47, "Subsection",
      CellTags->"T1.6.76.1"]},
  "T1.6.77.1"->{
    Cell[448318, 12119, 60, 1, 47, "Subsection",
      CellTags->"T1.6.77.1"]},
  "T1.6.78.1"->{
    Cell[448709, 12136, 60, 1, 47, "Subsection",
      CellTags->"T1.6.78.1"]},
  "T1.6.79.1"->{
    Cell[449131, 12153, 57, 1, 47, "Subsection",
      CellTags->"T1.6.79.1"]},
  "T1.6.80.1"->{
    Cell[454463, 12292, 57, 1, 47, "Subsection",
      CellTags->"T1.6.80.1"]},
  "T1.6.81.1"->{
    Cell[456965, 12366, 59, 1, 47, "Subsection",
      CellTags->"T1.6.81.1"]},
  "T1.6.82.1"->{
    Cell[457330, 12382, 57, 1, 47, "Subsection",
      CellTags->"T1.6.82.1"]},
  "T1.6.83.1"->{
    Cell[460590, 12475, 59, 1, 47, "Subsection",
      CellTags->"T1.6.83.1"]},
  "T1.6.84.1"->{
    Cell[460906, 12489, 61, 1, 47, "Subsection",
      CellTags->"T1.6.84.1"]},
  "T1.6.85.1"->{
    Cell[461403, 12508, 61, 1, 47, "Subsection",
      CellTags->"T1.6.85.1"]},
  "T1.6.86.1"->{
    Cell[461945, 12528, 57, 1, 47, "Subsection",
      CellTags->"T1.6.86.1"]},
  "T1.6.87.1"->{
    Cell[464291, 12604, 59, 1, 47, "Subsection",
      CellTags->"T1.6.87.1"]},
  "T1.6.88.1"->{
    Cell[464628, 12619, 57, 1, 47, "Subsection",
      CellTags->"T1.6.88.1"]},
  "T1.6.89.1"->{
    Cell[466958, 12692, 59, 1, 47, "Subsection",
      CellTags->"T1.6.89.1"]},
  "T1.6.90.1"->{
    Cell[467247, 12706, 57, 1, 47, "Subsection",
      CellTags->"T1.6.90.1"]},
  "T1.6.91.1"->{
    Cell[468775, 12759, 59, 1, 47, "Subsection",
      CellTags->"T1.6.91.1"]},
  "T1.6.92.1"->{
    Cell[469079, 12773, 57, 1, 47, "Subsection",
      CellTags->"T1.6.92.1"]},
  "T1.6.93.1"->{
    Cell[470465, 12822, 59, 1, 47, "Subsection",
      CellTags->"T1.6.93.1"]},
  "T1.6.94.1"->{
    Cell[470743, 12836, 57, 1, 47, "Subsection",
      CellTags->"T1.6.94.1"]},
  "T1.6.95.1"->{
    Cell[474691, 12953, 59, 1, 47, "Subsection",
      CellTags->"T1.6.95.1"]},
  "T1.6.96.1"->{
    Cell[475004, 12967, 61, 1, 47, "Subsection",
      CellTags->"T1.6.96.1"]},
  "T1.6.97.1"->{
    Cell[475379, 12983, 61, 1, 47, "Subsection",
      CellTags->"T1.6.97.1"]},
  "T1.6.98.1"->{
    Cell[475754, 12999, 58, 1, 47, "Subsection",
      CellTags->"T1.6.98.1"]},
  "T1.6.99.1"->{
    Cell[481923, 13177, 60, 1, 47, "Subsection",
      CellTags->"T1.6.99.1"]},
  "T1.6.100.1"->{
    Cell[482381, 13200, 62, 1, 47, "Subsection",
      CellTags->"T1.6.100.1"]},
  "T1.6.101.1"->{
    Cell[482882, 13223, 62, 1, 47, "Subsection",
      CellTags->"T1.6.101.1"]},
  "T1.6.102.1"->{
    Cell[483422, 13248, 62, 1, 47, "Subsection",
      CellTags->"T1.6.102.1"]},
  "T1.6.103.1"->{
    Cell[483923, 13271, 62, 1, 47, "Subsection",
      CellTags->"T1.6.103.1"]},
  "T1.6.104.1"->{
    Cell[484424, 13294, 62, 1, 47, "Subsection",
      CellTags->"T1.6.104.1"]},
  "T1.6.105.1"->{
    Cell[484964, 13319, 59, 1, 47, "Subsection",
      CellTags->"T1.6.105.1"]},
  "T1.6.106.1"->{
    Cell[485737, 13343, 61, 1, 47, "Subsection",
      CellTags->"T1.6.106.1"]},
  "T1.6.107.1"->{
    Cell[485959, 13355, 59, 1, 47, "Subsection",
      CellTags->"T1.6.107.1"]},
  "T1.6.108.1"->{
    Cell[486842, 13383, 61, 1, 47, "Subsection",
      CellTags->"T1.6.108.1"]},
  "T1.6.109.1"->{
    Cell[487114, 13397, 59, 1, 47, "Subsection",
      CellTags->"T1.6.109.1"]},
  "T1.6.110.1"->{
    Cell[487834, 13420, 61, 1, 47, "Subsection",
      CellTags->"T1.6.110.1"]},
  "T1.6.111.1"->{
    Cell[488056, 13432, 59, 1, 47, "Subsection",
      CellTags->"T1.6.111.1"]},
  "T1.6.112.1"->{
    Cell[488528, 13449, 61, 1, 47, "Subsection",
      CellTags->"T1.6.112.1"]},
  "T1.6.113.1"->{
    Cell[488750, 13461, 59, 1, 47, "Subsection",
      CellTags->"T1.6.113.1"]},
  "T1.6.114.1"->{
    Cell[495906, 13650, 61, 1, 47, "Subsection",
      CellTags->"T1.6.114.1"]},
  "T1.6.115.1"->{
    Cell[496508, 13671, 61, 1, 47, "Subsection",
      CellTags->"T1.6.115.1"]},
  "T1.6.116.1"->{
    Cell[497301, 13708, 61, 1, 47, "Subsection",
      CellTags->"T1.6.116.1"]},
  "T1.6.117.1"->{
    Cell[498098, 13745, 62, 1, 47, "Subsection",
      CellTags->"T1.6.117.1"]},
  "T1.6.118.1"->{
    Cell[499283, 13772, 62, 1, 47, "Subsection",
      CellTags->"T1.6.118.1"]},
  "T1.6.119.1"->{
    Cell[500431, 13798, 62, 1, 47, "Subsection",
      CellTags->"T1.6.119.1"]},
  "T1.6.120.1"->{
    Cell[501577, 13824, 61, 1, 47, "Subsection",
      CellTags->"T1.6.120.1"]},
  "Tsymmetry"->{
    Cell[508097, 13967, 55, 1, 47, "Subsection",
      CellTags->"Tsymmetry"]},
  "specialTVR"->{
    Cell[508300, 13978, 70, 1, 47, "Subsection",
      CellTags->"specialTVR"]},
  "T1.6.121.1"->{
    Cell[519089, 14166, 60, 1, 47, "Subsection",
      CellTags->"T1.6.121.1"]},
  "T1.6.122.1"->{
    Cell[521094, 14227, 61, 1, 47, "Subsection",
      CellTags->"T1.6.122.1"]},
  "T1.6.123.1"->{
    Cell[522147, 14259, 60, 1, 47, "Subsection",
      CellTags->"T1.6.123.1"]},
  "T1.6.124.1"->{
    Cell[522835, 14283, 58, 1, 47, "Subsection",
      CellTags->"T1.6.124.1"]},
  "T1.7.1"->{
    Cell[523587, 14311, 108, 2, 39, "Section",
      CellTags->"T1.7.1"]},
  "T1.7.1.1"->{
    Cell[523720, 14317, 70, 1, 47, "Subsection",
      CellTags->"T1.7.1.1"]},
  "TLRdef"->{
    Cell[525585, 14380, 216, 7, 109, "Text",
      CellTags->"TLRdef"]},
  "tlrule17"->{
    Cell[536074, 14654, 78, 1, 33, "Text",
      CellTags->"tlrule17"]},
  "T1.7.2.1"->{
    Cell[562602, 15348, 50, 1, 31, "Subsection",
      CellTags->"T1.7.2.1"]},
  "T1.7.3.1"->{
    Cell[568103, 15480, 54, 1, 31, "Subsection",
      CellTags->"T1.7.3.1"]},
  "$TTable"->{
    Cell[569641, 15517, 52, 1, 31, "Subsection",
      CellTags->"$TTable"]},
  "T1.7.4.1"->{
    Cell[569783, 15526, 59, 1, 31, "Subsection",
      CellTags->"T1.7.4.1"]},
  "T1.8.1"->{
    Cell[579394, 15717, 92, 2, 39, "Section",
      CellTags->"T1.8.1"]},
  "T1.9.1"->{
    Cell[587864, 15895, 93, 2, 39, "Section",
      CellTags->"T1.9.1"]},
  "T1.9.1.1"->{
    Cell[587982, 15901, 54, 1, 47, "Subsection",
      CellTags->"T1.9.1.1"]},
  "T1.10.1"->{
    Cell[592169, 16020, 95, 2, 39, "Section",
      CellTags->"T1.10.1"]},
  "T1.11.1"->{
    Cell[608742, 16457, 101, 2, 39, "Section",
      CellTags->"T1.11.1"]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"T1.1", 613435, 16620},
  {"T1.2", 613513, 16623},
  {"T1.1.1", 613594, 16626},
  {"T1.2.1.1", 613682, 16629},
  {"T1.2.1", 613776, 16632},
  {"T1.3.1", 613862, 16635},
  {"T1.3.1.1", 613951, 16638},
  {"T1.3.2.1", 614044, 16641},
  {"T1.3.3.1", 614137, 16644},
  {"T1.3.4.1", 614230, 16647},
  {"T1.3.5.1", 614323, 16650},
  {"T1.3.6.1", 614416, 16653},
  {"T1.3.7.1", 614509, 16656},
  {"T1.3.7.1.1", 614604, 16659},
  {"T1.3.7.2.1", 614704, 16662},
  {"T1.3.7.3.1", 614804, 16665},
  {"T1.3.7.4.1", 614904, 16668},
  {"T1.3.7.5.1", 615004, 16671},
  {"T1.3.7.6.1", 615104, 16674},
  {"T1.3.7.7.1", 615204, 16677},
  {"T1.3.7.8.1", 615304, 16680},
  {"T1.4.1", 615400, 16683},
  {"T1.4.1.1", 615489, 16686},
  {"T1.4.2.1", 615583, 16689},
  {"T1.4.3.1", 615677, 16692},
  {"T1.4.4.1", 615771, 16695},
  {"T1.5.1", 615863, 16698},
  {"T1.5.1.1", 615952, 16701},
  {"T1.5.1.1.1", 616048, 16704},
  {"T1.5.2.1", 616147, 16707},
  {"T1.5.2.1.1", 616243, 16710},
  {"T1.5.3.1", 616342, 16713},
  {"T1.5.3.1.1", 616438, 16716},
  {"T1.5.3.2.1", 616539, 16719},
  {"T1.6.1", 616636, 16722},
  {"T1.6.1.1", 616726, 16725},
  {"T1.6.2.1", 616820, 16728},
  {"T1.6.2.1.1", 616916, 16731},
  {"T1.6.2.2.1", 617018, 16734},
  {"T1.6.2.3.1", 617120, 16737},
  {"T1.6.2.4.1", 617222, 16740},
  {"T1.6.3.1", 617322, 16743},
  {"T1.6.4.1", 617416, 16746},
  {"T1.6.5.1", 617511, 16749},
  {"T1.6.6.1", 617605, 16752},
  {"T1.6.7.1", 617700, 16755},
  {"T1.6.8.1", 617795, 16758},
  {"T1.6.9.1", 617890, 16761},
  {"T1.6.10.1", 617985, 16764},
  {"T1.6.11.1", 618081, 16767},
  {"T1.6.12.1", 618177, 16770},
  {"T1.6.13.1", 618273, 16773},
  {"T1.6.14.1", 618369, 16776},
  {"T1.6.15.1", 618465, 16779},
  {"T1.6.16.1", 618561, 16782},
  {"T1.6.17.1", 618657, 16785},
  {"T1.6.18.1", 618753, 16788},
  {"T1.6.19.1", 618849, 16791},
  {"T1.6.20.1", 618945, 16794},
  {"T1.6.21.1", 619041, 16797},
  {"T1.6.22.1", 619137, 16800},
  {"T1.6.23.1", 619233, 16803},
  {"T1.6.24.1", 619329, 16806},
  {"T1.6.25.1", 619425, 16809},
  {"T1.6.26.1", 619521, 16812},
  {"T1.6.27.1", 619617, 16815},
  {"T1.6.28.1", 619713, 16818},
  {"T1.6.29.1", 619809, 16821},
  {"T1.6.30.1", 619905, 16824},
  {"T1.6.31.1", 620001, 16827},
  {"T1.6.32.1", 620097, 16830},
  {"T1.6.33.1", 620193, 16833},
  {"T1.6.34.1", 620289, 16836},
  {"T1.6.35.1", 620385, 16839},
  {"T1.6.36.1", 620481, 16842},
  {"T1.6.37.1", 620577, 16845},
  {"T1.6.38.1", 620673, 16848},
  {"T1.6.39.1", 620769, 16851},
  {"T1.6.40.1", 620865, 16854},
  {"T1.6.41.1", 620961, 16857},
  {"T1.6.42.1", 621057, 16860},
  {"T1.6.43.1", 621153, 16863},
  {"T1.6.44.1", 621249, 16866},
  {"T1.6.45.1", 621345, 16869},
  {"T1.6.46.1", 621441, 16872},
  {"T1.6.47.1", 621537, 16875},
  {"T1.6.48.1", 621633, 16878},
  {"T1.6.49.1", 621729, 16881},
  {"T1.6.50.1", 621825, 16884},
  {"T1.6.51.1", 621921, 16887},
  {"T1.6.52.1", 622017, 16890},
  {"T1.6.53.1", 622113, 16893},
  {"T1.6.54.1", 622210, 16896},
  {"T1.6.55.1", 622307, 16899},
  {"T1.6.56.1", 622404, 16902},
  {"T1.6.57.1", 622501, 16905},
  {"T1.6.58.1", 622598, 16908},
  {"T1.6.59.1", 622695, 16911},
  {"T1.6.60.1", 622792, 16914},
  {"T1.6.61.1", 622889, 16917},
  {"T1.6.62.1", 622986, 16920},
  {"T1.6.63.1", 623083, 16923},
  {"T1.6.64.1", 623180, 16926},
  {"T1.6.65.1", 623277, 16929},
  {"T1.6.66.1", 623374, 16932},
  {"T1.6.67.1", 623471, 16935},
  {"T1.6.68.1", 623568, 16938},
  {"T1.6.69.1", 623665, 16941},
  {"T1.6.70.1", 623762, 16944},
  {"T1.6.70.1.1", 623861, 16947},
  {"T1.6.70.2.1", 623965, 16950},
  {"T1.6.71.1", 624067, 16953},
  {"T1.6.72.1", 624164, 16956},
  {"T1.6.73.1", 624261, 16959},
  {"T1.6.74.1", 624358, 16962},
  {"T1.6.75.1", 624455, 16965},
  {"T1.6.76.1", 624552, 16968},
  {"T1.6.77.1", 624649, 16971},
  {"T1.6.78.1", 624746, 16974},
  {"T1.6.79.1", 624843, 16977},
  {"T1.6.80.1", 624940, 16980},
  {"T1.6.81.1", 625037, 16983},
  {"T1.6.82.1", 625134, 16986},
  {"T1.6.83.1", 625231, 16989},
  {"T1.6.84.1", 625328, 16992},
  {"T1.6.85.1", 625425, 16995},
  {"T1.6.86.1", 625522, 16998},
  {"T1.6.87.1", 625619, 17001},
  {"T1.6.88.1", 625716, 17004},
  {"T1.6.89.1", 625813, 17007},
  {"T1.6.90.1", 625910, 17010},
  {"T1.6.91.1", 626007, 17013},
  {"T1.6.92.1", 626104, 17016},
  {"T1.6.93.1", 626201, 17019},
  {"T1.6.94.1", 626298, 17022},
  {"T1.6.95.1", 626395, 17025},
  {"T1.6.96.1", 626492, 17028},
  {"T1.6.97.1", 626589, 17031},
  {"T1.6.98.1", 626686, 17034},
  {"T1.6.99.1", 626783, 17037},
  {"T1.6.100.1", 626881, 17040},
  {"T1.6.101.1", 626980, 17043},
  {"T1.6.102.1", 627079, 17046},
  {"T1.6.103.1", 627178, 17049},
  {"T1.6.104.1", 627277, 17052},
  {"T1.6.105.1", 627376, 17055},
  {"T1.6.106.1", 627475, 17058},
  {"T1.6.107.1", 627574, 17061},
  {"T1.6.108.1", 627673, 17064},
  {"T1.6.109.1", 627772, 17067},
  {"T1.6.110.1", 627871, 17070},
  {"T1.6.111.1", 627970, 17073},
  {"T1.6.112.1", 628069, 17076},
  {"T1.6.113.1", 628168, 17079},
  {"T1.6.114.1", 628267, 17082},
  {"T1.6.115.1", 628366, 17085},
  {"T1.6.116.1", 628465, 17088},
  {"T1.6.117.1", 628564, 17091},
  {"T1.6.118.1", 628663, 17094},
  {"T1.6.119.1", 628762, 17097},
  {"T1.6.120.1", 628861, 17100},
  {"Tsymmetry", 628959, 17103},
  {"specialTVR", 629057, 17106},
  {"T1.6.121.1", 629156, 17109},
  {"T1.6.122.1", 629255, 17112},
  {"T1.6.123.1", 629354, 17115},
  {"T1.6.124.1", 629453, 17118},
  {"T1.7.1", 629548, 17121},
  {"T1.7.1.1", 629639, 17124},
  {"TLRdef", 629732, 17127},
  {"tlrule17", 629821, 17130},
  {"T1.7.2.1", 629910, 17133},
  {"T1.7.3.1", 630005, 17136},
  {"$TTable", 630099, 17139},
  {"T1.7.4.1", 630193, 17142},
  {"T1.8.1", 630286, 17145},
  {"T1.9.1", 630374, 17148},
  {"T1.9.1.1", 630464, 17151},
  {"T1.10.1", 630558, 17154},
  {"T1.11.1", 630648, 17157}
  }
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1727, 52, 73, 2, 115, "Title",
  CellTags->"T1.1"],
Cell[1803, 56, 177, 6, 33, "Text"],
Cell[1983, 64, 434, 14, 109, "Text",
  CellTags->"T1.2"],
Cell[2420, 80, 55, 1, 33, "Text"],
Cell[2478, 83, 157, 4, 33, "Text"],
Cell[2638, 89, 131, 3, 33, "Text"],
Cell[2772, 94, 137, 3, 33, "Text"],
Cell[2912, 99, 186, 6, 33, "Text"],
Cell[3101, 107, 158, 6, 33, "Text"],
Cell[3262, 115, 171, 6, 33, "Text"],
Cell[3436, 123, 162, 6, 33, "Text"],
Cell[3601, 131, 162, 6, 33, "Text"],
Cell[3766, 139, 162, 6, 33, "Text"],
Cell[3931, 147, 167, 6, 33, "Text"],
Cell[4101, 155, 156, 3, 33, "Text"],
Cell[4260, 160, 1081, 21, 261, "Text"],
Cell[5344, 183, 147, 3, 33, "Text"],
Cell[5494, 188, 499, 10, 109, "Text"],
Cell[5996, 200, 94, 3, 33, "Text"],
Cell[6093, 205, 176, 4, 33, "Text"],
Cell[6272, 211, 228, 6, 52, "Text"],
Cell[6503, 219, 55, 1, 30, "Input"],
Cell[6561, 222, 180, 4, 33, "Text"],
Cell[6744, 228, 212, 4, 52, "Text"],
Cell[6959, 234, 262, 8, 33, "Text"],

Cell[CellGroupData[{
Cell[7246, 246, 75, 1, 59, "Section"],
Cell[7324, 249, 102, 2, 33, "Text"],
Cell[7429, 253, 95, 2, 33, "Text"],
Cell[7527, 257, 120, 2, 33, "Text"],
Cell[7650, 261, 121, 2, 33, "Text"],
Cell[7774, 265, 108, 2, 33, "Text"],
Cell[7885, 269, 103, 2, 33, "Text"],
Cell[7991, 273, 104, 2, 33, "Text"],
Cell[8098, 277, 105, 2, 33, "Text"],
Cell[8206, 281, 99, 2, 33, "Text"],
Cell[8308, 285, 104, 2, 33, "Text"],
Cell[8415, 289, 95, 2, 33, "Text"],
Cell[8513, 293, 104, 2, 28, "SmallText"],
Cell[8620, 297, 100, 2, 28, "SmallText"],
Cell[8723, 301, 101, 2, 28, "SmallText"],
Cell[8827, 305, 105, 2, 28, "SmallText"],
Cell[8935, 309, 101, 2, 28, "SmallText"],
Cell[9039, 313, 101, 2, 28, "SmallText"],
Cell[9143, 317, 101, 2, 28, "SmallText"],
Cell[9247, 321, 101, 2, 28, "SmallText"],
Cell[9351, 325, 113, 2, 33, "Text"],
Cell[9467, 329, 143, 3, 33, "Text"],
Cell[9613, 334, 141, 3, 33, "Text"],
Cell[9757, 339, 530, 15, 34, "Text"],
Cell[10290, 356, 547, 15, 34, "Text"],
Cell[10840, 373, 106, 2, 33, "Text"],
Cell[10949, 377, 110, 2, 33, "Text"],
Cell[11062, 381, 109, 2, 28, "SmallText"],
Cell[11174, 385, 189, 7, 33, "Text"],
Cell[11366, 394, 100, 2, 28, "SmallText"],
Cell[11469, 398, 105, 2, 33, "Text"],
Cell[11577, 402, 115, 2, 28, "SmallText"],
Cell[11695, 406, 115, 2, 28, "SmallText"],
Cell[11813, 410, 151, 3, 33, "Text"],
Cell[11967, 415, 101, 2, 33, "Text"],
Cell[12071, 419, 103, 2, 33, "Text"],
Cell[12177, 423, 485, 13, 31, "SmallText"],
Cell[12665, 438, 485, 13, 31, "SmallText"],
Cell[13153, 453, 485, 13, 31, "SmallText"],
Cell[13641, 468, 485, 13, 31, "SmallText"],
Cell[14129, 483, 103, 2, 33, "Text"],
Cell[14235, 487, 523, 15, 37, "Text"],
Cell[14761, 504, 103, 2, 33, "Text"],
Cell[14867, 508, 523, 15, 37, "Text"],
Cell[15393, 525, 523, 15, 37, "Text"],
Cell[15919, 542, 523, 15, 37, "Text"],
Cell[16445, 559, 101, 2, 33, "Text"],
Cell[16549, 563, 104, 2, 33, "Text"],
Cell[16656, 567, 102, 2, 33, "Text"],
Cell[16761, 571, 102, 2, 33, "Text"],
Cell[16866, 575, 102, 2, 33, "Text"],
Cell[16971, 579, 113, 2, 33, "Text"],
Cell[17087, 583, 104, 2, 33, "Text"],
Cell[17194, 587, 102, 2, 33, "Text"],
Cell[17299, 591, 104, 2, 33, "Text"],
Cell[17406, 595, 102, 2, 33, "Text"],
Cell[17511, 599, 104, 2, 33, "Text"],
Cell[17618, 603, 104, 2, 33, "Text"],
Cell[17725, 607, 102, 2, 33, "Text"],
Cell[17830, 611, 104, 2, 33, "Text"],
Cell[17937, 615, 102, 2, 33, "Text"],
Cell[18042, 619, 104, 2, 33, "Text"],
Cell[18149, 623, 102, 2, 33, "Text"],
Cell[18254, 627, 104, 2, 33, "Text"],
Cell[18361, 631, 102, 2, 33, "Text"],
Cell[18466, 635, 104, 2, 33, "Text"],
Cell[18573, 639, 102, 2, 33, "Text"],
Cell[18678, 643, 104, 2, 33, "Text"],
Cell[18785, 647, 102, 2, 33, "Text"],
Cell[18890, 651, 104, 2, 33, "Text"],
Cell[18997, 655, 106, 2, 33, "Text"],
Cell[19106, 659, 102, 2, 33, "Text"],
Cell[19211, 663, 104, 2, 33, "Text"],
Cell[19318, 667, 102, 2, 33, "Text"],
Cell[19423, 671, 104, 2, 33, "Text"],
Cell[19530, 675, 106, 2, 33, "Text"],
Cell[19639, 679, 106, 2, 33, "Text"],
Cell[19748, 683, 106, 2, 33, "Text"],
Cell[19857, 687, 102, 2, 33, "Text"],
Cell[19962, 691, 104, 2, 33, "Text"],
Cell[20069, 695, 113, 2, 33, "Text"],
Cell[20185, 699, 102, 2, 33, "Text"],
Cell[20290, 703, 104, 2, 33, "Text"],
Cell[20397, 707, 106, 2, 33, "Text"],
Cell[20506, 711, 102, 2, 33, "Text"],
Cell[20611, 715, 104, 2, 33, "Text"],
Cell[20718, 719, 106, 2, 33, "Text"],
Cell[20827, 723, 102, 2, 33, "Text"],
Cell[20932, 727, 104, 2, 33, "Text"],
Cell[21039, 731, 102, 2, 33, "Text"],
Cell[21144, 735, 104, 2, 33, "Text"],
Cell[21251, 739, 106, 2, 33, "Text"],
Cell[21360, 743, 106, 2, 33, "Text"],
Cell[21469, 747, 102, 2, 33, "Text"],
Cell[21574, 751, 104, 2, 33, "Text"],
Cell[21681, 755, 102, 2, 33, "Text"],
Cell[21786, 759, 104, 2, 33, "Text"],
Cell[21893, 763, 102, 2, 33, "Text"],
Cell[21998, 767, 104, 2, 33, "Text"],
Cell[22105, 771, 106, 2, 33, "Text"],
Cell[22214, 775, 106, 2, 33, "Text"],
Cell[22323, 779, 102, 2, 33, "Text"],
Cell[22428, 783, 104, 2, 33, "Text"],
Cell[22535, 787, 106, 2, 33, "Text"],
Cell[22644, 791, 106, 2, 33, "Text"],
Cell[22753, 795, 102, 2, 33, "Text"],
Cell[22858, 799, 104, 2, 33, "Text"],
Cell[22965, 803, 102, 2, 33, "Text"],
Cell[23070, 807, 103, 2, 28, "SmallText"],
Cell[23176, 811, 103, 2, 28, "SmallText"],
Cell[23282, 815, 104, 2, 33, "Text"],
Cell[23389, 819, 102, 2, 33, "Text"],
Cell[23494, 823, 104, 2, 33, "Text"],
Cell[23601, 827, 102, 2, 33, "Text"],
Cell[23706, 831, 105, 2, 33, "Text"],
Cell[23814, 835, 105, 2, 33, "Text"],
Cell[23922, 839, 105, 2, 33, "Text"],
Cell[24030, 843, 105, 2, 33, "Text"],
Cell[24138, 847, 102, 2, 33, "Text"],
Cell[24243, 851, 102, 2, 33, "Text"],
Cell[24348, 855, 104, 2, 33, "Text"],
Cell[24455, 859, 102, 2, 33, "Text"],
Cell[24560, 863, 104, 2, 33, "Text"],
Cell[24667, 867, 106, 2, 33, "Text"],
Cell[24776, 871, 106, 2, 33, "Text"],
Cell[24885, 875, 102, 2, 33, "Text"],
Cell[24990, 879, 104, 2, 33, "Text"],
Cell[25097, 883, 102, 2, 33, "Text"],
Cell[25202, 887, 104, 2, 33, "Text"],
Cell[25309, 891, 102, 2, 33, "Text"],
Cell[25414, 895, 104, 2, 33, "Text"],
Cell[25521, 899, 102, 2, 33, "Text"],
Cell[25626, 903, 104, 2, 33, "Text"],
Cell[25733, 907, 102, 2, 33, "Text"],
Cell[25838, 911, 104, 2, 33, "Text"],
Cell[25945, 915, 106, 2, 33, "Text"],
Cell[26054, 919, 106, 2, 33, "Text"],
Cell[26163, 923, 103, 2, 33, "Text"],
Cell[26269, 927, 105, 2, 33, "Text"],
Cell[26377, 931, 107, 2, 33, "Text"],
Cell[26487, 935, 107, 2, 33, "Text"],
Cell[26597, 939, 107, 2, 33, "Text"],
Cell[26707, 943, 107, 2, 33, "Text"],
Cell[26817, 947, 107, 2, 33, "Text"],
Cell[26927, 951, 104, 2, 33, "Text"],
Cell[27034, 955, 106, 2, 33, "Text"],
Cell[27143, 959, 104, 2, 33, "Text"],
Cell[27250, 963, 106, 2, 33, "Text"],
Cell[27359, 967, 104, 2, 33, "Text"],
Cell[27466, 971, 106, 2, 33, "Text"],
Cell[27575, 975, 104, 2, 33, "Text"],
Cell[27682, 979, 106, 2, 33, "Text"],
Cell[27791, 983, 104, 2, 33, "Text"],
Cell[27898, 987, 106, 2, 33, "Text"],
Cell[28007, 991, 106, 2, 33, "Text"],
Cell[28116, 995, 106, 2, 33, "Text"],
Cell[28225, 999, 106, 2, 33, "Text"],
Cell[28334, 1003, 106, 2, 33, "Text"],
Cell[28443, 1007, 106, 2, 33, "Text"],
Cell[28552, 1011, 106, 2, 33, "Text"],
Cell[28661, 1015, 105, 2, 33, "Text"],
Cell[28769, 1019, 106, 2, 33, "Text"],
Cell[28878, 1023, 105, 2, 33, "Text"],
Cell[28986, 1027, 103, 2, 33, "Text"],
Cell[29092, 1031, 117, 2, 33, "Text"],
Cell[29212, 1035, 101, 2, 33, "Text"],
Cell[29316, 1039, 95, 2, 33, "Text"],
Cell[29414, 1043, 99, 2, 33, "Text"],
Cell[29516, 1047, 104, 2, 33, "Text"],
Cell[29623, 1051, 101, 2, 33, "Text"],
Cell[29727, 1055, 102, 2, 33, "Text"],
Cell[29832, 1059, 105, 2, 33, "Text"],
Cell[29940, 1063, 93, 2, 33, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[30070, 1070, 93, 2, 39, "Section",
  CellTags->"T1.1.1"],
Cell[30166, 1074, 342, 10, 60, "SmallText"],
Cell[30511, 1086, 118, 3, 33, "Text"],
Cell[30632, 1091, 1691, 43, 87, "Text"],
Cell[32326, 1136, 222, 5, 90, "Text"],
Cell[32551, 1143, 50, 1, 30, "Input"],
Cell[32604, 1146, 64, 1, 30, "Input"],
Cell[32671, 1149, 89, 1, 30, "Input"],
Cell[32763, 1152, 56, 1, 30, "Input"],
Cell[32822, 1155, 107, 3, 33, "Text"],
Cell[32932, 1160, 178, 5, 30, "Input"],
Cell[33113, 1167, 220, 6, 28, "SmallText"],
Cell[33336, 1175, 233, 4, 50, "Input"],
Cell[33572, 1181, 57, 1, 30, "Input"],
Cell[33632, 1184, 58, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[33715, 1189, 78, 1, 43, "Subsubsection",
  CellTags->"T1.2.1.1"],
Cell[33796, 1192, 512, 11, 109, "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[34357, 1209, 86, 2, 39, "Section",
  CellTags->"T1.2.1"],
Cell[34446, 1213, 83, 1, 30, "Input"],
Cell[34532, 1216, 257, 4, 70, "Input"],
Cell[34792, 1222, 631, 9, 170, "Input"],
Cell[35426, 1233, 157, 2, 50, "Input"],
Cell[35586, 1237, 120, 2, 30, "Input"],
Cell[35709, 1241, 135, 2, 30, "Input"],
Cell[35847, 1245, 135, 2, 30, "Input"],
Cell[35985, 1249, 147, 2, 30, "Input"],
Cell[36135, 1253, 123, 2, 30, "Input"],
Cell[36261, 1257, 334, 5, 90, "Input"],
Cell[36598, 1264, 211, 3, 70, "Input"],
Cell[36812, 1269, 227, 3, 70, "Input"],
Cell[37042, 1274, 176, 3, 50, "Input"],
Cell[37221, 1279, 172, 3, 50, "Input"],
Cell[37396, 1284, 194, 3, 50, "Input"],
Cell[37593, 1289, 177, 3, 50, "Input"],
Cell[37773, 1294, 168, 3, 50, "Input"],
Cell[37944, 1299, 149, 2, 30, "Input"],
Cell[38096, 1303, 149, 2, 30, "Input"],
Cell[38248, 1307, 149, 2, 30, "Input"],
Cell[38400, 1311, 149, 2, 30, "Input"],
Cell[38552, 1315, 149, 2, 30, "Input"],
Cell[38704, 1319, 449, 6, 130, "Input"],
Cell[39156, 1327, 189, 3, 70, "Input"],
Cell[39348, 1332, 182, 3, 70, "Input"],
Cell[39533, 1337, 144, 2, 30, "Input"],
Cell[39680, 1341, 248, 3, 70, "Input"],
Cell[39931, 1346, 179, 3, 70, "Input"],
Cell[40113, 1351, 60, 1, 30, "Input"],
Cell[40176, 1354, 56, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[40269, 1360, 112, 2, 39, "Section",
  CellTags->"T1.3.1"],
Cell[40384, 1364, 798, 24, 33, "Text"],
Cell[41185, 1390, 57, 1, 30, "Input"],
Cell[41245, 1393, 73, 0, 33, "Text"],
Cell[41321, 1395, 61, 1, 30, "Input"],
Cell[41385, 1398, 115, 3, 33, "Text"],
Cell[41503, 1403, 58, 1, 30, "Input"],
Cell[41564, 1406, 754, 24, 33, "Text"],

Cell[CellGroupData[{
Cell[42343, 1434, 63, 1, 47, "Subsection",
  CellTags->"T1.3.1.1"],
Cell[42409, 1437, 60, 1, 30, "Input"],
Cell[42472, 1440, 265, 5, 70, "Input"],
Cell[42740, 1447, 265, 5, 70, "Input"],
Cell[43008, 1454, 174, 3, 50, "Input"],
Cell[43185, 1459, 115, 3, 33, "Text"],
Cell[43303, 1464, 213, 4, 50, "Input"],
Cell[43519, 1470, 208, 4, 30, "Input"],
Cell[43730, 1476, 225, 4, 50, "Input"],
Cell[43958, 1482, 256, 5, 50, "Input"],
Cell[44217, 1489, 146, 2, 30, "Input"],
Cell[44366, 1493, 163, 2, 30, "Input"],
Cell[44532, 1497, 205, 4, 50, "Input"],
Cell[44740, 1503, 182, 3, 50, "Input"],
Cell[44925, 1508, 513, 13, 138, "Input"],
Cell[45441, 1523, 108, 2, 30, "Input"],
Cell[45552, 1527, 108, 2, 30, "Input"],
Cell[45663, 1531, 108, 2, 30, "Input"],
Cell[45774, 1535, 108, 2, 30, "Input"],
Cell[45885, 1539, 108, 2, 30, "Input"],
Cell[45996, 1543, 76, 1, 30, "Input"],
Cell[46075, 1546, 79, 1, 30, "Input"],
Cell[46157, 1549, 119, 2, 30, "Input"],
Cell[46279, 1553, 311, 7, 50, "Input"],
Cell[46593, 1562, 154, 3, 30, "Input"],
Cell[46750, 1567, 179, 3, 30, "Input"],
Cell[46932, 1572, 119, 2, 30, "Input"],
Cell[47054, 1576, 154, 3, 30, "Input"],
Cell[47211, 1581, 311, 7, 50, "Input"],
Cell[47525, 1590, 596, 11, 110, "Input"],
Cell[48124, 1603, 686, 12, 150, "Input"],
Cell[48813, 1617, 621, 12, 130, "Input"],
Cell[49437, 1631, 702, 13, 130, "Input"],
Cell[50142, 1646, 202, 3, 30, "Input"],
Cell[50347, 1651, 728, 13, 130, "Input"],
Cell[51078, 1666, 619, 11, 110, "Input"],
Cell[51700, 1679, 426, 8, 90, "Input"],
Cell[52129, 1689, 619, 11, 110, "Input"],
Cell[52751, 1702, 172, 3, 30, "Input"],
Cell[52926, 1707, 426, 8, 90, "Input"],
Cell[53355, 1717, 550, 10, 110, "Input"],
Cell[53908, 1729, 135, 2, 30, "Input"],
Cell[54046, 1733, 546, 10, 110, "Input"],
Cell[54595, 1745, 108, 2, 30, "Input"],
Cell[54706, 1749, 107, 3, 52, "Text"],
Cell[54816, 1754, 630, 11, 130, "Input"],
Cell[55449, 1767, 628, 11, 130, "Input"],
Cell[56080, 1780, 624, 12, 130, "Input"],
Cell[56707, 1794, 707, 13, 130, "Input"],
Cell[57417, 1809, 733, 13, 130, "Input"],
Cell[58153, 1824, 637, 12, 110, "Input"],
Cell[58793, 1838, 428, 8, 90, "Input"],
Cell[59224, 1848, 553, 10, 110, "Input"],
Cell[59780, 1860, 549, 10, 110, "Input"],
Cell[60332, 1872, 627, 11, 110, "Input"],
Cell[60962, 1885, 625, 11, 110, "Input"],
Cell[61590, 1898, 621, 12, 130, "Input"],
Cell[62214, 1912, 702, 13, 130, "Input"],
Cell[62919, 1927, 728, 13, 130, "Input"],
Cell[63650, 1942, 619, 11, 110, "Input"],
Cell[64272, 1955, 426, 8, 90, "Input"],
Cell[64701, 1965, 550, 10, 110, "Input"],
Cell[65254, 1977, 546, 10, 110, "Input"],
Cell[65803, 1989, 115, 2, 30, "Input"],
Cell[65921, 1993, 115, 2, 30, "Input"],
Cell[66039, 1997, 115, 2, 30, "Input"],
Cell[66157, 2001, 115, 2, 30, "Input"],
Cell[66275, 2005, 115, 2, 30, "Input"],
Cell[66393, 2009, 133, 2, 31, "Input"],
Cell[66529, 2013, 121, 2, 30, "Input"],
Cell[66653, 2017, 214, 4, 50, "Input"],
Cell[66870, 2023, 111, 2, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[67018, 2030, 58, 1, 47, "Subsection",
  CellTags->"T1.3.2.1"],
Cell[67079, 2033, 4534, 78, 1279, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[71650, 2116, 59, 1, 47, "Subsection",
  CellTags->"T1.3.3.1"],
Cell[71712, 2119, 571, 10, 154, "Input"],
Cell[72286, 2131, 591, 10, 154, "Input"],
Cell[72880, 2143, 591, 10, 154, "Input"],
Cell[73474, 2155, 591, 10, 154, "Input"],
Cell[74068, 2167, 591, 10, 154, "Input"],
Cell[74662, 2179, 591, 10, 154, "Input"],
Cell[75256, 2191, 591, 10, 154, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[75884, 2206, 60, 1, 47, "Subsection",
  CellTags->"T1.3.4.1"],
Cell[75947, 2209, 685, 14, 130, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[76669, 2228, 54, 1, 47, "Subsection",
  CellTags->"T1.3.5.1"],
Cell[76726, 2231, 129, 3, 48, "Input"],
Cell[76858, 2236, 2089, 37, 470, "Input"],
Cell[78950, 2275, 193, 3, 50, "Input"],
Cell[79146, 2280, 306, 9, 29, "Text"],
Cell[79455, 2291, 2024, 34, 363, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[81516, 2330, 59, 1, 47, "Subsection",
  CellTags->"T1.3.6.1"],
Cell[81578, 2333, 486, 9, 130, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[82101, 2347, 50, 1, 47, "Subsection",
  CellTags->"T1.3.7.1"],
Cell[82154, 2350, 87, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[82266, 2355, 57, 1, 43, "Subsubsection",
  CellTags->"T1.3.7.1.1"],
Cell[82326, 2358, 779, 28, 37, "Text"],
Cell[83108, 2388, 217, 4, 50, "Input"],
Cell[83328, 2394, 154, 3, 30, "Input"],
Cell[83485, 2399, 156, 3, 30, "Input"],
Cell[83644, 2404, 160, 3, 30, "Input"],
Cell[83807, 2409, 150, 3, 30, "Input"],
Cell[83960, 2414, 158, 3, 30, "Input"],
Cell[84121, 2419, 148, 3, 30, "Input"],
Cell[84272, 2424, 148, 3, 30, "Input"],
Cell[84423, 2429, 156, 3, 30, "Input"],
Cell[84582, 2434, 152, 3, 30, "Input"],
Cell[84737, 2439, 132, 2, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[84906, 2446, 53, 1, 43, "Subsubsection",
  CellTags->"T1.3.7.2.1"],
Cell[84962, 2449, 71, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[85070, 2455, 54, 1, 43, "Subsubsection",
  CellTags->"T1.3.7.3.1"],
Cell[85127, 2458, 51, 1, 30, "Input"],
Cell[85181, 2461, 72, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[85290, 2467, 58, 1, 43, "Subsubsection",
  CellTags->"T1.3.7.4.1"],
Cell[85351, 2470, 74, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[85462, 2476, 54, 1, 43, "Subsubsection",
  CellTags->"T1.3.7.5.1"],
Cell[85519, 2479, 112, 2, 30, "Input"],
Cell[85634, 2483, 121, 2, 30, "Input"],
Cell[85758, 2487, 76, 1, 30, "Input"],
Cell[85837, 2490, 152, 2, 30, "Input"],
Cell[85992, 2494, 134, 2, 30, "Input"],
Cell[86129, 2498, 133, 2, 30, "Input"],
Cell[86265, 2502, 116, 2, 30, "Input"],
Cell[86384, 2506, 167, 2, 50, "Input"],
Cell[86554, 2510, 92, 1, 30, "Input"],
Cell[86649, 2513, 92, 1, 30, "Input"],
Cell[86744, 2516, 92, 1, 30, "Input"],
Cell[86839, 2519, 92, 1, 30, "Input"],
Cell[86934, 2522, 92, 1, 30, "Input"],
Cell[87029, 2525, 92, 1, 30, "Input"],
Cell[87124, 2528, 92, 1, 30, "Input"],
Cell[87219, 2531, 92, 1, 30, "Input"],
Cell[87314, 2534, 92, 1, 30, "Input"],
Cell[87409, 2537, 92, 1, 30, "Input"],
Cell[87504, 2540, 92, 1, 30, "Input"],
Cell[87599, 2543, 92, 1, 30, "Input"],
Cell[87694, 2546, 92, 1, 30, "Input"],
Cell[87789, 2549, 92, 1, 30, "Input"],
Cell[87884, 2552, 92, 1, 30, "Input"],
Cell[87979, 2555, 92, 1, 30, "Input"],
Cell[88074, 2558, 92, 1, 30, "Input"],
Cell[88169, 2561, 92, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[88298, 2567, 54, 1, 43, "Subsubsection",
  CellTags->"T1.3.7.6.1"],
Cell[88355, 2570, 375, 7, 70, "Input"],
Cell[88733, 2579, 72, 1, 30, "Input"],
Cell[88808, 2582, 74, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[88919, 2588, 54, 1, 43, "Subsubsection",
  CellTags->"T1.3.7.7.1"],
Cell[88976, 2591, 590, 14, 70, "Input"],
Cell[89569, 2607, 66, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[89672, 2613, 54, 1, 43, "Subsubsection",
  CellTags->"T1.3.7.8.1"],
Cell[89729, 2616, 215, 6, 30, "Input"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[90005, 2629, 104, 2, 39, "Section",
  CellTags->"T1.4.1"],
Cell[90112, 2633, 574, 11, 89, "Input"],
Cell[90689, 2646, 923, 15, 206, "Input"],
Cell[91615, 2663, 136, 4, 33, "Text"],
Cell[91754, 2669, 147, 2, 31, "Input"],
Cell[91904, 2673, 61, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[91990, 2678, 116, 3, 46, "Subsection",
  CellTags->"T1.4.1.1"],
Cell[92109, 2683, 213, 6, 30, "Input"],
Cell[92325, 2691, 72, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[92434, 2697, 114, 3, 46, "Subsection",
  CellTags->"T1.4.2.1"],
Cell[92551, 2702, 113, 5, 37, "Text"],
Cell[92667, 2709, 1952, 52, 72, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[94656, 2766, 205, 6, 46, "Subsection",
  CellTags->"T1.4.3.1"],
Cell[94864, 2774, 2381, 57, 73, "Input"],
Cell[97248, 2833, 157, 4, 30, "Input"],
Cell[97408, 2839, 1123, 24, 72, "Input"],
Cell[98534, 2865, 772, 19, 52, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[99343, 2889, 223, 6, 59, "Subsection",
  CellTags->"T1.4.4.1"],
Cell[99569, 2897, 3158, 73, 92, "Input"],
Cell[102730, 2972, 2826, 70, 93, "Input"],
Cell[105559, 3044, 2826, 70, 93, "Input"],
Cell[108388, 3116, 2826, 70, 93, "Input"],
Cell[111217, 3188, 157, 4, 30, "Input"],
Cell[111377, 3194, 157, 4, 30, "Input"],
Cell[111537, 3200, 143, 4, 30, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[111729, 3210, 97, 2, 39, "Section",
  CellTags->"T1.5.1"],

Cell[CellGroupData[{
Cell[111851, 3216, 65, 1, 47, "Subsection",
  CellTags->"T1.5.1.1"],
Cell[111919, 3219, 131, 3, 52, "Text"],
Cell[112053, 3224, 67, 0, 33, "Text"],
Cell[112123, 3226, 154, 5, 33, "Text"],
Cell[112280, 3233, 1216, 20, 135, "Input"],
Cell[113499, 3255, 1328, 22, 113, "Input"],
Cell[114830, 3279, 105, 2, 30, "Input"],
Cell[114938, 3283, 87, 1, 30, "Input"],
Cell[115028, 3286, 120, 2, 30, "Input"],
Cell[115151, 3290, 111, 2, 30, "Input"],
Cell[115265, 3294, 87, 1, 30, "Input"],
Cell[115355, 3297, 129, 2, 30, "Input"],
Cell[115487, 3301, 71, 1, 33, "Input"],
Cell[115561, 3304, 59, 0, 30, "Input"],
Cell[115623, 3306, 78, 1, 30, "Input"],
Cell[115704, 3309, 78, 1, 30, "Input"],
Cell[115785, 3312, 79, 1, 30, "Input"],
Cell[115867, 3315, 92, 1, 30, "Input"],
Cell[115962, 3318, 80, 1, 30, "Input"],
Cell[116045, 3321, 80, 1, 30, "Input"],
Cell[116128, 3324, 694, 11, 110, "Input"],
Cell[116825, 3337, 51, 1, 30, "Input"],
Cell[116879, 3340, 191, 4, 50, "Input"],
Cell[117073, 3346, 79, 1, 30, "Input"],
Cell[117155, 3349, 76, 0, 33, "Text"],
Cell[117234, 3351, 202, 6, 30, "Input"],
Cell[117439, 3359, 152, 4, 30, "Input"],
Cell[117594, 3365, 9785, 176, 403, "Input"],
Cell[127382, 3543, 9131, 163, 382, "Input"],
Cell[136516, 3708, 12045, 216, 493, "Input"],
Cell[148564, 3926, 11116, 201, 412, "Input"],
Cell[159683, 4129, 10160, 183, 494, "Input"],
Cell[169846, 4314, 68, 1, 30, "Input"],
Cell[169917, 4317, 804, 19, 70, "Input"],
Cell[170724, 4338, 753, 19, 70, "Input"],
Cell[171480, 4359, 753, 19, 70, "Input"],

Cell[CellGroupData[{
Cell[172258, 4382, 62, 1, 43, "Subsubsection",
  CellTags->"T1.5.1.1.1"],
Cell[172323, 4385, 2700, 62, 194, "Input"],
Cell[175026, 4449, 151, 2, 30, "Input"],
Cell[175180, 4453, 143, 2, 30, "Input"],
Cell[175326, 4457, 153, 2, 30, "Input"],
Cell[175482, 4461, 183, 3, 30, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[175714, 4470, 59, 1, 47, "Subsection",
  CellTags->"T1.5.2.1"],
Cell[175776, 4473, 1252, 20, 242, "Input"],

Cell[CellGroupData[{
Cell[177053, 4497, 53, 1, 43, "Subsubsection",
  CellTags->"T1.5.2.1.1"],
Cell[177109, 4500, 132, 2, 30, "Input"],
Cell[177244, 4504, 143, 2, 30, "Input"],
Cell[177390, 4508, 135, 3, 30, "Input"],
Cell[177528, 4513, 251, 9, 37, "Text"],
Cell[177782, 4524, 20, 0, 33, "Text"],
Cell[177805, 4526, 1281, 26, 93, "Input"],
Cell[179089, 4554, 20, 0, 33, "Text"],
Cell[179112, 4556, 759, 12, 93, "Input"],
Cell[179874, 4570, 20, 0, 33, "Text"],
Cell[179897, 4572, 552, 10, 73, "Input"],
Cell[180452, 4584, 20, 0, 33, "Text"],
Cell[180475, 4586, 549, 9, 97, "Input"],
Cell[181027, 4597, 20, 0, 33, "Text"],
Cell[181050, 4599, 799, 14, 106, "Input"],
Cell[181852, 4615, 20, 0, 33, "Text"],
Cell[181875, 4617, 1130, 24, 93, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[183054, 4647, 60, 1, 47, "Subsection",
  CellTags->"T1.5.3.1"],
Cell[183117, 4650, 176, 4, 33, "Text"],
Cell[183296, 4656, 139, 3, 52, "Text"],

Cell[CellGroupData[{
Cell[183460, 4663, 68, 1, 43, "Subsubsection",
  CellTags->"T1.5.3.1.1"],
Cell[183531, 4666, 369, 8, 67, "Input"],
Cell[183903, 4676, 781, 13, 177, "Input"],
Cell[184687, 4691, 1806, 32, 458, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[186530, 4728, 68, 1, 43, "Subsubsection",
  CellTags->"T1.5.3.2.1"],
Cell[186601, 4731, 304, 6, 67, "Input"],
Cell[186908, 4739, 185, 3, 50, "Input"],
Cell[187096, 4744, 62, 1, 30, "Input"],
Cell[187161, 4747, 1780, 33, 437, "Input"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[189002, 4787, 140, 2, 39, "Section",
  CellTags->"T1.6.1"],
Cell[189145, 4791, 272, 7, 52, "Text"],

Cell[CellGroupData[{
Cell[189442, 4802, 56, 1, 47, "Subsection",
  CellTags->"T1.6.1.1"],
Cell[189501, 4805, 77, 4, 33, "Text"],
Cell[189581, 4811, 7368, 171, 74, "Input"],
Cell[196952, 4984, 62, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[197051, 4990, 58, 1, 47, "Subsection",
  CellTags->"T1.6.2.1"],
Cell[197112, 4993, 81, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[197218, 4998, 139, 5, 43, "Subsubsection",
  CellTags->"T1.6.2.1.1"],
Cell[197360, 5005, 1088, 28, 34, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[198485, 5038, 139, 5, 43, "Subsubsection",
  CellTags->"T1.6.2.2.1"],
Cell[198627, 5045, 1160, 29, 34, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[199824, 5079, 139, 5, 43, "Subsubsection",
  CellTags->"T1.6.2.3.1"],
Cell[199966, 5086, 1160, 29, 34, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[201163, 5120, 139, 5, 43, "Subsubsection",
  CellTags->"T1.6.2.4.1"],
Cell[201305, 5127, 1160, 29, 34, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[202514, 5162, 58, 1, 47, "Subsection",
  CellTags->"T1.6.3.1"],
Cell[202575, 5165, 73, 1, 30, "Input"],
Cell[202651, 5168, 132, 2, 30, "Text"],
Cell[202786, 5172, 3415, 89, 32, "Input"],
Cell[206204, 5263, 66, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[206307, 5269, 159, 5, 47, "Subsection",
  CellTags->"T1.6.4.1"],
Cell[206469, 5276, 50, 1, 30, "Input"],
Cell[206522, 5279, 393, 9, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[206952, 5293, 58, 1, 47, "Subsection",
  CellTags->"T1.6.5.1"],
Cell[207013, 5296, 73, 1, 30, "Input"],
Cell[207089, 5299, 132, 2, 30, "Text"],
Cell[207224, 5303, 3415, 89, 32, "Input"],
Cell[210642, 5394, 66, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[210745, 5400, 159, 5, 47, "Subsection",
  CellTags->"T1.6.6.1"],
Cell[210907, 5407, 50, 1, 30, "Input"],
Cell[210960, 5410, 393, 9, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[211390, 5424, 159, 5, 47, "Subsection",
  CellTags->"T1.6.7.1"],
Cell[211552, 5431, 62, 1, 30, "Input"],
Cell[211617, 5434, 127, 3, 30, "Input"],
Cell[211747, 5439, 393, 9, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[212177, 5453, 159, 5, 47, "Subsection",
  CellTags->"T1.6.8.1"],
Cell[212339, 5460, 62, 1, 30, "Input"],
Cell[212404, 5463, 127, 3, 30, "Input"],
Cell[212534, 5468, 393, 9, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[212964, 5482, 56, 1, 47, "Subsection",
  CellTags->"T1.6.9.1"],
Cell[213023, 5485, 77, 4, 33, "Text"],
Cell[213103, 5491, 6612, 152, 94, "Input"],
Cell[219718, 5645, 62, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[219817, 5651, 59, 1, 47, "Subsection",
  CellTags->"T1.6.10.1"],
Cell[219879, 5654, 79, 1, 30, "Input"],
Cell[219961, 5657, 1084, 28, 34, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[221082, 5690, 57, 1, 47, "Subsection",
  CellTags->"T1.6.11.1"],
Cell[221142, 5693, 454, 20, 33, "Text"],
Cell[221599, 5715, 3684, 95, 32, "Input"],
Cell[225286, 5812, 62, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[225385, 5818, 57, 1, 47, "Subsection",
  CellTags->"T1.6.12.1"],
Cell[225445, 5821, 454, 20, 33, "Text"],
Cell[225902, 5843, 3684, 95, 32, "Input"],
Cell[229589, 5940, 62, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[229688, 5946, 57, 1, 47, "Subsection",
  CellTags->"T1.6.13.1"],
Cell[229748, 5949, 7313, 170, 95, "Input"],
Cell[237064, 6121, 62, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[237163, 6127, 68, 1, 47, "Subsection",
  CellTags->"T1.6.14.1"],
Cell[237234, 6130, 201, 4, 30, "Input"],
Cell[237438, 6136, 105, 2, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[237580, 6143, 59, 1, 47, "Subsection",
  CellTags->"T1.6.15.1"],
Cell[237642, 6146, 22655, 536, 9624, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[260334, 6687, 57, 1, 47, "Subsection",
  CellTags->"T1.6.16.1"],
Cell[260394, 6690, 77, 4, 33, "Text"],
Cell[260474, 6696, 3791, 96, 32, "Input"],
Cell[264268, 6794, 62, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[264367, 6800, 59, 1, 47, "Subsection",
  CellTags->"T1.6.17.1"],
Cell[264429, 6803, 38, 1, 30, "Input"],
Cell[264470, 6806, 79, 1, 30, "Input"],
Cell[264552, 6809, 401, 10, 30, "Input"],

Cell[CellGroupData[{
Cell[264978, 6823, 57, 1, 30, "Input"],
Cell[265038, 6826, 55, 1, 25, "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[265142, 6833, 57, 1, 47, "Subsection",
  CellTags->"T1.6.18.1"],
Cell[265202, 6836, 77, 4, 33, "Text"],
Cell[265282, 6842, 5270, 131, 95, "Input"],
Cell[270555, 6975, 62, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[270654, 6981, 59, 1, 47, "Subsection",
  CellTags->"T1.6.19.1"],
Cell[270716, 6984, 105, 2, 30, "Input"],
Cell[270824, 6988, 401, 10, 30, "Input"],

Cell[CellGroupData[{
Cell[271250, 7002, 57, 1, 30, "Input"],
Cell[271310, 7005, 55, 1, 25, "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[271414, 7012, 59, 1, 47, "Subsection",
  CellTags->"T1.6.20.1"],
Cell[271476, 7015, 133, 3, 30, "Input"],
Cell[271612, 7020, 401, 10, 30, "Input"],

Cell[CellGroupData[{
Cell[272038, 7034, 57, 1, 30, "Input"],
Cell[272098, 7037, 55, 1, 25, "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[272202, 7044, 57, 1, 47, "Subsection",
  CellTags->"T1.6.21.1"],
Cell[272262, 7047, 77, 4, 33, "Text"],
Cell[272342, 7053, 3179, 83, 54, "Input"],
Cell[275524, 7138, 62, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[275623, 7144, 59, 1, 47, "Subsection",
  CellTags->"T1.6.22.1"],
Cell[275685, 7147, 79, 1, 30, "Input"],
Cell[275767, 7150, 369, 9, 30, "Input"],

Cell[CellGroupData[{
Cell[276161, 7163, 57, 1, 30, "Input"],
Cell[276221, 7166, 55, 1, 25, "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[276325, 7173, 57, 1, 47, "Subsection",
  CellTags->"T1.6.23.1"],
Cell[276385, 7176, 77, 4, 33, "Text"],
Cell[276465, 7182, 3892, 100, 54, "Input"],
Cell[280360, 7284, 62, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[280459, 7290, 59, 1, 47, "Subsection",
  CellTags->"T1.6.24.1"],
Cell[280521, 7293, 79, 1, 30, "Input"],
Cell[280603, 7296, 393, 10, 30, "Input"],

Cell[CellGroupData[{
Cell[281021, 7310, 57, 1, 30, "Input"],
Cell[281081, 7313, 55, 1, 25, "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[281185, 7320, 57, 1, 47, "Subsection",
  CellTags->"T1.6.25.1"],
Cell[281245, 7323, 77, 4, 33, "Text"],
Cell[281325, 7329, 3447, 89, 54, "Input"],
Cell[284775, 7420, 62, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[284874, 7426, 59, 1, 47, "Subsection",
  CellTags->"T1.6.26.1"],
Cell[284936, 7429, 79, 1, 30, "Input"],
Cell[285018, 7432, 399, 10, 30, "Input"],

Cell[CellGroupData[{
Cell[285442, 7446, 57, 1, 30, "Input"],
Cell[285502, 7449, 55, 1, 25, "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[285606, 7456, 57, 1, 47, "Subsection",
  CellTags->"T1.6.27.1"],
Cell[285666, 7459, 169, 7, 33, "Text"],
Cell[285838, 7468, 5043, 126, 96, "Input"],
Cell[290884, 7596, 62, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[290983, 7602, 59, 1, 47, "Subsection",
  CellTags->"T1.6.28.1"],
Cell[291045, 7605, 46, 1, 30, "Input"],
Cell[291094, 7608, 773, 16, 93, "Input"],
Cell[291870, 7626, 50, 0, 33, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[291957, 7631, 58, 1, 47, "Subsection",
  CellTags->"T1.6.29.1"],
Cell[292018, 7634, 77, 4, 33, "Text"],
Cell[292098, 7640, 3179, 83, 54, "Input"],
Cell[295280, 7725, 62, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[295379, 7731, 59, 1, 47, "Subsection",
  CellTags->"T1.6.30.1"],
Cell[295441, 7734, 79, 1, 30, "Input"],
Cell[295523, 7737, 376, 9, 30, "Input"],

Cell[CellGroupData[{
Cell[295924, 7750, 69, 1, 30, "Input"],
Cell[295996, 7753, 55, 1, 25, "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[296100, 7760, 58, 1, 47, "Subsection",
  CellTags->"T1.6.31.1"],
Cell[296161, 7763, 292, 14, 33, "Text"],
Cell[296456, 7779, 2494, 68, 32, "Input"],
Cell[298953, 7849, 62, 1, 30, "Input"],
Cell[299018, 7852, 2268, 66, 54, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[301323, 7923, 59, 1, 47, "Subsection",
  CellTags->"T1.6.32.1"],
Cell[301385, 7926, 291, 14, 33, "Text"],
Cell[301679, 7942, 46, 1, 30, "Input"],
Cell[301728, 7945, 659, 18, 30, "Input"],
Cell[302390, 7965, 50, 0, 33, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[302477, 7970, 61, 1, 47, "Subsection",
  CellTags->"T1.6.33.1"],
Cell[302541, 7973, 291, 14, 33, "Text"],
Cell[302835, 7989, 84, 1, 30, "Input"],
Cell[302922, 7992, 665, 18, 30, "Input"],
Cell[303590, 8012, 50, 0, 33, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[303677, 8017, 58, 1, 47, "Subsection",
  CellTags->"T1.6.34.1"],
Cell[303738, 8020, 394, 17, 52, "Text"],
Cell[304135, 8039, 5065, 127, 96, "Input"],
Cell[309203, 8168, 62, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[309302, 8174, 59, 1, 47, "Subsection",
  CellTags->"T1.6.35.1"],
Cell[309364, 8177, 105, 2, 30, "Input"],
Cell[309472, 8181, 1002, 27, 31, "Input"],
Cell[310477, 8210, 50, 0, 33, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[310564, 8215, 58, 1, 47, "Subsection",
  CellTags->"T1.6.36.1"],
Cell[310625, 8218, 209, 7, 52, "Text"],
Cell[310837, 8227, 6354, 147, 75, "Input"],
Cell[317194, 8376, 62, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[317293, 8382, 59, 1, 47, "Subsection",
  CellTags->"T1.6.37.1"],
Cell[317355, 8385, 178, 7, 33, "Text"],
Cell[317536, 8394, 61, 1, 30, "Input"],
Cell[317600, 8397, 563, 14, 30, "Input"],

Cell[CellGroupData[{
Cell[318188, 8415, 57, 1, 30, "Input"],
Cell[318248, 8418, 55, 1, 25, "Print"]
}, Closed]],
Cell[318318, 8422, 643, 16, 30, "Input"],
Cell[318964, 8440, 106, 5, 90, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[319107, 8450, 61, 1, 47, "Subsection",
  CellTags->"T1.6.38.1"],
Cell[319171, 8453, 111, 2, 30, "Input"],
Cell[319285, 8457, 425, 11, 30, "Input"],
Cell[319713, 8470, 50, 1, 30, "Input"],
Cell[319766, 8473, 76, 1, 30, "Input"],
Cell[319845, 8476, 52, 1, 30, "Input"],
Cell[319900, 8479, 128, 2, 30, "Input"],
Cell[320031, 8483, 816, 18, 31, "Input"],
Cell[320850, 8503, 50, 0, 33, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[320937, 8508, 61, 1, 47, "Subsection",
  CellTags->"T1.6.39.1"],
Cell[321001, 8511, 111, 2, 30, "Input"],
Cell[321115, 8515, 66, 1, 30, "Input"],
Cell[321184, 8518, 52, 1, 30, "Input"],
Cell[321239, 8521, 128, 2, 30, "Input"],
Cell[321370, 8525, 555, 12, 31, "Input"],
Cell[321928, 8539, 50, 0, 33, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[322015, 8544, 61, 1, 47, "Subsection",
  CellTags->"T1.6.40.1"],
Cell[322079, 8547, 141, 2, 30, "Input"],
Cell[322223, 8551, 128, 2, 30, "Input"],
Cell[322354, 8555, 417, 10, 30, "Input"],
Cell[322774, 8567, 50, 0, 33, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[322861, 8572, 58, 1, 47, "Subsection",
  CellTags->"T1.6.41.1"],
Cell[322922, 8575, 77, 4, 33, "Text"],
Cell[323002, 8581, 4873, 123, 54, "Input"],
Cell[327878, 8706, 62, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[327977, 8712, 59, 1, 47, "Subsection",
  CellTags->"T1.6.42.1"],
Cell[328039, 8715, 162, 3, 30, "Input"],
Cell[328204, 8720, 460, 11, 30, "Input"],

Cell[CellGroupData[{
Cell[328689, 8735, 57, 1, 30, "Input"],
Cell[328749, 8738, 55, 1, 25, "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[328853, 8745, 68, 1, 47, "Subsection",
  CellTags->"T1.6.43.1"],
Cell[328924, 8748, 111, 3, 33, "Text"],
Cell[329038, 8753, 77, 4, 33, "Text"],
Cell[329118, 8759, 5395, 138, 75, "Input"],
Cell[334516, 8899, 62, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[334615, 8905, 57, 1, 47, "Subsection",
  CellTags->"T1.6.44.1"],
Cell[334675, 8908, 77, 4, 33, "Text"],
Cell[334755, 8914, 4938, 127, 75, "Input"],
Cell[339696, 9043, 9473, 263, 56, "Text"],
Cell[349172, 9308, 62, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[349271, 9314, 59, 1, 47, "Subsection",
  CellTags->"T1.6.45.1"],
Cell[349333, 9317, 132, 3, 30, "Input"],
Cell[349468, 9322, 444, 11, 30, "Input"],

Cell[CellGroupData[{
Cell[349937, 9337, 57, 1, 30, "Input"],
Cell[349997, 9340, 55, 1, 25, "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[350101, 9347, 61, 1, 47, "Subsection",
  CellTags->"T1.6.46.1"],
Cell[350165, 9350, 194, 5, 30, "Input"],
Cell[350362, 9357, 450, 11, 30, "Input"],

Cell[CellGroupData[{
Cell[350837, 9372, 59, 1, 30, "Input"],
Cell[350899, 9375, 55, 1, 25, "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[351003, 9382, 57, 1, 47, "Subsection",
  CellTags->"T1.6.47.1"],
Cell[351063, 9385, 77, 4, 33, "Text"],
Cell[351143, 9391, 5742, 138, 75, "Input"],
Cell[356888, 9531, 62, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[356987, 9537, 59, 1, 47, "Subsection",
  CellTags->"T1.6.48.1"],
Cell[357049, 9540, 132, 3, 30, "Input"],
Cell[357184, 9545, 456, 11, 30, "Input"],

Cell[CellGroupData[{
Cell[357665, 9560, 57, 1, 30, "Input"],
Cell[357725, 9563, 55, 1, 25, "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[357829, 9570, 61, 1, 47, "Subsection",
  CellTags->"T1.6.49.1"],
Cell[357893, 9573, 194, 5, 30, "Input"],
Cell[358090, 9580, 466, 11, 50, "Input"],

Cell[CellGroupData[{
Cell[358581, 9595, 59, 1, 30, "Input"],
Cell[358643, 9598, 55, 1, 25, "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[358747, 9605, 57, 1, 47, "Subsection",
  CellTags->"T1.6.50.1"],
Cell[358807, 9608, 77, 4, 33, "Text"],
Cell[358887, 9614, 12332, 293, 161, "Input"],
Cell[371222, 9909, 62, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[371321, 9915, 59, 1, 47, "Subsection",
  CellTags->"T1.6.51.1"],
Cell[371383, 9918, 46, 1, 30, "Input"],
Cell[371432, 9921, 432, 11, 30, "Input"],
Cell[371867, 9934, 104, 3, 52, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[372008, 9942, 57, 1, 47, "Subsection",
  CellTags->"T1.6.52.1"],
Cell[372068, 9945, 157, 7, 33, "Text"],
Cell[372228, 9954, 5042, 126, 91, "Input"],
Cell[377273, 10082, 62, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[377372, 10088, 59, 1, 47, "Subsection",
  CellTags->"T1.6.53.1"],
Cell[377434, 10091, 147, 3, 30, "Input"],
Cell[377584, 10096, 142, 3, 30, "Input"],

Cell[CellGroupData[{
Cell[377751, 10103, 57, 1, 30, "Input"],
Cell[377811, 10106, 55, 1, 25, "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[377915, 10113, 61, 1, 47, "Subsection",
  CellTags->"T1.6.54.1"],
Cell[377979, 10116, 172, 3, 30, "Input"],
Cell[378154, 10121, 148, 3, 30, "Input"],

Cell[CellGroupData[{
Cell[378327, 10128, 59, 1, 30, "Input"],
Cell[378389, 10131, 55, 1, 25, "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[378493, 10138, 61, 1, 47, "Subsection",
  CellTags->"T1.6.55.1"],
Cell[378557, 10141, 172, 3, 30, "Input"],
Cell[378732, 10146, 148, 3, 30, "Input"],

Cell[CellGroupData[{
Cell[378905, 10153, 59, 1, 30, "Input"],
Cell[378967, 10156, 55, 1, 25, "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[379071, 10163, 57, 1, 47, "Subsection",
  CellTags->"T1.6.56.1"],
Cell[379131, 10166, 147, 7, 33, "Text"],
Cell[379281, 10175, 5562, 141, 75, "Input"],
Cell[384846, 10318, 62, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[384945, 10324, 59, 1, 47, "Subsection",
  CellTags->"T1.6.57.1"],
Cell[385007, 10327, 89, 1, 30, "Input"],
Cell[385099, 10330, 142, 3, 30, "Input"],

Cell[CellGroupData[{
Cell[385266, 10337, 57, 1, 30, "Input"],
Cell[385326, 10340, 55, 1, 25, "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[385430, 10347, 57, 1, 47, "Subsection",
  CellTags->"T1.6.58.1"],
Cell[385490, 10350, 138, 6, 33, "Text"],
Cell[385631, 10358, 4317, 107, 96, "Input"],
Cell[389951, 10467, 62, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[390050, 10473, 59, 1, 47, "Subsection",
  CellTags->"T1.6.59.1"],
Cell[390112, 10476, 105, 2, 30, "Input"],
Cell[390220, 10480, 158, 4, 30, "Input"],

Cell[CellGroupData[{
Cell[390403, 10488, 57, 1, 30, "Input"],
Cell[390463, 10491, 55, 1, 25, "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[390567, 10498, 57, 1, 47, "Subsection",
  CellTags->"T1.6.60.1"],
Cell[390627, 10501, 156, 7, 33, "Text"],
Cell[390786, 10510, 4817, 120, 75, "Input"],
Cell[395606, 10632, 62, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[395705, 10638, 59, 1, 47, "Subsection",
  CellTags->"T1.6.61.1"],
Cell[395767, 10641, 169, 3, 30, "Input"],
Cell[395939, 10646, 185, 4, 30, "Input"],

Cell[CellGroupData[{
Cell[396149, 10654, 57, 1, 30, "Input"],
Cell[396209, 10657, 55, 1, 25, "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[396313, 10664, 61, 1, 47, "Subsection",
  CellTags->"T1.6.62.1"],
Cell[396377, 10667, 194, 3, 30, "Input"],
Cell[396574, 10672, 191, 4, 30, "Input"],

Cell[CellGroupData[{
Cell[396790, 10680, 59, 1, 30, "Input"],
Cell[396852, 10683, 55, 1, 25, "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[396956, 10690, 61, 1, 47, "Subsection",
  CellTags->"T1.6.63.1"],
Cell[397020, 10693, 218, 4, 30, "Input"],
Cell[397241, 10699, 191, 4, 30, "Input"],

Cell[CellGroupData[{
Cell[397457, 10707, 59, 1, 30, "Input"],
Cell[397519, 10710, 55, 1, 25, "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[397623, 10717, 57, 1, 47, "Subsection",
  CellTags->"T1.6.64.1"],
Cell[397683, 10720, 146, 7, 33, "Text"],
Cell[397832, 10729, 10108, 237, 175, "Input"],
Cell[407943, 10968, 62, 1, 30, "Input"],
Cell[408008, 10971, 50, 1, 30, "Input"],
Cell[408061, 10974, 127, 2, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[408225, 10981, 59, 1, 47, "Subsection",
  CellTags->"T1.6.65.1"],
Cell[408287, 10984, 116, 2, 30, "Input"],
Cell[408406, 10988, 185, 4, 30, "Input"],

Cell[CellGroupData[{
Cell[408616, 10996, 57, 1, 30, "Input"],
Cell[408676, 10999, 55, 1, 25, "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[408780, 11006, 61, 1, 47, "Subsection",
  CellTags->"T1.6.66.1"],
Cell[408844, 11009, 152, 3, 30, "Input"],
Cell[408999, 11014, 191, 4, 30, "Input"],

Cell[CellGroupData[{
Cell[409215, 11022, 59, 1, 30, "Input"],
Cell[409277, 11025, 55, 1, 25, "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[409381, 11032, 61, 1, 47, "Subsection",
  CellTags->"T1.6.67.1"],
Cell[409445, 11035, 152, 3, 30, "Input"],
Cell[409600, 11040, 191, 4, 30, "Input"],

Cell[CellGroupData[{
Cell[409816, 11048, 59, 1, 30, "Input"],
Cell[409878, 11051, 55, 1, 25, "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[409982, 11058, 57, 1, 47, "Subsection",
  CellTags->"T1.6.68.1"],
Cell[410042, 11061, 138, 6, 33, "Text"],
Cell[410183, 11069, 6617, 157, 97, "Input"],
Cell[416803, 11228, 62, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[416902, 11234, 59, 1, 47, "Subsection",
  CellTags->"T1.6.69.1"],
Cell[416964, 11237, 116, 2, 30, "Input"],
Cell[417083, 11241, 178, 4, 30, "Input"],

Cell[CellGroupData[{
Cell[417286, 11249, 57, 1, 30, "Input"],
Cell[417346, 11252, 55, 1, 25, "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[417450, 11259, 57, 1, 47, "Subsection",
  CellTags->"T1.6.70.1"],

Cell[CellGroupData[{
Cell[417532, 11264, 56, 1, 43, "Subsubsection",
  CellTags->"T1.6.70.1.1"],
Cell[417591, 11267, 72, 1, 30, "Input"],
Cell[417666, 11270, 193, 4, 31, "Input"],
Cell[417862, 11276, 269, 4, 31, "Input"],
Cell[418134, 11282, 941, 24, 73, "Input"],
Cell[419078, 11308, 83, 1, 31, "Input"],
Cell[419164, 11311, 83, 1, 31, "Input"],
Cell[419250, 11314, 83, 1, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[419370, 11320, 56, 1, 43, "Subsubsection",
  CellTags->"T1.6.70.2.1"],
Cell[419429, 11323, 779, 18, 63, "Input"],
Cell[420211, 11343, 83, 1, 31, "Input"],
Cell[420297, 11346, 83, 1, 31, "Input"],
Cell[420383, 11349, 83, 1, 31, "Input"],
Cell[420469, 11352, 47, 1, 30, "Input"],
Cell[420519, 11355, 85, 5, 33, "Text"],
Cell[420607, 11362, 5185, 132, 118, "Input"],
Cell[425795, 11496, 62, 1, 30, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[425906, 11503, 59, 1, 47, "Subsection",
  CellTags->"T1.6.71.1"],
Cell[425968, 11506, 46, 1, 30, "Input"],
Cell[426017, 11509, 164, 4, 30, "Input"],
Cell[426184, 11515, 49, 0, 33, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[426270, 11520, 57, 1, 47, "Subsection",
  CellTags->"T1.6.72.1"],
Cell[426330, 11523, 85, 5, 33, "Text"],
Cell[426418, 11530, 9126, 208, 264, "Input"],
Cell[435547, 11740, 62, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[435646, 11746, 59, 1, 47, "Subsection",
  CellTags->"T1.6.73.1"],
Cell[435708, 11749, 51, 1, 30, "Input"],
Cell[435762, 11752, 140, 3, 30, "Input"],
Cell[435905, 11757, 49, 0, 33, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[435991, 11762, 57, 1, 47, "Subsection",
  CellTags->"T1.6.74.1"],
Cell[436051, 11765, 77, 4, 33, "Text"],
Cell[436131, 11771, 2490, 68, 32, "Input"],
Cell[438624, 11841, 66, 1, 30, "Input"],
Cell[438693, 11844, 2377, 65, 32, "Input"],
Cell[441073, 11911, 66, 1, 30, "Input"],
Cell[441142, 11914, 2600, 67, 32, "Input"],
Cell[443745, 11983, 66, 1, 30, "Input"],
Cell[443814, 11986, 3551, 90, 32, "Input"],
Cell[447368, 12078, 66, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[447471, 12084, 60, 1, 47, "Subsection",
  CellTags->"T1.6.75.1"],
Cell[447534, 12087, 77, 1, 30, "Input"],
Cell[447614, 12090, 160, 4, 30, "Input"],
Cell[447777, 12096, 49, 0, 33, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[447863, 12101, 60, 1, 47, "Subsection",
  CellTags->"T1.6.76.1"],
Cell[447926, 12104, 124, 2, 30, "Input"],
Cell[448053, 12108, 176, 4, 30, "Input"],
Cell[448232, 12114, 49, 0, 33, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[448318, 12119, 60, 1, 47, "Subsection",
  CellTags->"T1.6.77.1"],
Cell[448381, 12122, 49, 1, 30, "Input"],
Cell[448433, 12125, 187, 4, 30, "Input"],
Cell[448623, 12131, 49, 0, 33, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[448709, 12136, 60, 1, 47, "Subsection",
  CellTags->"T1.6.78.1"],
Cell[448772, 12139, 77, 1, 30, "Input"],
Cell[448852, 12142, 190, 4, 30, "Input"],
Cell[449045, 12148, 49, 0, 33, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[449131, 12153, 57, 1, 47, "Subsection",
  CellTags->"T1.6.79.1"],
Cell[449191, 12156, 135, 4, 33, "Text"],
Cell[449329, 12162, 66, 1, 30, "Input"],
Cell[449398, 12165, 187, 3, 30, "Text"],
Cell[449588, 12170, 4773, 114, 54, "Input"],
Cell[454364, 12286, 62, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[454463, 12292, 57, 1, 47, "Subsection",
  CellTags->"T1.6.80.1"],
Cell[454523, 12295, 236, 6, 33, "Text"],
Cell[454762, 12303, 2101, 55, 32, "Input"],
Cell[456866, 12360, 62, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[456965, 12366, 59, 1, 47, "Subsection",
  CellTags->"T1.6.81.1"],
Cell[457027, 12369, 74, 1, 30, "Input"],
Cell[457104, 12372, 137, 3, 30, "Input"],
Cell[457244, 12377, 49, 0, 33, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[457330, 12382, 57, 1, 47, "Subsection",
  CellTags->"T1.6.82.1"],
Cell[457390, 12385, 77, 4, 33, "Text"],
Cell[457470, 12391, 3018, 76, 32, "Input"],
Cell[460491, 12469, 62, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[460590, 12475, 59, 1, 47, "Subsection",
  CellTags->"T1.6.83.1"],
Cell[460652, 12478, 74, 1, 30, "Input"],
Cell[460729, 12481, 140, 3, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[460906, 12489, 61, 1, 47, "Subsection",
  CellTags->"T1.6.84.1"],
Cell[460970, 12492, 121, 2, 30, "Input"],
Cell[461094, 12496, 90, 1, 30, "Input"],
Cell[461187, 12499, 179, 4, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[461403, 12508, 61, 1, 47, "Subsection",
  CellTags->"T1.6.85.1"],
Cell[461467, 12511, 166, 3, 30, "Input"],
Cell[461636, 12516, 90, 1, 30, "Input"],
Cell[461729, 12519, 179, 4, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[461945, 12528, 57, 1, 47, "Subsection",
  CellTags->"T1.6.86.1"],
Cell[462005, 12531, 77, 4, 33, "Text"],
Cell[462085, 12537, 2104, 59, 32, "Input"],
Cell[464192, 12598, 62, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[464291, 12604, 59, 1, 47, "Subsection",
  CellTags->"T1.6.87.1"],
Cell[464353, 12607, 74, 1, 30, "Input"],
Cell[464430, 12610, 161, 4, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[464628, 12619, 57, 1, 47, "Subsection",
  CellTags->"T1.6.88.1"],
Cell[464688, 12622, 77, 4, 33, "Text"],
Cell[464768, 12628, 2088, 56, 32, "Input"],
Cell[466859, 12686, 62, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[466958, 12692, 59, 1, 47, "Subsection",
  CellTags->"T1.6.89.1"],
Cell[467020, 12695, 46, 1, 30, "Input"],
Cell[467069, 12698, 141, 3, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[467247, 12706, 57, 1, 47, "Subsection",
  CellTags->"T1.6.90.1"],
Cell[467307, 12709, 77, 4, 33, "Text"],
Cell[467387, 12715, 1286, 36, 32, "Input"],
Cell[468676, 12753, 62, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[468775, 12759, 59, 1, 47, "Subsection",
  CellTags->"T1.6.91.1"],
Cell[468837, 12762, 74, 1, 30, "Input"],
Cell[468914, 12765, 128, 3, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[469079, 12773, 57, 1, 47, "Subsection",
  CellTags->"T1.6.92.1"],
Cell[469139, 12776, 77, 4, 33, "Text"],
Cell[469219, 12782, 1144, 32, 32, "Input"],
Cell[470366, 12816, 62, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[470465, 12822, 59, 1, 47, "Subsection",
  CellTags->"T1.6.93.1"],
Cell[470527, 12825, 46, 1, 30, "Input"],
Cell[470576, 12828, 130, 3, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[470743, 12836, 57, 1, 47, "Subsection",
  CellTags->"T1.6.94.1"],
Cell[470803, 12839, 114, 2, 29, "Text"],
Cell[470920, 12843, 1616, 44, 35, "Text"],
Cell[472539, 12889, 2050, 56, 32, "Input"],
Cell[474592, 12947, 62, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[474691, 12953, 59, 1, 47, "Subsection",
  CellTags->"T1.6.95.1"],
Cell[474753, 12956, 74, 1, 30, "Input"],
Cell[474830, 12959, 137, 3, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[475004, 12967, 61, 1, 47, "Subsection",
  CellTags->"T1.6.96.1"],
Cell[475068, 12970, 128, 3, 30, "Input"],
Cell[475199, 12975, 143, 3, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[475379, 12983, 61, 1, 47, "Subsection",
  CellTags->"T1.6.97.1"],
Cell[475443, 12986, 128, 3, 30, "Input"],
Cell[475574, 12991, 143, 3, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[475754, 12999, 58, 1, 47, "Subsection",
  CellTags->"T1.6.98.1"],
Cell[475815, 13002, 39, 0, 33, "Text"],
Cell[475857, 13004, 2876, 77, 33, "Text"],
Cell[478736, 13083, 73, 1, 29, "Text"],
Cell[478812, 13086, 3007, 83, 32, "Input"],
Cell[481822, 13171, 64, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[481923, 13177, 60, 1, 47, "Subsection",
  CellTags->"T1.6.99.1"],
Cell[481986, 13180, 48, 1, 30, "Input"],
Cell[482037, 13183, 154, 3, 30, "Input"],

Cell[CellGroupData[{
Cell[482216, 13190, 58, 1, 30, "Input"],
Cell[482277, 13193, 55, 1, 25, "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[482381, 13200, 62, 1, 47, "Subsection",
  CellTags->"T1.6.100.1"],
Cell[482446, 13203, 85, 1, 30, "Input"],
Cell[482534, 13206, 157, 3, 30, "Input"],

Cell[CellGroupData[{
Cell[482716, 13213, 59, 1, 30, "Input"],
Cell[482778, 13216, 55, 1, 25, "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[482882, 13223, 62, 1, 47, "Subsection",
  CellTags->"T1.6.101.1"],
Cell[482947, 13226, 124, 3, 30, "Input"],
Cell[483074, 13231, 157, 3, 30, "Input"],

Cell[CellGroupData[{
Cell[483256, 13238, 59, 1, 30, "Input"],
Cell[483318, 13241, 55, 1, 25, "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[483422, 13248, 62, 1, 47, "Subsection",
  CellTags->"T1.6.102.1"],
Cell[483487, 13251, 85, 1, 30, "Input"],
Cell[483575, 13254, 157, 3, 30, "Input"],

Cell[CellGroupData[{
Cell[483757, 13261, 59, 1, 30, "Input"],
Cell[483819, 13264, 55, 1, 25, "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[483923, 13271, 62, 1, 47, "Subsection",
  CellTags->"T1.6.103.1"],
Cell[483988, 13274, 85, 1, 30, "Input"],
Cell[484076, 13277, 157, 3, 30, "Input"],

Cell[CellGroupData[{
Cell[484258, 13284, 59, 1, 30, "Input"],
Cell[484320, 13287, 55, 1, 25, "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[484424, 13294, 62, 1, 47, "Subsection",
  CellTags->"T1.6.104.1"],
Cell[484489, 13297, 124, 3, 30, "Input"],
Cell[484616, 13302, 157, 3, 30, "Input"],

Cell[CellGroupData[{
Cell[484798, 13309, 59, 1, 30, "Input"],
Cell[484860, 13312, 55, 1, 25, "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[484964, 13319, 59, 1, 47, "Subsection",
  CellTags->"T1.6.105.1"],
Cell[485026, 13322, 59, 1, 31, "Text"],
Cell[485088, 13325, 154, 2, 33, "Text"],
Cell[485245, 13329, 455, 9, 90, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[485737, 13343, 61, 1, 47, "Subsection",
  CellTags->"T1.6.106.1"],
Cell[485801, 13346, 48, 1, 30, "Input"],
Cell[485852, 13349, 70, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[485959, 13355, 59, 1, 47, "Subsection",
  CellTags->"T1.6.107.1"],
Cell[486021, 13358, 59, 1, 31, "Text"],
Cell[486083, 13361, 154, 2, 33, "Text"],
Cell[486240, 13365, 565, 13, 90, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[486842, 13383, 61, 1, 47, "Subsection",
  CellTags->"T1.6.108.1"],
Cell[486906, 13386, 48, 1, 30, "Input"],
Cell[486957, 13389, 120, 3, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[487114, 13397, 59, 1, 47, "Subsection",
  CellTags->"T1.6.109.1"],
Cell[487176, 13400, 59, 1, 31, "Text"],
Cell[487238, 13403, 154, 2, 33, "Text"],
Cell[487395, 13407, 402, 8, 90, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[487834, 13420, 61, 1, 47, "Subsection",
  CellTags->"T1.6.110.1"],
Cell[487898, 13423, 48, 1, 30, "Input"],
Cell[487949, 13426, 70, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[488056, 13432, 59, 1, 47, "Subsection",
  CellTags->"T1.6.111.1"],
Cell[488118, 13435, 59, 1, 31, "Text"],
Cell[488180, 13438, 311, 6, 70, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[488528, 13449, 61, 1, 47, "Subsection",
  CellTags->"T1.6.112.1"],
Cell[488592, 13452, 48, 1, 30, "Input"],
Cell[488643, 13455, 70, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[488750, 13461, 59, 1, 47, "Subsection",
  CellTags->"T1.6.113.1"],
Cell[488812, 13464, 43, 0, 33, "Text"],
Cell[488858, 13466, 105, 1, 30, "Text"],
Cell[488966, 13469, 6709, 171, 76, "Input"],
Cell[495678, 13642, 191, 3, 31, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[495906, 13650, 61, 1, 47, "Subsection",
  CellTags->"T1.6.114.1"],
Cell[495970, 13653, 76, 1, 30, "Input"],
Cell[496049, 13656, 422, 10, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[496508, 13671, 61, 1, 47, "Subsection",
  CellTags->"T1.6.115.1"],
Cell[496572, 13674, 692, 29, 498, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[497301, 13708, 61, 1, 47, "Subsection",
  CellTags->"T1.6.116.1"],
Cell[497365, 13711, 696, 29, 498, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[498098, 13745, 62, 1, 47, "Subsection",
  CellTags->"T1.6.117.1"],
Cell[498163, 13748, 1083, 19, 310, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[499283, 13772, 62, 1, 47, "Subsection",
  CellTags->"T1.6.118.1"],
Cell[499348, 13775, 1046, 18, 290, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[500431, 13798, 62, 1, 47, "Subsection",
  CellTags->"T1.6.119.1"],
Cell[500496, 13801, 1044, 18, 290, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[501577, 13824, 61, 1, 47, "Subsection",
  CellTags->"T1.6.120.1"],
Cell[501641, 13827, 240, 6, 102, "Input"],

Cell[CellGroupData[{
Cell[501906, 13837, 4713, 86, 176, "Input"],
Cell[506622, 13925, 357, 6, 76, "Print"]
}, Closed]],
Cell[506994, 13934, 40, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[507059, 13939, 586, 11, 131, "Input"],
Cell[507648, 13952, 357, 6, 76, "Print"]
}, Closed]],
Cell[508020, 13961, 40, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[508097, 13967, 55, 1, 47, "Subsection",
  CellTags->"Tsymmetry"],
Cell[508155, 13970, 108, 3, 48, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[508300, 13978, 70, 1, 47, "Subsection",
  CellTags->"specialTVR"],
Cell[508373, 13981, 579, 11, 192, "Input"],
Cell[508955, 13994, 1683, 27, 462, "Input"],
Cell[510641, 14023, 1597, 26, 444, "Input"],
Cell[512241, 14051, 3547, 56, 1002, "Input"],
Cell[515791, 14109, 3261, 52, 930, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[519089, 14166, 60, 1, 47, "Subsection",
  CellTags->"T1.6.121.1"],
Cell[519152, 14169, 394, 7, 90, "Input"],
Cell[519549, 14178, 427, 8, 90, "Input"],
Cell[519979, 14188, 427, 8, 90, "Input"],
Cell[520409, 14198, 425, 8, 90, "Input"],
Cell[520837, 14208, 220, 14, 246, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[521094, 14227, 61, 1, 47, "Subsection",
  CellTags->"T1.6.122.1"],
Cell[521158, 14230, 273, 5, 70, "Input"],
Cell[521434, 14237, 273, 5, 70, "Input"],
Cell[521710, 14244, 133, 3, 48, "Input"],
Cell[521846, 14249, 264, 5, 70, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[522147, 14259, 60, 1, 47, "Subsection",
  CellTags->"T1.6.123.1"],
Cell[522210, 14262, 194, 4, 50, "Input"],
Cell[522407, 14268, 194, 4, 50, "Input"],
Cell[522604, 14274, 194, 4, 50, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[522835, 14283, 58, 1, 47, "Subsection",
  CellTags->"T1.6.124.1"],
Cell[522896, 14286, 185, 4, 50, "Input"],
Cell[523084, 14292, 185, 4, 50, "Input"],
Cell[523272, 14298, 87, 1, 30, "Input"],
Cell[523362, 14301, 85, 1, 30, "Input"],
Cell[523450, 14304, 88, 1, 30, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[523587, 14311, 108, 2, 39, "Section",
  CellTags->"T1.7.1"],

Cell[CellGroupData[{
Cell[523720, 14317, 70, 1, 47, "Subsection",
  CellTags->"T1.7.1.1"],
Cell[523793, 14320, 45, 0, 33, "Text"],
Cell[523841, 14322, 57, 1, 30, "Input"],
Cell[523901, 14325, 145, 3, 30, "Input"],
Cell[524049, 14330, 501, 13, 30, "Input"],
Cell[524553, 14345, 127, 2, 30, "Input"],
Cell[524683, 14349, 503, 13, 30, "Input"],
Cell[525189, 14364, 129, 2, 30, "Input"],
Cell[525321, 14368, 84, 3, 33, "Text"],
Cell[525408, 14373, 60, 1, 30, "Input"],
Cell[525471, 14376, 111, 2, 30, "Input"],
Cell[525585, 14380, 216, 7, 109, "Text",
  CellTags->"TLRdef"],
Cell[525804, 14389, 112, 2, 30, "Input"],
Cell[525919, 14393, 438, 10, 30, "Input"],
Cell[526360, 14405, 26, 0, 33, "Text"],
Cell[526389, 14407, 928, 16, 203, "Input"],
Cell[527320, 14425, 72, 1, 30, "Text"],
Cell[527395, 14428, 926, 16, 203, "Input"],
Cell[528324, 14446, 82, 1, 29, "Text"],
Cell[528409, 14449, 750, 15, 158, "Input"],
Cell[529162, 14466, 82, 1, 29, "Text"],
Cell[529247, 14469, 771, 15, 158, "Input"],
Cell[530021, 14486, 85, 1, 29, "Text"],
Cell[530109, 14489, 836, 17, 192, "Input"],
Cell[530948, 14508, 37, 0, 33, "Text"],
Cell[530988, 14510, 658, 13, 113, "Input"],
Cell[531649, 14525, 37, 0, 33, "Text"],
Cell[531689, 14527, 673, 14, 113, "Input"],
Cell[532365, 14543, 35, 0, 33, "Text"],
Cell[532403, 14545, 730, 11, 274, "Input"],
Cell[533136, 14558, 31, 0, 33, "Text"],
Cell[533170, 14560, 224, 6, 109, "Text"],
Cell[533397, 14568, 31, 0, 33, "Text"],
Cell[533431, 14570, 204, 5, 84, "Input"],
Cell[533638, 14577, 148, 4, 52, "Text"],
Cell[533789, 14583, 443, 15, 246, "Input"],
Cell[534235, 14600, 443, 15, 246, "Input"],
Cell[534681, 14617, 49, 0, 33, "Text"],
Cell[534733, 14619, 374, 6, 116, "Input"],
Cell[535110, 14627, 49, 0, 33, "Text"],
Cell[535162, 14629, 374, 6, 116, "Input"],
Cell[535539, 14637, 25, 0, 33, "Text"],
Cell[535567, 14639, 255, 6, 50, "Input"],
Cell[535825, 14647, 246, 5, 50, "Input"],
Cell[536074, 14654, 78, 1, 33, "Text",
  CellTags->"tlrule17"],
Cell[536155, 14657, 530, 9, 168, "Input"],
Cell[536688, 14668, 986, 19, 271, "Input"],
Cell[537677, 14689, 986, 19, 271, "Input"],
Cell[538666, 14710, 37, 0, 33, "Text"],
Cell[538706, 14712, 499, 8, 168, "Input"],
Cell[539208, 14722, 287, 5, 50, "Input"],
Cell[539498, 14729, 358, 7, 65, "Input"],
Cell[539859, 14738, 6408, 176, 162, "Text"],
Cell[546270, 14916, 895, 17, 171, "Input"],
Cell[547168, 14935, 12636, 333, 105, "Text"],
Cell[559807, 15270, 954, 16, 216, "Input"],
Cell[560764, 15288, 65, 0, 33, "Text"],
Cell[560832, 15290, 432, 8, 70, "Input"],
Cell[561267, 15300, 65, 1, 30, "Input"],
Cell[561335, 15303, 70, 1, 30, "Input"],
Cell[561408, 15306, 92, 1, 30, "Input"],
Cell[561503, 15309, 92, 1, 30, "Input"],
Cell[561598, 15312, 92, 1, 30, "Input"],
Cell[561693, 15315, 92, 1, 30, "Input"],
Cell[561788, 15318, 92, 1, 30, "Input"],
Cell[561883, 15321, 92, 1, 30, "Input"],
Cell[561978, 15324, 74, 1, 30, "Input"],
Cell[562055, 15327, 88, 1, 30, "Input"],
Cell[562146, 15330, 108, 2, 30, "Input"],
Cell[562257, 15334, 250, 6, 30, "Input"],
Cell[562510, 15342, 55, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[562602, 15348, 50, 1, 31, "Subsection",
  CellTags->"T1.7.2.1"],
Cell[562655, 15351, 86, 3, 52, "Text"],
Cell[562744, 15356, 668, 12, 105, "Input"],
Cell[563415, 15370, 202, 4, 50, "Input"],
Cell[563620, 15376, 202, 4, 50, "Input"],
Cell[563825, 15382, 668, 12, 129, "Input"],
Cell[564496, 15396, 163, 2, 30, "Input"],
Cell[564662, 15400, 163, 2, 30, "Input"],
Cell[564828, 15404, 505, 9, 69, "Input"],
Cell[565336, 15415, 164, 2, 30, "Input"],
Cell[565503, 15419, 164, 2, 30, "Input"],
Cell[565670, 15423, 545, 9, 82, "Input"],
Cell[566218, 15434, 556, 9, 97, "Input"],
Cell[566777, 15445, 22, 0, 33, "Text"],
Cell[566802, 15447, 130, 3, 30, "Input"],
Cell[566935, 15452, 87, 1, 30, "Input"],
Cell[567025, 15455, 501, 9, 91, "Input"],
Cell[567529, 15466, 537, 9, 100, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[568103, 15480, 54, 1, 31, "Subsection",
  CellTags->"T1.7.3.1"],
Cell[568160, 15483, 61, 1, 30, "Input"],
Cell[568224, 15486, 1380, 26, 250, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[569641, 15517, 52, 1, 31, "Subsection",
  CellTags->"$TTable"],
Cell[569696, 15520, 50, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[569783, 15526, 59, 1, 31, "Subsection",
  CellTags->"T1.7.4.1"],
Cell[569845, 15529, 105, 2, 30, "Input"],
Cell[569953, 15533, 273, 5, 70, "Input"],
Cell[570229, 15540, 191, 3, 50, "Input"],
Cell[570423, 15545, 8922, 166, 1070, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[579394, 15717, 92, 2, 39, "Section",
  CellTags->"T1.8.1"],
Cell[579489, 15721, 154, 2, 70, "Input"],
Cell[579646, 15725, 167, 2, 50, "Input"],
Cell[579816, 15729, 8011, 161, 525, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[587864, 15895, 93, 2, 39, "Section",
  CellTags->"T1.9.1"],

Cell[CellGroupData[{
Cell[587982, 15901, 54, 1, 47, "Subsection",
  CellTags->"T1.9.1.1"],
Cell[588039, 15904, 61, 0, 33, "Text"],
Cell[588103, 15906, 185, 3, 50, "Input"],
Cell[588291, 15911, 64, 1, 30, "Input"],
Cell[588358, 15914, 71, 1, 30, "Input"],
Cell[588432, 15917, 53, 1, 30, "Input"],
Cell[588488, 15920, 1836, 32, 190, "Input"],
Cell[590327, 15954, 53, 1, 30, "Input"],
Cell[590383, 15957, 129, 3, 30, "Input"],
Cell[590515, 15962, 81, 1, 30, "Input"],
Cell[590599, 15965, 48, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[590672, 15970, 137, 3, 30, "Input"],
Cell[590812, 15975, 325, 6, 25, "Print"]
}, Closed]],
Cell[591152, 15984, 143, 2, 27, "Input"],
Cell[591298, 15988, 49, 1, 30, "Input"],
Cell[591350, 15991, 53, 1, 30, "Input"],
Cell[591406, 15994, 45, 1, 30, "Input"],
Cell[591454, 15997, 53, 1, 30, "Input"],
Cell[591510, 16000, 43, 1, 30, "Input"],
Cell[591556, 16003, 50, 1, 30, "Input"],
Cell[591609, 16006, 511, 8, 210, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[592169, 16020, 95, 2, 39, "Section",
  CellTags->"T1.10.1"],
Cell[592267, 16024, 111, 3, 33, "Text"],

Cell[CellGroupData[{
Cell[592403, 16031, 1676, 54, 912, "Input"],
Cell[594082, 16087, 14611, 364, 334, "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[608742, 16457, 101, 2, 39, "Section",
  CellTags->"T1.11.1"],
Cell[608846, 16461, 45, 1, 30, "Input"],
Cell[608894, 16464, 2309, 55, 966, "Input"],
Cell[611206, 16521, 66, 1, 30, "Input"],
Cell[611275, 16524, 89, 3, 33, "Text"],

Cell[CellGroupData[{
Cell[611389, 16531, 114, 2, 30, "Input"],
Cell[611506, 16535, 75, 1, 29, "Output"]
}, Closed]],
Cell[611596, 16539, 55, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[611676, 16543, 137, 2, 30, "Input"],
Cell[611816, 16547, 64, 1, 29, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[611917, 16553, 62, 1, 30, "Input"],
Cell[611982, 16556, 64, 1, 29, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[612083, 16562, 44, 1, 30, "Input"],
Cell[612130, 16565, 130, 2, 29, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[612297, 16572, 48, 1, 30, "Input"],
Cell[612348, 16575, 59, 1, 29, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[612444, 16581, 46, 1, 30, "Input"],
Cell[612493, 16584, 58, 1, 29, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[612588, 16590, 43, 1, 30, "Input"],
Cell[612634, 16593, 69, 1, 29, "Output"]
}, Closed]],
Cell[612718, 16597, 35, 1, 30, "Input"]
}, Closed]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

